@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.bnf;
@@ -51,4 +51,19 @@ public interface BnfVisitor {
      */
     void visitRuleOptional(Rule rule);
 
+    /**
+     * Visit an OR list of optional rules.
+     *
+     * @param list the optional rules
+     */
+    void visitRuleOptional(ArrayList<Rule> list);
+
+    /**
+     * Visit a rule with non-standard extension.
+     *
+     * @param rule the rule
+     * @param compatibility whether this rule exists for compatibility only
+     */
+    void visitRuleExtension(Rule rule, boolean compatibility);
+
 }
