@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.bnf;
@@ -22,6 +22,7 @@ public class RuleFixed implements Rule {
     public static final int HEX_START = 10, CONCAT = 11;
     public static final int AZ_UNDERSCORE = 12, AF = 13, DIGIT = 14;
     public static final int OPEN_BRACKET = 15, CLOSE_BRACKET = 16;
+    public static final int JSON_TEXT = 17;
 
     private final int type;
 
@@ -115,6 +116,7 @@ public class RuleFixed implements Rule {
             }
             break;
         case ANY_WORD:
+        case JSON_TEXT:
             while (s.length() > 0 && !Bnf.startWithSpace(s)) {
                 s = s.substring(1);
             }
@@ -208,4 +210,9 @@ public class RuleFixed implements Rule {
         return false;
     }
 
+    @Override
+    public String toString() {
+        return "#" + type;
+    }
+
 }
