@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.bnf;
@@ -44,4 +44,9 @@ public class RuleRepeat implements Rule {
         return true;
     }
 
+    @Override
+    public String toString() {
+        return comma ? ", ..." : " ...";
+    }
+
 }
