@@ -1,15 +1,15 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.bnf.context;
 
 import java.sql.DatabaseMetaData;
+import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
-import org.h2.util.New;
 
 /**
  * Contains meta data information about a table or a view.
@@ -89,10 +89,20 @@ public class DbTableOrView {
      * Read the column for this table from the database meta data.
      *
      * @param meta the database meta data
+     * @param ps prepared statement with custom query for H2 database, null for
+     *           others
+     * @throws SQLException on failure
      */
-    public void readColumns(DatabaseMetaData meta) throws SQLException {
-        ResultSet rs = meta.getColumns(null, schema.name, name, null);
-        ArrayList<DbColumn> list = New.arrayList();
+    public void readColumns(DatabaseMetaData meta, PreparedStatement ps) throws SQLException {
+        ResultSet rs;
+        if (schema.getContents().isH2()) {
+            ps.setString(1, schema.name);
+            ps.setString(2, name);
+            rs = ps.executeQuery();
+        } else {
+            rs = meta.getColumns(null, schema.name, name, null);
+        }
+        ArrayList<DbColumn> list = new ArrayList<>();
         while (rs.next()) {
             DbColumn column = DbColumn.getColumn(schema.getContents(), rs);
             list.add(column);
