@@ -1,14 +1,13 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command;
 
 import java.io.IOException;
 import java.util.ArrayList;
-
-import org.h2.engine.Constants;
+import org.h2.engine.GeneratedKeysMode;
 import org.h2.engine.SessionRemote;
 import org.h2.engine.SysProperties;
 import org.h2.expression.ParameterInterface;
@@ -17,9 +16,11 @@ import org.h2.message.DbException;
 import org.h2.message.Trace;
 import org.h2.result.ResultInterface;
 import org.h2.result.ResultRemote;
-import org.h2.util.New;
+import org.h2.result.ResultWithGeneratedKeys;
+import org.h2.util.Utils;
 import org.h2.value.Transfer;
 import org.h2.value.Value;
+import org.h2.value.ValueLob;
 import org.h2.value.ValueNull;
 
 /**
@@ -45,7 +46,7 @@ public class CommandRemote implements CommandInterface {
         this.transferList = transferList;
         trace = session.getTrace();
         this.sql = sql;
-        parameters = New.arrayList();
+        parameters = Utils.newSmallArrayList();
         prepare(session, true);
         // set session late because prepare might fail - in this case we don't
         // need to close the object
@@ -56,8 +57,7 @@ public class CommandRemote implements CommandInterface {
 
     @Override
     public void stop() {
-        // Must never be called, because remote result is not lazy.
-        throw DbException.throwInternalError();
+        // Ignore
     }
 
     private void prepare(SessionRemote s, boolean createParams) {
@@ -66,14 +66,9 @@ public class CommandRemote implements CommandInterface {
             try {
                 Transfer transfer = transferList.get(i);
 
-                boolean v16 = s.getClientVersion() >= Constants.TCP_PROTOCOL_VERSION_16;
-
                 if (createParams) {
-                    s.traceOperation(v16 ? "SESSION_PREPARE_READ_PARAMS2"
-                            : "SESSION_PREPARE_READ_PARAMS", id);
-                    transfer.writeInt(
-                            v16 ? SessionRemote.SESSION_PREPARE_READ_PARAMS2
-                                    : SessionRemote.SESSION_PREPARE_READ_PARAMS)
+                    s.traceOperation("SESSION_PREPARE_READ_PARAMS2", id);
+                    transfer.writeInt(SessionRemote.SESSION_PREPARE_READ_PARAMS2)
                             .writeInt(id).writeString(sql);
                 } else {
                     s.traceOperation("SESSION_PREPARE", id);
@@ -84,7 +79,7 @@ public class CommandRemote implements CommandInterface {
                 isQuery = transfer.readBoolean();
                 readonly = transfer.readBoolean();
 
-                cmdType = v16 && createParams ? transfer.readInt() : UNKNOWN;
+                cmdType = createParams ? transfer.readInt() : UNKNOWN;
 
                 int paramCount = transfer.readInt();
                 if (createParams) {
@@ -153,7 +148,7 @@ public class CommandRemote implements CommandInterface {
     }
 
     @Override
-    public ResultInterface executeQuery(int maxRows, boolean scrollable) {
+    public ResultInterface executeQuery(long maxRows, boolean scrollable) {
         checkParameters();
         synchronized (session) {
             int objectId = session.getNextId();
@@ -163,8 +158,8 @@ public class CommandRemote implements CommandInterface {
                 Transfer transfer = transferList.get(i);
                 try {
                     session.traceOperation("COMMAND_EXECUTE_QUERY", id);
-                    transfer.writeInt(SessionRemote.COMMAND_EXECUTE_QUERY).
-                        writeInt(id).writeInt(objectId).writeInt(maxRows);
+                    transfer.writeInt(SessionRemote.COMMAND_EXECUTE_QUERY).writeInt(id).writeInt(objectId);
+                    transfer.writeRowCount(maxRows);
                     int fetch;
                     if (session.isClustered() || scrollable) {
                         fetch = Integer.MAX_VALUE;
@@ -194,10 +189,14 @@ public class CommandRemote implements CommandInterface {
     }
 
     @Override
-    public int executeUpdate() {
+    public ResultWithGeneratedKeys executeUpdate(Object generatedKeysRequest) {
         checkParameters();
+        int generatedKeysMode = GeneratedKeysMode.valueOf(generatedKeysRequest);
+        boolean readGeneratedKeys = generatedKeysMode != GeneratedKeysMode.NONE;
+        int objectId = readGeneratedKeys ? session.getNextId() : 0;
         synchronized (session) {
-            int updateCount = 0;
+            long updateCount = 0L;
+            ResultRemote generatedKeys = null;
             boolean autoCommit = false;
             for (int i = 0, count = 0; i < transferList.size(); i++) {
                 prepareIfRequired();
@@ -206,9 +205,36 @@ public class CommandRemote implements CommandInterface {
                     session.traceOperation("COMMAND_EXECUTE_UPDATE", id);
                     transfer.writeInt(SessionRemote.COMMAND_EXECUTE_UPDATE).writeInt(id);
                     sendParameters(transfer);
+                    transfer.writeInt(generatedKeysMode);
+                    switch (generatedKeysMode) {
+                    case GeneratedKeysMode.COLUMN_NUMBERS: {
+                        int[] keys = (int[]) generatedKeysRequest;
+                        transfer.writeInt(keys.length);
+                        for (int key : keys) {
+                            transfer.writeInt(key);
+                        }
+                        break;
+                    }
+                    case GeneratedKeysMode.COLUMN_NAMES: {
+                        String[] keys = (String[]) generatedKeysRequest;
+                        transfer.writeInt(keys.length);
+                        for (String key : keys) {
+                            transfer.writeString(key);
+                        }
+                        break;
+                    }
+                    }
                     session.done(transfer);
-                    updateCount = transfer.readInt();
+                    updateCount = transfer.readRowCount();
                     autoCommit = transfer.readBoolean();
+                    if (readGeneratedKeys) {
+                        int columnCount = transfer.readInt();
+                        if (generatedKeys != null) {
+                            generatedKeys.close();
+                            generatedKeys = null;
+                        }
+                        generatedKeys = new ResultRemote(session, transfer, objectId, columnCount, Integer.MAX_VALUE);
+                    }
                 } catch (IOException e) {
                     session.removeServer(e, i--, ++count);
                 }
@@ -216,7 +242,10 @@ public class CommandRemote implements CommandInterface {
             session.setAutoCommitFromServer(autoCommit);
             session.autoCommitIfCluster();
             session.readSessionState();
-            return updateCount;
+            if (generatedKeys != null) {
+                return new ResultWithGeneratedKeys.WithKeys(updateCount, generatedKeys);
+            }
+            return ResultWithGeneratedKeys.of(updateCount);
         }
     }
 
@@ -261,8 +290,8 @@ public class CommandRemote implements CommandInterface {
         try {
             for (ParameterInterface p : parameters) {
                 Value v = p.getParamValue();
-                if (v != null) {
-                    v.remove();
+                if (v instanceof ValueLob) {
+                    ((ValueLob) v).remove();
                 }
             }
         } catch (DbException e) {
