@@ -1,70 +1,80 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
 import java.util.ArrayList;
 import java.util.HashSet;
-import java.util.Iterator;
 import org.h2.api.ErrorCode;
+import org.h2.command.CommandContainer;
 import org.h2.command.CommandInterface;
 import org.h2.command.Parser;
 import org.h2.command.Prepared;
 import org.h2.constraint.Constraint;
 import org.h2.constraint.ConstraintReferential;
+import org.h2.constraint.ConstraintUnique;
 import org.h2.engine.Constants;
 import org.h2.engine.Database;
 import org.h2.engine.DbObject;
 import org.h2.engine.Right;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.expression.Expression;
 import org.h2.expression.ExpressionVisitor;
 import org.h2.index.Index;
 import org.h2.index.IndexType;
 import org.h2.message.DbException;
 import org.h2.result.ResultInterface;
+import org.h2.result.SearchRow;
 import org.h2.schema.Schema;
 import org.h2.schema.SchemaObject;
 import org.h2.schema.Sequence;
 import org.h2.schema.TriggerObject;
 import org.h2.table.Column;
 import org.h2.table.Table;
+import org.h2.table.TableBase;
 import org.h2.table.TableView;
-import org.h2.util.New;
+import org.h2.util.HasSQL;
+import org.h2.util.Utils;
 
 /**
  * This class represents the statements
  * ALTER TABLE ADD,
  * ALTER TABLE ADD IF NOT EXISTS,
  * ALTER TABLE ALTER COLUMN,
- * ALTER TABLE ALTER COLUMN RESTART,
  * ALTER TABLE ALTER COLUMN SELECTIVITY,
  * ALTER TABLE ALTER COLUMN SET DEFAULT,
- * ALTER TABLE ALTER COLUMN SET NOT NULL,
+ * ALTER TABLE ALTER COLUMN DROP DEFAULT,
+ * ALTER TABLE ALTER COLUMN DROP EXPRESSION,
  * ALTER TABLE ALTER COLUMN SET NULL,
+ * ALTER TABLE ALTER COLUMN DROP NULL,
  * ALTER TABLE ALTER COLUMN SET VISIBLE,
  * ALTER TABLE ALTER COLUMN SET INVISIBLE,
  * ALTER TABLE DROP COLUMN
  */
-public class AlterTableAlterColumn extends SchemaCommand {
+public class AlterTableAlterColumn extends CommandWithColumns {
 
     private String tableName;
     private Column oldColumn;
     private Column newColumn;
     private int type;
+    /**
+     * Default or on update expression.
+     */
     private Expression defaultExpression;
     private Expression newSelectivity;
+    private Expression usingExpression;
+    private boolean addFirst;
     private String addBefore;
     private String addAfter;
     private boolean ifTableExists;
     private boolean ifNotExists;
     private ArrayList<Column> columnsToAdd;
     private ArrayList<Column> columnsToRemove;
-    private boolean newVisibility;
+    private boolean booleanFlag;
 
-    public AlterTableAlterColumn(Session session, Schema schema) {
+    public AlterTableAlterColumn(SessionLocal session, Schema schema) {
         super(session, schema);
     }
 
@@ -80,6 +90,13 @@ public class AlterTableAlterColumn extends SchemaCommand {
         this.oldColumn = oldColumn;
     }
 
+    /**
+     * Add the column as the first column of the table.
+     */
+    public void setAddFirst() {
+        addFirst = true;
+    }
+
     public void setAddBefore(String before) {
         this.addBefore = before;
     }
@@ -89,19 +106,18 @@ public class AlterTableAlterColumn extends SchemaCommand {
     }
 
     @Override
-    public int update() {
-        session.commit(true);
+    public long update() {
         Database db = session.getDatabase();
-        Table table = getSchema().findTableOrView(session, tableName);
+        Table table = getSchema().resolveTableOrView(session, tableName);
         if (table == null) {
             if (ifTableExists) {
                 return 0;
             }
             throw DbException.get(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_1, tableName);
         }
-        session.getUser().checkRight(table, Right.ALL);
+        session.getUser().checkTableRight(table, Right.SCHEMA_OWNER);
         table.checkSupportAlter();
-        table.lock(session, true, true);
+        table.lock(session, Table.EXCLUSIVE_LOCK);
         if (newColumn != null) {
             checkDefaultReferencesTable(table, newColumn.getDefaultExpression());
             checkClustering(newColumn);
@@ -114,7 +130,7 @@ public class AlterTableAlterColumn extends SchemaCommand {
         }
         switch (type) {
         case CommandInterface.ALTER_TABLE_ALTER_COLUMN_NOT_NULL: {
-            if (!oldColumn.isNullable()) {
+            if (oldColumn == null || !oldColumn.isNullable()) {
                 // no change
                 break;
             }
@@ -123,8 +139,8 @@ public class AlterTableAlterColumn extends SchemaCommand {
             db.updateMeta(session, table);
             break;
         }
-        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_NULL: {
-            if (oldColumn.isNullable()) {
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_DROP_NOT_NULL: {
+            if (oldColumn == null || oldColumn.isNullable()) {
                 // no change
                 break;
             }
@@ -133,27 +149,72 @@ public class AlterTableAlterColumn extends SchemaCommand {
             db.updateMeta(session, table);
             break;
         }
-        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_DEFAULT: {
-            Sequence sequence = oldColumn == null ? null : oldColumn.getSequence();
-            checkDefaultReferencesTable(table, defaultExpression);
-            oldColumn.setSequence(null);
-            oldColumn.setDefaultExpression(session, defaultExpression);
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_DEFAULT:
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_DROP_EXPRESSION:  {
+            if (oldColumn == null) {
+                break;
+            }
+            if (oldColumn.isIdentity()) {
+                break;
+            }
+            if (defaultExpression != null) {
+                if (oldColumn.isGenerated()) {
+                    break;
+                }
+                checkDefaultReferencesTable(table, defaultExpression);
+                oldColumn.setDefaultExpression(session, defaultExpression);
+            } else {
+                if (type == CommandInterface.ALTER_TABLE_ALTER_COLUMN_DROP_EXPRESSION != oldColumn.isGenerated()) {
+                    break;
+                }
+                oldColumn.setDefaultExpression(session, null);
+            }
+            db.updateMeta(session, table);
+            break;
+        }
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_DROP_IDENTITY:  {
+            if (oldColumn == null) {
+                break;
+            }
+            Sequence sequence = oldColumn.getSequence();
+            if (sequence == null) {
+                break;
+            }
+            oldColumn.setSequence(null, false);
             removeSequence(table, sequence);
             db.updateMeta(session, table);
             break;
         }
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_ON_UPDATE: {
+            if (oldColumn == null) {
+                break;
+            }
+            if (defaultExpression != null) {
+                if (oldColumn.isIdentity() || oldColumn.isGenerated()) {
+                    break;
+                }
+                checkDefaultReferencesTable(table, defaultExpression);
+                oldColumn.setOnUpdateExpression(session, defaultExpression);
+            } else {
+                oldColumn.setOnUpdateExpression(session, null);
+            }
+            db.updateMeta(session, table);
+            break;
+        }
         case CommandInterface.ALTER_TABLE_ALTER_COLUMN_CHANGE_TYPE: {
+            if (oldColumn == null) {
+                break;
+            }
             // if the change is only increasing the precision, then we don't
             // need to copy the table because the length is only a constraint,
             // and does not affect the storage structure.
-            if (oldColumn.isWideningConversion(newColumn)) {
-                convertAutoIncrementColumn(table, newColumn);
+            if (oldColumn.isWideningConversion(newColumn) && usingExpression == null) {
+                convertIdentityColumn(table, newColumn);
                 oldColumn.copy(newColumn);
                 db.updateMeta(session, table);
             } else {
-                oldColumn.setSequence(null);
+                oldColumn.setSequence(null, false);
                 oldColumn.setDefaultExpression(session, null);
-                oldColumn.setConvertNullToDefault(false);
                 if (oldColumn.isNullable() && !newColumn.isNullable()) {
                     checkNoNullValues(table);
                 } else if (!oldColumn.isNullable() && newColumn.isNullable()) {
@@ -162,51 +223,64 @@ public class AlterTableAlterColumn extends SchemaCommand {
                 if (oldColumn.getVisible() ^ newColumn.getVisible()) {
                     oldColumn.setVisible(newColumn.getVisible());
                 }
-                convertAutoIncrementColumn(table, newColumn);
-                copyData(table);
+                convertIdentityColumn(table, newColumn);
+                copyData(table, null, true);
             }
             table.setModified();
             break;
         }
         case CommandInterface.ALTER_TABLE_ADD_COLUMN: {
             // ifNotExists only supported for single column add
-            if (ifNotExists && columnsToAdd.size() == 1 &&
+            if (ifNotExists && columnsToAdd != null && columnsToAdd.size() == 1 &&
                     table.doesColumnExist(columnsToAdd.get(0).getName())) {
                 break;
             }
-            for (Column column : columnsToAdd) {
-                if (column.isAutoIncrement()) {
-                    int objId = getObjectId();
-                    column.convertAutoIncrementToSequence(session, getSchema(), objId,
-                            table.isTemporary());
-                }
+            ArrayList<Sequence> sequences = generateSequences(columnsToAdd, false);
+            if (columnsToAdd != null) {
+                changePrimaryKeysToNotNull(columnsToAdd);
             }
-            copyData(table);
+            copyData(table, sequences, true);
             break;
         }
         case CommandInterface.ALTER_TABLE_DROP_COLUMN: {
             if (table.getColumns().length - columnsToRemove.size() < 1) {
-                throw DbException.get(ErrorCode.CANNOT_DROP_LAST_COLUMN,
-                        columnsToRemove.get(0).getSQL());
+                throw DbException.get(ErrorCode.CANNOT_DROP_LAST_COLUMN, columnsToRemove.get(0).getTraceSQL());
             }
             table.dropMultipleColumnsConstraintsAndIndexes(session, columnsToRemove);
-            copyData(table);
+            copyData(table, null, false);
             break;
         }
         case CommandInterface.ALTER_TABLE_ALTER_COLUMN_SELECTIVITY: {
+            if (oldColumn == null) {
+                break;
+            }
             int value = newSelectivity.optimize(session).getValue(session).getInt();
             oldColumn.setSelectivity(value);
             db.updateMeta(session, table);
             break;
         }
-        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_VISIBILITY: {
-            oldColumn.setVisible(newVisibility);
-            table.setModified();
-            db.updateMeta(session, table);
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_VISIBILITY:
+            if (oldColumn == null) {
+                break;
+            }
+            if (oldColumn.getVisible() != booleanFlag) {
+                oldColumn.setVisible(booleanFlag);
+                table.setModified();
+                db.updateMeta(session, table);
+            }
+            break;
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_DEFAULT_ON_NULL:
+            if (oldColumn == null) {
+                break;
+            }
+            if (oldColumn.isDefaultOnNull() != booleanFlag) {
+                oldColumn.setDefaultOnNull(booleanFlag);
+                table.setModified();
+                db.updateMeta(session, table);
+            }
             break;
-        }
         default:
-            DbException.throwInternalError("type=" + type);
+            throw DbException.getInternalError("type=" + type);
         }
         return 0;
     }
@@ -215,34 +289,32 @@ public class AlterTableAlterColumn extends SchemaCommand {
         if (defaultExpression == null) {
             return;
         }
-        HashSet<DbObject> dependencies = New.hashSet();
+        HashSet<DbObject> dependencies = new HashSet<>();
         ExpressionVisitor visitor = ExpressionVisitor
                 .getDependenciesVisitor(dependencies);
         defaultExpression.isEverything(visitor);
         if (dependencies.contains(table)) {
-            throw DbException.get(ErrorCode.COLUMN_IS_REFERENCED_1,
-                    defaultExpression.getSQL());
+            throw DbException.get(ErrorCode.COLUMN_IS_REFERENCED_1, defaultExpression.getTraceSQL());
         }
     }
 
     private void checkClustering(Column c) {
         if (!Constants.CLUSTERING_DISABLED
                 .equals(session.getDatabase().getCluster())
-                && c.isAutoIncrement()) {
+                && c.hasIdentityOptions()) {
             throw DbException.getUnsupportedException(
-                    "CLUSTERING && auto-increment columns");
+                    "CLUSTERING && identity columns");
         }
     }
 
-    private void convertAutoIncrementColumn(Table table, Column c) {
-        if (c.isAutoIncrement()) {
+    private void convertIdentityColumn(Table table, Column c) {
+        if (c.hasIdentityOptions()) {
             if (c.isPrimaryKey()) {
-                c.setOriginalSQL("IDENTITY");
-            } else {
-                int objId = getObjectId();
-                c.convertAutoIncrementToSequence(session, getSchema(), objId,
-                        table.isTemporary());
+                addConstraintCommand(
+                        Parser.newPrimaryKeyConstraintCommand(session, table.getSchema(), table.getName(), c));
             }
+            int objId = getObjectId();
+            c.initializeSequence(session, getSchema(), objId, table.isTemporary());
         }
     }
 
@@ -255,7 +327,7 @@ public class AlterTableAlterColumn extends SchemaCommand {
         }
     }
 
-    private void copyData(Table table) {
+    private void copyData(Table table, ArrayList<Sequence> sequences, boolean createConstraints) {
         if (table.isTemporary()) {
             throw DbException.getUnsupportedException("TEMP TABLE");
         }
@@ -263,25 +335,31 @@ public class AlterTableAlterColumn extends SchemaCommand {
         String baseName = table.getName();
         String tempName = db.getTempTableName(baseName, session);
         Column[] columns = table.getColumns();
-        ArrayList<Column> newColumns = New.arrayList();
+        ArrayList<Column> newColumns = new ArrayList<>(columns.length);
         Table newTable = cloneTableStructure(table, columns, db, tempName, newColumns);
+        if (sequences != null) {
+            for (Sequence sequence : sequences) {
+                table.addSequence(sequence);
+            }
+        }
         try {
             // check if a view would become invalid
             // (because the column to drop is referenced or so)
             checkViews(table, newTable);
         } catch (DbException e) {
-            execute("DROP TABLE " + newTable.getName(), true);
-            throw DbException.get(ErrorCode.VIEW_IS_INVALID_2, e, getSQL(), e.getMessage());
+            StringBuilder builder = new StringBuilder("DROP TABLE ");
+            newTable.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS);
+            execute(builder.toString());
+            throw e;
         }
         String tableName = table.getName();
-        ArrayList<TableView> views = table.getViews();
-        if (views != null) {
-            views = New.arrayList(views);
-            for (TableView view : views) {
-                table.removeView(view);
-            }
+        ArrayList<TableView> dependentViews = new ArrayList<>(table.getDependentViews());
+        for (TableView view : dependentViews) {
+            table.removeDependentView(view);
         }
-        execute("DROP TABLE " + table.getSQL() + " IGNORE", true);
+        StringBuilder builder = new StringBuilder("DROP TABLE ");
+        table.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS).append(" IGNORE");
+        execute(builder.toString());
         db.renameSchemaObject(session, newTable, tableName);
         for (DbObject child : newTable.getChildren()) {
             if (child instanceof Sequence) {
@@ -306,11 +384,12 @@ public class AlterTableAlterColumn extends SchemaCommand {
                 db.renameSchemaObject(session, so, name);
             }
         }
-        if (views != null) {
-            for (TableView view : views) {
-                String sql = view.getCreateSQL(true, true);
-                execute(sql, true);
-            }
+        if (createConstraints) {
+            createConstraints();
+        }
+        for (TableView view : dependentViews) {
+            String sql = view.getCreateSQL(true, true);
+            execute(sql);
         }
     }
 
@@ -319,37 +398,42 @@ public class AlterTableAlterColumn extends SchemaCommand {
         for (Column col : columns) {
             newColumns.add(col.getClone());
         }
-        if (type == CommandInterface.ALTER_TABLE_DROP_COLUMN) {
+        switch (type) {
+        case CommandInterface.ALTER_TABLE_DROP_COLUMN:
             for (Column removeCol : columnsToRemove) {
                 Column foundCol = null;
-                for (Iterator<Column> it = newColumns.iterator(); it.hasNext();) {
-                    Column newCol = it.next();
-                    if (newCol.getName() == removeCol.getName()) {
+                for (Column newCol : newColumns) {
+                    if (newCol.getName().equals(removeCol.getName())) {
                         foundCol = newCol;
                         break;
                     }
                 }
                 if (foundCol == null) {
-                    throw DbException.throwInternalError(removeCol.getCreateSQL());
+                    throw DbException.getInternalError(removeCol.getCreateSQL());
                 }
                 newColumns.remove(foundCol);
             }
-        } else if (type == CommandInterface.ALTER_TABLE_ADD_COLUMN) {
+            break;
+        case CommandInterface.ALTER_TABLE_ADD_COLUMN: {
             int position;
-            if (addBefore != null) {
+            if (addFirst) {
+                position = 0;
+            } else if (addBefore != null) {
                 position = table.getColumn(addBefore).getColumnId();
             } else if (addAfter != null) {
                 position = table.getColumn(addAfter).getColumnId() + 1;
             } else {
                 position = columns.length;
             }
-            for (Column column : columnsToAdd) {
-                newColumns.add(position++, column);
+            if (columnsToAdd != null) {
+                for (Column column : columnsToAdd) {
+                    newColumns.add(position++, column);
+                }
             }
-        } else if (type == CommandInterface.ALTER_TABLE_ALTER_COLUMN_CHANGE_TYPE) {
-            int position = oldColumn.getColumnId();
-            newColumns.remove(position);
-            newColumns.add(position, newColumn);
+            break;
+        }
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_CHANGE_TYPE:
+            newColumns.set(oldColumn.getColumnId(), newColumn);
         }
 
         // create a table object in order to get the SQL statement
@@ -366,41 +450,44 @@ public class AlterTableAlterColumn extends SchemaCommand {
         data.persistData = table.isPersistData();
         data.persistIndexes = table.isPersistIndexes();
         data.isHidden = table.isHidden();
-        data.create = true;
         data.session = session;
         Table newTable = getSchema().createTable(data);
         newTable.setComment(table.getComment());
-        StringBuilder buff = new StringBuilder();
-        buff.append(newTable.getCreateSQL());
-        StringBuilder columnList = new StringBuilder();
+        String newTableSQL = newTable.getCreateSQLForMeta();
+        StringBuilder columnNames = new StringBuilder();
+        StringBuilder columnValues = new StringBuilder();
         for (Column nc : newColumns) {
-            if (columnList.length() > 0) {
-                columnList.append(", ");
+            if (nc.isGenerated()) {
+                continue;
             }
-            if (type == CommandInterface.ALTER_TABLE_ADD_COLUMN &&
-                    columnsToAdd.contains(nc)) {
-                Expression def = nc.getDefaultExpression();
-                columnList.append(def == null ? "NULL" : def.getSQL());
-            } else {
-                columnList.append(nc.getSQL());
+            switch (type) {
+            case CommandInterface.ALTER_TABLE_ADD_COLUMN:
+                if (columnsToAdd != null && columnsToAdd.contains(nc)) {
+                    if (usingExpression != null) {
+                        usingExpression.getUnenclosedSQL(addColumn(nc, columnNames, columnValues),
+                                HasSQL.DEFAULT_SQL_FLAGS);
+                    }
+                    continue;
+                }
+                break;
+            case CommandInterface.ALTER_TABLE_ALTER_COLUMN_CHANGE_TYPE:
+                if (nc.equals(newColumn) && usingExpression != null) {
+                    usingExpression.getUnenclosedSQL(addColumn(nc, columnNames, columnValues),
+                            HasSQL.DEFAULT_SQL_FLAGS);
+                    continue;
+                }
             }
+            nc.getSQL(addColumn(nc, columnNames, columnValues), HasSQL.DEFAULT_SQL_FLAGS);
         }
-        buff.append(" AS SELECT ");
-        if (columnList.length() == 0) {
-            // special case: insert into test select * from
-            buff.append('*');
-        } else {
-            buff.append(columnList);
-        }
-        buff.append(" FROM ").append(table.getSQL());
-        String newTableSQL = buff.toString();
         String newTableName = newTable.getName();
         Schema newTableSchema = newTable.getSchema();
         newTable.removeChildrenAndResources(session);
 
-        execute(newTableSQL, true);
+        execute(newTableSQL);
         newTable = newTableSchema.getTableOrView(session, newTableName);
-        ArrayList<String> triggers = New.arrayList();
+        ArrayList<String> children = Utils.newSmallArrayList();
+        ArrayList<String> triggers = Utils.newSmallArrayList();
+        boolean hasDelegateIndex = false;
         for (DbObject child : table.getChildren()) {
             if (child instanceof Sequence) {
                 continue;
@@ -417,9 +504,9 @@ public class AlterTableAlterColumn extends SchemaCommand {
             if (child instanceof TableView) {
                 continue;
             } else if (child.getType() == DbObject.TABLE_OR_VIEW) {
-                DbException.throwInternalError();
+                throw DbException.getInternalError();
             }
-            String quotedName = Parser.quoteIdentifier(tempName + "_" + child.getName());
+            String quotedName = Parser.quoteIdentifier(tempName + "_" + child.getName(), HasSQL.DEFAULT_SQL_FLAGS);
             String sql = null;
             if (child instanceof ConstraintReferential) {
                 ConstraintReferential r = (ConstraintReferential) child;
@@ -434,10 +521,51 @@ public class AlterTableAlterColumn extends SchemaCommand {
                 if (child instanceof TriggerObject) {
                     triggers.add(sql);
                 } else {
-                    execute(sql, true);
+                    if (!hasDelegateIndex) {
+                        Index index = null;
+                        if (child instanceof ConstraintUnique) {
+                            ConstraintUnique constraint = (ConstraintUnique) child;
+                            if (constraint.getConstraintType() == Constraint.Type.PRIMARY_KEY) {
+                                index = constraint.getIndex();
+                            }
+                        } else if (child instanceof Index) {
+                            index = (Index) child;
+                        }
+                        if (index != null
+                                && TableBase.getMainIndexColumn(index.getIndexType(), index.getIndexColumns())
+                                        != SearchRow.ROWID_INDEX) {
+                            execute(sql);
+                            hasDelegateIndex = true;
+                            continue;
+                        }
+                    }
+                    children.add(sql);
                 }
             }
         }
+        StringBuilder builder = newTable.getSQL(new StringBuilder(128).append("INSERT INTO "), //
+                HasSQL.DEFAULT_SQL_FLAGS)
+            .append('(').append(columnNames).append(") OVERRIDING SYSTEM VALUE SELECT ");
+        if (columnValues.length() == 0) {
+            // special case: insert into test select * from
+            builder.append('*');
+        } else {
+            builder.append(columnValues);
+        }
+        table.getSQL(builder.append(" FROM "), HasSQL.DEFAULT_SQL_FLAGS);
+        try {
+            execute(builder.toString());
+        } catch (Throwable t) {
+            // data was not inserted due to data conversion error or some
+            // unexpected reason
+            builder = new StringBuilder("DROP TABLE ");
+            newTable.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS);
+            execute(builder.toString());
+            throw t;
+        }
+        for (String sql : children) {
+            execute(sql);
+        }
         table.setModified();
         // remove the sequences from the columns (except dropped columns)
         // otherwise the sequence is dropped if the table is dropped
@@ -445,15 +573,26 @@ public class AlterTableAlterColumn extends SchemaCommand {
             Sequence seq = col.getSequence();
             if (seq != null) {
                 table.removeSequence(seq);
-                col.setSequence(null);
+                col.setSequence(null, false);
             }
         }
         for (String sql : triggers) {
-            execute(sql, true);
+            execute(sql);
         }
         return newTable;
     }
 
+    private static StringBuilder addColumn(Column column, StringBuilder columnNames, StringBuilder columnValues) {
+        if (columnNames.length() > 0) {
+            columnNames.append(", ");
+        }
+        column.getSQL(columnNames, HasSQL.DEFAULT_SQL_FLAGS);
+        if (columnValues.length() > 0) {
+            columnValues.append(", ");
+        }
+        return columnValues;
+    }
+
     /**
      * Check that all views and other dependent objects.
      */
@@ -486,48 +625,51 @@ public class AlterTableAlterColumn extends SchemaCommand {
     private void checkViewsAreValid(DbObject tableOrView) {
         for (DbObject view : tableOrView.getChildren()) {
             if (view instanceof TableView) {
-                String sql = ((TableView) view).getQuery();
+                String sql = ((TableView) view).getQuerySQL();
                 // check if the query is still valid
                 // do not execute, not even with limit 1, because that could
                 // have side effects or take a very long time
-                session.prepare(sql);
+                try {
+                    session.prepare(sql);
+                } catch (DbException e) {
+                    throw DbException.get(ErrorCode.COLUMN_IS_REFERENCED_1, e, view.getTraceSQL());
+                }
                 checkViewsAreValid(view);
             }
         }
     }
 
-    private void execute(String sql, boolean ddl) {
+    private void execute(String sql) {
         Prepared command = session.prepare(sql);
-        command.update();
-        if (ddl) {
-            session.commit(true);
-        }
+        CommandContainer commandContainer = new CommandContainer(session, sql, command);
+        commandContainer.executeUpdate(null);
     }
 
     private void checkNullable(Table table) {
+        if (oldColumn.isIdentity()) {
+            throw DbException.get(ErrorCode.COLUMN_MUST_NOT_BE_NULLABLE_1, oldColumn.getName());
+        }
         for (Index index : table.getIndexes()) {
             if (index.getColumnIndex(oldColumn) < 0) {
                 continue;
             }
             IndexType indexType = index.getIndexType();
-            if (indexType.isPrimaryKey() || indexType.isHash()) {
-                throw DbException.get(
-                        ErrorCode.COLUMN_IS_PART_OF_INDEX_1, index.getSQL());
+            if (indexType.isPrimaryKey()) {
+                throw DbException.get(ErrorCode.COLUMN_MUST_NOT_BE_NULLABLE_1, oldColumn.getName());
             }
         }
     }
 
     private void checkNoNullValues(Table table) {
-        String sql = "SELECT COUNT(*) FROM " +
-                table.getSQL() + " WHERE " +
-                oldColumn.getSQL() + " IS NULL";
+        StringBuilder builder = new StringBuilder("SELECT COUNT(*) FROM ");
+        table.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS).append(" WHERE ");
+        oldColumn.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS).append(" IS NULL");
+        String sql = builder.toString();
         Prepared command = session.prepare(sql);
         ResultInterface result = command.query(0);
         result.next();
         if (result.currentRow()[0].getInt() > 0) {
-            throw DbException.get(
-                    ErrorCode.COLUMN_CONTAINS_NULL_VALUES_1,
-                    oldColumn.getSQL());
+            throw DbException.get(ErrorCode.COLUMN_CONTAINS_NULL_VALUES_1, oldColumn.getTraceSQL());
         }
     }
 
@@ -539,10 +681,24 @@ public class AlterTableAlterColumn extends SchemaCommand {
         newSelectivity = selectivity;
     }
 
+    /**
+     * Set default or on update expression.
+     *
+     * @param defaultExpression default or on update expression
+     */
     public void setDefaultExpression(Expression defaultExpression) {
         this.defaultExpression = defaultExpression;
     }
 
+    /**
+     * Set using expression.
+     *
+     * @param usingExpression using expression
+     */
+    public void setUsingExpression(Expression usingExpression) {
+        this.usingExpression = usingExpression;
+    }
+
     public void setNewColumn(Column newColumn) {
         this.newColumn = newColumn;
     }
@@ -556,15 +712,19 @@ public class AlterTableAlterColumn extends SchemaCommand {
         this.ifNotExists = ifNotExists;
     }
 
-    public void setNewColumns(ArrayList<Column> columnsToAdd) {
-        this.columnsToAdd = columnsToAdd;
+    @Override
+    public void addColumn(Column column) {
+        if (columnsToAdd == null) {
+            columnsToAdd = new ArrayList<>();
+        }
+        columnsToAdd.add(column);
     }
 
     public void setColumnsToRemove(ArrayList<Column> columnsToRemove) {
         this.columnsToRemove = columnsToRemove;
     }
 
-    public void setVisible(boolean visible) {
-        this.newVisibility = visible;
+    public void setBooleanFlag(boolean booleanFlag) {
+        this.booleanFlag = booleanFlag;
     }
 }
