@@ -1,13 +1,12 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
 import org.h2.command.CommandInterface;
-import org.h2.engine.Right;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
 import org.h2.table.TableView;
 
@@ -20,7 +19,7 @@ public class AlterView extends DefineCommand {
     private boolean ifExists;
     private TableView view;
 
-    public AlterView(Session session) {
+    public AlterView(SessionLocal session) {
         super(session);
     }
 
@@ -33,12 +32,11 @@ public class AlterView extends DefineCommand {
     }
 
     @Override
-    public int update() {
-        session.commit(true);
+    public long update() {
         if (view == null && ifExists) {
             return 0;
         }
-        session.getUser().checkRight(view, Right.ALL);
+        session.getUser().checkSchemaOwner(view.getSchema());
         DbException e = view.recompile(session, false, true);
         if (e != null) {
             throw e;
