@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
@@ -8,50 +8,43 @@ package org.h2.command.ddl;
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
 import org.h2.engine.Database;
-import org.h2.engine.Session;
-import org.h2.engine.UserAggregate;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
 import org.h2.schema.Schema;
+import org.h2.schema.UserAggregate;
 
 /**
  * This class represents the statement
  * CREATE AGGREGATE
  */
-public class CreateAggregate extends DefineCommand {
+public class CreateAggregate extends SchemaCommand {
 
-    private Schema schema;
     private String name;
     private String javaClassMethod;
     private boolean ifNotExists;
     private boolean force;
 
-    public CreateAggregate(Session session) {
-        super(session);
+    public CreateAggregate(SessionLocal session, Schema schema) {
+        super(session, schema);
     }
 
     @Override
-    public int update() {
-        session.commit(true);
+    public long update() {
         session.getUser().checkAdmin();
         Database db = session.getDatabase();
-        if (db.findAggregate(name) != null || schema.findFunction(name) != null) {
+        Schema schema = getSchema();
+        if (schema.findFunctionOrAggregate(name) != null) {
             if (!ifNotExists) {
-                throw DbException.get(
-                        ErrorCode.FUNCTION_ALIAS_ALREADY_EXISTS_1, name);
+                throw DbException.get(ErrorCode.FUNCTION_ALIAS_ALREADY_EXISTS_1, name);
             }
         } else {
             int id = getObjectId();
-            UserAggregate aggregate = new UserAggregate(
-                    db, id, name, javaClassMethod, force);
-            db.addDatabaseObject(session, aggregate);
+            UserAggregate aggregate = new UserAggregate(schema, id, name, javaClassMethod, force);
+            db.addSchemaObject(session, aggregate);
         }
         return 0;
     }
 
-    public void setSchema(Schema schema) {
-        this.schema = schema;
-    }
-
     public void setName(String name) {
         this.name = name;
     }
