@@ -1,15 +1,15 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
 import java.util.ArrayList;
-import org.h2.engine.Session;
+
+import org.h2.engine.SessionLocal;
 import org.h2.schema.Schema;
 import org.h2.table.Column;
-import org.h2.util.New;
 
 /**
  * The data required to create a table.
@@ -34,7 +34,7 @@ public class CreateTableData {
     /**
      * The column list.
      */
-    public ArrayList<Column> columns = New.arrayList();
+    public ArrayList<Column> columns = new ArrayList<>();
 
     /**
      * Whether this is a temporary table.
@@ -56,15 +56,10 @@ public class CreateTableData {
      */
     public boolean persistData;
 
-    /**
-     * Whether to create a new table.
-     */
-    public boolean create;
-
     /**
      * The session.
      */
-    public Session session;
+    public SessionLocal session;
 
     /**
      * The table engine to use for creating the table.
