@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
@@ -9,7 +9,7 @@ import org.h2.api.ErrorCode;
 import org.h2.api.Trigger;
 import org.h2.command.CommandInterface;
 import org.h2.engine.Database;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
 import org.h2.schema.Schema;
 import org.h2.schema.TriggerObject;
@@ -36,7 +36,7 @@ public class CreateTrigger extends SchemaCommand {
     private boolean force;
     private boolean onRollback;
 
-    public CreateTrigger(Session session, Schema schema) {
+    public CreateTrigger(SessionLocal session, Schema schema) {
         super(session, schema);
     }
 
@@ -85,8 +85,8 @@ public class CreateTrigger extends SchemaCommand {
     }
 
     @Override
-    public int update() {
-        session.commit(true);
+    public long update() {
+        session.getUser().checkAdmin();
         Database db = session.getDatabase();
         if (getSchema().findTrigger(triggerName) != null) {
             if (ifNotExists) {
@@ -96,10 +96,18 @@ public class CreateTrigger extends SchemaCommand {
                     ErrorCode.TRIGGER_ALREADY_EXISTS_1,
                     triggerName);
         }
-        if ((typeMask & Trigger.SELECT) == Trigger.SELECT && rowBased) {
-            throw DbException.get(
-                    ErrorCode.TRIGGER_SELECT_AND_ROW_BASED_NOT_SUPPORTED,
-                    triggerName);
+        if ((typeMask & Trigger.SELECT) != 0) {
+            if (rowBased) {
+                throw DbException.get(ErrorCode.INVALID_TRIGGER_FLAGS_1, "SELECT + FOR EACH ROW");
+            }
+            if (onRollback) {
+                throw DbException.get(ErrorCode.INVALID_TRIGGER_FLAGS_1, "SELECT + ROLLBACK");
+            }
+        } else if ((typeMask & (Trigger.INSERT | Trigger.UPDATE | Trigger.DELETE)) == 0) {
+            if (onRollback) {
+                throw DbException.get(ErrorCode.INVALID_TRIGGER_FLAGS_1, "(!INSERT & !UPDATE & !DELETE) + ROLLBACK");
+            }
+            throw DbException.getInternalError();
         }
         int id = getObjectId();
         Table table = getSchema().getTableOrView(session, tableName);
