@@ -1,12 +1,12 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
 import org.h2.command.CommandInterface;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 
 /**
  * This class represents the statement
@@ -16,12 +16,12 @@ public class DeallocateProcedure extends DefineCommand {
 
     private String procedureName;
 
-    public DeallocateProcedure(Session session) {
+    public DeallocateProcedure(SessionLocal session) {
         super(session);
     }
 
     @Override
-    public int update() {
+    public long update() {
         session.removeProcedure(procedureName);
         return 0;
     }
