@@ -1,12 +1,12 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
 import org.h2.command.Prepared;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.result.ResultInterface;
 
 /**
@@ -26,7 +26,7 @@ public abstract class DefineCommand extends Prepared {
      *
      * @param session the session
      */
-    DefineCommand(Session session) {
+    DefineCommand(SessionLocal session) {
         super(session);
     }
 
