@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
@@ -8,7 +8,7 @@ package org.h2.command.ddl;
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
 import org.h2.engine.Database;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
 import org.h2.schema.Constant;
 import org.h2.schema.Schema;
@@ -17,12 +17,12 @@ import org.h2.schema.Schema;
  * This class represents the statement
  * DROP CONSTANT
  */
-public class DropConstant extends SchemaCommand {
+public class DropConstant extends SchemaOwnerCommand {
 
     private String constantName;
     private boolean ifExists;
 
-    public DropConstant(Session session, Schema schema) {
+    public DropConstant(SessionLocal session, Schema schema) {
         super(session, schema);
     }
 
@@ -35,11 +35,9 @@ public class DropConstant extends SchemaCommand {
     }
 
     @Override
-    public int update() {
-        session.getUser().checkAdmin();
-        session.commit(true);
+    long update(Schema schema) {
         Database db = session.getDatabase();
-        Constant constant = getSchema().findConstant(constantName);
+        Constant constant = schema.findConstant(constantName);
         if (constant == null) {
             if (!ifExists) {
                 throw DbException.get(ErrorCode.CONSTANT_NOT_FOUND_1, constantName);
