@@ -1,11 +1,11 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.ddl;
 
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.schema.Schema;
 
 /**
@@ -21,7 +21,7 @@ public abstract class SchemaCommand extends DefineCommand {
      * @param session the session
      * @param schema the schema
      */
-    public SchemaCommand(Session session, Schema schema) {
+    public SchemaCommand(SessionLocal session, Schema schema) {
         super(session);
         this.schema = schema;
     }
@@ -31,7 +31,7 @@ public abstract class SchemaCommand extends DefineCommand {
      *
      * @return the schema
      */
-    protected Schema getSchema() {
+    protected final Schema getSchema() {
         return schema;
     }
 
