@@ -1,13 +1,13 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.dml;
 
 import org.h2.command.CommandInterface;
 import org.h2.command.Prepared;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.result.ResultInterface;
 
 /**
@@ -15,12 +15,12 @@ import org.h2.result.ResultInterface;
  */
 public class NoOperation extends Prepared {
 
-    public NoOperation(Session session) {
+    public NoOperation(SessionLocal session) {
         super(session);
     }
 
     @Override
-    public int update() {
+    public long update() {
         return 0;
     }
 
