@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.command.dml;
@@ -17,50 +17,54 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.Comparator;
 import java.util.Set;
+import java.util.TreeMap;
+import java.util.TreeSet;
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
-import org.h2.command.Parser;
 import org.h2.constraint.Constraint;
+import org.h2.constraint.Constraint.Type;
 import org.h2.engine.Comment;
 import org.h2.engine.Constants;
 import org.h2.engine.Database;
 import org.h2.engine.DbObject;
 import org.h2.engine.Right;
+import org.h2.engine.RightOwner;
 import org.h2.engine.Role;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.engine.Setting;
-import org.h2.engine.SysProperties;
 import org.h2.engine.User;
-import org.h2.engine.UserAggregate;
-import org.h2.engine.UserDataType;
 import org.h2.expression.Expression;
 import org.h2.expression.ExpressionColumn;
 import org.h2.index.Cursor;
 import org.h2.index.Index;
 import org.h2.message.DbException;
+import org.h2.mvstore.DataUtils;
 import org.h2.result.LocalResult;
 import org.h2.result.ResultInterface;
 import org.h2.result.Row;
 import org.h2.schema.Constant;
+import org.h2.schema.Domain;
 import org.h2.schema.Schema;
 import org.h2.schema.SchemaObject;
 import org.h2.schema.Sequence;
 import org.h2.schema.TriggerObject;
+import org.h2.schema.UserDefinedFunction;
 import org.h2.table.Column;
 import org.h2.table.PlanItem;
 import org.h2.table.Table;
 import org.h2.table.TableType;
+import org.h2.util.HasSQL;
 import org.h2.util.IOUtils;
 import org.h2.util.MathUtils;
-import org.h2.util.StatementBuilder;
 import org.h2.util.StringUtils;
 import org.h2.util.Utils;
+import org.h2.value.TypeInfo;
 import org.h2.value.Value;
-import org.h2.value.ValueString;
+import org.h2.value.ValueVarchar;
 
 /**
  * This class represents the statement
@@ -68,6 +72,16 @@ import org.h2.value.ValueString;
  */
 public class ScriptCommand extends ScriptBase {
 
+    private static final Comparator<? super DbObject> BY_NAME_COMPARATOR = (o1, o2) -> {
+        if (o1 instanceof SchemaObject && o2 instanceof SchemaObject) {
+            int cmp = ((SchemaObject) o1).getSchema().getName().compareTo(((SchemaObject) o2).getSchema().getName());
+            if (cmp != 0) {
+                return cmp;
+            }
+        }
+        return o1.getName().compareTo(o2.getName());
+    };
+
     private Charset charset = StandardCharsets.UTF_8;
     private Set<String> schemaNames;
     private Collection<Table> tables;
@@ -80,6 +94,9 @@ public class ScriptCommand extends ScriptBase {
     // true if we're generating the DROP statements
     private boolean drop;
     private boolean simple;
+    private boolean withColumns;
+    private boolean version = true;
+
     private LocalResult result;
     private String lineSeparatorString;
     private byte[] lineSeparator;
@@ -88,7 +105,7 @@ public class ScriptCommand extends ScriptBase {
     private int nextLobId;
     private int lobBlockSize = Constants.IO_BUFFER_SIZE;
 
-    public ScriptCommand(Session session) {
+    public ScriptCommand(SessionLocal session) {
         super(session);
     }
 
@@ -135,13 +152,12 @@ public class ScriptCommand extends ScriptBase {
     }
 
     private LocalResult createResult() {
-        Expression[] expressions = { new ExpressionColumn(
-                session.getDatabase(), new Column("SCRIPT", Value.STRING)) };
-        return new LocalResult(session, expressions, 1);
+        return new LocalResult(session, new Expression[] {
+                new ExpressionColumn(session.getDatabase(), new Column("SCRIPT", TypeInfo.TYPE_VARCHAR)) }, 1, 1);
     }
 
     @Override
-    public ResultInterface query(int maxrows) {
+    public ResultInterface query(long maxrows) {
         session.getUser().checkAdmin();
         reset();
         Database db = session.getDatabase();
@@ -161,6 +177,9 @@ public class ScriptCommand extends ScriptBase {
             if (out != null) {
                 buffer = new byte[Constants.IO_BUFFER_SIZE];
             }
+            if (version) {
+                add("-- H2 " + Constants.VERSION, true);
+            }
             if (settings) {
                 for (Setting setting : db.getAllSettings()) {
                     if (setting.getName().equals(SetTypes.getTypeName(
@@ -175,42 +194,47 @@ public class ScriptCommand extends ScriptBase {
             if (out != null) {
                 add("", true);
             }
-            for (User user : db.getAllUsers()) {
-                add(user.getCreateSQL(passwords), false);
-            }
-            for (Role role : db.getAllRoles()) {
-                add(role.getCreateSQL(true), false);
+            RightOwner[] rightOwners = db.getAllUsersAndRoles().toArray(new RightOwner[0]);
+            // ADMIN users first, other users next, roles last
+            Arrays.sort(rightOwners, (o1, o2) -> {
+                boolean b = o1 instanceof User;
+                if (b != o2 instanceof User) {
+                    return b ? -1 : 1;
+                }
+                if (b) {
+                    b = ((User) o1).isAdmin();
+                    if (b != ((User) o2).isAdmin()) {
+                        return b ? -1 : 1;
+                    }
+                }
+                return o1.getName().compareTo(o2.getName());
+            });
+            for (RightOwner rightOwner : rightOwners) {
+                if (rightOwner instanceof User) {
+                    add(((User) rightOwner).getCreateSQL(passwords), false);
+                } else {
+                    add(((Role) rightOwner).getCreateSQL(true), false);
+                }
             }
+            ArrayList<Schema> schemas = new ArrayList<>();
             for (Schema schema : db.getAllSchemas()) {
                 if (excludeSchema(schema)) {
                     continue;
                 }
+                schemas.add(schema);
                 add(schema.getCreateSQL(), false);
             }
-            for (UserDataType datatype : db.getAllUserDataTypes()) {
-                if (drop) {
-                    add(datatype.getDropSQL(), false);
+            dumpDomains(schemas);
+            for (Schema schema : schemas) {
+                for (Constant constant : sorted(schema.getAllConstants(), Constant.class)) {
+                    add(constant.getCreateSQL(), false);
                 }
-                add(datatype.getCreateSQL(), false);
-            }
-            for (SchemaObject obj : db.getAllSchemaObjects(
-                    DbObject.CONSTANT)) {
-                if (excludeSchema(obj.getSchema())) {
-                    continue;
-                }
-                Constant constant = (Constant) obj;
-                add(constant.getCreateSQL(), false);
             }
 
-            final ArrayList<Table> tables = db.getAllTablesAndViews(false);
+            final ArrayList<Table> tables = db.getAllTablesAndViews();
             // sort by id, so that views are after tables and views on views
             // after the base views
-            Collections.sort(tables, new Comparator<Table>() {
-                @Override
-                public int compare(Table t1, Table t2) {
-                    return t1.getId() - t2.getId();
-                }
-            });
+            tables.sort(Comparator.comparingInt(Table::getId));
 
             // Generate the DROP XXX  ... IF EXISTS
             for (Table table : tables) {
@@ -223,7 +247,7 @@ public class ScriptCommand extends ScriptBase {
                 if (table.isHidden()) {
                     continue;
                 }
-                table.lock(session, false, false);
+                table.lock(session, Table.READ_LOCK);
                 String sql = table.getCreateSQL();
                 if (sql == null) {
                     // null for metadata tables
@@ -233,32 +257,25 @@ public class ScriptCommand extends ScriptBase {
                     add(table.getDropSQL(), false);
                 }
             }
-            for (SchemaObject obj : db.getAllSchemaObjects(
-                    DbObject.FUNCTION_ALIAS)) {
-                if (excludeSchema(obj.getSchema())) {
-                    continue;
+            for (Schema schema : schemas) {
+                for (UserDefinedFunction userDefinedFunction : sorted(schema.getAllFunctionsAndAggregates(),
+                        UserDefinedFunction.class)) {
+                    if (drop) {
+                        add(userDefinedFunction.getDropSQL(), false);
+                    }
+                    add(userDefinedFunction.getCreateSQL(), false);
                 }
-                if (drop) {
-                    add(obj.getDropSQL(), false);
-                }
-                add(obj.getCreateSQL(), false);
             }
-            for (UserAggregate agg : db.getAllAggregates()) {
-                if (drop) {
-                    add(agg.getDropSQL(), false);
+            for (Schema schema : schemas) {
+                for (Sequence sequence : sorted(schema.getAllSequences(), Sequence.class)) {
+                    if (sequence.getBelongsToTable()) {
+                        continue;
+                    }
+                    if (drop) {
+                        add(sequence.getDropSQL(), false);
+                    }
+                    add(sequence.getCreateSQL(), false);
                 }
-                add(agg.getCreateSQL(), false);
-            }
-            for (SchemaObject obj : db.getAllSchemaObjects(
-                    DbObject.SEQUENCE)) {
-                if (excludeSchema(obj.getSchema())) {
-                    continue;
-                }
-                Sequence sequence = (Sequence) obj;
-                if (drop) {
-                    add(sequence.getDropSQL(), false);
-                }
-                add(sequence.getCreateSQL(), false);
             }
 
             // Generate CREATE TABLE and INSERT...VALUES
@@ -273,7 +290,7 @@ public class ScriptCommand extends ScriptBase {
                 if (table.isHidden()) {
                     continue;
                 }
-                table.lock(session, false, false);
+                table.lock(session, Table.READ_LOCK);
                 String createTableSql = table.getCreateSQL();
                 if (createTableSql == null) {
                     // null for metadata tables
@@ -290,11 +307,12 @@ public class ScriptCommand extends ScriptBase {
                     }
                 }
                 if (TableType.TABLE == tableType) {
-                    if (table.canGetRowCount()) {
-                        String rowcount = "-- " +
-                                table.getRowCountApproximation() +
-                                " +/- SELECT COUNT(*) FROM " + table.getSQL();
-                        add(rowcount, false);
+                    if (table.canGetRowCount(session)) {
+                        StringBuilder builder = new StringBuilder("-- ")
+                                .append(table.getRowCountApproximation(session))
+                                .append(" +/- SELECT COUNT(*) FROM ");
+                        table.getSQL(builder, HasSQL.TRACE_SQL_FLAGS);
+                        add(builder.toString(), false);
                     }
                     if (data) {
                         count = generateInsertValues(count, table);
@@ -310,66 +328,41 @@ public class ScriptCommand extends ScriptBase {
             }
             if (tempLobTableCreated) {
                 add("DROP TABLE IF EXISTS SYSTEM_LOB_STREAM", true);
-                add("CALL SYSTEM_COMBINE_BLOB(-1)", true);
                 add("DROP ALIAS IF EXISTS SYSTEM_COMBINE_CLOB", true);
                 add("DROP ALIAS IF EXISTS SYSTEM_COMBINE_BLOB", true);
                 tempLobTableCreated = false;
             }
             // Generate CREATE CONSTRAINT ...
-            final ArrayList<SchemaObject> constraints = db.getAllSchemaObjects(
-                    DbObject.CONSTRAINT);
-            Collections.sort(constraints, new Comparator<SchemaObject>() {
-                @Override
-                public int compare(SchemaObject c1, SchemaObject c2) {
-                    return ((Constraint) c1).compareTo((Constraint) c2);
-                }
-            });
-            for (SchemaObject obj : constraints) {
-                if (excludeSchema(obj.getSchema())) {
-                    continue;
-                }
-                Constraint constraint = (Constraint) obj;
-                if (excludeTable(constraint.getTable())) {
-                    continue;
-                }
-                if (constraint.getTable().isHidden()) {
-                    continue;
-                }
-                if (Constraint.Type.PRIMARY_KEY != constraint.getConstraintType()) {
-                    add(constraint.getCreateSQLWithoutIndexes(), false);
-                }
-            }
-            // Generate CREATE TRIGGER ...
-            for (SchemaObject obj : db.getAllSchemaObjects(DbObject.TRIGGER)) {
-                if (excludeSchema(obj.getSchema())) {
-                    continue;
-                }
-                TriggerObject trigger = (TriggerObject) obj;
-                if (excludeTable(trigger.getTable())) {
-                    continue;
-                }
-                add(trigger.getCreateSQL(), false);
-            }
-            // Generate GRANT ...
-            for (Right right : db.getAllRights()) {
-                DbObject object = right.getGrantedObject();
-                if (object != null) {
-                    if (object instanceof Schema) {
-                        if (excludeSchema((Schema) object)) {
-                            continue;
-                        }
-                    } else if (object instanceof Table) {
-                        Table table = (Table) object;
-                        if (excludeSchema(table.getSchema())) {
-                            continue;
-                        }
-                        if (excludeTable(table)) {
-                            continue;
-                        }
+            ArrayList<Constraint> constraints = new ArrayList<>();
+            for (Schema schema : schemas) {
+                for (Constraint constraint : schema.getAllConstraints()) {
+                    if (excludeTable(constraint.getTable())) {
+                        continue;
+                    }
+                    Type constraintType = constraint.getConstraintType();
+                    if (constraintType != Type.DOMAIN && constraint.getTable().isHidden()) {
+                        continue;
+                    }
+                    if (constraintType != Constraint.Type.PRIMARY_KEY) {
+                        constraints.add(constraint);
                     }
                 }
-                add(right.getCreateSQL(), false);
             }
+            constraints.sort(null);
+            for (Constraint constraint : constraints) {
+                add(constraint.getCreateSQLWithoutIndexes(), false);
+            }
+            // Generate CREATE TRIGGER ...
+            for (Schema schema : schemas) {
+                for (TriggerObject trigger : schema.getAllTriggers()) {
+                    if (excludeTable(trigger.getTable())) {
+                        continue;
+                    }
+                    add(trigger.getCreateSQL(), false);
+                }
+            }
+            // Generate GRANT ...
+            dumpRights(db);
             // Generate COMMENT ON ...
             for (Comment comment : db.getAllComments()) {
                 add(comment.getCreateSQL(), false);
@@ -388,95 +381,223 @@ public class ScriptCommand extends ScriptBase {
         return r;
     }
 
+    private void dumpDomains(ArrayList<Schema> schemas) throws IOException {
+        TreeMap<Domain, TreeSet<Domain>> referencingDomains = new TreeMap<>(BY_NAME_COMPARATOR);
+        TreeSet<Domain> known = new TreeSet<>(BY_NAME_COMPARATOR);
+        for (Schema schema : schemas) {
+            for (Domain domain : sorted(schema.getAllDomains(), Domain.class)) {
+                Domain parent = domain.getDomain();
+                if (parent == null) {
+                    addDomain(domain);
+                } else {
+                    TreeSet<Domain> set = referencingDomains.get(parent);
+                    if (set == null) {
+                        set = new TreeSet<>(BY_NAME_COMPARATOR);
+                        referencingDomains.put(parent, set);
+                    }
+                    set.add(domain);
+                    if (parent.getDomain() == null || !schemas.contains(parent.getSchema())) {
+                        known.add(parent);
+                    }
+                }
+            }
+        }
+        while (!referencingDomains.isEmpty()) {
+            TreeSet<Domain> known2 = new TreeSet<>(BY_NAME_COMPARATOR);
+            for (Domain d : known) {
+                TreeSet<Domain> set = referencingDomains.remove(d);
+                if (set != null) {
+                    for (Domain d2 : set) {
+                        addDomain(d2);
+                        known2.add(d2);
+                    }
+                }
+            }
+            known = known2;
+        }
+    }
+
+    private void dumpRights(Database db) throws IOException {
+        Right[] rights = db.getAllRights().toArray(new Right[0]);
+        Arrays.sort(rights, (o1, o2) -> {
+            Role r1 = o1.getGrantedRole(), r2 = o2.getGrantedRole();
+            if ((r1 == null) != (r2 == null)) {
+                return r1 == null ? -1 : 1;
+            }
+            if (r1 == null) {
+                DbObject g1 = o1.getGrantedObject(), g2 = o2.getGrantedObject();
+                if ((g1 == null) != (g2 == null)) {
+                    return g1 == null ? -1 : 1;
+                }
+                if (g1 != null) {
+                    if (g1 instanceof Schema != g2 instanceof Schema) {
+                        return g1 instanceof Schema ? -1 : 1;
+                    }
+                    int cmp = g1.getName().compareTo(g2.getName());
+                    if (cmp != 0) {
+                        return cmp;
+                    }
+                }
+            } else {
+                int cmp = r1.getName().compareTo(r2.getName());
+                if (cmp != 0) {
+                    return cmp;
+                }
+            }
+            return o1.getGrantee().getName().compareTo(o2.getGrantee().getName());
+        });
+        for (Right right : rights) {
+            DbObject object = right.getGrantedObject();
+            if (object != null) {
+                if (object instanceof Schema) {
+                    if (excludeSchema((Schema) object)) {
+                        continue;
+                    }
+                } else if (object instanceof Table) {
+                    Table table = (Table) object;
+                    if (excludeSchema(table.getSchema())) {
+                        continue;
+                    }
+                    if (excludeTable(table)) {
+                        continue;
+                    }
+                }
+            }
+            add(right.getCreateSQL(), false);
+        }
+    }
+
+    private void addDomain(Domain domain) throws IOException {
+        if (drop) {
+            add(domain.getDropSQL(), false);
+        }
+        add(domain.getCreateSQL(), false);
+    }
+
+    private static <T extends DbObject> T[] sorted(Collection<T> collection, Class<T> clazz) {
+        @SuppressWarnings("unchecked")
+        T[] array = collection.toArray((T[]) java.lang.reflect.Array.newInstance(clazz, 0));
+        Arrays.sort(array, BY_NAME_COMPARATOR);
+        return array;
+    }
+
     private int generateInsertValues(int count, Table table) throws IOException {
         PlanItem plan = table.getBestPlanItem(session, null, null, -1, null, null);
         Index index = plan.getIndex();
         Cursor cursor = index.find(session, null, null);
         Column[] columns = table.getColumns();
-        StatementBuilder buff = new StatementBuilder("INSERT INTO ");
-        buff.append(table.getSQL()).append('(');
-        for (Column col : columns) {
-            buff.appendExceptFirst(", ");
-            buff.append(Parser.quoteIdentifier(col.getName()));
+        boolean withGenerated = false, withGeneratedAlwaysAsIdentity = false;
+        for (Column c : columns) {
+            if (c.isGeneratedAlways()) {
+                if (c.isIdentity()) {
+                    withGeneratedAlwaysAsIdentity = true;
+                } else {
+                    withGenerated = true;
+                }
+            }
         }
-        buff.append(") VALUES");
+        StringBuilder builder = new StringBuilder("INSERT INTO ");
+        table.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS);
+        if (withGenerated || withGeneratedAlwaysAsIdentity || withColumns) {
+            builder.append('(');
+            boolean needComma = false;
+            for (Column column : columns) {
+                if (!column.isGenerated()) {
+                    if (needComma) {
+                        builder.append(", ");
+                    }
+                    needComma = true;
+                    column.getSQL(builder, HasSQL.DEFAULT_SQL_FLAGS);
+                }
+            }
+            builder.append(')');
+            if (withGeneratedAlwaysAsIdentity) {
+                builder.append(" OVERRIDING SYSTEM VALUE");
+            }
+        }
+        builder.append(" VALUES");
         if (!simple) {
-            buff.append('\n');
+            builder.append('\n');
         }
-        buff.append('(');
-        String ins = buff.toString();
-        buff = null;
+        builder.append('(');
+        String ins = builder.toString();
+        builder = null;
+        int columnCount = columns.length;
         while (cursor.next()) {
             Row row = cursor.get();
-            if (buff == null) {
-                buff = new StatementBuilder(ins);
+            if (builder == null) {
+                builder = new StringBuilder(ins);
             } else {
-                buff.append(",\n(");
+                builder.append(",\n(");
             }
-            for (int j = 0; j < row.getColumnCount(); j++) {
-                if (j > 0) {
-                    buff.append(", ");
+            boolean needComma = false;
+            for (int i = 0; i < columnCount; i++) {
+                if (columns[i].isGenerated()) {
+                    continue;
                 }
-                Value v = row.getValue(j);
-                if (v.getPrecision() > lobBlockSize) {
+                if (needComma) {
+                    builder.append(", ");
+                }
+                needComma = true;
+                Value v = row.getValue(i);
+                if (v.getType().getPrecision() > lobBlockSize) {
                     int id;
-                    if (v.getType() == Value.CLOB) {
+                    if (v.getValueType() == Value.CLOB) {
                         id = writeLobStream(v);
-                        buff.append("SYSTEM_COMBINE_CLOB(").append(id).append(')');
-                    } else if (v.getType() == Value.BLOB) {
+                        builder.append("SYSTEM_COMBINE_CLOB(").append(id).append(')');
+                    } else if (v.getValueType() == Value.BLOB) {
                         id = writeLobStream(v);
-                        buff.append("SYSTEM_COMBINE_BLOB(").append(id).append(')');
+                        builder.append("SYSTEM_COMBINE_BLOB(").append(id).append(')');
                     } else {
-                        buff.append(v.getSQL());
+                        v.getSQL(builder, HasSQL.NO_CASTS);
                     }
                 } else {
-                    buff.append(v.getSQL());
+                    v.getSQL(builder, HasSQL.NO_CASTS);
                 }
             }
-            buff.append(')');
+            builder.append(')');
             count++;
             if ((count & 127) == 0) {
                 checkCanceled();
             }
-            if (simple || buff.length() > Constants.IO_BUFFER_SIZE) {
-                add(buff.toString(), true);
-                buff = null;
+            if (simple || builder.length() > Constants.IO_BUFFER_SIZE) {
+                add(builder.toString(), true);
+                builder = null;
             }
         }
-        if (buff != null) {
-            add(buff.toString(), true);
+        if (builder != null) {
+            add(builder.toString(), true);
         }
         return count;
     }
 
     private int writeLobStream(Value v) throws IOException {
         if (!tempLobTableCreated) {
-            add("CREATE TABLE IF NOT EXISTS SYSTEM_LOB_STREAM" +
+            add("CREATE CACHED LOCAL TEMPORARY TABLE IF NOT EXISTS SYSTEM_LOB_STREAM" +
                     "(ID INT NOT NULL, PART INT NOT NULL, " +
-                    "CDATA VARCHAR, BDATA BINARY)",
+                    "CDATA VARCHAR, BDATA VARBINARY)",
                     true);
-            add("CREATE PRIMARY KEY SYSTEM_LOB_STREAM_PRIMARY_KEY " +
-                    "ON SYSTEM_LOB_STREAM(ID, PART)", true);
-            add("CREATE ALIAS IF NOT EXISTS " + "SYSTEM_COMBINE_CLOB FOR \"" +
-                    this.getClass().getName() + ".combineClob\"", true);
-            add("CREATE ALIAS IF NOT EXISTS " + "SYSTEM_COMBINE_BLOB FOR \"" +
-                    this.getClass().getName() + ".combineBlob\"", true);
+            add("ALTER TABLE SYSTEM_LOB_STREAM ADD CONSTRAINT SYSTEM_LOB_STREAM_PRIMARY_KEY PRIMARY KEY(ID, PART)",
+                    true);
+            String className = getClass().getName();
+            add("CREATE ALIAS IF NOT EXISTS " + "SYSTEM_COMBINE_CLOB FOR '" + className + ".combineClob'", true);
+            add("CREATE ALIAS IF NOT EXISTS " + "SYSTEM_COMBINE_BLOB FOR '" + className + ".combineBlob'", true);
             tempLobTableCreated = true;
         }
         int id = nextLobId++;
-        switch (v.getType()) {
+        switch (v.getValueType()) {
         case Value.BLOB: {
             byte[] bytes = new byte[lobBlockSize];
             try (InputStream input = v.getInputStream()) {
                 for (int i = 0;; i++) {
                     StringBuilder buff = new StringBuilder(lobBlockSize * 2);
                     buff.append("INSERT INTO SYSTEM_LOB_STREAM VALUES(").append(id)
-                            .append(", ").append(i).append(", NULL, '");
+                            .append(", ").append(i).append(", NULL, X'");
                     int len = IOUtils.readFully(input, bytes, lobBlockSize);
                     if (len <= 0) {
                         break;
                     }
-                    buff.append(StringUtils.convertBytesToHex(bytes, len)).append("')");
+                    StringUtils.convertBytesToHex(buff, bytes, len).append("')");
                     String sql = buff.toString();
                     add(sql, true);
                 }
@@ -495,7 +616,7 @@ public class ScriptCommand extends ScriptBase {
                     if (len == 0) {
                         break;
                     }
-                    buff.append(StringUtils.quoteStringSQL(new String(chars, 0, len))).
+                    StringUtils.quoteStringSQL(buff, new String(chars, 0, len)).
                         append(", NULL)");
                     String sql = buff.toString();
                     add(sql, true);
@@ -504,7 +625,7 @@ public class ScriptCommand extends ScriptBase {
             break;
         }
         default:
-            DbException.throwInternalError("type:" + v.getType());
+            throw DbException.getInternalError("type:" + v.getValueType());
         }
         return id;
     }
@@ -517,6 +638,7 @@ public class ScriptCommand extends ScriptBase {
      * @param conn a connection
      * @param id the lob id
      * @return a stream for the combined data
+     * @throws SQLException on failure
      */
     public static InputStream combineBlob(Connection conn, int id)
             throws SQLException {
@@ -548,7 +670,7 @@ public class ScriptCommand extends ScriptBase {
                         }
                         current = null;
                     } catch (SQLException e) {
-                        throw DbException.convertToIOException(e);
+                        throw DataUtils.convertToIOException(e);
                     }
                 }
             }
@@ -561,7 +683,7 @@ public class ScriptCommand extends ScriptBase {
                 try {
                     rs.close();
                 } catch (SQLException e) {
-                    throw DbException.convertToIOException(e);
+                    throw DataUtils.convertToIOException(e);
                 }
             }
         };
@@ -574,6 +696,7 @@ public class ScriptCommand extends ScriptBase {
      * @param conn a connection
      * @param id the lob id
      * @return a reader for the combined data
+     * @throws SQLException on failure
      */
     public static Reader combineClob(Connection conn, int id) throws SQLException {
         if (id < 0) {
@@ -604,7 +727,7 @@ public class ScriptCommand extends ScriptBase {
                         }
                         current = null;
                     } catch (SQLException e) {
-                        throw DbException.convertToIOException(e);
+                        throw DataUtils.convertToIOException(e);
                     }
                 }
             }
@@ -617,7 +740,7 @@ public class ScriptCommand extends ScriptBase {
                 try {
                     rs.close();
                 } catch (SQLException e) {
-                    throw DbException.convertToIOException(e);
+                    throw DataUtils.convertToIOException(e);
                 }
             }
             @Override
@@ -654,7 +777,7 @@ public class ScriptCommand extends ScriptBase {
     private void reset() {
         result = null;
         buffer = null;
-        lineSeparatorString = SysProperties.LINE_SEPARATOR;
+        lineSeparatorString = System.lineSeparator();
         lineSeparator = lineSeparatorString.getBytes(charset);
     }
 
@@ -664,7 +787,7 @@ public class ScriptCommand extends ScriptBase {
         }
         if (tables != null) {
             // if filtering on specific tables, only include those schemas
-            for (Table table : schema.getAllTablesAndViews()) {
+            for (Table table : schema.getAllTablesAndViews(session)) {
                 if (tables.contains(table)) {
                     return false;
                 }
@@ -704,12 +827,10 @@ public class ScriptCommand extends ScriptBase {
             }
             out.write(buffer, 0, len);
             if (!insert) {
-                Value[] row = { ValueString.get(s) };
-                result.addRow(row);
+                result.addRow(ValueVarchar.get(s));
             }
         } else {
-            Value[] row = { ValueString.get(s) };
-            result.addRow(row);
+            result.addRow(ValueVarchar.get(s));
         }
     }
 
@@ -717,6 +838,14 @@ public class ScriptCommand extends ScriptBase {
         this.simple = simple;
     }
 
+    public void setWithColumns(boolean withColumns) {
+        this.withColumns = withColumns;
+    }
+
+    public void setVersion(boolean version) {
+        this.version = version;
+    }
+
     public void setCharset(Charset charset) {
         this.charset = charset;
     }
