@@ -1,12 +1,12 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
 
+import java.io.File;
 import java.io.IOException;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -16,19 +16,24 @@ import org.h2.api.ErrorCode;
 import org.h2.command.dml.SetTypes;
 import org.h2.message.DbException;
 import org.h2.security.SHA256;
-import org.h2.store.fs.FilePathEncrypt;
-import org.h2.store.fs.FilePathRec;
 import org.h2.store.fs.FileUtils;
-import org.h2.util.New;
+import org.h2.store.fs.encrypt.FilePathEncrypt;
+import org.h2.store.fs.rec.FilePathRec;
+import org.h2.util.IOUtils;
+import org.h2.util.NetworkConnectionInfo;
 import org.h2.util.SortedProperties;
 import org.h2.util.StringUtils;
+import org.h2.util.TimeZoneProvider;
 import org.h2.util.Utils;
 
 /**
  * Encapsulates the connection settings, including user name and password.
  */
 public class ConnectionInfo implements Cloneable {
-    private static final HashSet<String> KNOWN_SETTINGS = New.hashSet();
+
+    private static final HashSet<String> KNOWN_SETTINGS;
+
+    private static final HashSet<String> IGNORED_BY_PARSER;
 
     private Properties prop = new Properties();
     private String originalURL;
@@ -38,6 +43,8 @@ public class ConnectionInfo implements Cloneable {
     private byte[] fileEncryptionKey;
     private byte[] userPasswordHash;
 
+    private TimeZoneProvider timeZone;
+
     /**
      * The database name
      */
@@ -48,6 +55,8 @@ public class ConnectionInfo implements Cloneable {
     private boolean persistent;
     private boolean unnamed;
 
+    private NetworkConnectionInfo networkConnectionInfo;
+
     /**
      * Create a connection info object.
      *
@@ -64,21 +73,36 @@ public class ConnectionInfo implements Cloneable {
      * Create a connection info object.
      *
      * @param u the database URL (must start with jdbc:h2:)
-     * @param info the connection properties
+     * @param info the connection properties or {@code null}
+     * @param user the user name or {@code null}
+     * @param password
+     *            the password as {@code String} or {@code char[]}, or
+     *            {@code null}
      */
-    public ConnectionInfo(String u, Properties info) {
+    public ConnectionInfo(String u, Properties info, String user, Object password) {
         u = remapURL(u);
-        this.originalURL = u;
+        originalURL = url = u;
         if (!u.startsWith(Constants.START_URL)) {
-            throw DbException.getInvalidValueException("url", u);
+            throw getFormatException();
+        }
+        if (info != null) {
+            readProperties(info);
+        }
+        if (user != null) {
+            prop.put("USER", user);
+        }
+        if (password != null) {
+            prop.put("PASSWORD", password);
         }
-        this.url = u;
-        readProperties(info);
         readSettingsFromURL();
+        Object timeZoneName = prop.remove("TIME ZONE");
+        if (timeZoneName != null) {
+            timeZone = TimeZoneProvider.ofId(timeZoneName.toString());
+        }
         setUserName(removeProperty("USER", ""));
-        convertPasswords();
         name = url.substring(Constants.START_URL.length());
         parseName();
+        convertPasswords();
         String recoverTest = removeProperty("RECOVER_TEST", null);
         if (recoverTest != null) {
             FilePathRec.register();
@@ -92,26 +116,73 @@ public class ConnectionInfo implements Cloneable {
     }
 
     static {
-        ArrayList<String> list = SetTypes.getTypes();
-        HashSet<String> set = KNOWN_SETTINGS;
-        set.addAll(list);
-        String[] connectionTime = { "ACCESS_MODE_DATA", "AUTOCOMMIT", "CIPHER",
-                "CREATE", "CACHE_TYPE", "FILE_LOCK", "IGNORE_UNKNOWN_SETTINGS",
-                "IFEXISTS", "INIT", "PASSWORD", "RECOVER", "RECOVER_TEST",
-                "USER", "AUTO_SERVER", "AUTO_SERVER_PORT", "NO_UPGRADE",
-                "AUTO_RECONNECT", "OPEN_NEW", "PAGE_SIZE", "PASSWORD_HASH", "JMX" };
-        for (String key : connectionTime) {
-            if (SysProperties.CHECK && set.contains(key)) {
-                DbException.throwInternalError(key);
+        String[] commonSettings = { //
+                "ACCESS_MODE_DATA", "AUTO_RECONNECT", "AUTO_SERVER", "AUTO_SERVER_PORT", //
+                "CACHE_TYPE", //
+                "FILE_LOCK", //
+                "JMX", //
+                "NETWORK_TIMEOUT", //
+                "OLD_INFORMATION_SCHEMA", "OPEN_NEW", //
+                "PAGE_SIZE", //
+                "RECOVER", //
+        };
+        String[] settings = { //
+                "AUTHREALM", "AUTHZPWD", "AUTOCOMMIT", //
+                "CIPHER", "CREATE", //
+                "FORBID_CREATION", //
+                "IGNORE_UNKNOWN_SETTINGS", "IFEXISTS", "INIT", //
+                "NO_UPGRADE", //
+                "PASSWORD", "PASSWORD_HASH", //
+                "RECOVER_TEST", //
+                "USER" //
+        };
+        HashSet<String> set = new HashSet<>(128);
+        set.addAll(SetTypes.getTypes());
+        for (String setting : commonSettings) {
+            if (!set.add(setting)) {
+                throw DbException.getInternalError(setting);
             }
-            set.add(key);
         }
+        for (String setting : settings) {
+            if (!set.add(setting)) {
+                throw DbException.getInternalError(setting);
+            }
+        }
+        KNOWN_SETTINGS = set;
+        settings = new String[] { //
+                "ASSERT", //
+                "BINARY_COLLATION", //
+                "DB_CLOSE_ON_EXIT", //
+                "PAGE_STORE", //
+                "UUID_COLLATION", //
+        };
+        set = new HashSet<>(32);
+        for (String setting : commonSettings) {
+            set.add(setting);
+        }
+        for (String setting : settings) {
+            set.add(setting);
+        }
+        IGNORED_BY_PARSER = set;
     }
 
     private static boolean isKnownSetting(String s) {
         return KNOWN_SETTINGS.contains(s);
     }
 
+    /**
+     * Returns whether setting with the specified name should be ignored by
+     * parser.
+     *
+     * @param name
+     *            the name of the setting
+     * @return whether setting with the specified name should be ignored by
+     *         parser
+     */
+    public static boolean isIgnoredByParser(String name) {
+        return IGNORED_BY_PARSER.contains(name);
+    }
+
     @Override
     public ConnectionInfo clone() throws CloneNotSupportedException {
         ConnectionInfo clone = (ConnectionInfo) super.clone();
@@ -145,11 +216,7 @@ public class ConnectionInfo implements Cloneable {
             persistent = true;
         }
         if (persistent && !remote) {
-            if ("/".equals(SysProperties.FILE_SEPARATOR)) {
-                name = name.replace('\\', '/');
-            } else {
-                name = name.replace('/', '\\');
-            }
+            name = IOUtils.nameSeparatorsToNative(name);
         }
     }
 
@@ -165,7 +232,7 @@ public class ConnectionInfo implements Cloneable {
             boolean absolute = FileUtils.isAbsolute(name);
             String n;
             String prefix = null;
-            if (dir.endsWith(SysProperties.FILE_SEPARATOR)) {
+            if (dir.endsWith(File.separator)) {
                 dir = dir.substring(0, dir.length() - 1);
             }
             if (absolute) {
@@ -173,7 +240,7 @@ public class ConnectionInfo implements Cloneable {
             } else {
                 n  = FileUtils.unwrap(name);
                 prefix = name.substring(0, name.length() - n.length());
-                n = dir + SysProperties.FILE_SEPARATOR + n;
+                n = dir + File.separatorChar + n;
             }
             String normalizedName = FileUtils.unwrap(FileUtils.toRealPath(n));
             if (normalizedName.equals(absDir) || !normalizedName.startsWith(absDir)) {
@@ -192,7 +259,7 @@ public class ConnectionInfo implements Cloneable {
                         absDir);
             }
             if (!absolute) {
-                name = prefix + dir + SysProperties.FILE_SEPARATOR + FileUtils.unwrap(name);
+                name = prefix + dir + File.separatorChar + FileUtils.unwrap(name);
             }
         }
     }
@@ -225,8 +292,7 @@ public class ConnectionInfo implements Cloneable {
     }
 
     private void readProperties(Properties info) {
-        Object[] list = new Object[info.size()];
-        info.keySet().toArray(list);
+        Object[] list = info.keySet().toArray();
         DbSettings s = null;
         for (Object k : list) {
             String key = StringUtils.toUpperEnglish(k.toString());
@@ -248,14 +314,15 @@ public class ConnectionInfo implements Cloneable {
     }
 
     private void readSettingsFromURL() {
-        DbSettings dbSettings = DbSettings.getInstance(null);
+        DbSettings defaultSettings = DbSettings.DEFAULT;
         int idx = url.indexOf(';');
         if (idx >= 0) {
             String settings = url.substring(idx + 1);
             url = url.substring(0, idx);
+            String unknownSetting = null;
             String[] list = StringUtils.arraySplit(settings, ';', false);
             for (String setting : list) {
-                if (setting.length() == 0) {
+                if (setting.isEmpty()) {
                     continue;
                 }
                 int equal = setting.indexOf('=');
@@ -265,20 +332,32 @@ public class ConnectionInfo implements Cloneable {
                 String value = setting.substring(equal + 1);
                 String key = setting.substring(0, equal);
                 key = StringUtils.toUpperEnglish(key);
-                if (!isKnownSetting(key) && !dbSettings.containsKey(key)) {
-                    throw DbException.get(ErrorCode.UNSUPPORTED_SETTING_1, key);
+                if (isKnownSetting(key) || defaultSettings.containsKey(key)) {
+                    String old = prop.getProperty(key);
+                    if (old != null && !old.equals(value)) {
+                        throw DbException.get(ErrorCode.DUPLICATE_PROPERTY_1, key);
+                    }
+                    prop.setProperty(key, value);
+                } else {
+                    unknownSetting = key;
                 }
-                String old = prop.getProperty(key);
-                if (old != null && !old.equals(value)) {
-                    throw DbException.get(ErrorCode.DUPLICATE_PROPERTY_1, key);
-                }
-                prop.setProperty(key, value);
             }
+            if (unknownSetting != null //
+                    && !Utils.parseBoolean(prop.getProperty("IGNORE_UNKNOWN_SETTINGS"), false, false)) {
+                throw DbException.get(ErrorCode.UNSUPPORTED_SETTING_1, unknownSetting);
+            }
+        }
+    }
+
+    private void preservePasswordForAuthentication(Object password) {
+        if ((!isRemote() || isSSL()) &&  prop.containsKey("AUTHREALM") && password!=null) {
+            prop.put("AUTHZPWD",password instanceof char[] ? new String((char[])password) : password);
         }
     }
 
     private char[] removePassword() {
         Object p = prop.remove("PASSWORD");
+        preservePasswordForAuthentication(p);
         if (p == null) {
             return new char[0];
         } else if (p instanceof char[]) {
@@ -307,10 +386,8 @@ public class ConnectionInfo implements Cloneable {
             if (space < 0) {
                 throw DbException.get(ErrorCode.WRONG_PASSWORD_FORMAT);
             }
-            char[] np = new char[password.length - space - 1];
-            char[] filePassword = new char[space];
-            System.arraycopy(password, space + 1, np, 0, np.length);
-            System.arraycopy(password, 0, filePassword, 0, space);
+            char[] np = Arrays.copyOfRange(password, space + 1, password.length);
+            char[] filePassword = Arrays.copyOf(password, space);
             Arrays.fill(password, (char) 0);
             password = np;
             fileEncryptionKey = FilePathEncrypt.getPasswordBytes(filePassword);
@@ -324,7 +401,7 @@ public class ConnectionInfo implements Cloneable {
         if (passwordHash) {
             return StringUtils.convertHexToBytes(new String(password));
         }
-        if (userName.length() == 0 && password.length == 0) {
+        if (userName.isEmpty() && password.length == 0) {
             return new byte[0];
         }
         return SHA256.getKeyPasswordHash(userName, password);
@@ -337,16 +414,8 @@ public class ConnectionInfo implements Cloneable {
      * @param defaultValue the default value
      * @return the value
      */
-    boolean getProperty(String key, boolean defaultValue) {
-        String x = getProperty(key, null);
-        if (x == null) {
-            return defaultValue;
-        }
-        // support 0 / 1 (like the parser)
-        if (x.length() == 1 && Character.isDigit(x.charAt(0))) {
-            return Integer.parseInt(x) != 0;
-        }
-        return Boolean.parseBoolean(x);
+    public boolean getProperty(String key, boolean defaultValue) {
+        return Utils.parseBoolean(getProperty(key, null), defaultValue, false);
     }
 
     /**
@@ -357,8 +426,7 @@ public class ConnectionInfo implements Cloneable {
      * @return the value
      */
     public boolean removeProperty(String key, boolean defaultValue) {
-        String x = removeProperty(key, null);
-        return x == null ? defaultValue : Boolean.parseBoolean(x);
+        return Utils.parseBoolean(removeProperty(key, null), defaultValue, false);
     }
 
     /**
@@ -370,7 +438,7 @@ public class ConnectionInfo implements Cloneable {
      */
     String removeProperty(String key, String defaultValue) {
         if (SysProperties.CHECK && !isKnownSetting(key)) {
-            DbException.throwInternalError(key);
+            throw DbException.getInternalError(key);
         }
         Object x = prop.remove(key);
         return x == null ? defaultValue : x.toString();
@@ -382,42 +450,28 @@ public class ConnectionInfo implements Cloneable {
      * @return the database name
      */
     public String getName() {
-        if (persistent) {
-            if (nameNormalized == null) {
-                if (!SysProperties.IMPLICIT_RELATIVE_PATH) {
-                    if (!FileUtils.isAbsolute(name)) {
-                        if (name.indexOf("./") < 0 &&
-                                name.indexOf(".\\") < 0 &&
-                                name.indexOf(":/") < 0 &&
-                                name.indexOf(":\\") < 0) {
-                            // the name could start with "./", or
-                            // it could start with a prefix such as "nio:./"
-                            // for Windows, the path "\test" is not considered
-                            // absolute as the drive letter is missing,
-                            // but we consider it absolute
-                            throw DbException.get(
-                                    ErrorCode.URL_RELATIVE_TO_CWD,
-                                    originalURL);
-                        }
-                    }
-                }
-                String suffix = Constants.SUFFIX_PAGE_FILE;
-                String n;
-                if (FileUtils.exists(name + suffix)) {
-                    n = FileUtils.toRealPath(name + suffix);
-                } else {
-                    suffix = Constants.SUFFIX_MV_FILE;
-                    n = FileUtils.toRealPath(name + suffix);
-                }
-                String fileName = FileUtils.getName(n);
-                if (fileName.length() < suffix.length() + 1) {
-                    throw DbException.get(ErrorCode.INVALID_DATABASE_NAME_1, name);
-                }
-                nameNormalized = n.substring(0, n.length() - suffix.length());
-            }
-            return nameNormalized;
+        if (!persistent) {
+            return name;
         }
-        return name;
+        if (nameNormalized == null) {
+            if (!FileUtils.isAbsolute(name) && !name.contains("./") && !name.contains(".\\") && !name.contains(":/")
+                    && !name.contains(":\\")) {
+                // the name could start with "./", or
+                // it could start with a prefix such as "nioMapped:./"
+                // for Windows, the path "\test" is not considered
+                // absolute as the drive letter is missing,
+                // but we consider it absolute
+                throw DbException.get(ErrorCode.URL_RELATIVE_TO_CWD, originalURL);
+            }
+            String suffix = Constants.SUFFIX_MV_FILE;
+            String n = FileUtils.toRealPath(name + suffix);
+            String fileName = FileUtils.getName(n);
+            if (fileName.length() < suffix.length() + 1) {
+                throw DbException.get(ErrorCode.INVALID_DATABASE_NAME_1, name);
+            }
+            nameNormalized = n.substring(0, n.length() - suffix.length());
+        }
+        return nameNormalized;
     }
 
     /**
@@ -457,9 +511,7 @@ public class ConnectionInfo implements Cloneable {
      * @return the property keys
      */
     String[] getKeys() {
-        String[] keys = new String[prop.size()];
-        prop.keySet().toArray(keys);
-        return keys;
+        return prop.keySet().toArray(new String[prop.size()]);
     }
 
     /**
@@ -470,7 +522,7 @@ public class ConnectionInfo implements Cloneable {
      */
     String getProperty(String key) {
         Object value = prop.get(key);
-        if (value == null || !(value instanceof String)) {
+        if (!(value instanceof String)) {
             return null;
         }
         return value.toString();
@@ -485,7 +537,7 @@ public class ConnectionInfo implements Cloneable {
      */
     int getProperty(String key, int defaultValue) {
         if (SysProperties.CHECK && !isKnownSetting(key)) {
-            DbException.throwInternalError(key);
+            throw DbException.getInternalError(key);
         }
         String s = getProperty(key);
         return s == null ? defaultValue : Integer.parseInt(s);
@@ -500,7 +552,7 @@ public class ConnectionInfo implements Cloneable {
      */
     public String getProperty(String key, String defaultValue) {
         if (SysProperties.CHECK && !isKnownSetting(key)) {
-            DbException.throwInternalError(key);
+            throw DbException.getInternalError(key);
         }
         String s = getProperty(key);
         return s == null ? defaultValue : s;
@@ -618,13 +670,21 @@ public class ConnectionInfo implements Cloneable {
     }
 
     /**
-     * Generate an URL format exception.
+     * Returns the time zone.
+     *
+     * @return the time zone
+     */
+    public TimeZoneProvider getTimeZone() {
+        return timeZone;
+    }
+
+    /**
+     * Generate a URL format exception.
      *
      * @return the exception
      */
     DbException getFormatException() {
-        String format = Constants.URL_FORMAT;
-        return DbException.get(ErrorCode.URL_FORMAT_ERROR_2, format, url);
+        return DbException.get(ErrorCode.URL_FORMAT_ERROR_2, Constants.URL_FORMAT, url);
     }
 
     /**
@@ -638,15 +698,30 @@ public class ConnectionInfo implements Cloneable {
         this.name = serverKey;
     }
 
+    /**
+     * Returns the network connection information, or {@code null}.
+     *
+     * @return the network connection information, or {@code null}
+     */
+    public NetworkConnectionInfo getNetworkConnectionInfo() {
+        return networkConnectionInfo;
+    }
+
+    /**
+     * Sets the network connection information.
+     *
+     * @param networkConnectionInfo the network connection information
+     */
+    public void setNetworkConnectionInfo(NetworkConnectionInfo networkConnectionInfo) {
+        this.networkConnectionInfo = networkConnectionInfo;
+    }
+
     public DbSettings getDbSettings() {
-        DbSettings defaultSettings = DbSettings.getInstance(null);
-        HashMap<String, String> s = null;
+        DbSettings defaultSettings = DbSettings.DEFAULT;
+        HashMap<String, String> s = new HashMap<>(DbSettings.TABLE_SIZE);
         for (Object k : prop.keySet()) {
             String key = k.toString();
             if (!isKnownSetting(key) && defaultSettings.containsKey(key)) {
-                if (s == null) {
-                    s = New.hashMap();
-                }
                 s.put(key, prop.getProperty(key));
             }
         }
@@ -655,7 +730,7 @@ public class ConnectionInfo implements Cloneable {
 
     private static String remapURL(String url) {
         String urlMap = SysProperties.URL_MAP;
-        if (urlMap != null && urlMap.length() > 0) {
+        if (urlMap != null && !urlMap.isEmpty()) {
             try {
                 SortedProperties prop;
                 prop = SortedProperties.loadProperties(urlMap);
@@ -665,7 +740,7 @@ public class ConnectionInfo implements Cloneable {
                     prop.store(urlMap);
                 } else {
                     url2 = url2.trim();
-                    if (url2.length() > 0) {
+                    if (!url2.isEmpty()) {
                         return url2;
                     }
                 }
@@ -676,4 +751,11 @@ public class ConnectionInfo implements Cloneable {
         return url;
     }
 
+    /**
+     * Clear authentication properties.
+     */
+    public void cleanAuthenticationInfo() {
+        removeProperty("AUTHREALM", false);
+        removeProperty("AUTHZPWD", false);
+    }
 }
