@@ -1,11 +1,10 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
 
-import java.nio.charset.Charset;
 import java.sql.ResultSet;
 
 /**
@@ -16,22 +15,13 @@ public class Constants {
     /**
      * The build date is updated for each public release.
      */
-    public static final String BUILD_DATE = "2017-06-10";
+    public static final String BUILD_DATE = "2022-06-13";
 
     /**
-     * The build date of the last stable release.
+     * Sequential version number. Even numbers are used for official releases,
+     * odd numbers are used for development builds.
      */
-    public static final String BUILD_DATE_STABLE = "2017-04-23";
-
-    /**
-     * The build id is incremented for each public release.
-     */
-    public static final int BUILD_ID = 196;
-
-    /**
-     * The build id of the last stable release.
-     */
-    public static final int BUILD_ID_STABLE = 195;
+    public static final int BUILD_ID = 214;
 
     /**
      * Whether this is a snapshot version.
@@ -47,69 +37,48 @@ public class Constants {
     public static final String BUILD_VENDOR_AND_VERSION = null;
 
     /**
-     * The TCP protocol version number 6.
+     * The TCP protocol version number 17.
+     * @since 1.4.197 (2018-03-18)
      */
-    public static final int TCP_PROTOCOL_VERSION_6 = 6;
+    public static final int TCP_PROTOCOL_VERSION_17 = 17;
 
     /**
-     * The TCP protocol version number 7.
+     * The TCP protocol version number 18.
+     * @since 1.4.198 (2019-02-22)
      */
-    public static final int TCP_PROTOCOL_VERSION_7 = 7;
+    public static final int TCP_PROTOCOL_VERSION_18 = 18;
 
     /**
-     * The TCP protocol version number 8.
+     * The TCP protocol version number 19.
+     * @since 1.4.200 (2019-10-14)
      */
-    public static final int TCP_PROTOCOL_VERSION_8 = 8;
+    public static final int TCP_PROTOCOL_VERSION_19 = 19;
 
     /**
-     * The TCP protocol version number 9.
+     * The TCP protocol version number 20.
+     * @since 2.0.202 (2021-11-25)
      */
-    public static final int TCP_PROTOCOL_VERSION_9 = 9;
+    public static final int TCP_PROTOCOL_VERSION_20 = 20;
 
     /**
-     * The TCP protocol version number 10.
+     * Minimum supported version of TCP protocol.
      */
-    public static final int TCP_PROTOCOL_VERSION_10 = 10;
+    public static final int TCP_PROTOCOL_VERSION_MIN_SUPPORTED = TCP_PROTOCOL_VERSION_17;
 
     /**
-     * The TCP protocol version number 11.
+     * Maximum supported version of TCP protocol.
      */
-    public static final int TCP_PROTOCOL_VERSION_11 = 11;
-
-    /**
-     * The TCP protocol version number 12.
-     */
-    public static final int TCP_PROTOCOL_VERSION_12 = 12;
-
-    /**
-     * The TCP protocol version number 13.
-     */
-    public static final int TCP_PROTOCOL_VERSION_13 = 13;
-
-    /**
-     * The TCP protocol version number 14.
-     */
-    public static final int TCP_PROTOCOL_VERSION_14 = 14;
-
-    /**
-     * The TCP protocol version number 15.
-     */
-    public static final int TCP_PROTOCOL_VERSION_15 = 15;
-
-    /**
-     * The TCP protocol version number 16.
-     */
-    public static final int TCP_PROTOCOL_VERSION_16 = 16;
+    public static final int TCP_PROTOCOL_VERSION_MAX_SUPPORTED = TCP_PROTOCOL_VERSION_20;
 
     /**
      * The major version of this database.
      */
-    public static final int VERSION_MAJOR = 1;
+    public static final int VERSION_MAJOR = 2;
 
     /**
      * The minor version of this database.
      */
-    public static final int VERSION_MINOR = 4;
+    public static final int VERSION_MINOR = 1;
 
     /**
      * The lock mode that means no locking is used at all.
@@ -151,6 +120,11 @@ public class Constants {
      */
     public static final int ALLOW_LITERALS_NUMBERS = 1;
 
+    /**
+     * SNAPSHOT isolation level of transaction.
+     */
+    public static final int TRANSACTION_SNAPSHOT = 6;
+
     /**
      * Whether searching in Blob values should be supported.
      */
@@ -161,11 +135,6 @@ public class Constants {
      */
     public static final int CACHE_MIN_RECORDS = 16;
 
-    /**
-     * The default cache size in KB for each GB of RAM.
-     */
-    public static final int CACHE_SIZE_DEFAULT = 64 * 1024;
-
     /**
      * The default cache type.
      */
@@ -224,20 +193,10 @@ public class Constants {
      */
     public static final int DEFAULT_MAX_LENGTH_INPLACE_LOB = 256;
 
-    /**
-     * The default value for the maximum transaction log size.
-     */
-    public static final long DEFAULT_MAX_LOG_SIZE = 16 * 1024 * 1024;
-
-    /**
-     * The default value for the MAX_MEMORY_UNDO setting.
-     */
-    public static final int DEFAULT_MAX_MEMORY_UNDO = 50000;
-
     /**
      * The default for the setting MAX_OPERATION_MEMORY.
      */
-    public static final int DEFAULT_MAX_OPERATION_MEMORY = 100000;
+    public static final int DEFAULT_MAX_OPERATION_MEMORY = 100_000;
 
     /**
      * The default page size to use for new databases.
@@ -295,20 +254,39 @@ public class Constants {
      */
     public static final int LOCK_SLEEP = 1000;
 
+    /**
+     * The maximum allowed length of identifiers.
+     */
+    public static final int MAX_IDENTIFIER_LENGTH = 256;
+
+    /**
+     * The maximum number of columns in a table, select statement or row value.
+     */
+    public static final int MAX_COLUMNS = 16_384;
+
+    /**
+     * The maximum allowed length for character string, binary string, and other
+     * data types based on them; excluding LOB data types.
+     * <p>
+     * This needs to be less than (2^31-8)/2 to avoid running into the limit on
+     * encoding data fields when storing rows.
+     */
+    public static final int MAX_STRING_LENGTH = 1000_000_000;
+
+    /**
+     * The maximum allowed precision of numeric data types.
+     */
+    public static final int MAX_NUMERIC_PRECISION = 100_000;
+
+    /**
+     * The maximum allowed cardinality of array.
+     */
+    public static final int MAX_ARRAY_CARDINALITY = 65_536;
+
     /**
      * The highest possible parameter index.
      */
-    public static final int MAX_PARAMETER_INDEX = 100000;
-
-    /**
-     * The memory needed by a object of class Data
-     */
-    public static final int MEMORY_DATA = 24;
-
-    /**
-     * This value is used to calculate the average memory usage.
-     */
-    public static final int MEMORY_FACTOR = 64;
+    public static final int MAX_PARAMETER_INDEX = 100_000;
 
     /**
      * The memory needed by a regular object with at least one field.
@@ -318,21 +296,9 @@ public class Constants {
     public static final int MEMORY_OBJECT = 24;
 
     /**
-     * The memory needed by an object of class PageBtree.
+     * The memory needed by an array.
      */
-    public static final int MEMORY_PAGE_BTREE =
-            112 + MEMORY_DATA + 2 * MEMORY_OBJECT;
-
-    /**
-     * The memory needed by an object of class PageData.
-     */
-    public static final int MEMORY_PAGE_DATA =
-            144 + MEMORY_DATA + 3 * MEMORY_OBJECT;
-
-    /**
-     * The memory needed by an object of class PageDataOverflow.
-     */
-    public static final int MEMORY_PAGE_DATA_OVERFLOW = 96 + MEMORY_DATA;
+    public static final int MEMORY_ARRAY = 24;
 
     /**
      * The memory needed by a pointer.
@@ -346,11 +312,6 @@ public class Constants {
      */
     public static final int MEMORY_ROW = 40;
 
-    /**
-     * The minimum write delay that causes commits to be delayed.
-     */
-    public static final int MIN_WRITE_DELAY = 5;
-
     /**
      * The name prefix used for indexes that are not explicitly named.
      */
@@ -367,6 +328,11 @@ public class Constants {
      */
     public static final String PREFIX_PRIMARY_KEY = "PRIMARY_KEY_";
 
+    /**
+     * The name prefix used for query aliases that are not explicitly named.
+     */
+    public static final String PREFIX_QUERY_ALIAS = "QUERY_ALIAS_";
+
     /**
      * Every user belongs to this role.
      */
@@ -377,11 +343,31 @@ public class Constants {
      */
     public static final int SALT_LEN = 8;
 
+    /**
+     * The identity of INFORMATION_SCHEMA.
+     */
+    public static final int INFORMATION_SCHEMA_ID = -1;
+
+    /**
+     * The identity of PUBLIC schema.
+     */
+    public static final int MAIN_SCHEMA_ID = 0;
+
     /**
      * The name of the default schema.
      */
     public static final String SCHEMA_MAIN = "PUBLIC";
 
+    /**
+     * The identity of pg_catalog schema.
+     */
+    public static final int PG_CATALOG_SCHEMA_ID = -1_000;
+
+    /**
+     * The name of the pg_catalog schema.
+     */
+    public static final String SCHEMA_PG_CATALOG = "PG_CATALOG";
+
     /**
      * The default selectivity (used if the selectivity is not calculated).
      */
@@ -390,7 +376,7 @@ public class Constants {
     /**
      * The number of distinct values to keep in memory when running ANALYZE.
      */
-    public static final int SELECTIVITY_DISTINCT_COUNT = 10000;
+    public static final int SELECTIVITY_DISTINCT_COUNT = 10_000;
 
     /**
      * The default directory name of the server properties file for the H2
@@ -414,22 +400,6 @@ public class Constants {
      */
     public static final String START_URL = "jdbc:h2:";
 
-    /**
-     * The file name suffix of all database files.
-     */
-    public static final String SUFFIX_DB_FILE = ".db";
-
-    /**
-     * The file name suffix of large object files.
-     */
-    public static final String SUFFIX_LOB_FILE = ".lob.db";
-
-    /**
-     * The suffix of the directory name used if LOB objects are stored in a
-     * directory.
-     */
-    public static final String SUFFIX_LOBS_DIRECTORY = ".lobs.db";
-
     /**
      * The file name suffix of file lock files that are used to make sure a
      * database is open by only one process at any time.
@@ -441,10 +411,6 @@ public class Constants {
      */
     public static final String SUFFIX_OLD_DATABASE_FILE = ".data.db";
 
-    /**
-     * The file name suffix of page files.
-     */
-    public static final String SUFFIX_PAGE_FILE = ".h2.db";
     /**
      * The file name suffix of a MVStore file.
      */
@@ -472,15 +438,11 @@ public class Constants {
     public static final String SUFFIX_TRACE_FILE = ".trace.db";
 
     /**
-     * The delay that is to be used if throttle has been enabled.
+     * How often we check to see if we need to apply a throttling delay if SET
+     * THROTTLE has been used.
      */
     public static final int THROTTLE_DELAY = 50;
 
-    /**
-     * The maximum size of an undo log block.
-     */
-    public static final int UNDO_BLOCK_SIZE = 1024 * 1024;
-
     /**
      * The database URL format in simplified Backus-Naur form.
      */
@@ -493,16 +455,11 @@ public class Constants {
      */
     public static final String USER_PACKAGE = "org.h2.dynamic";
 
-    /**
-     * Name of the character encoding format.
-     */
-    public static final Charset UTF8 = Charset.forName("UTF-8");
-
     /**
      * The maximum time in milliseconds to keep the cost of a view.
      * 10000 means 10 seconds.
      */
-    public static final int VIEW_COST_CACHE_MAX_AGE = 10000;
+    public static final int VIEW_COST_CACHE_MAX_AGE = 10_000;
 
     /**
      * The name of the index cache that is used for temporary view (subqueries
@@ -515,44 +472,37 @@ public class Constants {
      */
     public static final int QUERY_STATISTICS_MAX_ENTRIES = 100;
 
-    private Constants() {
-        // utility class
-    }
+    /**
+     * Announced version for PgServer.
+     */
+    public static final String PG_VERSION = "8.2.23";
 
     /**
-     * Get the version of this product, consisting of major version, minor
+     * The version of this product, consisting of major version, minor
      * version, and build id.
-     *
-     * @return the version number
      */
-    public static String getVersion() {
-        String version = VERSION_MAJOR + "." + VERSION_MINOR + "." + BUILD_ID;
+    public static final String VERSION;
+
+    /**
+     * The complete version number of this database, consisting of
+     * the major version, the minor version, the build id, and the build date.
+     */
+    public static final String FULL_VERSION;
+
+    static {
+        String version = VERSION_MAJOR + "." + VERSION_MINOR + '.' + BUILD_ID;
         if (BUILD_VENDOR_AND_VERSION != null) {
-            version += "_" + BUILD_VENDOR_AND_VERSION;
+            version += '_' + BUILD_VENDOR_AND_VERSION;
         }
         if (BUILD_SNAPSHOT) {
             version += "-SNAPSHOT";
         }
-        return version;
+        VERSION = version;
+        FULL_VERSION = version + (" (" + BUILD_DATE + ')');
     }
 
-    /**
-     * Get the last stable version name.
-     *
-     * @return the version number
-     */
-    public static Object getVersionStable() {
-        return "1.4." + BUILD_ID_STABLE;
-    }
-
-    /**
-     * Get the complete version number of this database, consisting of
-     * the major version, the minor version, the build id, and the build date.
-     *
-     * @return the complete version
-     */
-    public static String getFullVersion() {
-        return getVersion() + " (" + BUILD_DATE + ")";
+    private Constants() {
+        // utility class
     }
 
 }
