@@ -1,22 +1,28 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
 
 import java.util.HashMap;
+import java.util.Map;
+import java.util.Objects;
 import org.h2.api.ErrorCode;
 import org.h2.command.CommandInterface;
-import org.h2.command.Parser;
 import org.h2.command.dml.SetTypes;
 import org.h2.message.DbException;
 import org.h2.message.Trace;
-import org.h2.store.FileLock;
+import org.h2.security.auth.AuthenticationException;
+import org.h2.security.auth.AuthenticationInfo;
+import org.h2.security.auth.Authenticator;
+import org.h2.store.fs.FileUtils;
+import org.h2.util.DateTimeUtils;
 import org.h2.util.MathUtils;
-import org.h2.util.New;
+import org.h2.util.ParserUtil;
 import org.h2.util.StringUtils;
 import org.h2.util.ThreadDeadlockDetector;
+import org.h2.util.TimeZoneProvider;
 import org.h2.util.Utils;
 
 /**
@@ -24,58 +30,86 @@ import org.h2.util.Utils;
  * It is also responsible for opening and creating new databases.
  * This is a singleton class.
  */
-public class Engine implements SessionFactory {
+public final class Engine {
 
-    private static final Engine INSTANCE = new Engine();
-    private static final HashMap<String, Database> DATABASES = New.hashMap();
+    private static final Map<String, DatabaseHolder> DATABASES = new HashMap<>();
 
-    private volatile long wrongPasswordDelay =
-            SysProperties.DELAY_WRONG_PASSWORD_MIN;
-    private boolean jmx;
+    private static volatile long WRONG_PASSWORD_DELAY = SysProperties.DELAY_WRONG_PASSWORD_MIN;
 
-    private Engine() {
-        // use getInstance()
+    private static boolean JMX;
+
+    static {
         if (SysProperties.THREAD_DEADLOCK_DETECTOR) {
             ThreadDeadlockDetector.init();
         }
     }
 
-    public static Engine getInstance() {
-        return INSTANCE;
-    }
-
-    private Session openSession(ConnectionInfo ci, boolean ifExists,
+    private static SessionLocal openSession(ConnectionInfo ci, boolean ifExists, boolean forbidCreation,
             String cipher) {
         String name = ci.getName();
         Database database;
         ci.removeProperty("NO_UPGRADE", false);
         boolean openNew = ci.getProperty("OPEN_NEW", false);
-        if (openNew || ci.isUnnamedInMemory()) {
-            database = null;
-        } else {
-            database = DATABASES.get(name);
-        }
-        User user = null;
         boolean opened = false;
-        if (database == null) {
-            if (ifExists && !Database.exists(name)) {
-                throw DbException.get(ErrorCode.DATABASE_NOT_FOUND_1, name);
+        User user = null;
+        DatabaseHolder databaseHolder;
+        if (!ci.isUnnamedInMemory()) {
+            synchronized (DATABASES) {
+                databaseHolder = DATABASES.computeIfAbsent(name, (key) -> new DatabaseHolder());
             }
-            database = new Database(ci, cipher);
-            opened = true;
-            if (database.getAllUsers().size() == 0) {
-                // users is the last thing we add, so if no user is around,
-                // the database is new (or not initialized correctly)
-                user = new User(database, database.allocateObjectId(),
-                        ci.getUserName(), false);
-                user.setAdmin(true);
-                user.setUserPasswordHash(ci.getUserPasswordHash());
-                database.setMasterUser(user);
-            }
-            if (!ci.isUnnamedInMemory()) {
-                DATABASES.put(name, database);
+        } else {
+            databaseHolder = new DatabaseHolder();
+        }
+        synchronized (databaseHolder) {
+            database = databaseHolder.database;
+            if (database == null || openNew) {
+                if (ci.isPersistent()) {
+                    String p = ci.getProperty("MV_STORE");
+                    String fileName;
+                    if (p == null) {
+                        fileName = name + Constants.SUFFIX_MV_FILE;
+                        if (!FileUtils.exists(fileName)) {
+                            throwNotFound(ifExists, forbidCreation, name);
+                            fileName = name + Constants.SUFFIX_OLD_DATABASE_FILE;
+                            if (FileUtils.exists(fileName)) {
+                                throw DbException.getFileVersionError(fileName);
+                            }
+                            fileName = null;
+                        }
+                    } else {
+                        fileName = name + Constants.SUFFIX_MV_FILE;
+                        if (!FileUtils.exists(fileName)) {
+                            throwNotFound(ifExists, forbidCreation, name);
+                            fileName = null;
+                        }
+                    }
+                    if (fileName != null && !FileUtils.canWrite(fileName)) {
+                        ci.setProperty("ACCESS_MODE_DATA", "r");
+                    }
+                } else {
+                    throwNotFound(ifExists, forbidCreation, name);
+                }
+                database = new Database(ci, cipher);
+                opened = true;
+                boolean found = false;
+                for (RightOwner rightOwner : database.getAllUsersAndRoles()) {
+                    if (rightOwner instanceof User) {
+                        found = true;
+                        break;
+                    }
+                }
+                if (!found) {
+                    // users is the last thing we add, so if no user is around,
+                    // the database is new (or not initialized correctly)
+                    user = new User(database, database.allocateObjectId(), ci.getUserName(), false);
+                    user.setAdmin(true);
+                    user.setUserPasswordHash(ci.getUserPasswordHash());
+                    database.setMasterUser(user);
+                }
+                databaseHolder.database = database;
             }
         }
+
         if (opened) {
             // start the thread when already synchronizing on the database
             // otherwise a deadlock can occur when the writer thread
@@ -87,10 +121,26 @@ public class Engine implements SessionFactory {
         }
         if (user == null) {
             if (database.validateFilePasswordHash(cipher, ci.getFilePasswordHash())) {
-                user = database.findUser(ci.getUserName());
-                if (user != null) {
-                    if (!user.validateUserPasswordHash(ci.getUserPasswordHash())) {
-                        user = null;
+                if (ci.getProperty("AUTHREALM")== null) {
+                    user = database.findUser(ci.getUserName());
+                    if (user != null) {
+                        if (!user.validateUserPasswordHash(ci.getUserPasswordHash())) {
+                            user = null;
+                        }
+                    }
+                } else {
+                    Authenticator authenticator = database.getAuthenticator();
+                    if (authenticator==null) {
+                        throw DbException.get(ErrorCode.AUTHENTICATOR_NOT_AVAILABLE, name);
+                    } else {
+                        try {
+                            AuthenticationInfo authenticationInfo=new AuthenticationInfo(ci);
+                            user = database.getAuthenticator().authenticate(authenticationInfo, database);
+                        } catch (AuthenticationException authenticationError) {
+                            database.getTrace(Trace.DATABASE).error(authenticationError,
+                                "an error occurred during authentication; user: \"" +
+                                ci.getUserName() + "\"");
+                        }
                     }
                 }
             }
@@ -107,12 +157,17 @@ public class Engine implements SessionFactory {
             database.removeSession(null);
             throw er;
         }
+        //Prevent to set _PASSWORD
+        ci.cleanAuthenticationInfo();
         checkClustering(ci, database);
-        Session session = database.createSession(user);
+        SessionLocal session = database.createSession(user, ci.getNetworkConnectionInfo());
         if (session == null) {
             // concurrently closing
             return null;
         }
+        if (ci.getProperty("OLD_INFORMATION_SCHEMA", false)) {
+            session.setOldInformationSchema(true);
+        }
         if (ci.getProperty("JMX", false)) {
             try {
                 Utils.callStaticMethod(
@@ -121,41 +176,30 @@ public class Engine implements SessionFactory {
                 database.removeSession(session);
                 throw DbException.get(ErrorCode.FEATURE_NOT_SUPPORTED_1, e, "JMX");
             }
-            jmx = true;
+            JMX = true;
         }
         return session;
     }
 
+    private static void throwNotFound(boolean ifExists, boolean forbidCreation, String name) {
+        if (ifExists) {
+            throw DbException.get(ErrorCode.DATABASE_NOT_FOUND_WITH_IF_EXISTS_1, name);
+        }
+        if (forbidCreation) {
+            throw DbException.get(ErrorCode.REMOTE_DATABASE_NOT_FOUND_1, name);
+        }
+    }
+
     /**
      * Open a database connection with the given connection information.
      *
      * @param ci the connection information
      * @return the session
      */
-    @Override
-    public Session createSession(ConnectionInfo ci) {
-        return INSTANCE.createSessionAndValidate(ci);
-    }
-
-    private Session createSessionAndValidate(ConnectionInfo ci) {
+    public static SessionLocal createSession(ConnectionInfo ci) {
         try {
-            ConnectionInfo backup = null;
-            String lockMethodName = ci.getProperty("FILE_LOCK", null);
-            int fileLockMethod = FileLock.getFileLockMethod(lockMethodName);
-            if (fileLockMethod == FileLock.LOCK_SERIALIZED) {
-                // In serialized mode, database instance sharing is not possible
-                ci.setProperty("OPEN_NEW", "TRUE");
-                try {
-                    backup = ci.clone();
-                } catch (CloneNotSupportedException e) {
-                    throw DbException.convert(e);
-                }
-            }
-            Session session = openSession(ci);
+            SessionLocal session = openSession(ci);
             validateUserAndPassword(true);
-            if (backup != null) {
-                session.setConnectionInfo(backup);
-            }
             return session;
         } catch (DbException e) {
             if (e.getErrorCode() == ErrorCode.WRONG_USER_OR_PASSWORD) {
@@ -165,45 +209,53 @@ public class Engine implements SessionFactory {
         }
     }
 
-    private synchronized Session openSession(ConnectionInfo ci) {
+    private static SessionLocal openSession(ConnectionInfo ci) {
         boolean ifExists = ci.removeProperty("IFEXISTS", false);
+        boolean forbidCreation = ci.removeProperty("FORBID_CREATION", false);
         boolean ignoreUnknownSetting = ci.removeProperty(
                 "IGNORE_UNKNOWN_SETTINGS", false);
         String cipher = ci.removeProperty("CIPHER", null);
         String init = ci.removeProperty("INIT", null);
-        Session session;
-        for (int i = 0;; i++) {
-            session = openSession(ci, ifExists, cipher);
+        SessionLocal session;
+        long start = System.nanoTime();
+        for (;;) {
+            session = openSession(ci, ifExists, forbidCreation, cipher);
             if (session != null) {
                 break;
             }
             // we found a database that is currently closing
             // wait a bit to avoid a busy loop (the method is synchronized)
-            if (i > 60 * 1000) {
-                // retry at most 1 minute
+            if (System.nanoTime() - start > DateTimeUtils.NANOS_PER_MINUTE) {
                 throw DbException.get(ErrorCode.DATABASE_ALREADY_OPEN_1,
                         "Waited for database closing longer than 1 minute");
             }
             try {
                 Thread.sleep(1);
             } catch (InterruptedException e) {
-                // ignore
+                throw DbException.get(ErrorCode.DATABASE_CALLED_AT_SHUTDOWN);
             }
         }
         synchronized (session) {
             session.setAllowLiterals(true);
-            DbSettings defaultSettings = DbSettings.getDefaultSettings();
+            DbSettings defaultSettings = DbSettings.DEFAULT;
             for (String setting : ci.getKeys()) {
                 if (defaultSettings.containsKey(setting)) {
                     // database setting are only used when opening the database
                     continue;
                 }
                 String value = ci.getProperty(setting);
+                StringBuilder builder = new StringBuilder("SET ").append(setting).append(' ');
+                if (!ParserUtil.isSimpleIdentifier(setting, false, false)) {
+                    if (!setting.equalsIgnoreCase("TIME ZONE")) {
+                        throw DbException.get(ErrorCode.UNSUPPORTED_SETTING_1, setting);
+                    }
+                    StringUtils.quoteStringSQL(builder, value);
+                } else {
+                    builder.append(value);
+                }
                 try {
-                    CommandInterface command = session.prepareCommand(
-                            "SET " + Parser.quoteIdentifier(setting) + " " + value,
-                            Integer.MAX_VALUE);
-                    command.executeUpdate();
+                    CommandInterface command = session.prepareLocal(builder.toString());
+                    command.executeUpdate(null);
                 } catch (DbException e) {
                     if (e.getErrorCode() == ErrorCode.ADMIN_RIGHTS_REQUIRED) {
                         session.getTrace().error(e, "admin rights required; user: \"" +
@@ -217,11 +269,14 @@ public class Engine implements SessionFactory {
                     }
                 }
             }
+            TimeZoneProvider timeZone = ci.getTimeZone();
+            if (timeZone != null) {
+                session.setTimeZone(timeZone);
+            }
             if (init != null) {
                 try {
-                    CommandInterface command = session.prepareCommand(init,
-                            Integer.MAX_VALUE);
-                    command.executeUpdate();
+                    CommandInterface command = session.prepareLocal(init);
+                    command.executeUpdate(null);
                 } catch (DbException e) {
                     if (!ignoreUnknownSetting) {
                         session.close();
@@ -245,7 +300,7 @@ public class Engine implements SessionFactory {
         String clusterDb = database.getCluster();
         if (!Constants.CLUSTERING_DISABLED.equals(clusterDb)) {
             if (!Constants.CLUSTERING_ENABLED.equals(clusterSession)) {
-                if (!StringUtils.equals(clusterSession, clusterDb)) {
+                if (!Objects.equals(clusterSession, clusterDb)) {
                     if (clusterDb.equals(Constants.CLUSTERING_DISABLED)) {
                         throw DbException.get(
                                 ErrorCode.CLUSTER_ERROR_DATABASE_RUNS_ALONE);
@@ -264,15 +319,17 @@ public class Engine implements SessionFactory {
      *
      * @param name the database name
      */
-    void close(String name) {
-        if (jmx) {
+    static void close(String name) {
+        if (JMX) {
             try {
                 Utils.callStaticMethod("org.h2.jmx.DatabaseInfo.unregisterMBean", name);
             } catch (Exception e) {
                 throw DbException.get(ErrorCode.FEATURE_NOT_SUPPORTED_1, e, "JMX");
             }
         }
-        DATABASES.remove(name);
+        synchronized (DATABASES) {
+            DATABASES.remove(name);
+        }
     }
 
     /**
@@ -292,14 +349,14 @@ public class Engine implements SessionFactory {
      * @param correct if the user name or the password was correct
      * @throws DbException the exception 'wrong user or password'
      */
-    private void validateUserAndPassword(boolean correct) {
+    private static void validateUserAndPassword(boolean correct) {
         int min = SysProperties.DELAY_WRONG_PASSWORD_MIN;
         if (correct) {
-            long delay = wrongPasswordDelay;
+            long delay = WRONG_PASSWORD_DELAY;
             if (delay > min && delay > 0) {
                 // the first correct password must be blocked,
                 // otherwise parallel attacks are possible
-                synchronized (INSTANCE) {
+                synchronized (Engine.class) {
                     // delay up to the last delay
                     // an attacker can't know how long it will be
                     delay = MathUtils.secureRandomInt((int) delay);
@@ -308,21 +365,21 @@ public class Engine implements SessionFactory {
                     } catch (InterruptedException e) {
                         // ignore
                     }
-                    wrongPasswordDelay = min;
+                    WRONG_PASSWORD_DELAY = min;
                 }
             }
         } else {
             // this method is not synchronized on the Engine, so that
             // regular successful attempts are not blocked
-            synchronized (INSTANCE) {
-                long delay = wrongPasswordDelay;
+            synchronized (Engine.class) {
+                long delay = WRONG_PASSWORD_DELAY;
                 int max = SysProperties.DELAY_WRONG_PASSWORD_MAX;
                 if (max <= 0) {
                     max = Integer.MAX_VALUE;
                 }
-                wrongPasswordDelay += wrongPasswordDelay;
-                if (wrongPasswordDelay > max || wrongPasswordDelay < 0) {
-                    wrongPasswordDelay = max;
+                WRONG_PASSWORD_DELAY += WRONG_PASSWORD_DELAY;
+                if (WRONG_PASSWORD_DELAY > max || WRONG_PASSWORD_DELAY < 0) {
+                    WRONG_PASSWORD_DELAY = max;
                 }
                 if (min > 0) {
                     // a bit more to protect against timing attacks
@@ -338,4 +395,14 @@ public class Engine implements SessionFactory {
         }
     }
 
+    private Engine() {
+    }
+
+    private static final class DatabaseHolder {
+
+        DatabaseHolder() {
+        }
+
+        volatile Database database;
+    }
 }
