@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
@@ -14,7 +14,7 @@ import org.h2.table.Table;
  * An access right. Rights are regular database objects, but have generated
  * names.
  */
-public class Right extends DbObjectBase {
+public final class Right extends DbObject {
 
     /**
      * The right bit mask that means: selecting from a table is allowed.
@@ -41,6 +41,12 @@ public class Right extends DbObjectBase {
      */
     public static final int ALTER_ANY_SCHEMA = 16;
 
+    /**
+     * The right bit mask that means: user is a schema owner. This mask isn't
+     * used in GRANT / REVOKE statements.
+     */
+    public static final int SCHEMA_OWNER = 32;
+
     /**
      * The right bit mask that means: select, insert, update, delete, and update
      * for this object is allowed.
@@ -68,21 +74,19 @@ public class Right extends DbObjectBase {
     private DbObject grantedObject;
 
     public Right(Database db, int id, RightOwner grantee, Role grantedRole) {
-        initDbObjectBase(db, id, "RIGHT_" + id, Trace.USER);
+        super(db, id, "RIGHT_" + id, Trace.USER);
         this.grantee = grantee;
         this.grantedRole = grantedRole;
     }
 
-    public Right(Database db, int id, RightOwner grantee, int grantedRight,
-            DbObject grantedObject) {
-        initDbObjectBase(db, id, "" + id, Trace.USER);
+    public Right(Database db, int id, RightOwner grantee, int grantedRight, DbObject grantedObject) {
+        super(db, id, Integer.toString(id), Trace.USER);
         this.grantee = grantee;
         this.grantedRight = grantedRight;
         this.grantedObject = grantedObject;
     }
 
-    private static boolean appendRight(StringBuilder buff, int right, int mask,
-            String name, boolean comma) {
+    private static boolean appendRight(StringBuilder buff, int right, int mask, String name, boolean comma) {
         if ((right & mask) != 0) {
             if (comma) {
                 buff.append(", ");
@@ -102,9 +106,8 @@ public class Right extends DbObjectBase {
             comma = appendRight(buff, grantedRight, SELECT, "SELECT", comma);
             comma = appendRight(buff, grantedRight, DELETE, "DELETE", comma);
             comma = appendRight(buff, grantedRight, INSERT, "INSERT", comma);
-            comma = appendRight(buff, grantedRight, ALTER_ANY_SCHEMA,
-                    "ALTER ANY SCHEMA", comma);
-            appendRight(buff, grantedRight, UPDATE, "UPDATE", comma);
+            comma = appendRight(buff, grantedRight, UPDATE, "UPDATE", comma);
+            appendRight(buff, grantedRight, ALTER_ANY_SCHEMA, "ALTER ANY SCHEMA", comma);
         }
         return buff.toString();
     }
@@ -121,33 +124,31 @@ public class Right extends DbObjectBase {
         return grantee;
     }
 
-    @Override
-    public String getDropSQL() {
-        return null;
-    }
-
     @Override
     public String getCreateSQLForCopy(Table table, String quotedName) {
         return getCreateSQLForCopy(table);
     }
 
     private String getCreateSQLForCopy(DbObject object) {
-        StringBuilder buff = new StringBuilder();
-        buff.append("GRANT ");
+        StringBuilder builder = new StringBuilder();
+        builder.append("GRANT ");
         if (grantedRole != null) {
-            buff.append(grantedRole.getSQL());
+            grantedRole.getSQL(builder, DEFAULT_SQL_FLAGS);
         } else {
-            buff.append(getRights());
+            builder.append(getRights());
             if (object != null) {
                 if (object instanceof Schema) {
-                    buff.append(" ON SCHEMA ").append(object.getSQL());
+                    builder.append(" ON SCHEMA ");
+                    object.getSQL(builder, DEFAULT_SQL_FLAGS);
                 } else if (object instanceof Table) {
-                    buff.append(" ON ").append(object.getSQL());
+                    builder.append(" ON ");
+                    object.getSQL(builder, DEFAULT_SQL_FLAGS);
                 }
             }
         }
-        buff.append(" TO ").append(grantee.getSQL());
-        return buff.toString();
+        builder.append(" TO ");
+        grantee.getSQL(builder, DEFAULT_SQL_FLAGS);
+        return builder.toString();
     }
 
     @Override
@@ -161,7 +162,7 @@ public class Right extends DbObjectBase {
     }
 
     @Override
-    public void removeChildrenAndResources(Session session) {
+    public void removeChildrenAndResources(SessionLocal session) {
         if (grantedRole != null) {
             grantee.revokeRole(grantedRole);
         } else {
@@ -176,7 +177,7 @@ public class Right extends DbObjectBase {
 
     @Override
     public void checkRename() {
-        DbException.throwInternalError();
+        throw DbException.getInternalError();
     }
 
     public void setRightMask(int rightMask) {
