@@ -1,11 +1,12 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.expression;
 
 import java.util.HashSet;
+import org.h2.command.query.AllColumnsForPlan;
 import org.h2.engine.DbObject;
 import org.h2.table.Column;
 import org.h2.table.ColumnResolver;
@@ -16,7 +17,7 @@ import org.h2.table.TableFilter;
  * The visitor pattern is used to iterate through all expressions of a query
  * to optimize a statement.
  */
-public class ExpressionVisitor {
+public final class ExpressionVisitor {
 
     /**
      * Is the value independent on unset parameters or on columns of a higher
@@ -32,10 +33,10 @@ public class ExpressionVisitor {
             new ExpressionVisitor(INDEPENDENT);
 
     /**
-     * Are all aggregates MIN(column), MAX(column), or COUNT(*) for the given
-     * table (getTable)?
+     * Are all aggregates MIN(column), MAX(column), COUNT(*), MEDIAN(column),
+     * ENVELOPE(count) for the given table (getTable)?
      */
-    public static final int OPTIMIZABLE_MIN_MAX_COUNT_ALL = 1;
+    public static final int OPTIMIZABLE_AGGREGATE = 1;
 
     /**
      * Does the expression return the same results for the same parameters?
@@ -60,6 +61,37 @@ public class ExpressionVisitor {
     public static final ExpressionVisitor EVALUATABLE_VISITOR =
             new ExpressionVisitor(EVALUATABLE);
 
+    /**
+     * Count of cached INDEPENDENT and EVALUATABLE visitors with different query
+     * level.
+     */
+    private static final int CACHED = 8;
+
+    /**
+     * INDEPENDENT listeners with query level 0, 1, ...
+     */
+    private static final ExpressionVisitor[] INDEPENDENT_VISITORS;
+
+    /**
+     * EVALUATABLE listeners with query level 0, 1, ...
+     */
+    private static final ExpressionVisitor[] EVALUATABLE_VISITORS;
+
+    static {
+        ExpressionVisitor[] a = new ExpressionVisitor[CACHED];
+        a[0] = INDEPENDENT_VISITOR;
+        for (int i = 1; i < CACHED; i++) {
+            a[i] = new ExpressionVisitor(INDEPENDENT, i);
+        }
+        INDEPENDENT_VISITORS = a;
+        a = new ExpressionVisitor[CACHED];
+        a[0] = EVALUATABLE_VISITOR;
+        for (int i = 1; i < CACHED; i++) {
+            a[i] = new ExpressionVisitor(EVALUATABLE, i);
+        }
+        EVALUATABLE_VISITORS = a;
+    }
+
     /**
      * Request to set the latest modification id (addDataModificationId).
      */
@@ -95,10 +127,20 @@ public class ExpressionVisitor {
      */
     public static final int QUERY_COMPARABLE = 8;
 
+    /**
+     * Get all referenced columns for the optimiser.
+     */
+    public static final int GET_COLUMNS1 = 9;
+
     /**
      * Get all referenced columns.
      */
-    public static final int GET_COLUMNS = 9;
+    public static final int GET_COLUMNS2 = 10;
+
+    /**
+     * Decrement query level of all expression columns.
+     */
+    public static final int DECREMENT_QUERY_LEVEL = 11;
 
     /**
      * The visitor singleton for the type QUERY_COMPARABLE.
@@ -108,21 +150,21 @@ public class ExpressionVisitor {
 
     private final int type;
     private final int queryLevel;
-    private final HashSet<DbObject> dependencies;
-    private final HashSet<Column> columns;
+    private final HashSet<?> set;
+    private final AllColumnsForPlan columns1;
     private final Table table;
     private final long[] maxDataModificationId;
     private final ColumnResolver resolver;
 
     private ExpressionVisitor(int type,
             int queryLevel,
-            HashSet<DbObject> dependencies,
-            HashSet<Column> columns, Table table, ColumnResolver resolver,
+            HashSet<?> set,
+            AllColumnsForPlan columns1, Table table, ColumnResolver resolver,
             long[] maxDataModificationId) {
         this.type = type;
         this.queryLevel = queryLevel;
-        this.dependencies = dependencies;
-        this.columns = columns;
+        this.set = set;
+        this.columns1 = columns1;
         this.table = table;
         this.resolver = resolver;
         this.maxDataModificationId = maxDataModificationId;
@@ -131,8 +173,18 @@ public class ExpressionVisitor {
     private ExpressionVisitor(int type) {
         this.type = type;
         this.queryLevel = 0;
-        this.dependencies = null;
-        this.columns = null;
+        this.set = null;
+        this.columns1 = null;
+        this.table = null;
+        this.resolver = null;
+        this.maxDataModificationId = null;
+    }
+
+    private ExpressionVisitor(int type, int queryLevel) {
+        this.type = type;
+        this.queryLevel = queryLevel;
+        this.set = null;
+        this.columns1 = null;
         this.table = null;
         this.resolver = null;
         this.maxDataModificationId = null;
@@ -157,7 +209,7 @@ public class ExpressionVisitor {
      * @return the new visitor
      */
     public static ExpressionVisitor getOptimizableVisitor(Table table) {
-        return new ExpressionVisitor(OPTIMIZABLE_MIN_MAX_COUNT_ALL, 0, null,
+        return new ExpressionVisitor(OPTIMIZABLE_AGGREGATE, 0, null,
                 null, table, null, null);
     }
 
@@ -168,7 +220,7 @@ public class ExpressionVisitor {
      * @param resolver the resolver
      * @return the new visitor
      */
-    static ExpressionVisitor getNotFromResolverVisitor(ColumnResolver resolver) {
+    public static ExpressionVisitor getNotFromResolverVisitor(ColumnResolver resolver) {
         return new ExpressionVisitor(NOT_FROM_RESOLVER, 0, null, null, null,
                 resolver, null);
     }
@@ -179,8 +231,19 @@ public class ExpressionVisitor {
      * @param columns the columns map
      * @return the new visitor
      */
-    public static ExpressionVisitor getColumnsVisitor(HashSet<Column> columns) {
-        return new ExpressionVisitor(GET_COLUMNS, 0, null, columns, null, null, null);
+    public static ExpressionVisitor getColumnsVisitor(AllColumnsForPlan columns) {
+        return new ExpressionVisitor(GET_COLUMNS1, 0, null, columns, null, null, null);
+    }
+
+    /**
+     * Create a new visitor to get all referenced columns.
+     *
+     * @param columns the columns map
+     * @param table table to gather columns from, or {@code null} to gather all columns
+     * @return the new visitor
+     */
+    public static ExpressionVisitor getColumnsVisitor(HashSet<Column> columns, Table table) {
+        return new ExpressionVisitor(GET_COLUMNS2, 0, columns, null, table, null, null);
     }
 
     public static ExpressionVisitor getMaxModificationIdVisitor() {
@@ -188,14 +251,31 @@ public class ExpressionVisitor {
                 null, null, null, new long[1]);
     }
 
+    /**
+     * Create a new visitor to decrement query level in columns with the
+     * specified resolvers.
+     *
+     * @param columnResolvers
+     *            column resolvers
+     * @param queryDecrement
+     *            0 to check whether operation is allowed, 1 to actually perform
+     *            the decrement
+     * @return the new visitor
+     */
+    public static ExpressionVisitor getDecrementQueryLevelVisitor(HashSet<ColumnResolver> columnResolvers,
+            int queryDecrement) {
+        return new ExpressionVisitor(DECREMENT_QUERY_LEVEL, queryDecrement, columnResolvers, null, null, null, null);
+    }
+
     /**
      * Add a new dependency to the set of dependencies.
      * This is used for GET_DEPENDENCIES visitors.
      *
      * @param obj the additional dependency.
      */
+    @SuppressWarnings("unchecked")
     public void addDependency(DbObject obj) {
-        dependencies.add(obj);
+        ((HashSet<DbObject>) set).add(obj);
     }
 
     /**
@@ -204,8 +284,21 @@ public class ExpressionVisitor {
      *
      * @param column the additional column.
      */
-    void addColumn(Column column) {
-        columns.add(column);
+    void addColumn1(Column column) {
+        columns1.add(column);
+    }
+
+    /**
+     * Add a new column to the set of columns.
+     * This is used for GET_COLUMNS2 visitors.
+     *
+     * @param column the additional column.
+     */
+    @SuppressWarnings("unchecked")
+    void addColumn2(Column column) {
+        if (table == null || table == column.getTable()) {
+            ((HashSet<Column>) set).add(column);
+        }
     }
 
     /**
@@ -214,19 +307,27 @@ public class ExpressionVisitor {
      *
      * @return the set
      */
+    @SuppressWarnings("unchecked")
     public HashSet<DbObject> getDependencies() {
-        return dependencies;
+        return (HashSet<DbObject>) set;
     }
 
     /**
      * Increment or decrement the query level.
      *
      * @param offset 1 to increment, -1 to decrement
-     * @return a clone of this expression visitor, with the changed query level
+     * @return this visitor or its clone with the changed query level
      */
     public ExpressionVisitor incrementQueryLevel(int offset) {
-        return new ExpressionVisitor(type, queryLevel + offset, dependencies,
-                columns, table, resolver, maxDataModificationId);
+        if (type == INDEPENDENT) {
+            offset += queryLevel;
+            return offset < CACHED ? INDEPENDENT_VISITORS[offset] : new ExpressionVisitor(INDEPENDENT, offset);
+        } else if (type == EVALUATABLE) {
+            offset += queryLevel;
+            return offset < CACHED ? EVALUATABLE_VISITORS[offset] : new ExpressionVisitor(EVALUATABLE, offset);
+        } else {
+            return this;
+        }
     }
 
     /**
@@ -239,6 +340,17 @@ public class ExpressionVisitor {
         return resolver;
     }
 
+    /**
+     * Get the set of column resolvers.
+     * This is used for {@link #DECREMENT_QUERY_LEVEL} visitors.
+     *
+     * @return the set
+     */
+    @SuppressWarnings("unchecked")
+    public HashSet<ColumnResolver> getColumnResolvers() {
+        return (HashSet<ColumnResolver>) set;
+    }
+
     /**
      * Update the field maxDataModificationId if this value is higher
      * than the current value.
@@ -264,6 +376,7 @@ public class ExpressionVisitor {
     }
 
     int getQueryLevel() {
+        assert type == INDEPENDENT || type == EVALUATABLE || type == DECREMENT_QUERY_LEVEL;
         return queryLevel;
     }
 
@@ -290,16 +403,14 @@ public class ExpressionVisitor {
      * Get the set of columns of all tables.
      *
      * @param filters the filters
-     * @return the set of columns
+     * @param allColumnsSet the on-demand all-columns set
      */
-    public static HashSet<Column> allColumnsForTableFilters(TableFilter[] filters) {
-        HashSet<Column> allColumnsSet = new HashSet<>();
+    public static void allColumnsForTableFilters(TableFilter[] filters, AllColumnsForPlan allColumnsSet) {
         for (TableFilter filter : filters) {
             if (filter.getSelect() != null) {
                 filter.getSelect().isEverything(ExpressionVisitor.getColumnsVisitor(allColumnsSet));
             }
         }
-        return allColumnsSet;
     }
 
 }
