@@ -89,6 +89,15 @@ public abstract class DataAnalysisOperation extends Expression {
         this.select = select;
     }
 
+    /**
+     * Returns the OVER condition.
+     *
+     * @return the OVER condition
+     */
+    public Window getOverCondition() {
+        return over;
+    }
+
     /**
      * Sets the OVER condition.
      *
