@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.fulltext;
@@ -23,21 +23,21 @@ import java.util.HashSet;
 import java.util.Set;
 import java.util.StringTokenizer;
 import java.util.UUID;
+
 import org.h2.api.Trigger;
 import org.h2.command.Parser;
-import org.h2.engine.Session;
-import org.h2.expression.Comparison;
-import org.h2.expression.ConditionAndOr;
+import org.h2.engine.SessionLocal;
 import org.h2.expression.Expression;
 import org.h2.expression.ExpressionColumn;
 import org.h2.expression.ValueExpression;
+import org.h2.expression.condition.Comparison;
+import org.h2.expression.condition.ConditionAndOr;
 import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
 import org.h2.tools.SimpleResultSet;
 import org.h2.util.IOUtils;
-import org.h2.util.New;
-import org.h2.util.StatementBuilder;
 import org.h2.util.StringUtils;
+import org.h2.util.Utils;
 
 /**
  * This class implements the native full text search.
@@ -75,7 +75,7 @@ public class FullText {
     private static final String SELECT_MAP_BY_WORD_ID =
             "SELECT ROWID FROM " + SCHEMA + ".MAP WHERE WORDID=?";
     private static final String SELECT_ROW_BY_ID =
-            "SELECT KEY, INDEXID FROM " + SCHEMA + ".ROWS WHERE ID=?";
+            "SELECT `KEY`, INDEXID FROM " + SCHEMA + ".ROWS WHERE ID=?";
 
     /**
      * The column name of the result set returned by the search method.
@@ -103,38 +103,34 @@ public class FullText {
      * </pre>
      *
      * @param conn the connection
+     * @throws SQLException on failure
      */
     public static void init(Connection conn) throws SQLException {
         Statement stat = conn.createStatement();
         stat.execute("CREATE SCHEMA IF NOT EXISTS " + SCHEMA);
         stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
-                ".INDEXES(ID INT AUTO_INCREMENT PRIMARY KEY, " +
-                "SCHEMA VARCHAR, TABLE VARCHAR, COLUMNS VARCHAR, " +
-                "UNIQUE(SCHEMA, TABLE))");
+                ".INDEXES(ID INT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, " +
+                "SCHEMA VARCHAR, `TABLE` VARCHAR, COLUMNS VARCHAR, " +
+                "UNIQUE(SCHEMA, `TABLE`))");
         stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
-                ".WORDS(ID INT AUTO_INCREMENT PRIMARY KEY, " +
+                ".WORDS(ID INT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, " +
                 "NAME VARCHAR, UNIQUE(NAME))");
         stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
-                ".ROWS(ID IDENTITY, HASH INT, INDEXID INT, " +
-                "KEY VARCHAR, UNIQUE(HASH, INDEXID, KEY))");
+                ".ROWS(ID BIGINT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, HASH INT, INDEXID INT, " +
+                "`KEY` VARCHAR, UNIQUE(HASH, INDEXID, `KEY`))");
         stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
                 ".MAP(ROWID INT, WORDID INT, PRIMARY KEY(WORDID, ROWID))");
         stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
                 ".IGNORELIST(LIST VARCHAR)");
         stat.execute("CREATE TABLE IF NOT EXISTS " + SCHEMA +
-                ".SETTINGS(KEY VARCHAR PRIMARY KEY, VALUE VARCHAR)");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_CREATE_INDEX FOR \"" +
-                FullText.class.getName() + ".createIndex\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_DROP_INDEX FOR \"" +
-                FullText.class.getName() + ".dropIndex\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_SEARCH FOR \"" +
-                FullText.class.getName() + ".search\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_SEARCH_DATA FOR \"" +
-                FullText.class.getName() + ".searchData\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_REINDEX FOR \"" +
-                FullText.class.getName() + ".reindex\"");
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_DROP_ALL FOR \"" +
-                FullText.class.getName() + ".dropAll\"");
+                ".SETTINGS(`KEY` VARCHAR PRIMARY KEY, `VALUE` VARCHAR)");
+        String className = FullText.class.getName();
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_CREATE_INDEX FOR '" + className + ".createIndex'");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_DROP_INDEX FOR '" + className + ".dropIndex'");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_SEARCH FOR '" + className + ".search'");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_SEARCH_DATA FOR '" + className + ".searchData'");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_REINDEX FOR '" + className + ".reindex'");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_DROP_ALL FOR '" + className + ".dropAll'");
         FullTextSettings setting = FullTextSettings.getInstance(conn);
         ResultSet rs = stat.executeQuery("SELECT * FROM " + SCHEMA +
                 ".IGNORELIST");
@@ -170,12 +166,13 @@ public class FullText {
      * @param schema the schema name of the table (case sensitive)
      * @param table the table name (case sensitive)
      * @param columnList the column list (null for all columns)
+     * @throws SQLException on failure
      */
     public static void createIndex(Connection conn, String schema,
             String table, String columnList) throws SQLException {
         init(conn);
         PreparedStatement prep = conn.prepareStatement("INSERT INTO " + SCHEMA
-                + ".INDEXES(SCHEMA, TABLE, COLUMNS) VALUES(?, ?, ?)");
+                + ".INDEXES(SCHEMA, `TABLE`, COLUMNS) VALUES(?, ?, ?)");
         prep.setString(1, schema);
         prep.setString(2, table);
         prep.setString(3, columnList);
@@ -189,6 +186,7 @@ public class FullText {
      * usually not needed, as the index is kept up-to-date automatically.
      *
      * @param conn the connection
+     * @throws SQLException on failure
      */
     public static void reindex(Connection conn) throws SQLException {
         init(conn);
@@ -215,12 +213,13 @@ public class FullText {
      * @param conn the connection
      * @param schema the schema name of the table (case sensitive)
      * @param table the table name (case sensitive)
+     * @throws SQLException on failure
      */
     public static void dropIndex(Connection conn, String schema, String table)
             throws SQLException {
         init(conn);
         PreparedStatement prep = conn.prepareStatement("SELECT ID FROM " + SCHEMA
-                + ".INDEXES WHERE SCHEMA=? AND TABLE=?");
+                + ".INDEXES WHERE SCHEMA=? AND `TABLE`=?");
         prep.setString(1, schema);
         prep.setString(2, table);
         ResultSet rs = prep.executeQuery();
@@ -257,6 +256,7 @@ public class FullText {
      * Drops all full text indexes from the database.
      *
      * @param conn the connection
+     * @throws SQLException on failure
      */
     public static void dropAll(Connection conn) throws SQLException {
         init(conn);
@@ -284,6 +284,7 @@ public class FullText {
      * @param limit the maximum number of rows or 0 for no limit
      * @param offset the offset or 0 for no offset
      * @return the result set
+     * @throws SQLException on failure
      */
     public static ResultSet search(Connection conn, String text, int limit,
             int offset) throws SQLException {
@@ -315,6 +316,7 @@ public class FullText {
      * @param limit the maximum number of rows or 0 for no limit
      * @param offset the offset or 0 for no offset
      * @return the result set
+     * @throws SQLException on failure
      */
     public static ResultSet searchData(Connection conn, String text, int limit,
             int offset) throws SQLException {
@@ -333,6 +335,7 @@ public class FullText {
      *
      * @param conn the connection
      * @param commaSeparatedList the list
+     * @throws SQLException on failure
      */
     public static void setIgnoreList(Connection conn, String commaSeparatedList)
             throws SQLException {
@@ -358,6 +361,7 @@ public class FullText {
      *
      * @param conn the connection
      * @param whitespaceChars the list of characters
+     * @throws SQLException on failure
      */
     public static void setWhitespaceChars(Connection conn,
             String whitespaceChars) throws SQLException {
@@ -382,6 +386,7 @@ public class FullText {
      * @param data the object
      * @param type the SQL type
      * @return the string
+     * @throws SQLException on failure
      */
     protected static String asString(Object data, int type) throws SQLException {
         if (data == null) {
@@ -445,8 +450,8 @@ public class FullText {
         if (data) {
             result.addColumn(FullText.FIELD_SCHEMA, Types.VARCHAR, 0, 0);
             result.addColumn(FullText.FIELD_TABLE, Types.VARCHAR, 0, 0);
-            result.addColumn(FullText.FIELD_COLUMNS, Types.ARRAY, 0, 0);
-            result.addColumn(FullText.FIELD_KEYS, Types.ARRAY, 0, 0);
+            result.addColumn(FullText.FIELD_COLUMNS, Types.ARRAY, "VARCHAR ARRAY", 0, 0);
+            result.addColumn(FullText.FIELD_KEYS, Types.ARRAY, "VARCHAR ARRAY", 0, 0);
         } else {
             result.addColumn(FullText.FIELD_QUERY, Types.VARCHAR, 0, 0);
         }
@@ -461,17 +466,17 @@ public class FullText {
      * @param key the primary key condition as a string
      * @return an array containing the column name list and the data list
      */
-    protected static Object[][] parseKey(Connection conn, String key) {
-        ArrayList<String> columns = New.arrayList();
-        ArrayList<String> data = New.arrayList();
+    protected static String[][] parseKey(Connection conn, String key) {
+        ArrayList<String> columns = Utils.newSmallArrayList();
+        ArrayList<String> data = Utils.newSmallArrayList();
         JdbcConnection c = (JdbcConnection) conn;
-        Session session = (Session) c.getSession();
+        SessionLocal session = (SessionLocal) c.getSession();
         Parser p = new Parser(session);
         Expression expr = p.parseExpression(key);
-        addColumnData(columns, data, expr);
-        Object[] col = columns.toArray();
-        Object[] dat = data.toArray();
-        Object[][] columnData = { col, dat };
+        addColumnData(session, columns, data, expr);
+        String[] col = columns.toArray(new String[0]);
+        String[] dat = data.toArray(new String[0]);
+        String[][] columnData = { col, dat };
         return columnData;
     }
 
@@ -482,6 +487,7 @@ public class FullText {
      * @param data the object
      * @param type the SQL type
      * @return the SQL String
+     * @throws SQLException on failure
      */
     protected static String quoteSQL(Object data, int type) throws SQLException {
         if (data == null) {
@@ -511,9 +517,12 @@ public class FullText {
         case Types.LONGVARBINARY:
         case Types.BINARY:
             if (data instanceof UUID) {
-                return "'" + data.toString() + "'";
+                return "'" + data + "'";
             }
-            return "'" + StringUtils.convertBytesToHex((byte[]) data) + "'";
+            byte[] bytes = (byte[]) data;
+            StringBuilder builder = new StringBuilder(bytes.length * 2 + 2).append('\'');
+            StringUtils.convertBytesToHex(builder, bytes).append('\'');
+            return builder.toString();
         case Types.CLOB:
         case Types.JAVA_OBJECT:
         case Types.OTHER:
@@ -535,11 +544,13 @@ public class FullText {
      *
      * @param conn the database connection
      * @param prefix the prefix
+     * @throws SQLException on failure
      */
     protected static void removeAllTriggers(Connection conn, String prefix)
             throws SQLException {
         Statement stat = conn.createStatement();
-        ResultSet rs = stat.executeQuery("SELECT * FROM INFORMATION_SCHEMA.TRIGGERS");
+        ResultSet rs = stat.executeQuery(
+                "SELECT DISTINCT TRIGGER_SCHEMA, TRIGGER_NAME FROM INFORMATION_SCHEMA.TRIGGERS");
         Statement stat2 = conn.createStatement();
         while (rs.next()) {
             String schema = rs.getString("TRIGGER_SCHEMA");
@@ -558,6 +569,7 @@ public class FullText {
      * @param index the column indices (will be modified)
      * @param keys the key list
      * @param columns the column list
+     * @throws SQLException on failure
      */
     protected static void setColumns(int[] index, ArrayList<String> keys,
             ArrayList<String> columns) throws SQLException {
@@ -587,6 +599,7 @@ public class FullText {
      * @param offset the offset
      * @param data whether the raw data should be returned
      * @return the result set
+     * @throws SQLException on failure
      */
     protected static ResultSet search(Connection conn, String text, int limit,
             int offset, boolean data) throws SQLException {
@@ -595,7 +608,7 @@ public class FullText {
             // this is just to query the result set columns
             return result;
         }
-        if (text == null || text.trim().length() == 0) {
+        if (text == null || StringUtils.isWhitespaceOrEmpty(text)) {
             return result;
         }
         FullTextSettings setting = FullTextSettings.getInstance(conn);
@@ -615,7 +628,7 @@ public class FullText {
             if (wId == null) {
                 continue;
             }
-            prepSelectMapByWordId.setInt(1, wId.intValue());
+            prepSelectMapByWordId.setInt(1, wId);
             ResultSet rs = prepSelectMapByWordId.executeQuery();
             while (rs.next()) {
                 Integer rId = rs.getInt(1);
@@ -642,7 +655,7 @@ public class FullText {
                 int indexId = rs.getInt(2);
                 IndexInfo index = setting.getIndexInfo(indexId);
                 if (data) {
-                    Object[][] columnData = parseKey(conn, key);
+                    String[][] columnData = parseKey(conn, key);
                     result.addRow(
                             index.schema,
                             index.table,
@@ -664,23 +677,21 @@ public class FullText {
         return result;
     }
 
-    private static void addColumnData(ArrayList<String> columns,
-            ArrayList<String> data, Expression expr) {
+    private static void addColumnData(SessionLocal session, ArrayList<String> columns, ArrayList<String> data,
+            Expression expr) {
         if (expr instanceof ConditionAndOr) {
             ConditionAndOr and = (ConditionAndOr) expr;
-            Expression left = and.getExpression(true);
-            Expression right = and.getExpression(false);
-            addColumnData(columns, data, left);
-            addColumnData(columns, data, right);
+            addColumnData(session, columns, data, and.getSubexpression(0));
+            addColumnData(session, columns, data, and.getSubexpression(1));
         } else {
             Comparison comp = (Comparison) expr;
-            ExpressionColumn ec = (ExpressionColumn) comp.getExpression(true);
-            ValueExpression ev = (ValueExpression) comp.getExpression(false);
-            String columnName = ec.getColumnName();
+            ExpressionColumn ec = (ExpressionColumn) comp.getSubexpression(0);
+            String columnName = ec.getColumnName(session, -1);
             columns.add(columnName);
-            if (ev == null) {
+            if (expr.getSubexpressionCount() == 1) {
                 data.add(null);
             } else {
+                ValueExpression ev = (ValueExpression) comp.getSubexpression(1);
                 data.add(ev.getValue(null).getString());
             }
         }
@@ -746,6 +757,7 @@ public class FullText {
      * @param conn the database connection
      * @param schema the schema name
      * @param table the table name
+     * @throws SQLException on failure
      */
     private static void createTrigger(Connection conn, String schema,
             String table) throws SQLException {
@@ -759,7 +771,6 @@ public class FullText {
                     + StringUtils.quoteIdentifier(TRIGGER_PREFIX + table);
             stat.execute("DROP TRIGGER IF EXISTS " + trigger);
             if (create) {
-                boolean multiThread = FullTextTrigger.isMultiThread(conn);
                 StringBuilder buff = new StringBuilder(
                         "CREATE TRIGGER IF NOT EXISTS ");
                 // unless multithread, trigger needs to be called on rollback as well,
@@ -767,16 +778,13 @@ public class FullText {
                 // (not the user connection)
                 buff.append(trigger).
                         append(" AFTER INSERT, UPDATE, DELETE");
-                if(!multiThread) {
-                    buff.append(", ROLLBACK");
-                }
-                buff.append(" ON ").
-                        append(StringUtils.quoteIdentifier(schema)).
-                        append('.').
-                        append(StringUtils.quoteIdentifier(table)).
+                buff.append(" ON ");
+                StringUtils.quoteIdentifier(buff, schema).
+                        append('.');
+                StringUtils.quoteIdentifier(buff, table).
                         append(" FOR EACH ROW CALL \"").
                         append(FullText.FullTextTrigger.class.getName()).
-                        append('\"');
+                        append('"');
                 stat.execute(buff.toString());
             }
         }
@@ -788,6 +796,7 @@ public class FullText {
      * @param conn the database connection
      * @param schema the schema name
      * @param table the table name
+     * @throws SQLException on failure
      */
     private static void indexExistingRows(Connection conn, String schema,
             String table) throws SQLException {
@@ -862,8 +871,6 @@ public class FullText {
         private FullTextSettings          setting;
         private IndexInfo                 index;
         private int[]                     columnTypes;
-        private final PreparedStatement[] prepStatements = new PreparedStatement[SQL.length];
-        private boolean                   useOwnConnection;
 
         private static final int INSERT_WORD = 0;
         private static final int INSERT_ROW  = 1;
@@ -872,17 +879,18 @@ public class FullText {
         private static final int DELETE_MAP  = 4;
         private static final int SELECT_ROW  = 5;
 
-        private static final String SQL[] = {
+        private static final String[] SQL = {
             "MERGE INTO " + SCHEMA + ".WORDS(NAME) KEY(NAME) VALUES(?)",
-            "INSERT INTO " + SCHEMA + ".ROWS(HASH, INDEXID, KEY) VALUES(?, ?, ?)",
+            "INSERT INTO " + SCHEMA + ".ROWS(HASH, INDEXID, `KEY`) VALUES(?, ?, ?)",
             "INSERT INTO " + SCHEMA + ".MAP(ROWID, WORDID) VALUES(?, ?)",
-            "DELETE FROM " + SCHEMA + ".ROWS WHERE HASH=? AND INDEXID=? AND KEY=?",
+            "DELETE FROM " + SCHEMA + ".ROWS WHERE HASH=? AND INDEXID=? AND `KEY`=?",
             "DELETE FROM " + SCHEMA + ".MAP WHERE ROWID=? AND WORDID=?",
-            "SELECT ID FROM " + SCHEMA + ".ROWS WHERE HASH=? AND INDEXID=? AND KEY=?"
+            "SELECT ID FROM " + SCHEMA + ".ROWS WHERE HASH=? AND INDEXID=? AND `KEY`=?"
         };
 
         /**
          * INTERNAL
+         * @see Trigger#init(Connection, String, String, String, boolean, int)
          */
         @Override
         public void init(Connection conn, String schemaName, String triggerName,
@@ -891,13 +899,13 @@ public class FullText {
             if (!setting.isInitialized()) {
                 FullText.init(conn);
             }
-            ArrayList<String> keyList = New.arrayList();
+            ArrayList<String> keyList = Utils.newSmallArrayList();
             DatabaseMetaData meta = conn.getMetaData();
             ResultSet rs = meta.getColumns(null,
                     StringUtils.escapeMetaDataPattern(schemaName),
                     StringUtils.escapeMetaDataPattern(tableName),
                     null);
-            ArrayList<String> columnList = New.arrayList();
+            ArrayList<String> columnList = Utils.newSmallArrayList();
             while (rs.next()) {
                 columnList.add(rs.getString("COLUMN_NAME"));
             }
@@ -924,10 +932,10 @@ public class FullText {
             if (keyList.isEmpty()) {
                 throw throwException("No primary key for table " + tableName);
             }
-            ArrayList<String> indexList = New.arrayList();
+            ArrayList<String> indexList = Utils.newSmallArrayList();
             PreparedStatement prep = conn.prepareStatement(
                     "SELECT ID, COLUMNS FROM " + SCHEMA + ".INDEXES" +
-                    " WHERE SCHEMA=? AND TABLE=?");
+                    " WHERE SCHEMA=? AND `TABLE`=?");
             prep.setString(1, schemaName);
             prep.setString(2, tableName);
             rs = prep.executeQuery();
@@ -946,34 +954,11 @@ public class FullText {
             index.indexColumns = new int[indexList.size()];
             setColumns(index.indexColumns, indexList, columnList);
             setting.addIndexInfo(index);
-
-            useOwnConnection = isMultiThread(conn);
-            if(!useOwnConnection) {
-                for (int i = 0; i < SQL.length; i++) {
-                    prepStatements[i] = conn.prepareStatement(SQL[i],
-                            Statement.RETURN_GENERATED_KEYS);
-                }
-            }
-        }
-
-        /**
-         * Check whether the database is in multi-threaded mode.
-         *
-         * @param conn the connection
-         * @return true if the multi-threaded mode is used
-         */
-        static boolean isMultiThread(Connection conn)
-                throws SQLException {
-            try (Statement stat = conn.createStatement()) {
-                ResultSet rs = stat.executeQuery(
-                                "SELECT value FROM information_schema.settings" +
-                                " WHERE name = 'MULTI_THREADED'");
-                return rs.next() && !"0".equals(rs.getString(1));
-            }
         }
 
         /**
          * INTERNAL
+         * @see Trigger#fire(Connection, Object[], Object[])
          */
         @Override
         public void fire(Connection conn, Object[] oldRow, Object[] newRow)
@@ -1016,8 +1001,9 @@ public class FullText {
          *
          * @param conn to use
          * @param row the row
+         * @throws SQLException on failure
          */
-        protected void insert(Connection conn, Object[] row) throws SQLException {
+        private void insert(Connection conn, Object[] row) throws SQLException {
             PreparedStatement prepInsertRow = null;
             PreparedStatement prepInsertMap = null;
             try {
@@ -1040,10 +1026,8 @@ public class FullText {
                     prepInsertMap.execute();
                 }
             } finally {
-                if (useOwnConnection) {
-                    IOUtils.closeSilently(prepInsertRow);
-                    IOUtils.closeSilently(prepInsertMap);
-                }
+                IOUtils.closeSilently(prepInsertRow);
+                IOUtils.closeSilently(prepInsertMap);
             }
         }
 
@@ -1052,8 +1036,9 @@ public class FullText {
          *
          * @param conn to use
          * @param row the row
+         * @throws SQLException on failure
          */
-        protected void delete(Connection conn, Object[] row) throws SQLException {
+        private void delete(Connection conn, Object[] row) throws SQLException {
             PreparedStatement prepSelectRow = null;
             PreparedStatement prepDeleteMap = null;
             PreparedStatement prepDeleteRow = null;
@@ -1081,11 +1066,9 @@ public class FullText {
                     prepDeleteRow.executeUpdate();
                 }
             } finally {
-                if (useOwnConnection) {
-                    IOUtils.closeSilently(prepSelectRow);
-                    IOUtils.closeSilently(prepDeleteMap);
-                    IOUtils.closeSilently(prepDeleteRow);
-                }
+                IOUtils.closeSilently(prepSelectRow);
+                IOUtils.closeSilently(prepDeleteMap);
+                IOUtils.closeSilently(prepDeleteRow);
             }
         }
 
@@ -1133,31 +1116,31 @@ public class FullText {
                 Arrays.sort(wordIds);
                 return wordIds;
             } finally {
-                if (useOwnConnection) {
-                    IOUtils.closeSilently(prepInsertWord);
-                }
+                IOUtils.closeSilently(prepInsertWord);
             }
         }
 
         private String getKey(Object[] row) throws SQLException {
-            StatementBuilder buff = new StatementBuilder();
-            for (int columnIndex : index.keys) {
-                buff.appendExceptFirst(" AND ");
-                buff.append(StringUtils.quoteIdentifier(index.columns[columnIndex]));
+            StringBuilder builder = new StringBuilder();
+            int[] keys = index.keys;
+            for (int i = 0, l = keys.length; i < l; i++) {
+                if (i > 0) {
+                    builder.append(" AND ");
+                }
+                int columnIndex = keys[i];
+                StringUtils.quoteIdentifier(builder, index.columns[columnIndex]);
                 Object o = row[columnIndex];
                 if (o == null) {
-                    buff.append(" IS NULL");
+                    builder.append(" IS NULL");
                 } else {
-                    buff.append('=').append(quoteSQL(o, columnTypes[columnIndex]));
+                    builder.append('=').append(quoteSQL(o, columnTypes[columnIndex]));
                 }
             }
-            return buff.toString();
+            return builder.toString();
         }
 
-        private PreparedStatement getStatement(Connection conn, int index) throws SQLException {
-            return useOwnConnection ?
-                    conn.prepareStatement(SQL[index], Statement.RETURN_GENERATED_KEYS)
-                    : prepStatements[index];
+        private static PreparedStatement getStatement(Connection conn, int index) throws SQLException {
+            return conn.prepareStatement(SQL[index], Statement.RETURN_GENERATED_KEYS);
         }
 
     }
