@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.index;
@@ -10,7 +10,7 @@ package org.h2.index;
  */
 public class IndexType {
 
-    private boolean primaryKey, persistent, unique, hash, scan, spatial, affinity;
+    private boolean primaryKey, persistent, unique, hash, scan, spatial;
     private boolean belongsToConstraint;
 
     /**
@@ -71,17 +71,6 @@ public class IndexType {
         return type;
     }
 
-    /**
-     * Create an affinity index.
-     *
-     * @return the index type
-     */
-    public static IndexType createAffinity() {
-        IndexType type = new IndexType();
-        type.affinity = true;
-        return type;
-    }
-
     /**
      * Create a scan pseudo-index.
      *
@@ -159,15 +148,6 @@ public class IndexType {
         return unique;
     }
 
-    /**
-     * Does this index represent an affinity key?
-     *
-     * @return true if it does
-     */
-    public boolean isAffinity() {
-        return affinity;
-    }
-
     /**
      * Get the SQL snippet to create such an index.
      *
