@@ -1,12 +1,14 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.index;
 
 import java.util.ArrayList;
-import org.h2.engine.Session;
+
+import org.h2.command.query.AllColumnsForPlan;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
 import org.h2.result.Row;
 import org.h2.result.SearchRow;
@@ -19,55 +21,56 @@ import org.h2.table.TableFilter;
 /**
  * The index implementation for meta data tables.
  */
-public class MetaIndex extends BaseIndex {
+public class MetaIndex extends Index {
 
     private final MetaTable meta;
     private final boolean scan;
 
     public MetaIndex(MetaTable meta, IndexColumn[] columns, boolean scan) {
-        initBaseIndex(meta, 0, null, columns, IndexType.createNonUnique(true));
+        super(meta, 0, null, columns, 0, IndexType.createNonUnique(true));
         this.meta = meta;
         this.scan = scan;
     }
 
     @Override
-    public void close(Session session) {
+    public void close(SessionLocal session) {
         // nothing to do
     }
 
     @Override
-    public void add(Session session, Row row) {
+    public void add(SessionLocal session, Row row) {
         throw DbException.getUnsupportedException("META");
     }
 
     @Override
-    public void remove(Session session, Row row) {
+    public void remove(SessionLocal session, Row row) {
         throw DbException.getUnsupportedException("META");
     }
 
     @Override
-    public Cursor find(Session session, SearchRow first, SearchRow last) {
+    public Cursor find(SessionLocal session, SearchRow first, SearchRow last) {
         ArrayList<Row> rows = meta.generateRows(session, first, last);
         return new MetaCursor(rows);
     }
 
     @Override
-    public double getCost(Session session, int[] masks,
-            TableFilter[] filters, int filter, SortOrder sortOrder) {
+    public double getCost(SessionLocal session, int[] masks,
+            TableFilter[] filters, int filter, SortOrder sortOrder,
+            AllColumnsForPlan allColumnsSet) {
         if (scan) {
             return 10 * MetaTable.ROW_COUNT_APPROXIMATION;
         }
         return getCostRangeIndex(masks, MetaTable.ROW_COUNT_APPROXIMATION,
-                filters, filter, sortOrder, false);
+                filters, filter, sortOrder, false, allColumnsSet);
     }
 
     @Override
-    public void truncate(Session session) {
+    public void truncate(SessionLocal session) {
         throw DbException.getUnsupportedException("META");
     }
 
     @Override
-    public void remove(Session session) {
+    public void remove(SessionLocal session) {
         throw DbException.getUnsupportedException("META");
     }
 
@@ -80,6 +83,14 @@ public class MetaIndex extends BaseIndex {
         return super.getColumnIndex(col);
     }
 
+    @Override
+    public boolean isFirstColumn(Column column) {
+        if (scan) {
+            return false;
+        }
+        return super.isFirstColumn(column);
+    }
+
     @Override
     public void checkRename() {
         throw DbException.getUnsupportedException("META");
@@ -96,22 +107,12 @@ public class MetaIndex extends BaseIndex {
     }
 
     @Override
-    public boolean canGetFirstOrLast() {
-        return false;
-    }
-
-    @Override
-    public Cursor findFirstOrLast(Session session, boolean first) {
-        throw DbException.getUnsupportedException("META");
-    }
-
-    @Override
-    public long getRowCount(Session session) {
+    public long getRowCount(SessionLocal session) {
         return MetaTable.ROW_COUNT_APPROXIMATION;
     }
 
     @Override
-    public long getRowCountApproximation() {
+    public long getRowCountApproximation(SessionLocal session) {
         return MetaTable.ROW_COUNT_APPROXIMATION;
     }
 
