@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.jdbc;
@@ -19,9 +19,11 @@ import java.sql.Ref;
 import java.sql.ResultSetMetaData;
 import java.sql.RowId;
 import java.sql.SQLException;
+import java.sql.SQLType;
 import java.sql.SQLXML;
 import java.sql.Time;
 import java.sql.Timestamp;
+import java.util.BitSet;
 import java.util.Calendar;
 import java.util.HashMap;
 import java.util.Map;
@@ -29,26 +31,41 @@ import org.h2.api.ErrorCode;
 import org.h2.expression.ParameterInterface;
 import org.h2.message.DbException;
 import org.h2.message.TraceObject;
-import org.h2.util.BitField;
-import org.h2.util.New;
 import org.h2.value.ValueNull;
 
 /**
  * Represents a callable statement.
- *
+ * <p>
+ * Thread safety: the callable statement is not thread-safe. If the same
+ * callable statement is used by multiple threads access to it must be
+ * synchronized. The single synchronized block must include assignment of
+ * parameters, execution of the command and all operations with its result.
+ * </p>
+ * <pre>
+ * synchronized (call) {
+ *     call.setInt(1, 10);
+ *     try (ResultSet rs = call.executeQuery()) {
+ *         while (rs.next) {
+ *             // Do something
+ *         }
+ *     }
+ * }
+ * synchronized (call) {
+ *     call.setInt(1, 15);
+ *     updateCount = call.executeUpdate();
+ * }
+ * </pre>
  * @author Sergi Vladykin
  * @author Thomas Mueller
  */
-public class JdbcCallableStatement extends JdbcPreparedStatement implements
-        CallableStatement, JdbcCallableStatementBackwardsCompat {
+public final class JdbcCallableStatement extends JdbcPreparedStatement implements CallableStatement {
 
-    private BitField outParameters;
+    private BitSet outParameters;
     private int maxOutParameters;
     private HashMap<String, Integer> namedParameters;
 
-    JdbcCallableStatement(JdbcConnection conn, String sql, int id,
-            int resultSetType, int resultSetConcurrency) {
-        super(conn, sql, id, resultSetType, resultSetConcurrency, false);
+    JdbcCallableStatement(JdbcConnection conn, String sql, int id, int resultSetType, int resultSetConcurrency) {
+        super(conn, sql, id, resultSetType, resultSetConcurrency, null);
         setTrace(session.getTrace(), TraceObject.CALLABLE_STATEMENT, id);
     }
 
@@ -82,6 +99,36 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
         }
     }
 
+    /**
+     * Executes a statement (insert, update, delete, create, drop)
+     * and returns the update count.
+     * If another result set exists for this statement, this will be closed
+     * (even if this statement fails).
+     *
+     * If auto commit is on, this statement will be committed.
+     * If the statement is a DDL statement (create, drop, alter) and does not
+     * throw an exception, the current transaction (if any) is committed after
+     * executing the statement.
+     *
+     * @return the update count (number of row affected by an insert, update or
+     *         delete, or 0 if no rows or the statement was a create, drop,
+     *         commit or rollback)
+     * @throws SQLException if this object is closed or invalid
+     */
+    @Override
+    public long executeLargeUpdate() throws SQLException {
+        try {
+            checkClosed();
+            if (command.isQuery()) {
+                super.executeQuery();
+                return 0;
+            }
+            return super.executeLargeUpdate();
+        } catch (Exception e) {
+            throw logAndConvert(e);
+        }
+    }
+
     /**
      * Registers the given OUT parameter.
      *
@@ -323,11 +370,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Returns the value of the specified column as a java.sql.Date.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(columnIndex, LocalDate.class)} instead.
+     * </p>
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(int, Class)
      */
     @Override
     public Date getDate(int parameterIndex) throws SQLException {
@@ -337,11 +389,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Returns the value of the specified column as a java.sql.Time.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(columnIndex, LocalTime.class)} instead.
+     * </p>
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(int, Class)
      */
     @Override
     public Time getTime(int parameterIndex) throws SQLException {
@@ -351,11 +408,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Returns the value of the specified column as a java.sql.Timestamp.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(columnIndex, LocalDateTime.class)} instead.
+     * </p>
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(int, Class)
      */
     @Override
     public Timestamp getTimestamp(int parameterIndex) throws SQLException {
@@ -455,12 +517,17 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     /**
      * Returns the value of the specified column as a java.sql.Date using a
      * specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(columnIndex, LocalDate.class)} instead.
+     * </p>
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @param cal the calendar
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(int, Class)
      */
     @Override
     public Date getDate(int parameterIndex, Calendar cal) throws SQLException {
@@ -471,12 +538,17 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     /**
      * Returns the value of the specified column as a java.sql.Time using a
      * specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(columnIndex, LocalTime.class)} instead.
+     * </p>
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @param cal the calendar
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(int, Class)
      */
     @Override
     public Time getTime(int parameterIndex, Calendar cal) throws SQLException {
@@ -487,16 +559,20 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     /**
      * Returns the value of the specified column as a java.sql.Timestamp using a
      * specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(columnIndex, LocalDateTime.class)} instead.
+     * </p>
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @param cal the calendar
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(int, Class)
      */
     @Override
-    public Timestamp getTimestamp(int parameterIndex, Calendar cal)
-            throws SQLException {
+    public Timestamp getTimestamp(int parameterIndex, Calendar cal) throws SQLException {
         checkRegistered(parameterIndex);
         return getOpenResultSet().getTimestamp(parameterIndex, cal);
     }
@@ -512,28 +588,37 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     /**
      * Returns the value of the specified column as a java.sql.Timestamp using a
      * specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(parameterName, LocalDateTime.class)} instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param cal the calendar
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(String, Class)
      */
     @Override
-    public Timestamp getTimestamp(String parameterName, Calendar cal)
-            throws SQLException {
+    public Timestamp getTimestamp(String parameterName, Calendar cal) throws SQLException {
         return getTimestamp(getIndexForName(parameterName), cal);
     }
 
     /**
      * Returns the value of the specified column as a java.sql.Time using a
      * specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(parameterName, LocalTime.class)} instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param cal the calendar
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(String, Class)
      */
     @Override
     public Time getTime(String parameterName, Calendar cal) throws SQLException {
@@ -543,12 +628,17 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     /**
      * Returns the value of the specified column as a java.sql.Date using a
      * specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(parameterName, LocalDate.class)} instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param cal the calendar
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(String, Class)
      */
     @Override
     public Date getDate(String parameterName, Calendar cal) throws SQLException {
@@ -641,11 +731,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Returns the value of the specified column as a java.sql.Timestamp.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(parameterName, LocalDateTime.class)} instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(String, Class)
      */
     @Override
     public Timestamp getTimestamp(String parameterName) throws SQLException {
@@ -654,11 +749,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Returns the value of the specified column as a java.sql.Time.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(parameterName, LocalTime.class)} instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(String, Class)
      */
     @Override
     public Time getTime(String parameterName) throws SQLException {
@@ -667,11 +767,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Returns the value of the specified column as a java.sql.Date.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code getObject(parameterName, LocalDate.class)} instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @return the value
      * @throws SQLException if the column is not found or if this object is
      *             closed
+     * @see #getObject(String, Class)
      */
     @Override
     public Date getDate(String parameterName) throws SQLException {
@@ -843,21 +948,30 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     }
 
     /**
-     * [Not supported] Returns the value of the specified column as a SQLXML
-     * object.
+     * Returns the value of the specified column as a SQLXML object.
+     *
+     * @param parameterIndex the parameter index (1, 2, ...)
+     * @return the value
+     * @throws SQLException if the column is not found or if this object is
+     *             closed
      */
     @Override
     public SQLXML getSQLXML(int parameterIndex) throws SQLException {
-        throw unsupported("SQLXML");
+        checkRegistered(parameterIndex);
+        return getOpenResultSet().getSQLXML(parameterIndex);
     }
 
     /**
-     * [Not supported] Returns the value of the specified column as a SQLXML
-     * object.
+     * Returns the value of the specified column as a SQLXML object.
+     *
+     * @param parameterName the parameter name
+     * @return the value
+     * @throws SQLException if the column is not found or if this object is
+     *             closed
      */
     @Override
     public SQLXML getSQLXML(String parameterName) throws SQLException {
-        throw unsupported("SQLXML");
+        return getSQLXML(getIndexForName(parameterName));
     }
 
     /**
@@ -976,45 +1090,60 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     /**
      * Sets the timestamp using a specified time zone. The value will be
      * converted to the local time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code setObject(parameterName, value)} with
+     * {@link java.time.LocalDateTime} parameter instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param x the value
      * @param cal the calendar
      * @throws SQLException if this object is closed
+     * @see #setObject(String, Object)
      */
     @Override
-    public void setTimestamp(String parameterName, Timestamp x, Calendar cal)
-            throws SQLException {
+    public void setTimestamp(String parameterName, Timestamp x, Calendar cal) throws SQLException {
         setTimestamp(getIndexForName(parameterName), x, cal);
     }
 
     /**
      * Sets the time using a specified time zone. The value will be converted to
      * the local time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code setObject(parameterName, value)} with {@link java.time.LocalTime}
+     * parameter instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param x the value
      * @param cal the calendar
      * @throws SQLException if this object is closed
+     * @see #setObject(String, Object)
      */
     @Override
-    public void setTime(String parameterName, Time x, Calendar cal)
-            throws SQLException {
+    public void setTime(String parameterName, Time x, Calendar cal) throws SQLException {
         setTime(getIndexForName(parameterName), x, cal);
     }
 
     /**
      * Sets the date using a specified time zone. The value will be converted to
      * the local time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code setObject(parameterName, value)} with {@link java.time.LocalDate}
+     * parameter instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param x the value
      * @param cal the calendar
      * @throws SQLException if this object is closed
+     * @see #setObject(String, Object)
      */
     @Override
-    public void setDate(String parameterName, Date x, Calendar cal)
-            throws SQLException {
+    public void setDate(String parameterName, Date x, Calendar cal) throws SQLException {
         setDate(getIndexForName(parameterName), x, cal);
     }
 
@@ -1080,6 +1209,38 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
         setObject(getIndexForName(parameterName), x, targetSqlType, scale);
     }
 
+    /**
+     * Sets the value of a parameter. The object is converted, if required, to
+     * the specified data type before sending to the database.
+     * Objects of unknown classes are serialized (on the client side).
+     *
+     * @param parameterName the parameter name
+     * @param x the value, null is allowed
+     * @param targetSqlType the type
+     * @throws SQLException if this object is closed
+     */
+    @Override
+    public void setObject(String parameterName, Object x, SQLType targetSqlType) throws SQLException {
+        setObject(getIndexForName(parameterName), x, targetSqlType);
+    }
+
+    /**
+     * Sets the value of a parameter. The object is converted, if required, to
+     * the specified data type before sending to the database.
+     * Objects of unknown classes are serialized (on the client side).
+     *
+     * @param parameterName the parameter name
+     * @param x the value, null is allowed
+     * @param targetSqlType the type
+     * @param scaleOrLength is ignored
+     * @throws SQLException if this object is closed
+     */
+    @Override
+    public void setObject(String parameterName, Object x, SQLType targetSqlType, int scaleOrLength)
+            throws SQLException {
+        setObject(getIndexForName(parameterName), x, targetSqlType, scaleOrLength);
+    }
+
     /**
      * Sets the value of a parameter as an input stream.
      * This method does not close the stream.
@@ -1114,23 +1275,34 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Sets the value of a parameter.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code setObject(parameterName, value)} with
+     * {@link java.time.LocalDateTime} parameter instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param x the value
      * @throws SQLException if this object is closed
+     * @see #setObject(String, Object)
      */
     @Override
-    public void setTimestamp(String parameterName, Timestamp x)
-            throws SQLException {
+    public void setTimestamp(String parameterName, Timestamp x) throws SQLException {
         setTimestamp(getIndexForName(parameterName), x);
     }
 
     /**
      * Sets the time using a specified time zone.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code setObject(parameterName, value)} with {@link java.time.LocalTime}
+     * parameter instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param x the value
      * @throws SQLException if this object is closed
+     * @see #setObject(String, Object)
      */
     @Override
     public void setTime(String parameterName, Time x) throws SQLException {
@@ -1139,10 +1311,16 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
 
     /**
      * Sets the value of a parameter.
+     * <p>
+     * Usage of this method is discouraged. Use
+     * {@code setObject(parameterName, value)} with {@link java.time.LocalDate}
+     * parameter instead.
+     * </p>
      *
      * @param parameterName the parameter name
      * @param x the value
      * @throws SQLException if this object is closed
+     * @see #setObject(String, Object)
      */
     @Override
     public void setDate(String parameterName, Date x) throws SQLException {
@@ -1555,19 +1733,27 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     }
 
     /**
-     * [Not supported] Sets the value of a parameter as a SQLXML object.
+     * Sets the value of a parameter as a SQLXML object.
+     *
+     * @param parameterName the parameter name
+     * @param x the value
+     * @throws SQLException if this object is closed
      */
     @Override
     public void setSQLXML(String parameterName, SQLXML x)
             throws SQLException {
-        throw unsupported("SQLXML");
+        setSQLXML(getIndexForName(parameterName), x);
     }
 
     /**
-     * [Not supported]
+     * Returns the value of the specified column as a Java object of the
+     * specified type.
      *
      * @param parameterIndex the parameter index (1, 2, ...)
      * @param type the class of the returned value
+     * @return the value
+     * @throws SQLException if the column is not found or if this object is
+     *             closed
      */
     @Override
     public <T> T getObject(int parameterIndex, Class<T> type) throws SQLException {
@@ -1575,10 +1761,14 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
     }
 
     /**
-     * [Not supported]
+     * Returns the value of the specified column as a Java object of the
+     * specified type.
      *
      * @param parameterName the parameter name
      * @param type the class of the returned value
+     * @return the value
+     * @throws SQLException if the column is not found or if this object is
+     *             closed
      */
     @Override
     public <T> T getObject(String parameterName, Class<T> type) throws SQLException {
@@ -1608,7 +1798,7 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
                 maxOutParameters = Math.min(
                         getParameterMetaData().getParameterCount(),
                         getCheckedMetaData().getColumnCount());
-                outParameters = new BitField();
+                outParameters = new BitSet();
             }
             checkIndexBounds(parameterIndex);
             ParameterInterface param = command.getParameters().get(--parameterIndex);
@@ -1638,7 +1828,7 @@ public class JdbcCallableStatement extends JdbcPreparedStatement implements
             if (namedParameters == null) {
                 ResultSetMetaData meta = getCheckedMetaData();
                 int columnCount = meta.getColumnCount();
-                HashMap<String, Integer> map = New.hashMap(columnCount);
+                HashMap<String, Integer> map = new HashMap<>();
                 for (int i = 1; i <= columnCount; i++) {
                     map.put(meta.getColumnLabel(i), i);
                 }
