@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.jdbcx;
@@ -10,6 +10,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
+
 import javax.sql.ConnectionEvent;
 import javax.sql.ConnectionEventListener;
 import javax.sql.StatementEventListener;
@@ -17,11 +18,12 @@ import javax.sql.XAConnection;
 import javax.transaction.xa.XAException;
 import javax.transaction.xa.XAResource;
 import javax.transaction.xa.Xid;
+
 import org.h2.api.ErrorCode;
 import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
 import org.h2.message.TraceObject;
-import org.h2.util.New;
+import org.h2.util.Utils;
 
 
 /**
@@ -29,7 +31,7 @@ import org.h2.util.New;
  * An application developer usually does not use this interface.
  * It is used by the transaction manager internally.
  */
-public class JdbcXAConnection extends TraceObject implements XAConnection,
+public final class JdbcXAConnection extends TraceObject implements XAConnection,
         XAResource {
 
     private final JdbcDataSourceFactory factory;
@@ -39,14 +41,10 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
 
     // this connection is replaced whenever getConnection is called
     private volatile Connection handleConn;
-    private final ArrayList<ConnectionEventListener> listeners = New.arrayList();
+    private final ArrayList<ConnectionEventListener> listeners = Utils.newSmallArrayList();
     private Xid currentTransaction;
     private boolean prepared;
 
-    static {
-        org.h2.Driver.load();
-    }
-
     JdbcXAConnection(JdbcDataSourceFactory factory, int id,
             JdbcConnection physicalConn) {
         this.factory = factory;
@@ -113,7 +111,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
      */
     @Override
     public void addConnectionEventListener(ConnectionEventListener listener) {
-        debugCode("addConnectionEventListener(listener);");
+        debugCode("addConnectionEventListener(listener)");
         listeners.add(listener);
     }
 
@@ -124,7 +122,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
      */
     @Override
     public void removeConnectionEventListener(ConnectionEventListener listener) {
-        debugCode("removeConnectionEventListener(listener);");
+        debugCode("removeConnectionEventListener(listener)");
         listeners.remove(listener);
     }
 
@@ -132,7 +130,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
      * INTERNAL
      */
     void closedHandle() {
-        debugCode("closedHandle();");
+        debugCodeCall("closedHandle");
         ConnectionEvent event = new ConnectionEvent(this);
         // go backward so that a listener can remove itself
         // (otherwise we need to clone the list)
@@ -174,7 +172,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
      */
     @Override
     public boolean isSameRM(XAResource xares) {
-        debugCode("isSameRM(xares);");
+        debugCode("isSameRM(xares)");
         return xares == this;
     }
 
@@ -191,11 +189,10 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
         debugCodeCall("recover", quoteFlags(flag));
         checkOpen();
         try (Statement stat = physicalConn.createStatement()) {
-            ResultSet rs = stat.executeQuery("SELECT * FROM " +
-                    "INFORMATION_SCHEMA.IN_DOUBT ORDER BY TRANSACTION");
-            ArrayList<Xid> list = New.arrayList();
+            ResultSet rs = stat.executeQuery("SELECT * FROM INFORMATION_SCHEMA.IN_DOUBT ORDER BY TRANSACTION_NAME");
+            ArrayList<Xid> list = Utils.newSmallArrayList();
             while (rs.next()) {
-                String tid = rs.getString("TRANSACTION");
+                String tid = rs.getString("TRANSACTION_NAME");
                 int id = getNextId(XID);
                 Xid xid = new JdbcXid(factory, id, tid);
                 list.add(xid);
@@ -222,7 +219,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     @Override
     public int prepare(Xid xid) throws XAException {
         if (isDebugEnabled()) {
-            debugCode("prepare("+JdbcXid.toString(xid)+");");
+            debugCode("prepare(" + quoteXid(xid) + ')');
         }
         checkOpen();
         if (!currentTransaction.equals(xid)) {
@@ -230,7 +227,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
         }
 
         try (Statement stat = physicalConn.createStatement()) {
-            stat.execute("PREPARE COMMIT " + JdbcXid.toString(xid));
+            stat.execute(JdbcXid.toString(new StringBuilder("PREPARE COMMIT \""), xid).append('"').toString());
             prepared = true;
         } catch (SQLException e) {
             throw convertException(e);
@@ -247,7 +244,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     @Override
     public void forget(Xid xid) {
         if (isDebugEnabled()) {
-            debugCode("forget("+JdbcXid.toString(xid)+");");
+            debugCode("forget(" + quoteXid(xid) + ')');
         }
         prepared = false;
     }
@@ -260,12 +257,13 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     @Override
     public void rollback(Xid xid) throws XAException {
         if (isDebugEnabled()) {
-            debugCode("rollback("+JdbcXid.toString(xid)+");");
+            debugCode("rollback(" + quoteXid(xid) + ')');
         }
         try {
             if (prepared) {
                 try (Statement stat = physicalConn.createStatement()) {
-                    stat.execute("ROLLBACK TRANSACTION " + JdbcXid.toString(xid));
+                    stat.execute(JdbcXid.toString( //
+                            new StringBuilder("ROLLBACK TRANSACTION \""), xid).append('"').toString());
                 }
                 prepared = false;
             } else {
@@ -287,7 +285,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     @Override
     public void end(Xid xid, int flags) throws XAException {
         if (isDebugEnabled()) {
-            debugCode("end("+JdbcXid.toString(xid)+", "+quoteFlags(flags)+");");
+            debugCode("end(" + quoteXid(xid) + ", " + quoteFlags(flags) + ')');
         }
         // TODO transaction end: implement this method
         if (flags == TMSUSPEND) {
@@ -308,7 +306,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     @Override
     public void start(Xid xid, int flags) throws XAException {
         if (isDebugEnabled()) {
-            debugCode("start("+JdbcXid.toString(xid)+", "+quoteFlags(flags)+");");
+            debugCode("start(" + quoteXid(xid) + ", " + quoteFlags(flags) + ')');
         }
         if (flags == TMRESUME) {
             return;
@@ -338,7 +336,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     @Override
     public void commit(Xid xid, boolean onePhase) throws XAException {
         if (isDebugEnabled()) {
-            debugCode("commit("+JdbcXid.toString(xid)+", "+onePhase+");");
+            debugCode("commit(" + quoteXid(xid) + ", " + onePhase + ')');
         }
 
         try {
@@ -346,7 +344,8 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
                 physicalConn.commit();
             } else {
                 try (Statement stat = physicalConn.createStatement()) {
-                    stat.execute("COMMIT TRANSACTION " + JdbcXid.toString(xid));
+                    stat.execute(
+                            JdbcXid.toString(new StringBuilder("COMMIT TRANSACTION \""), xid).append('"').toString());
                     prepared = false;
                 }
             }
@@ -391,6 +390,10 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
         return xa;
     }
 
+    private static String quoteXid(Xid xid) {
+        return JdbcXid.toString(new StringBuilder(), xid).toString().replace('-', '$');
+    }
+
     private static String quoteFlags(int flags) {
         StringBuilder buff = new StringBuilder();
         if ((flags & XAResource.TMENDRSCAN) != 0) {
@@ -423,7 +426,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
         if (buff.length() == 0) {
             buff.append("|XAResource.TMNOFLAGS");
         }
-        return buff.toString().substring(1);
+        return buff.substring(1);
     }
 
     private void checkOpen() throws XAException {
@@ -435,7 +438,7 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
     /**
      * A pooled connection.
      */
-    class PooledJdbcConnection extends JdbcConnection {
+    final class PooledJdbcConnection extends JdbcConnection {
 
         private boolean isClosed;
 
@@ -463,11 +466,11 @@ public class JdbcXAConnection extends TraceObject implements XAConnection,
         }
 
         @Override
-        protected synchronized void checkClosed(boolean write) {
+        protected synchronized void checkClosed() {
             if (isClosed) {
                 throw DbException.get(ErrorCode.OBJECT_CLOSED);
             }
-            super.checkClosed(write);
+            super.checkClosed();
         }
 
     }
