@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.message;
@@ -8,12 +8,15 @@ package org.h2.message;
 import java.math.BigDecimal;
 import java.sql.SQLException;
 import java.util.Map;
+import java.util.concurrent.atomic.AtomicIntegerArray;
+
+import org.h2.api.ErrorCode;
 import org.h2.util.StringUtils;
 
 /**
  * The base class for objects that can print trace information about themselves.
  */
-public class TraceObject {
+public abstract class TraceObject {
 
     /**
      * The trace type id  for callable statements.
@@ -90,11 +93,19 @@ public class TraceObject {
      */
     protected static final int ARRAY = 16;
 
-    private static final int LAST = ARRAY + 1;
-    private static final int[] ID = new int[LAST];
+    /**
+     * The trace type id  for SQLXML objects.
+     */
+    protected static final int SQLXML = 17;
+
+    private static final int LAST = SQLXML + 1;
+    private static final AtomicIntegerArray ID = new AtomicIntegerArray(LAST);
+
     private static final String[] PREFIX = { "call", "conn", "dbMeta", "prep",
             "rs", "rsMeta", "sp", "ex", "stat", "blob", "clob", "pMeta", "ds",
-            "xads", "xares", "xid", "ar" };
+            "xads", "xares", "xid", "ar", "sqlxml" };
+
+    private static final SQLException SQL_OOME = DbException.SQL_OOME;
 
     /**
      * The trace module used by this object.
@@ -119,6 +130,7 @@ public class TraceObject {
 
     /**
      * INTERNAL
+     * @return id
      */
     public int getTraceId() {
         return id;
@@ -126,6 +138,7 @@ public class TraceObject {
 
     /**
      * INTERNAL
+     * @return object name
      */
     public String getTraceObjectName() {
         return PREFIX[traceType] + id;
@@ -138,7 +151,7 @@ public class TraceObject {
      * @return the new trace object id
      */
     protected static int getNextId(int type) {
-        return ID[type]++;
+        return ID.getAndIncrement(type);
     }
 
     /**
@@ -146,7 +159,7 @@ public class TraceObject {
      *
      * @return true if it is
      */
-    protected boolean isDebugEnabled() {
+    protected final boolean isDebugEnabled() {
         return trace.isDebugEnabled();
     }
 
@@ -155,7 +168,7 @@ public class TraceObject {
      *
      * @return true if it is
      */
-    protected boolean isInfoEnabled() {
+    protected final boolean isInfoEnabled() {
         return trace.isInfoEnabled();
     }
 
@@ -168,11 +181,10 @@ public class TraceObject {
      * @param newId the trace object id of the created object
      * @param value the value to assign this new object to
      */
-    protected void debugCodeAssign(String className, int newType, int newId,
-            String value) {
+    protected final void debugCodeAssign(String className, int newType, int newId, String value) {
         if (trace.isDebugEnabled()) {
-            trace.debugCode(className + " " + PREFIX[newType] +
-                    newId + " = " + getTraceObjectName() + "." + value + ";");
+            trace.debugCode(className + ' ' + PREFIX[newType] + newId + " = " + getTraceObjectName() + '.' + value
+                    + ';');
         }
     }
 
@@ -182,9 +194,9 @@ public class TraceObject {
      *
      * @param methodName the method name
      */
-    protected void debugCodeCall(String methodName) {
+    protected final void debugCodeCall(String methodName) {
         if (trace.isDebugEnabled()) {
-            trace.debugCode(getTraceObjectName() + "." + methodName + "();");
+            trace.debugCode(getTraceObjectName() + '.' + methodName + "();");
         }
     }
 
@@ -196,10 +208,9 @@ public class TraceObject {
      * @param methodName the method name
      * @param param one single long parameter
      */
-    protected void debugCodeCall(String methodName, long param) {
+    protected final void debugCodeCall(String methodName, long param) {
         if (trace.isDebugEnabled()) {
-            trace.debugCode(getTraceObjectName() + "." +
-                    methodName + "(" + param + ");");
+            trace.debugCode(getTraceObjectName() + '.' + methodName + '(' + param + ");");
         }
     }
 
@@ -211,10 +222,9 @@ public class TraceObject {
      * @param methodName the method name
      * @param param one single string parameter
      */
-    protected void debugCodeCall(String methodName, String param) {
+    protected final void debugCodeCall(String methodName, String param) {
         if (trace.isDebugEnabled()) {
-            trace.debugCode(getTraceObjectName() + "." +
-                    methodName + "(" + quote(param) + ");");
+            trace.debugCode(getTraceObjectName() + '.' + methodName + '(' + quote(param) + ");");
         }
     }
 
@@ -223,9 +233,9 @@ public class TraceObject {
      *
      * @param text the trace text
      */
-    protected void debugCode(String text) {
+    protected final void debugCode(String text) {
         if (trace.isDebugEnabled()) {
-            trace.debugCode(getTraceObjectName() + "." + text);
+            trace.debugCode(getTraceObjectName() + '.' + text + ';');
         }
     }
 
@@ -301,8 +311,9 @@ public class TraceObject {
         if (x == null) {
             return "null";
         }
-        return "org.h2.util.StringUtils.convertHexToBytes(\"" +
-                StringUtils.convertBytesToHex(x) + "\")";
+        StringBuilder builder = new StringBuilder(x.length * 2 + 45)
+                .append("org.h2.util.StringUtils.convertHexToBytes(\"");
+        return StringUtils.convertBytesToHex(builder, x).append("\")").toString();
     }
 
     /**
@@ -348,17 +359,29 @@ public class TraceObject {
      * @param ex the exception
      * @return the SQL exception object
      */
-    protected SQLException logAndConvert(Exception ex) {
-        SQLException e = DbException.toSQLException(ex);
-        if (trace == null) {
-            DbException.traceThrowable(e);
-        } else {
-            int errorCode = e.getErrorCode();
-            if (errorCode >= 23000 && errorCode < 24000) {
-                trace.info(e, "exception");
+    protected SQLException logAndConvert(Throwable ex) {
+        SQLException e = null;
+        try {
+            e = DbException.toSQLException(ex);
+            if (trace == null) {
+                DbException.traceThrowable(e);
             } else {
-                trace.error(e, "exception");
+                int errorCode = e.getErrorCode();
+                if (errorCode >= 23000 && errorCode < 24000) {
+                    trace.info(e, "exception");
+                } else {
+                    trace.error(e, "exception");
+                }
             }
+        } catch(Throwable another) {
+            if (e == null) {
+                try {
+                    e = new SQLException("GeneralError", "HY000", ErrorCode.GENERAL_ERROR_1, ex);
+                } catch (OutOfMemoryError | NoClassDefFoundError ignored) {
+                    return SQL_OOME;
+                }
+            }
+            e.addSuppressed(another);
         }
         return e;
     }
