@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.mvstore;
@@ -11,6 +11,7 @@ import java.io.PrintWriter;
 import java.io.Writer;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
+import java.nio.charset.StandardCharsets;
 import java.sql.Timestamp;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -20,11 +21,12 @@ import org.h2.compress.CompressDeflate;
 import org.h2.compress.CompressLZF;
 import org.h2.compress.Compressor;
 import org.h2.engine.Constants;
-import org.h2.message.DbException;
-import org.h2.mvstore.type.DataType;
+import org.h2.mvstore.tx.TransactionStore;
+import org.h2.mvstore.type.BasicDataType;
 import org.h2.mvstore.type.StringDataType;
 import org.h2.store.fs.FilePath;
 import org.h2.store.fs.FileUtils;
+import org.h2.util.Utils;
 
 /**
  * Utility methods used in combination with the MVStore.
@@ -34,7 +36,8 @@ public class MVStoreTool {
     /**
      * Runs this tool.
      * Options are case sensitive. Supported options are:
-     * <table summary="command line options">
+     * <table>
+     * <caption>Command line options</caption>
      * <tr><td>[-dump &lt;fileName&gt;]</td>
      * <td>Dump the contends of the file</td></tr>
      * <tr><td>[-info &lt;fileName&gt;]</td>
@@ -107,24 +110,32 @@ public class MVStoreTool {
         }
         long size = FileUtils.size(fileName);
         pw.printf("File %s, %d bytes, %d MB\n", fileName, size, size / 1024 / 1024);
-        FileChannel file = null;
         int blockSize = MVStore.BLOCK_SIZE;
         TreeMap<Integer, Long> mapSizesTotal =
-                new TreeMap<Integer, Long>();
+                new TreeMap<>();
         long pageSizeTotal = 0;
-        try {
-            file = FilePath.get(fileName).open("r");
+        try (FileChannel file = FilePath.get(fileName).open("r")) {
             long fileSize = file.size();
             int len = Long.toHexString(fileSize).length();
             ByteBuffer block = ByteBuffer.allocate(4096);
             long pageCount = 0;
-            for (long pos = 0; pos < fileSize;) {
+            for (long pos = 0; pos < fileSize; ) {
                 block.rewind();
-                DataUtils.readFully(file, pos, block);
+                // Bugfix - An MVStoreException that wraps EOFException is
+                // thrown when partial writes happens in the case of power off
+                // or file system issues.
+                // So we should skip the broken block at end of the DB file.
+                try {
+                    DataUtils.readFully(file, pos, block);
+                } catch (MVStoreException e) {
+                    pos += blockSize;
+                    pw.printf("ERROR illegal position %d%n", pos);
+                    continue;
+                }
                 block.rewind();
                 int headerType = block.get();
                 if (headerType == 'H') {
-                    String header = new String(block.array(), DataUtils.LATIN).trim();
+                    String header = new String(block.array(), StandardCharsets.ISO_8859_1).trim();
                     pw.printf("%0" + len + "x fileHeader %s%n",
                             pos, header);
                     pos += blockSize;
@@ -135,10 +146,10 @@ public class MVStoreTool {
                     continue;
                 }
                 block.position(0);
-                Chunk c = null;
+                Chunk c;
                 try {
                     c = Chunk.readChunkHeader(block, pos);
-                } catch (IllegalStateException e) {
+                } catch (MVStoreException e) {
                     pos += blockSize;
                     continue;
                 }
@@ -157,7 +168,7 @@ public class MVStoreTool {
                 int remaining = c.pageCount;
                 pageCount += c.pageCount;
                 TreeMap<Integer, Integer> mapSizes =
-                        new TreeMap<Integer, Integer>();
+                        new TreeMap<>();
                 int pageSizeSum = 0;
                 while (remaining > 0) {
                     int start = p;
@@ -171,23 +182,24 @@ public class MVStoreTool {
                     int pageSize = chunk.getInt();
                     // check value (ignored)
                     chunk.getShort();
+                    /*int pageNo =*/ DataUtils.readVarInt(chunk);
                     int mapId = DataUtils.readVarInt(chunk);
                     int entries = DataUtils.readVarInt(chunk);
                     int type = chunk.get();
                     boolean compressed = (type & DataUtils.PAGE_COMPRESSED) != 0;
-                    boolean node = (type & 1) != 0;
+                    boolean node = (type & DataUtils.PAGE_TYPE_NODE) != 0;
                     if (details) {
                         pw.printf(
                                 "+%0" + len +
-                                "x %s, map %x, %d entries, %d bytes, maxLen %x%n",
+                                        "x %s, map %x, %d entries, %d bytes, maxLen %x%n",
                                 p,
                                 (node ? "node" : "leaf") +
-                                (compressed ? " compressed" : ""),
+                                        (compressed ? " compressed" : ""),
                                 mapId,
                                 node ? entries + 1 : entries,
                                 pageSize,
                                 DataUtils.getPageMaxLength(DataUtils.getPagePos(0, 0, pageSize, 0))
-                                );
+                        );
                     }
                     p += pageSize;
                     Integer mapSize = mapSizes.get(mapId);
@@ -220,12 +232,11 @@ public class MVStoreTool {
                     if (mapId == 0 && details) {
                         ByteBuffer data;
                         if (compressed) {
-                            boolean fast = !((type & DataUtils.PAGE_COMPRESSED_HIGH) ==
-                                    DataUtils.PAGE_COMPRESSED_HIGH);
+                            boolean fast = (type & DataUtils.PAGE_COMPRESSED_HIGH) != DataUtils.PAGE_COMPRESSED_HIGH;
                             Compressor compressor = getCompressor(fast);
                             int lenAdd = DataUtils.readVarInt(chunk);
                             int compLen = pageSize + start - chunk.position();
-                            byte[] comp = DataUtils.newBytes(compLen);
+                            byte[] comp = Utils.newBytes(compLen);
                             chunk.get(comp);
                             int l = compLen + lenAdd;
                             data = ByteBuffer.allocate(l);
@@ -242,8 +253,8 @@ public class MVStoreTool {
                             for (int i = 0; i < entries; i++) {
                                 long cp = children[i];
                                 pw.printf("    %d children < %s @ " +
-                                        "chunk %x +%0" +
-                                        len + "x%n",
+                                                "chunk %x +%0" +
+                                                len + "x%n",
                                         counts[i],
                                         keys[i],
                                         DataUtils.getPageChunkId(cp),
@@ -251,7 +262,7 @@ public class MVStoreTool {
                             }
                             long cp = children[entries];
                             pw.printf("    %d children >= %s @ chunk %x +%0" +
-                                    len + "x%n",
+                                            len + "x%n",
                                     counts[entries],
                                     keys.length >= entries ? null : keys[entries],
                                     DataUtils.getPageChunkId(cp),
@@ -273,7 +284,7 @@ public class MVStoreTool {
                             for (int i = 0; i <= entries; i++) {
                                 long cp = children[i];
                                 pw.printf("    %d children @ chunk %x +%0" +
-                                        len + "x%n",
+                                                len + "x%n",
                                         counts[i],
                                         DataUtils.getPageChunkId(cp),
                                         DataUtils.getPageOffset(cp));
@@ -293,7 +304,7 @@ public class MVStoreTool {
                             "+%0" + len + "x chunkFooter %s%n",
                             footerPos,
                             new String(chunk.array(), chunk.position(),
-                                    Chunk.FOOTER_LENGTH, DataUtils.LATIN).trim());
+                                    Chunk.FOOTER_LENGTH, StandardCharsets.ISO_8859_1).trim());
                 } catch (IllegalArgumentException e) {
                     // too far
                     pw.printf("ERROR illegal footer position %d%n", footerPos);
@@ -312,15 +323,8 @@ public class MVStoreTool {
         } catch (IOException e) {
             pw.println("ERROR: " + e);
             e.printStackTrace(pw);
-        } finally {
-            if (file != null) {
-                try {
-                    file.close();
-                } catch (IOException e) {
-                    // ignore
-                }
-            }
         }
+        // ignore
         pw.flush();
     }
 
@@ -343,21 +347,20 @@ public class MVStoreTool {
             return "File not found: " + fileName;
         }
         long fileLength = FileUtils.size(fileName);
-        MVStore store = new MVStore.Builder().
-                fileName(fileName).
-                readOnly().open();
-        try {
-            MVMap<String, String> meta = store.getMetaMap();
+        try (MVStore store = new MVStore.Builder().
+                fileName(fileName).recoveryMode().
+                readOnly().open()) {
+            MVMap<String, String> layout = store.getLayoutMap();
             Map<String, Object> header = store.getStoreHeader();
             long fileCreated = DataUtils.readHexLong(header, "created", 0L);
-            TreeMap<Integer, Chunk> chunks = new TreeMap<Integer, Chunk>();
+            TreeMap<Integer, Chunk> chunks = new TreeMap<>();
             long chunkLength = 0;
             long maxLength = 0;
             long maxLengthLive = 0;
             long maxLengthNotEmpty = 0;
-            for (Entry<String, String> e : meta.entrySet()) {
+            for (Entry<String, String> e : layout.entrySet()) {
                 String k = e.getKey();
-                if (k.startsWith("chunk.")) {
+                if (k.startsWith(DataUtils.META_CHUNK)) {
                     Chunk c = Chunk.fromString(e.getValue());
                     chunks.put(c.id, c);
                     chunkLength += c.len * MVStore.BLOCK_SIZE;
@@ -400,8 +403,6 @@ public class MVStoreTool {
             pw.println("ERROR: " + e);
             e.printStackTrace(pw);
             return e.getMessage();
-        } finally {
-            store.close();
         }
         pw.flush();
         return null;
@@ -438,14 +439,25 @@ public class MVStoreTool {
         String tempName = fileName + Constants.SUFFIX_MV_STORE_TEMP_FILE;
         FileUtils.delete(tempName);
         compact(fileName, tempName, compress);
+        moveAtomicReplace(tempName, fileName);
+    }
+
+    /**
+     * Rename a file(s) of the named store, and try to atomically replace an
+     * existing file(s) of another store.
+     *
+     * @param sourceName the old fully qualified file name of the store
+     * @param destinationName the new fully qualified file name of the store
+     */
+    public static void moveAtomicReplace(String sourceName, String destinationName) {
         try {
-            FileUtils.moveAtomicReplace(tempName, fileName);
-        } catch (DbException e) {
-            String newName = fileName + Constants.SUFFIX_MV_STORE_NEW_FILE;
+            FileUtils.moveAtomicReplace(sourceName, destinationName);
+        } catch (MVStoreException e) {
+            String newName = destinationName + Constants.SUFFIX_MV_STORE_NEW_FILE;
             FileUtils.delete(newName);
-            FileUtils.move(tempName, newName);
-            FileUtils.delete(fileName);
-            FileUtils.move(newName, fileName);
+            FileUtils.move(sourceName, newName);
+            FileUtils.delete(destinationName);
+            FileUtils.move(newName, destinationName);
         }
     }
 
@@ -480,20 +492,23 @@ public class MVStoreTool {
      * @param compress whether to compress the data
      */
     public static void compact(String sourceFileName, String targetFileName, boolean compress) {
-        MVStore source = new MVStore.Builder().
-                fileName(sourceFileName).
-                readOnly().
-                open();
-        FileUtils.delete(targetFileName);
-        MVStore.Builder b = new MVStore.Builder().
+        try (MVStore source = new MVStore.Builder().
+                fileName(sourceFileName).readOnly().open()) {
+            // Bugfix - Add double "try-finally" statements to close source and target stores for
+            //releasing lock and file resources in these stores even if OOM occurs.
+            // Fix issues such as "Cannot delete file "/h2/data/test.mv.db.tempFile" [90025-197]"
+            //when client connects to this server and reopens this store database in this process.
+            // @since 2018-09-13 little-pan
+            FileUtils.delete(targetFileName);
+            MVStore.Builder b = new MVStore.Builder().
                 fileName(targetFileName);
-        if (compress) {
-            b.compress();
+            if (compress) {
+                b.compress();
+            }
+            try (MVStore target = b.open()) {
+                compact(source, target);
+            }
         }
-        MVStore target = b.open();
-        compact(source, target);
-        target.close();
-        source.close();
     }
 
     /**
@@ -503,30 +518,52 @@ public class MVStoreTool {
      * @param target the target store
      */
     public static void compact(MVStore source, MVStore target) {
-        MVMap<String, String> sourceMeta = source.getMetaMap();
-        MVMap<String, String> targetMeta = target.getMetaMap();
-        for (Entry<String, String> m : sourceMeta.entrySet()) {
-            String key = m.getKey();
-            if (key.startsWith("chunk.")) {
-                // ignore
-            } else if (key.startsWith("map.")) {
-                // ignore
-            } else if (key.startsWith("name.")) {
-                // ignore
-            } else if (key.startsWith("root.")) {
-                // ignore
-            } else {
-                targetMeta.put(key, m.getValue());
+        target.adoptMetaFrom(source);
+        int autoCommitDelay = target.getAutoCommitDelay();
+        boolean reuseSpace = target.getReuseSpace();
+        try {
+            target.setReuseSpace(false);  // disable unused chunks collection
+            target.setAutoCommitDelay(0); // disable autocommit
+            MVMap<String, String> sourceMeta = source.getMetaMap();
+            MVMap<String, String> targetMeta = target.getMetaMap();
+            for (Entry<String, String> m : sourceMeta.entrySet()) {
+                String key = m.getKey();
+                if (key.startsWith(DataUtils.META_MAP)) {
+                    // ignore
+                } else if (key.startsWith(DataUtils.META_NAME)) {
+                    // ignore
+                } else {
+                    targetMeta.put(key, m.getValue());
+                }
             }
-        }
-        for (String mapName : source.getMapNames()) {
-            MVMap.Builder<Object, Object> mp =
-                    new MVMap.Builder<Object, Object>().
-                    keyType(new GenericDataType()).
-                    valueType(new GenericDataType());
-            MVMap<Object, Object> sourceMap = source.openMap(mapName, mp);
-            MVMap<Object, Object> targetMap = target.openMap(mapName, mp);
-            targetMap.copyFrom(sourceMap);
+            // We are going to cheat a little bit in the copyFrom() by employing "incomplete" pages,
+            // which would be spared of saving, but save completed pages underneath,
+            // and those may appear as dead (non-reachable).
+            // That's why it is important to preserve all chunks
+            // created in the process, especially if retention time
+            // is set to a lower value, or even 0.
+            for (String mapName : source.getMapNames()) {
+                MVMap.Builder<Object, Object> mp = getGenericMapBuilder();
+                // This is a hack to preserve chunks occupancy rate accounting.
+                // It exposes design deficiency flaw in MVStore related to lack of
+                // map's type metadata.
+                // TODO: Introduce type metadata which will allow to open any store
+                // TODO: without prior knowledge of keys / values types and map implementation
+                // TODO: (MVMap vs MVRTreeMap, regular vs. singleWriter etc.)
+                if (mapName.startsWith(TransactionStore.UNDO_LOG_NAME_PREFIX)) {
+                    mp.singleWriter();
+                }
+                MVMap<Object, Object> sourceMap = source.openMap(mapName, mp);
+                MVMap<Object, Object> targetMap = target.openMap(mapName, mp);
+                targetMap.copyFrom(sourceMap);
+                targetMeta.put(MVMap.getMapKey(targetMap.getId()), sourceMeta.get(MVMap.getMapKey(sourceMap.getId())));
+            }
+            // this will end hacky mode of operation with incomplete pages
+            // end ensure that all pages are saved
+            target.commit();
+        } finally {
+            target.setAutoCommitDelay(autoCommitDelay);
+            target.setReuseSpace(reuseSpace);
         }
     }
 
@@ -540,7 +577,7 @@ public class MVStoreTool {
         long version = Long.MAX_VALUE;
         OutputStream ignore = new OutputStream() {
             @Override
-            public void write(int b) throws IOException {
+            public void write(int b) {
                 // ignore
             }
         };
@@ -568,7 +605,7 @@ public class MVStoreTool {
     }
 
     /**
-     * Roll back to a given revision into a a file called *.temp.
+     * Roll back to a given revision into a file called *.temp.
      *
      * @param fileName the file name
      * @param targetVersion the version to roll back to (Long.MAX_VALUE for the
@@ -608,10 +645,10 @@ public class MVStoreTool {
                     pos += blockSize;
                     continue;
                 }
-                Chunk c = null;
+                Chunk c;
                 try {
                     c = Chunk.readChunkHeader(block, pos);
-                } catch (IllegalStateException e) {
+                } catch (MVStoreException e) {
                     pos += blockSize;
                     continue;
                 }
@@ -664,38 +701,46 @@ public class MVStoreTool {
         return newestVersion;
     }
 
+    @SuppressWarnings({"rawtypes","unchecked"})
+    static MVMap.Builder<Object,Object> getGenericMapBuilder() {
+        return (MVMap.Builder)new MVMap.Builder<byte[],byte[]>().
+                keyType(GenericDataType.INSTANCE).
+                valueType(GenericDataType.INSTANCE);
+    }
+
     /**
      * A data type that can read any data that is persisted, and converts it to
      * a byte array.
      */
-    static class GenericDataType implements DataType {
+    private static class GenericDataType extends BasicDataType<byte[]> {
+        static GenericDataType INSTANCE = new GenericDataType();
+
+        private GenericDataType() {}
 
         @Override
-        public int compare(Object a, Object b) {
-            throw DataUtils.newUnsupportedOperationException("Can not compare");
+        public boolean isMemoryEstimationAllowed() {
+            return false;
         }
 
         @Override
-        public int getMemory(Object obj) {
-            return obj == null ? 0 : ((byte[]) obj).length * 8;
+        public int getMemory(byte[] obj) {
+            return obj == null ? 0 : obj.length * 8;
         }
 
         @Override
-        public void write(WriteBuffer buff, Object obj) {
+        public byte[][] createStorage(int size) {
+            return new byte[size][];
+        }
+
+        @Override
+        public void write(WriteBuffer buff, byte[] obj) {
             if (obj != null) {
-                buff.put((byte[]) obj);
+                buff.put(obj);
             }
         }
 
         @Override
-        public void write(WriteBuffer buff, Object[] obj, int len, boolean key) {
-            for (Object o : obj) {
-                write(buff, o);
-            }
-        }
-
-        @Override
-        public Object read(ByteBuffer buff) {
+        public byte[] read(ByteBuffer buff) {
             int len = buff.remaining();
             if (len == 0) {
                 return null;
@@ -704,15 +749,5 @@ public class MVStoreTool {
             buff.get(data);
             return data;
         }
-
-        @Override
-        public void read(ByteBuffer buff, Object[] obj, int len, boolean key) {
-            for (int i = 0; i < obj.length; i++) {
-                obj[i] = read(buff);
-            }
-        }
-
     }
-
-
 }
