@@ -69,7 +69,7 @@ public final class MVRTreeMap<V> extends MVMap<Spatial, V> {
         return new ContainsRTreeCursor<>(getRootPage(), x, keyType);
     }
 
-    private boolean contains(Page<Spatial,V> p, int index, Object key) {
+    private boolean contains(Page<Spatial,V> p, int index, Spatial key) {
         return keyType.contains(p.getKey(index), key);
     }
 
@@ -236,7 +236,7 @@ public final class MVRTreeMap<V> extends MVMap<Spatial, V> {
             // a new entry, we don't know where to add yet
             float min = Float.MAX_VALUE;
             for (int i = 0; i < p.getKeyCount(); i++) {
-                Object k = p.getKey(i);
+                Spatial k = p.getKey(i);
                 float areaIncrease = keyType.getAreaIncrease(k, key);
                 if (areaIncrease < min) {
                     index = i;
@@ -308,7 +308,7 @@ public final class MVRTreeMap<V> extends MVMap<Spatial, V> {
 
     private Page<Spatial,V> splitLinear(Page<Spatial,V> p) {
         int keyCount = p.getKeyCount();
-        ArrayList<Object> keys = new ArrayList<>(keyCount);
+        ArrayList<Spatial> keys = new ArrayList<>(keyCount);
         for (int i = 0; i < keyCount; i++) {
             keys.add(p.getKey(i));
         }
@@ -323,10 +323,10 @@ public final class MVRTreeMap<V> extends MVMap<Spatial, V> {
             extremes[1]--;
         }
         move(p, splitB, extremes[1]);
-        Object boundsA = keyType.createBoundingBox(splitA.getKey(0));
-        Object boundsB = keyType.createBoundingBox(splitB.getKey(0));
+        Spatial boundsA = keyType.createBoundingBox(splitA.getKey(0));
+        Spatial boundsB = keyType.createBoundingBox(splitB.getKey(0));
         while (p.getKeyCount() > 0) {
-            Object o = p.getKey(0);
+            Spatial o = p.getKey(0);
             float a = keyType.getAreaIncrease(boundsA, o);
             float b = keyType.getAreaIncrease(boundsB, o);
             if (a < b) {
@@ -350,12 +350,12 @@ public final class MVRTreeMap<V> extends MVMap<Spatial, V> {
         int ia = 0, ib = 0;
         int keyCount = p.getKeyCount();
         for (int a = 0; a < keyCount; a++) {
-            Object objA = p.getKey(a);
+            Spatial objA = p.getKey(a);
             for (int b = 0; b < keyCount; b++) {
                 if (a == b) {
                     continue;
                 }
-                Object objB = p.getKey(b);
+                Spatial objB = p.getKey(b);
                 float area = keyType.getCombinedArea(objA, objB);
                 if (area > largest) {
                     largest = area;
@@ -369,14 +369,14 @@ public final class MVRTreeMap<V> extends MVMap<Spatial, V> {
             ib--;
         }
         move(p, splitB, ib);
-        Object boundsA = keyType.createBoundingBox(splitA.getKey(0));
-        Object boundsB = keyType.createBoundingBox(splitB.getKey(0));
+        Spatial boundsA = keyType.createBoundingBox(splitA.getKey(0));
+        Spatial boundsB = keyType.createBoundingBox(splitB.getKey(0));
         while (p.getKeyCount() > 0) {
             float diff = 0, bestA = 0, bestB = 0;
             int best = 0;
             keyCount = p.getKeyCount();
             for (int i = 0; i < keyCount; i++) {
-                Object o = p.getKey(i);
+                Spatial o = p.getKey(i);
                 float incA = keyType.getAreaIncrease(boundsA, o);
                 float incB = keyType.getAreaIncrease(boundsB, o);
                 float d = Math.abs(incA - incB);
