@@ -1,11 +1,10 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.security;
 
-import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
 import org.h2.util.Bits;
 
@@ -47,10 +46,8 @@ public class XTEA implements BlockCipher {
 
     @Override
     public void encrypt(byte[] bytes, int off, int len) {
-        if (SysProperties.CHECK) {
-            if (len % ALIGN != 0) {
-                DbException.throwInternalError("unaligned len " + len);
-            }
+        if (len % ALIGN != 0) {
+            throw DbException.getInternalError("unaligned len " + len);
         }
         for (int i = off; i < off + len; i += 8) {
             encryptBlock(bytes, bytes, i);
@@ -59,10 +56,8 @@ public class XTEA implements BlockCipher {
 
     @Override
     public void decrypt(byte[] bytes, int off, int len) {
-        if (SysProperties.CHECK) {
-            if (len % ALIGN != 0) {
-                DbException.throwInternalError("unaligned len " + len);
-            }
+        if (len % ALIGN != 0) {
+            throw DbException.getInternalError("unaligned len " + len);
         }
         for (int i = off; i < off + len; i += 8) {
             decryptBlock(bytes, bytes, i);
