@@ -1,20 +1,26 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.server.web;
 
-import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.net.ServerSocket;
 import java.net.Socket;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.sql.Connection;
 import java.sql.SQLException;
-import java.text.SimpleDateFormat;
+import java.time.ZoneId;
+import java.time.ZonedDateTime;
+import java.time.format.DateTimeFormatter;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -23,18 +29,18 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Properties;
 import java.util.Set;
-import java.util.TimeZone;
 
 import org.h2.engine.Constants;
 import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
+import org.h2.security.SHA256;
 import org.h2.server.Service;
 import org.h2.server.ShutdownHandler;
 import org.h2.store.fs.FileUtils;
 import org.h2.util.JdbcUtils;
 import org.h2.util.MathUtils;
 import org.h2.util.NetUtils;
-import org.h2.util.New;
+import org.h2.util.NetworkConnectionInfo;
 import org.h2.util.SortedProperties;
 import org.h2.util.StringUtils;
 import org.h2.util.Tool;
@@ -52,6 +58,7 @@ public class WebServer implements Service {
         { "en", "English" },
         { "es", "Espa\u00f1ol" },
         { "fr", "Fran\u00e7ais" },
+        { "hi", "Hindi \u0939\u093f\u0902\u0926\u0940" },
         { "hu", "Magyar"},
         { "ko", "\ud55c\uad6d\uc5b4"},
         { "in", "Indonesia"},
@@ -76,6 +83,20 @@ public class WebServer implements Service {
     private static final String[] GENERIC = {
         "Generic JNDI Data Source|javax.naming.InitialContext|" +
                 "java:comp/env/jdbc/Test|sa",
+        "Generic Teradata|com.teradata.jdbc.TeraDriver|" +
+                "jdbc:teradata://whomooz/|",
+        "Generic Snowflake|com.snowflake.client.jdbc.SnowflakeDriver|" +
+                "jdbc:snowflake://accountName.snowflakecomputing.com|",
+        "Generic Redshift|com.amazon.redshift.jdbc42.Driver|" +
+                "jdbc:redshift://endpoint:5439/database|",
+        "Generic Impala|org.cloudera.impala.jdbc41.Driver|" +
+                "jdbc:impala://clustername:21050/default|",
+        "Generic Hive 2|org.apache.hive.jdbc.HiveDriver|" +
+                "jdbc:hive2://clustername:10000/default|",
+        "Generic Hive|org.apache.hadoop.hive.jdbc.HiveDriver|" +
+                "jdbc:hive://clustername:10000/default|",
+        "Generic Azure SQL|com.microsoft.sqlserver.jdbc.SQLServerDriver|" +
+                "jdbc:sqlserver://name.database.windows.net:1433|",
         "Generic Firebird Server|org.firebirdsql.jdbc.FBDriver|" +
                 "jdbc:firebirdsql:localhost:c:/temp/firebird/test|sysdba",
         "Generic SQLite|org.sqlite.JDBC|" +
@@ -90,13 +111,15 @@ public class WebServer implements Service {
                 "jdbc:sqlserver://localhost;DatabaseName=test|sa",
         "Generic PostgreSQL|org.postgresql.Driver|" +
                 "jdbc:postgresql:test|" ,
-        "Generic MySQL|com.mysql.jdbc.Driver|" +
+        "Generic MySQL|com.mysql.cj.jdbc.Driver|" +
                 "jdbc:mysql://localhost:3306/test|" ,
+        "Generic MariaDB|org.mariadb.jdbc.Driver|" +
+                "jdbc:mariadb://localhost:3306/test|" ,
         "Generic HSQLDB|org.hsqldb.jdbcDriver|" +
                 "jdbc:hsqldb:test;hsqldb.default_table_type=cached|sa" ,
-        "Generic Derby (Server)|org.apache.derby.jdbc.ClientDriver|" +
+        "Generic Derby (Server)|org.apache.derby.client.ClientAutoloadedDriver|" +
                 "jdbc:derby://localhost:1527/test;create=true|sa",
-        "Generic Derby (Embedded)|org.apache.derby.jdbc.EmbeddedDriver|" +
+        "Generic Derby (Embedded)|org.apache.derby.iapi.jdbc.AutoloadedDriver|" +
                 "jdbc:derby:test;create=true|sa",
         "Generic H2 (Server)|org.h2.Driver|" +
                 "jdbc:h2:tcp://localhost/~/test|sa",
@@ -136,21 +159,26 @@ public class WebServer implements Service {
     // private URLClassLoader urlClassLoader;
     private int port;
     private boolean allowOthers;
+    private String externalNames;
     private boolean isDaemon;
     private final Set<WebThread> running =
             Collections.synchronizedSet(new HashSet<WebThread>());
     private boolean ssl;
-    private final HashMap<String, ConnectionInfo> connInfoMap = New.hashMap();
+    private byte[] adminPassword;
+    private final HashMap<String, ConnectionInfo> connInfoMap = new HashMap<>();
 
     private long lastTimeoutCheck;
-    private final HashMap<String, WebSession> sessions = New.hashMap();
-    private final HashSet<String> languages = New.hashSet();
+    private final HashMap<String, WebSession> sessions = new HashMap<>();
+    private final HashSet<String> languages = new HashSet<>();
     private String startDateTime;
     private ServerSocket serverSocket;
+    private String host;
     private String url;
     private ShutdownHandler shutdownHandler;
     private Thread listenerThread;
-    private boolean ifExists;
+    private boolean ifExists = true;
+    private String key;
+    private boolean allowSecureCreation;
     private boolean trace;
     private TranslateThread translateThread;
     private boolean allowChunked = true;
@@ -163,6 +191,7 @@ public class WebServer implements Service {
      *
      * @param file the file name
      * @return the data
+     * @throws IOException on failure
      */
     byte[] getFile(String file) throws IOException {
         trace("getFile <" + file + ">");
@@ -175,22 +204,6 @@ public class WebServer implements Service {
         return data;
     }
 
-    /**
-     * Check if this is a simple name (only contains '.', '-', '_', letters, or
-     * digits).
-     *
-     * @param s the string
-     * @return true if it's a simple name
-     */
-    static boolean isSimpleName(String s) {
-        for (char c : s.toCharArray()) {
-            if (c != '.' && c != '_' && c != '-' && !Character.isLetterOrDigit(c)) {
-                return false;
-            }
-        }
-        return true;
-    }
-
     /**
      * Remove this web thread from the set of running threads.
      *
@@ -214,7 +227,7 @@ public class WebServer implements Service {
     WebSession getSession(String sessionId) {
         long now = System.currentTimeMillis();
         if (lastTimeoutCheck + SESSION_TIMEOUT < now) {
-            for (String id : New.arrayList(sessions.keySet())) {
+            for (String id : new ArrayList<>(sessions.keySet())) {
                 WebSession session = sessions.get(id);
                 if (session.lastAccess + SESSION_TIMEOUT < now) {
                     trace("timeout for " + id);
@@ -257,14 +270,42 @@ public class WebServer implements Service {
 
     String getStartDateTime() {
         if (startDateTime == null) {
-            SimpleDateFormat format = new SimpleDateFormat(
-                    "EEE, d MMM yyyy HH:mm:ss z", new Locale("en", ""));
-            format.setTimeZone(TimeZone.getTimeZone("GMT"));
-            startDateTime = format.format(System.currentTimeMillis());
+            startDateTime = DateTimeFormatter.ofPattern("EEE, d MMM yyyy HH:mm:ss z", Locale.ENGLISH)
+                .format(ZonedDateTime.now(ZoneId.of("UTC")));
         }
         return startDateTime;
     }
 
+    /**
+     * Returns the key for privileged connections.
+     *
+     * @return key key, or null
+     */
+    String getKey() {
+        return key;
+    }
+
+    /**
+     * Sets the key for privileged connections.
+     *
+     * @param key key, or null
+     */
+    public void setKey(String key) {
+        if (!allowOthers) {
+            this.key = key;
+        }
+    }
+
+    /**
+     * @param allowSecureCreation
+     *            whether creation of databases using the key should be allowed
+     */
+    public void setAllowSecureCreation(boolean allowSecureCreation) {
+        if (!allowOthers) {
+            this.allowSecureCreation = allowSecureCreation;
+        }
+    }
+
     @Override
     public void init(String... args) {
         // set the serverPropertiesDir, because it's used in loadProperties()
@@ -280,6 +321,8 @@ public class WebServer implements Service {
                 "webSSL", false);
         allowOthers = SortedProperties.getBooleanProperty(prop,
                 "webAllowOthers", false);
+        setExternalNames(SortedProperties.getStringProperty(prop, "webExternalNames", null));
+        setAdminPassword(SortedProperties.getStringProperty(prop, "webAdminPassword", null));
         commandHistoryString = prop.getProperty(COMMAND_HISTORY);
         for (int i = 0; args != null && i < args.length; i++) {
             String a = args[i];
@@ -289,6 +332,8 @@ public class WebServer implements Service {
                 ssl = true;
             } else if (Tool.isOption(a, "-webAllowOthers")) {
                 allowOthers = true;
+            }  else if (Tool.isOption(a, "-webExternalNames")) {
+                setExternalNames(args[++i]);
             } else if (Tool.isOption(a, "-webDaemon")) {
                 isDaemon = true;
             } else if (Tool.isOption(a, "-baseDir")) {
@@ -296,6 +341,10 @@ public class WebServer implements Service {
                 SysProperties.setBaseDir(baseDir);
             } else if (Tool.isOption(a, "-ifExists")) {
                 ifExists = true;
+            } else if (Tool.isOption(a, "-ifNotExists")) {
+                ifExists = false;
+            } else if (Tool.isOption(a, "-webAdminPassword")) {
+                setAdminPassword(args[++i]);
             } else if (Tool.isOption(a, "-properties")) {
                 // already set
                 i++;
@@ -319,6 +368,9 @@ public class WebServer implements Service {
         for (String[] lang : LANGUAGES) {
             languages.add(lang[0]);
         }
+        if (allowOthers) {
+            key = null;
+        }
         updateURL();
     }
 
@@ -328,10 +380,25 @@ public class WebServer implements Service {
         return url;
     }
 
+    /**
+     * @return host name
+     */
+    public String getHost() {
+        if (host == null) {
+            updateURL();
+        }
+        return host;
+    }
+
     private void updateURL() {
         try {
-            url = (ssl ? "https" : "http") + "://" +
-                    NetUtils.getLocalAddress() + ":" + port;
+            host = StringUtils.toLowerEnglish(NetUtils.getLocalAddress());
+            StringBuilder builder = new StringBuilder(ssl ? "https" : "http").append("://")
+                    .append(host).append(':').append(port);
+            if (key != null && serverSocket != null) {
+                builder.append("?key=").append(key);
+            }
+            url = builder.toString();
         } catch (NoClassDefFoundError e) {
             // Google App Engine does not allow java.net.InetAddress
         }
@@ -398,10 +465,10 @@ public class WebServer implements Service {
             }
         }
         // TODO server: using a boolean 'now' argument? a timeout?
-        for (WebSession session : New.arrayList(sessions.values())) {
+        for (WebSession session : new ArrayList<>(sessions.values())) {
             session.close();
         }
-        for (WebThread c : New.arrayList(running)) {
+        for (WebThread c : new ArrayList<>(running)) {
             try {
                 c.stopNow();
                 c.join(100);
@@ -455,8 +522,9 @@ public class WebServer implements Service {
         try {
             trace("translation: "+language);
             byte[] trans = getFile("_text_"+language+".prop");
-            trace("  "+new String(trans));
-            text = SortedProperties.fromLines(new String(trans, Constants.UTF8));
+            String s = new String(trans, StandardCharsets.UTF_8);
+            trace("  " + s);
+            text = SortedProperties.fromLines(s);
             // remove starting # (if not translated yet)
             for (Entry<Object, Object> entry : text.entrySet()) {
                 String value = (String) entry.getValue();
@@ -467,11 +535,11 @@ public class WebServer implements Service {
         } catch (IOException e) {
             DbException.traceThrowable(e);
         }
-        session.put("text", new HashMap<Object, Object>(text));
+        session.put("text", new HashMap<>(text));
     }
 
     ArrayList<HashMap<String, Object>> getSessions() {
-        ArrayList<HashMap<String, Object>> list = New.arrayList();
+        ArrayList<HashMap<String, Object>> list = new ArrayList<>(sessions.size());
         for (WebSession s : sessions.values()) {
             list.add(s.getInfo());
         }
@@ -489,6 +557,9 @@ public class WebServer implements Service {
     }
 
     void setAllowOthers(boolean b) {
+        if (b) {
+            key = null;
+        }
         allowOthers = b;
     }
 
@@ -497,6 +568,14 @@ public class WebServer implements Service {
         return allowOthers;
     }
 
+    void setExternalNames(String externalNames) {
+        this.externalNames = externalNames != null ? StringUtils.toLowerEnglish(externalNames) : null;
+    }
+
+    String getExternalNames() {
+        return externalNames;
+    }
+
     void setSSL(boolean b) {
         ssl = b;
     }
@@ -529,7 +608,7 @@ public class WebServer implements Service {
     }
 
     public ArrayList<String> getCommandHistoryList() {
-        ArrayList<String> result = New.arrayList();
+        ArrayList<String> result = new ArrayList<>();
         if (commandHistoryString == null) {
             return result;
         }
@@ -636,7 +715,7 @@ public class WebServer implements Service {
      * @return the list
      */
     synchronized ArrayList<ConnectionInfo> getSettings() {
-        ArrayList<ConnectionInfo> settings = New.arrayList();
+        ArrayList<ConnectionInfo> settings = new ArrayList<>();
         if (connInfoMap.size() == 0) {
             Properties prop = loadProperties();
             if (prop.size() == 0) {
@@ -647,7 +726,7 @@ public class WebServer implements Service {
                 }
             } else {
                 for (int i = 0;; i++) {
-                    String data = prop.getProperty(String.valueOf(i));
+                    String data = prop.getProperty(Integer.toString(i));
                     if (data == null) {
                         break;
                     }
@@ -674,14 +753,17 @@ public class WebServer implements Service {
                 Properties old = loadProperties();
                 prop = new SortedProperties();
                 prop.setProperty("webPort",
-                        "" + SortedProperties.getIntProperty(old,
-                        "webPort", port));
+                        Integer.toString(SortedProperties.getIntProperty(old, "webPort", port)));
                 prop.setProperty("webAllowOthers",
-                        "" + SortedProperties.getBooleanProperty(old,
-                        "webAllowOthers", allowOthers));
+                        Boolean.toString(SortedProperties.getBooleanProperty(old, "webAllowOthers", allowOthers)));
+                if (externalNames != null) {
+                    prop.setProperty("webExternalNames", externalNames);
+                }
                 prop.setProperty("webSSL",
-                        "" + SortedProperties.getBooleanProperty(old,
-                        "webSSL", ssl));
+                        Boolean.toString(SortedProperties.getBooleanProperty(old, "webSSL", ssl)));
+                if (adminPassword != null) {
+                    prop.setProperty("webAdminPassword", StringUtils.convertBytesToHex(adminPassword));
+                }
                 if (commandHistoryString != null) {
                     prop.setProperty(COMMAND_HISTORY, commandHistoryString);
                 }
@@ -691,7 +773,7 @@ public class WebServer implements Service {
             for (int i = 0; i < len; i++) {
                 ConnectionInfo info = settings.get(i);
                 if (info != null) {
-                    prop.setProperty(String.valueOf(len - i - 1), info.getString());
+                    prop.setProperty(Integer.toString(len - i - 1), info.getString());
                 }
             }
             if (!"null".equals(serverPropertiesDir)) {
@@ -712,36 +794,19 @@ public class WebServer implements Service {
      * @param databaseUrl the database URL
      * @param user the user name
      * @param password the password
+     * @param userKey the key of privileged user
+     * @param networkConnectionInfo the network connection information
      * @return the database connection
+     * @throws SQLException on failure
      */
     Connection getConnection(String driver, String databaseUrl, String user,
-            String password) throws SQLException {
+            String password, String userKey, NetworkConnectionInfo networkConnectionInfo) throws SQLException {
         driver = driver.trim();
         databaseUrl = databaseUrl.trim();
-        org.h2.Driver.load();
-        Properties p = new Properties();
-        p.setProperty("user", user.trim());
         // do not trim the password, otherwise an
         // encrypted H2 database with empty user password doesn't work
-        p.setProperty("password", password);
-        if (databaseUrl.startsWith("jdbc:h2:")) {
-            if (ifExists) {
-                databaseUrl += ";IFEXISTS=TRUE";
-            }
-            // PostgreSQL would throw a NullPointerException
-            // if it is loaded before the H2 driver
-            // because it can't deal with non-String objects in the connection
-            // Properties
-            return org.h2.Driver.load().connect(databaseUrl, p);
-        }
-//            try {
-//                Driver dr = (Driver) urlClassLoader.
-//                        loadClass(driver).newInstance();
-//                return dr.connect(url, p);
-//            } catch(ClassNotFoundException e2) {
-//                throw e2;
-//            }
-        return JdbcUtils.getConnection(driver, databaseUrl, p);
+        return JdbcUtils.getConnection(driver, databaseUrl, user.trim(), password, networkConnectionInfo,
+                ifExists && (!allowSecureCreation || key == null || !key.equals(userKey)));
     }
 
     /**
@@ -762,6 +827,7 @@ public class WebServer implements Service {
      *
      * @param conn the connection
      * @return the URL of the web site to access this connection
+     * @throws SQLException on failure
      */
     public String addSession(Connection conn) throws SQLException {
         WebSession session = createNewSession("local");
@@ -778,7 +844,7 @@ public class WebServer implements Service {
      */
     private class TranslateThread extends Thread {
 
-        private final File file = new File("translation.properties");
+        private final Path file = Paths.get("translation.properties");
         private final Map<Object, Object> translation;
         private volatile boolean stopNow;
 
@@ -787,7 +853,7 @@ public class WebServer implements Service {
         }
 
         public String getFileName() {
-            return file.getAbsolutePath();
+            return file.toAbsolutePath().toString();
         }
 
         public void stopNow() {
@@ -804,12 +870,12 @@ public class WebServer implements Service {
             while (!stopNow) {
                 try {
                     SortedProperties sp = new SortedProperties();
-                    if (file.exists()) {
-                        InputStream in = FileUtils.newInputStream(file.getName());
+                    if (Files.exists(file)) {
+                        InputStream in = Files.newInputStream(file);
                         sp.load(in);
                         translation.putAll(sp);
                     } else {
-                        OutputStream out = FileUtils.newOutputStream(file.getName(), false);
+                        OutputStream out = Files.newOutputStream(file);
                         sp.putAll(translation);
                         sp.store(out, "Translation");
                     }
@@ -851,4 +917,42 @@ public class WebServer implements Service {
         return allowChunked;
     }
 
+    byte[] getAdminPassword() {
+        return adminPassword;
+    }
+
+    void setAdminPassword(String password) {
+        if (password == null || password.isEmpty()) {
+            adminPassword = null;
+            return;
+        }
+        if (password.length() == 128) {
+            try {
+                adminPassword = StringUtils.convertHexToBytes(password);
+                return;
+            } catch (Exception ex) {}
+        }
+        byte[] salt = MathUtils.secureRandomBytes(32);
+        byte[] hash = SHA256.getHashWithSalt(password.getBytes(StandardCharsets.UTF_8), salt);
+        byte[] total = Arrays.copyOf(salt, 64);
+        System.arraycopy(hash, 0, total, 32, 32);
+        adminPassword = total;
+    }
+
+    /**
+     * Check the admin password.
+     *
+     * @param password the password to test
+     * @return true if admin password not configure, or admin password correct
+     */
+    boolean checkAdminPassword(String password) {
+        if (adminPassword == null) {
+            return false;
+        }
+        byte[] salt = Arrays.copyOf(adminPassword, 32);
+        byte[] hash = new byte[32];
+        System.arraycopy(adminPassword, 32, hash, 0, 32);
+        return Utils.compareSecure(hash, SHA256.getHashWithSalt(password.getBytes(StandardCharsets.UTF_8), salt));
+    }
+
 }
