@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.server.web;
@@ -22,6 +22,7 @@ import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
 import org.h2.util.IOUtils;
 import org.h2.util.NetUtils;
+import org.h2.util.NetworkConnectionInfo;
 import org.h2.util.StringUtils;
 import org.h2.util.Utils;
 
@@ -31,10 +32,16 @@ import org.h2.util.Utils;
  */
 class WebThread extends WebApp implements Runnable {
 
+    private static final byte[] RN = { '\r', '\n' };
+
+    private static final byte[] RNRN = { '\r', '\n', '\r', '\n' };
+
     protected OutputStream output;
     protected final Socket socket;
     private final Thread thread;
     private InputStream input;
+    private String host;
+    private int dataLength;
     private String ifModifiedSince;
 
     WebThread(Socket socket, WebServer server) {
@@ -54,6 +61,7 @@ class WebThread extends WebApp implements Runnable {
      * Wait until the thread is stopped.
      *
      * @param millis the maximum number of milliseconds to wait
+     * @throws InterruptedException if interrupted
      */
     void join(int millis) throws InterruptedException {
         thread.join(millis);
@@ -78,6 +86,9 @@ class WebThread extends WebApp implements Runnable {
         if (requestedFile.length() == 0) {
             return "index.do";
         }
+        if (requestedFile.charAt(0) == '?') {
+            return "index.do" + requestedFile;
+        }
         return requestedFile;
     }
 
@@ -107,103 +118,159 @@ class WebThread extends WebApp implements Runnable {
 
     @SuppressWarnings("unchecked")
     private boolean process() throws IOException {
-        boolean keepAlive = false;
         String head = readHeaderLine();
-        if (head.startsWith("GET ") || head.startsWith("POST ")) {
-            int begin = head.indexOf('/'), end = head.lastIndexOf(' ');
-            String file;
-            if (begin < 0 || end < begin) {
-                file = "";
-            } else {
-                file = head.substring(begin + 1, end).trim();
+        boolean get = head.startsWith("GET ");
+        if ((!get && !head.startsWith("POST ")) || !head.endsWith(" HTTP/1.1")) {
+            writeSimple("HTTP/1.1 400 Bad Request", "Bad request");
+            return false;
+        }
+        String file = StringUtils.trimSubstring(head, get ? 4 : 5, head.length() - 9);
+        if (file.isEmpty() || file.charAt(0) != '/') {
+            writeSimple("HTTP/1.1 400 Bad Request", "Bad request");
+            return false;
+        }
+        attributes = new Properties();
+        boolean keepAlive = parseHeader();
+        if (!checkHost(host)) {
+            return false;
+        }
+        file = file.substring(1);
+        trace(head + ": " + file);
+        file = getAllowedFile(file);
+        int paramIndex = file.indexOf('?');
+        session = null;
+        String key = null;
+        if (paramIndex >= 0) {
+            String attrib = file.substring(paramIndex + 1);
+            parseAttributes(attrib);
+            String sessionId = attributes.getProperty("jsessionid");
+            key = attributes.getProperty("key");
+            file = file.substring(0, paramIndex);
+            session = server.getSession(sessionId);
+        }
+        parseBodyAttributes();
+        file = processRequest(file,
+                new NetworkConnectionInfo(
+                        NetUtils.ipToShortForm(new StringBuilder(server.getSSL() ? "https://" : "http://"),
+                                socket.getLocalAddress().getAddress(), true) //
+                                .append(':').append(socket.getLocalPort()).toString(), //
+                        socket.getInetAddress().getAddress(), socket.getPort(), null));
+        if (file.length() == 0) {
+            // asynchronous request
+            return true;
+        }
+        String message;
+        if (cache && ifModifiedSince != null && ifModifiedSince.equals(server.getStartDateTime())) {
+            writeSimple("HTTP/1.1 304 Not Modified", (byte[]) null);
+            return keepAlive;
+        }
+        byte[] bytes = server.getFile(file);
+        if (bytes == null) {
+            writeSimple("HTTP/1.1 404 Not Found", "File not found: " + file);
+            return keepAlive;
+        }
+        if (session != null && file.endsWith(".jsp")) {
+            if (key != null) {
+                session.put("key", key);
             }
-            trace(head + ": " + file);
-            file = getAllowedFile(file);
-            attributes = new Properties();
-            int paramIndex = file.indexOf('?');
-            session = null;
-            if (paramIndex >= 0) {
-                String attrib = file.substring(paramIndex + 1);
-                parseAttributes(attrib);
-                String sessionId = attributes.getProperty("jsessionid");
-                file = file.substring(0, paramIndex);
-                session = server.getSession(sessionId);
-            }
-            keepAlive = parseHeader();
-            String hostAddr = socket.getInetAddress().getHostAddress();
-            file = processRequest(file, hostAddr);
-            if (file.length() == 0) {
-                // asynchronous request
-                return true;
-            }
-            String message;
-            byte[] bytes;
-            if (cache && ifModifiedSince != null &&
-                    ifModifiedSince.equals(server.getStartDateTime())) {
-                bytes = null;
-                message = "HTTP/1.1 304 Not Modified\r\n";
-            } else {
-                bytes = server.getFile(file);
-                if (bytes == null) {
-                    message = "HTTP/1.1 404 Not Found\r\n";
-                    bytes = ("File not found: " + file).getBytes(StandardCharsets.UTF_8);
-                    message += "Content-Length: " + bytes.length + "\r\n";
-                } else {
-                    if (session != null && file.endsWith(".jsp")) {
-                        String page = new String(bytes, StandardCharsets.UTF_8);
-                        if (SysProperties.CONSOLE_STREAM) {
-                            Iterator<String> it = (Iterator<String>) session.map.remove("chunks");
-                            if (it != null) {
-                                message = "HTTP/1.1 200 OK\r\n";
-                                message += "Content-Type: " + mimeType + "\r\n";
-                                message += "Cache-Control: no-cache\r\n";
-                                message += "Transfer-Encoding: chunked\r\n";
-                                message += "\r\n";
-                                trace(message);
-                                output.write(message.getBytes());
-                                while (it.hasNext()) {
-                                    String s = it.next();
-                                    s = PageParser.parse(s, session.map);
-                                    bytes = s.getBytes(StandardCharsets.UTF_8);
-                                    if (bytes.length == 0) {
-                                        continue;
-                                    }
-                                    output.write(Integer.toHexString(bytes.length).getBytes());
-                                    output.write("\r\n".getBytes());
-                                    output.write(bytes);
-                                    output.write("\r\n".getBytes());
-                                    output.flush();
-                                }
-                                output.write("0\r\n\r\n".getBytes());
-                                output.flush();
-                                return keepAlive;
-                            }
-                        }
-                        page = PageParser.parse(page, session.map);
-                        bytes = page.getBytes(StandardCharsets.UTF_8);
-                    }
+            String page = new String(bytes, StandardCharsets.UTF_8);
+            if (SysProperties.CONSOLE_STREAM) {
+                Iterator<String> it = (Iterator<String>) session.map.remove("chunks");
+                if (it != null) {
                     message = "HTTP/1.1 200 OK\r\n";
                     message += "Content-Type: " + mimeType + "\r\n";
-                    if (!cache) {
-                        message += "Cache-Control: no-cache\r\n";
-                    } else {
-                        message += "Cache-Control: max-age=10\r\n";
-                        message += "Last-Modified: " + server.getStartDateTime() + "\r\n";
+                    message += "Cache-Control: no-cache\r\n";
+                    message += "Transfer-Encoding: chunked\r\n";
+                    message += "\r\n";
+                    trace(message);
+                    output.write(message.getBytes(StandardCharsets.ISO_8859_1));
+                    while (it.hasNext()) {
+                        String s = it.next();
+                        s = PageParser.parse(s, session.map);
+                        bytes = s.getBytes(StandardCharsets.UTF_8);
+                        if (bytes.length == 0) {
+                            continue;
+                        }
+                        output.write(Integer.toHexString(bytes.length).getBytes(StandardCharsets.ISO_8859_1));
+                        output.write(RN);
+                        output.write(bytes);
+                        output.write(RN);
+                        output.flush();
                     }
-                    message += "Content-Length: " + bytes.length + "\r\n";
+                    output.write('0');
+                    output.write(RNRN);
+                    output.flush();
+                    return keepAlive;
                 }
             }
-            message += "\r\n";
-            trace(message);
-            output.write(message.getBytes());
-            if (bytes != null) {
-                output.write(bytes);
-            }
-            output.flush();
+            page = PageParser.parse(page, session.map);
+            bytes = page.getBytes(StandardCharsets.UTF_8);
         }
+        message = "HTTP/1.1 200 OK\r\n";
+        message += "Content-Type: " + mimeType + "\r\n";
+        if (!cache) {
+            message += "Cache-Control: no-cache\r\n";
+        } else {
+            message += "Cache-Control: max-age=10\r\n";
+            message += "Last-Modified: " + server.getStartDateTime() + "\r\n";
+        }
+        message += "Content-Length: " + bytes.length + "\r\n";
+        message += "\r\n";
+        trace(message);
+        output.write(message.getBytes(StandardCharsets.ISO_8859_1));
+        output.write(bytes);
+        output.flush();
         return keepAlive;
     }
 
+    private void writeSimple(String status, String text) throws IOException {
+        writeSimple(status, text != null ? text.getBytes(StandardCharsets.UTF_8) : null);
+    }
+
+    private void writeSimple(String status, byte[] bytes) throws IOException {
+        trace(status);
+        output.write(status.getBytes(StandardCharsets.ISO_8859_1));
+        if (bytes != null) {
+            output.write(RN);
+            String contentLength = "Content-Length: " + bytes.length;
+            trace(contentLength);
+            output.write(contentLength.getBytes(StandardCharsets.ISO_8859_1));
+            output.write(RNRN);
+            output.write(bytes);
+        } else {
+            output.write(RNRN);
+        }
+        output.flush();
+    }
+
+    private boolean checkHost(String host) throws IOException {
+        if (host == null) {
+            writeSimple("HTTP/1.1 400 Bad Request", "Bad request");
+            return false;
+        }
+        int index = host.indexOf(':');
+        if (index >= 0) {
+            host = host.substring(0, index);
+        }
+        if (host.isEmpty()) {
+            return false;
+        }
+        host = StringUtils.toLowerEnglish(host);
+        if (host.equals(server.getHost()) || host.equals("localhost") || host.equals("127.0.0.1")) {
+            return true;
+        }
+        String externalNames = server.getExternalNames();
+        if (externalNames != null && !externalNames.isEmpty()) {
+            for (String s : externalNames.split(",")) {
+                if (host.equals(s.trim())) {
+                    return true;
+                }
+            }
+        }
+        writeSimple("HTTP/1.1 404 Not Found", "Host " + host + " not found");
+        return false;
+    }
+
     private String readHeaderLine() throws IOException {
         StringBuilder buff = new StringBuilder();
         while (true) {
@@ -222,6 +289,17 @@ class WebThread extends WebApp implements Runnable {
         }
     }
 
+    private void parseBodyAttributes() throws IOException {
+        if (dataLength > 0) {
+            byte[] bytes = Utils.newBytes(dataLength);
+            for (int pos = 0; pos < dataLength;) {
+                pos += input.read(bytes, pos, dataLength - pos);
+            }
+            String s = new String(bytes, StandardCharsets.UTF_8);
+            parseAttributes(s);
+        }
+    }
+
     private void parseAttributes(String s) {
         trace("data=" + s);
         while (s != null) {
@@ -250,16 +328,15 @@ class WebThread extends WebApp implements Runnable {
         boolean keepAlive = false;
         trace("parseHeader");
         int len = 0;
+        host = null;
         ifModifiedSince = null;
         boolean multipart = false;
-        while (true) {
-            String line = readHeaderLine();
-            if (line == null) {
-                break;
-            }
+        for (String line; (line = readHeaderLine()) != null;) {
             trace(" " + line);
             String lower = StringUtils.toLowerEnglish(line);
-            if (lower.startsWith("if-modified-since")) {
+            if (lower.startsWith("host")) {
+                host = getHeaderLineValue(line);
+            } else if (lower.startsWith("if-modified-since")) {
                 ifModifiedSince = getHeaderLineValue(line);
             } else if (lower.startsWith("connection")) {
                 String conn = getHeaderLineValue(line);
@@ -278,7 +355,7 @@ class WebThread extends WebApp implements Runnable {
                 boolean isWebKit = lower.contains("webkit/");
                 if (isWebKit && session != null) {
                     // workaround for what seems to be a WebKit bug:
-                    // http://code.google.com/p/chromium/issues/detail?id=6402
+                    // https://bugs.chromium.org/p/chromium/issues/detail?id=6402
                     session.put("frame-border", "1");
                     session.put("frameset-border", "2");
                 }
@@ -310,25 +387,21 @@ class WebThread extends WebApp implements Runnable {
                         }
                     }
                 }
-            } else if (line.trim().length() == 0) {
+            } else if (StringUtils.isWhitespaceOrEmpty(line)) {
                 break;
             }
         }
+        dataLength = 0;
         if (multipart) {
             // not supported
-        } else if (session != null && len > 0) {
-            byte[] bytes = Utils.newBytes(len);
-            for (int pos = 0; pos < len;) {
-                pos += input.read(bytes, pos, len - pos);
-            }
-            String s = new String(bytes);
-            parseAttributes(s);
+        } else if (len > 0) {
+            dataLength = len;
         }
         return keepAlive;
     }
 
     private static String getHeaderLineValue(String line) {
-        return line.substring(line.indexOf(':') + 1).trim();
+        return StringUtils.trimSubstring(line, line.indexOf(':') + 1);
     }
 
     @Override
