@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  *  * Initial Developer: H2 Group
  */
 
@@ -142,7 +142,7 @@ function editKeyDown(row, object, event) {
 
 function getInnerText(el) {
     if (typeof el == "string") return el;
-    if (typeof el == "undefined") { return el };
+    if (typeof el == "undefined") return el;
     if (el.innerText) {
         // not needed but it is faster
         return el.innerText;
@@ -163,6 +163,10 @@ function getInnerText(el) {
     return str;
 }
 
+function isNullCell(td) {
+    return td.childNodes.length == 1 && (td.childNodes[0].nodeName == "I");
+}
+
 function resortTable(link) {
     // get the span
     var span;
@@ -171,30 +175,42 @@ function resortTable(link) {
             span = link.childNodes[ci];
         }
     }
-    var spantext = getInnerText(span);
     var td = link.parentNode;
     var column = td.cellIndex;
     var table = getParent(td,'TABLE');
-
-    if (table.rows.length <= 1) return;
+    var rows = table.rows;
+    if (rows.length <= 1) return;
 
     // detect sort type
-    var sortNumeric = false;
-    var x = getInnerText(table.rows[1].cells[column]);
-    if (x.match(/^[\d\.]+$/)) {
-        sortNumeric = true;
+    var sortNumeric = true;
+    for (i = 1; i < rows.length; i++) {
+        var td = rows[i].cells[column];
+        if (!isNullCell(td)) {
+            var x = getInnerText(td);
+            // H2 does not return numeric values with leading +, but may return
+            // values in scientific notation
+            if (!x.match(/^\-?\d*\.?\d+(?:[Ee][\+\-]?\d+)?$/)) {
+                sortNumeric = false;
+                break;
+            }
+        }
     }
     var newRows = new Array();
-    var rows = table.rows;
     for (i=1; i<rows.length; i++) {
         var o = new Object();
         o.data = rows[i];
         o.id = i;
-        if (sortNumeric) {
-            o.sort = parseFloat(getInnerText(o.data.cells[column]));
-            if (isNaN(o.sort)) o.sort = 0;
-        } else {
-            o.sort = getInnerText(o.data.cells[column]);
+        var td = o.data.cells[column];
+        var n = isNullCell(td);
+        o.isNull = n;
+        if (!n) {
+            var txt = getInnerText(td);
+            if (sortNumeric) {
+                o.sort = parseFloat(txt);
+                if (isNaN(o.sort)) o.sort = 0;
+            } else {
+                o.sort = txt;
+            }
         }
         newRows[i-1] = o;
     }
@@ -240,6 +256,10 @@ function getParent(el, pTagName) {
 }
 
 function sortCallback(ra, rb) {
+    if (ra.isNull) {
+        return rb.isNull ? (ra.id - rb.id) : -1;
+    } else if (rb.IsNull) {
+        return 1;
+    }
     return (ra.sort==rb.sort) ? (ra.id-rb.id) : (ra.sort<rb.sort ? -1 : 1);
 }
-
