@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.store;
@@ -13,6 +13,9 @@ import java.net.InetAddress;
 import java.net.ServerSocket;
 import java.net.Socket;
 import java.net.UnknownHostException;
+import java.nio.ByteBuffer;
+import java.nio.channels.FileChannel;
+import java.nio.file.Paths;
 import java.util.Properties;
 import org.h2.Driver;
 import org.h2.api.ErrorCode;
@@ -38,7 +41,6 @@ public class FileLock implements Runnable {
     private static final String MAGIC = "FileLock";
     private static final String FILE = "file";
     private static final String SOCKET = "socket";
-    private static final String SERIALIZED = "serialized";
     private static final int RANDOM_BYTES = 16;
     private static final int SLEEP_GAP = 25;
     private static final int TIME_GRANULARITY = 2000;
@@ -73,7 +75,7 @@ public class FileLock implements Runnable {
      */
     private long lastWrite;
 
-    private String method, ipAddress;
+    private String method;
     private Properties properties;
     private String uniqueId;
     private Thread watchdog;
@@ -101,7 +103,7 @@ public class FileLock implements Runnable {
     public synchronized void lock(FileLockMethod fileLockMethod) {
         checkServer();
         if (locked) {
-            DbException.throwInternalError("already locked");
+            throw DbException.getInternalError("already locked");
         }
         switch (fileLockMethod) {
         case FILE:
@@ -110,9 +112,6 @@ public class FileLock implements Runnable {
         case SOCKET:
             lockSocket();
             break;
-        case SERIALIZED:
-            lockSerialized();
-            break;
         case FS:
         case NO:
             break;
@@ -187,7 +186,7 @@ public class FileLock implements Runnable {
             try (OutputStream out = FileUtils.newOutputStream(fileName, false)) {
                 properties.store(out, MAGIC);
             }
-            lastWrite = FileUtils.lastModified(fileName);
+            lastWrite = aggressiveLastModified(fileName);
             if (trace.isDebugEnabled()) {
                 trace.debug("save " + properties);
             }
@@ -197,6 +196,28 @@ public class FileLock implements Runnable {
         }
     }
 
+    /**
+     * Aggressively read last modified time, to work-around remote filesystems.
+     *
+     * @param fileName file name to check
+     * @return last modified date/time in milliseconds UTC
+     */
+    private static long aggressiveLastModified(String fileName) {
+        /*
+         * Some remote filesystem, e.g. SMB on Windows, can cache metadata for
+         * 5-10 seconds. To work around that, do a one-byte read from the
+         * underlying file, which has the effect of invalidating the metadata
+         * cache.
+         */
+        try {
+            try (FileChannel f = FileChannel.open(Paths.get(fileName), FileUtils.RWS, FileUtils.NO_ATTRIBUTES);) {
+                ByteBuffer b = ByteBuffer.wrap(new byte[1]);
+                f.read(b);
+            }
+        } catch (IOException ignoreEx) {}
+        return FileUtils.lastModified(fileName);
+    }
+
     private void checkServer() {
         Properties prop = load();
         String server = prop.getProperty("server");
@@ -257,7 +278,7 @@ public class FileLock implements Runnable {
 
     private void waitUntilOld() {
         for (int i = 0; i < 2 * TIME_GRANULARITY / SLEEP_GAP; i++) {
-            long last = FileUtils.lastModified(fileName);
+            long last = aggressiveLastModified(fileName);
             long dist = System.currentTimeMillis() - last;
             if (dist < -TIME_GRANULARITY) {
                 // lock file modified in the future -
@@ -287,26 +308,6 @@ public class FileLock implements Runnable {
         properties.setProperty("id", uniqueId);
     }
 
-    private void lockSerialized() {
-        method = SERIALIZED;
-        FileUtils.createDirectories(FileUtils.getParent(fileName));
-        if (FileUtils.createFile(fileName)) {
-            properties = new SortedProperties();
-            properties.setProperty("method", String.valueOf(method));
-            setUniqueId();
-            save();
-        } else {
-            while (true) {
-                try {
-                    properties = load();
-                } catch (DbException e) {
-                    // ignore
-                }
-                return;
-            }
-        }
-    }
-
     private void lockFile() {
         method = FILE;
         properties = new SortedProperties();
@@ -350,11 +351,11 @@ public class FileLock implements Runnable {
         setUniqueId();
         // if this returns 127.0.0.1,
         // the computer is probably not networked
-        ipAddress = NetUtils.getLocalAddress();
+        String ipAddress = NetUtils.getLocalAddress();
         FileUtils.createDirectories(FileUtils.getParent(fileName));
         if (!FileUtils.createFile(fileName)) {
             waitUntilOld();
-            long read = FileUtils.lastModified(fileName);
+            long read = aggressiveLastModified(fileName);
             Properties p2 = load();
             String m2 = p2.getProperty("method", SOCKET);
             if (m2.equals(FILE)) {
@@ -388,7 +389,7 @@ public class FileLock implements Runnable {
                     throw getExceptionFatal("IOException", null);
                 }
             }
-            if (read != FileUtils.lastModified(fileName)) {
+            if (read != aggressiveLastModified(fileName)) {
                 throw getExceptionFatal("Concurrent update", null);
             }
             FileUtils.delete(fileName);
@@ -401,7 +402,7 @@ public class FileLock implements Runnable {
             serverSocket = NetUtils.createServerSocket(0, false);
             int port = serverSocket.getLocalPort();
             properties.setProperty("ipAddress", ipAddress);
-            properties.setProperty("port", String.valueOf(port));
+            properties.setProperty("port", Integer.toString(port));
         } catch (Exception e) {
             trace.debug(e, "lock");
             serverSocket = null;
@@ -461,13 +462,10 @@ public class FileLock implements Runnable {
             return FileLockMethod.NO;
         } else if (method.equalsIgnoreCase("SOCKET")) {
             return FileLockMethod.SOCKET;
-        } else if (method.equalsIgnoreCase("SERIALIZED")) {
-            return FileLockMethod.SERIALIZED;
         } else if (method.equalsIgnoreCase("FS")) {
             return FileLockMethod.FS;
         } else {
-            throw DbException.get(
-                    ErrorCode.UNSUPPORTED_LOCK_METHOD_1, method);
+            throw DbException.get(ErrorCode.UNSUPPORTED_LOCK_METHOD_1, method);
         }
     }
 
@@ -482,15 +480,11 @@ public class FileLock implements Runnable {
                 // trace.debug("watchdog check");
                 try {
                     if (!FileUtils.exists(fileName) ||
-                            FileUtils.lastModified(fileName) != lastWrite) {
+                            aggressiveLastModified(fileName) != lastWrite) {
                         save();
                     }
                     Thread.sleep(sleep);
-                } catch (OutOfMemoryError e) {
-                    // ignore
-                } catch (InterruptedException e) {
-                    // ignore
-                } catch (NullPointerException e) {
+                } catch (OutOfMemoryError | NullPointerException | InterruptedException e) {
                     // ignore
                 } catch (Exception e) {
                     trace.debug(e, "watchdog");
