@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.table;
@@ -8,13 +8,12 @@ package org.h2.table;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
-import java.util.HashSet;
-import org.h2.engine.Session;
+
+import org.h2.command.query.AllColumnsForPlan;
+import org.h2.engine.SessionLocal;
 import org.h2.expression.Expression;
 import org.h2.expression.ExpressionVisitor;
 import org.h2.message.Trace;
-import org.h2.table.TableFilter.TableFilterVisitor;
-import org.h2.util.New;
 
 /**
  * A possible query execution plan. The time required to execute a query depends
@@ -37,20 +36,17 @@ public class Plan {
     public Plan(TableFilter[] filters, int count, Expression condition) {
         this.filters = new TableFilter[count];
         System.arraycopy(filters, 0, this.filters, 0, count);
-        final ArrayList<Expression> allCond = New.arrayList();
-        final ArrayList<TableFilter> all = New.arrayList();
+        final ArrayList<Expression> allCond = new ArrayList<>();
+        final ArrayList<TableFilter> all = new ArrayList<>();
         if (condition != null) {
             allCond.add(condition);
         }
         for (int i = 0; i < count; i++) {
             TableFilter f = filters[i];
-            f.visit(new TableFilterVisitor() {
-                @Override
-                public void accept(TableFilter f) {
-                    all.add(f);
-                    if (f.getJoinCondition() != null) {
-                        allCond.add(f.getJoinCondition());
-                    }
+            f.visit(f1 -> {
+                all.add(f1);
+                if (f1.getJoinCondition() != null) {
+                    allCond.add(f1.getJoinCondition());
                 }
             });
         }
@@ -84,12 +80,11 @@ public class Plan {
         for (int i = 0; i < allFilters.length; i++) {
             TableFilter f = allFilters[i];
             setEvaluatable(f, true);
-            if (i < allFilters.length - 1 ||
-                    f.getSession().getDatabase().getSettings().earlyFilter) {
+            if (i < allFilters.length - 1) {
                 // the last table doesn't need the optimization,
                 // otherwise the expression is calculated twice unnecessarily
                 // (not that bad but not optimal)
-                f.optimizeFullCondition(false);
+                f.optimizeFullCondition();
             }
             f.removeUnusableIndexConditions();
         }
@@ -102,17 +97,16 @@ public class Plan {
      * Calculate the cost of this query plan.
      *
      * @param session the session
+     * @param allColumnsSet calculates all columns on-demand
      * @return the cost
      */
-    public double calculateCost(Session session) {
+    public double calculateCost(SessionLocal session, AllColumnsForPlan allColumnsSet) {
         Trace t = session.getTrace();
         if (t.isDebugEnabled()) {
             t.debug("Plan       : calculate cost for plan {0}", Arrays.toString(allFilters));
         }
         double cost = 1;
         boolean invalidPlan = false;
-        final HashSet<Column> allColumnsSet = ExpressionVisitor
-                .allColumnsForTableFilters(allFilters);
         for (int i = 0; i < allFilters.length; i++) {
             TableFilter tableFilter = allFilters[i];
             if (t.isDebugEnabled()) {
