@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.tools;
@@ -8,8 +8,6 @@ package org.h2.tools;
 import java.io.IOException;
 import java.io.PipedReader;
 import java.io.PipedWriter;
-import java.sql.Connection;
-import java.sql.DriverManager;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
@@ -17,19 +15,20 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
+import org.h2.jdbc.JdbcConnection;
 import org.h2.util.Tool;
 
 /**
  * Creates a cluster from a stand-alone database.
- * <br />
+ *
  * Copies a database to another location if required.
- * @h2.resource
  */
 public class CreateCluster extends Tool {
 
     /**
-     * Options are case sensitive. Supported options are:
+     * Options are case sensitive.
      * <table>
+     * <caption>Supported options</caption>
      * <tr><td>[-help] or [-?]</td>
      * <td>Print the list of options</td></tr>
      * <tr><td>[-urlSource "&lt;url&gt;"]</td>
@@ -43,9 +42,9 @@ public class CreateCluster extends Tool {
      * <tr><td>[-serverList &lt;list&gt;]</td>
      * <td>The comma separated list of host names or IP addresses</td></tr>
      * </table>
-     * @h2.resource
      *
      * @param args the command line arguments
+     * @throws SQLException on failure
      */
     public static void main(String... args) throws SQLException {
         new CreateCluster().runTool(args);
@@ -92,6 +91,7 @@ public class CreateCluster extends Tool {
      * @param user the user name
      * @param password the password
      * @param serverList the server list
+     * @throws SQLException on failure
      */
     public void execute(String urlSource, String urlTarget,
             String user, String password, String serverList) throws SQLException {
@@ -100,11 +100,9 @@ public class CreateCluster extends Tool {
 
     private static void process(String urlSource, String urlTarget,
             String user, String password, String serverList) throws SQLException {
-        org.h2.Driver.load();
-
         // use cluster='' so connecting is possible
         // even if the cluster is enabled
-        try (Connection connSource = DriverManager.getConnection(urlSource + ";CLUSTER=''", user, password);
+        try (JdbcConnection connSource = new JdbcConnection(urlSource + ";CLUSTER=''", null, user, password, false);
                 Statement statSource = connSource.createStatement()) {
             // enable the exclusive mode and close other connections,
             // so that data can't change while restoring the second database
@@ -122,7 +120,7 @@ public class CreateCluster extends Tool {
             String serverList) throws SQLException {
 
         // Delete the target database first.
-        try (Connection connTarget = DriverManager.getConnection(urlTarget + ";CLUSTER=''", user, password);
+        try (JdbcConnection connTarget = new JdbcConnection(urlTarget + ";CLUSTER=''", null, user, password, false);
                 Statement statTarget = connTarget.createStatement()) {
             statTarget.execute("DROP ALL OBJECTS DELETE FILES");
         }
@@ -131,7 +129,7 @@ public class CreateCluster extends Tool {
             Future<?> threadFuture = startWriter(pipeReader, statSource);
 
             // Read data from pipe reader, restore on target.
-            try (Connection connTarget = DriverManager.getConnection(urlTarget, user, password);
+            try (JdbcConnection connTarget = new JdbcConnection(urlTarget, null, user, password, false);
                     Statement statTarget = connTarget.createStatement()) {
                 RunScript.execute(connTarget, pipeReader);
 
@@ -154,27 +152,24 @@ public class CreateCluster extends Tool {
     }
 
     private static Future<?> startWriter(final PipedReader pipeReader,
-            final Statement statSource) {
+            final Statement statSource) throws IOException {
         final ExecutorService thread = Executors.newFixedThreadPool(1);
-
+        final PipedWriter pipeWriter = new PipedWriter(pipeReader);
         // Since exceptions cannot be thrown across thread boundaries, return
         // the task's future so we can check manually
-        Future<?> threadFuture = thread.submit(new Runnable() {
-            @Override
-            public void run() {
-                // If the creation of the piped writer fails, the reader will
-                // throw an IOException as soon as read() is called: IOException
-                // - if the pipe is broken, unconnected, closed, or an I/O error
-                // occurs. The reader's IOException will then trigger the
-                // finally{} that releases exclusive mode on the source DB.
-                try (final PipedWriter pipeWriter = new PipedWriter(pipeReader);
-                        final ResultSet rs = statSource.executeQuery("SCRIPT")) {
-                    while (rs.next()) {
-                        pipeWriter.write(rs.getString(1) + "\n");
-                    }
-                } catch (SQLException | IOException ex) {
-                    throw new IllegalStateException("Producing script from the source DB is failing.", ex);
+        Future<?> threadFuture = thread.submit(() -> {
+            // If the creation of the piped writer fails, the reader will
+            // throw an IOException as soon as read() is called: IOException
+            // - if the pipe is broken, unconnected, closed, or an I/O error
+            // occurs. The reader's IOException will then trigger the
+            // finally{} that releases exclusive mode on the source DB.
+            try (PipedWriter writer = pipeWriter;
+                    final ResultSet rs = statSource.executeQuery("SCRIPT")) {
+                while (rs.next()) {
+                    writer.write(rs.getString(1) + "\n");
                 }
+            } catch (SQLException | IOException ex) {
+                throw new IllegalStateException("Producing script from the source DB is failing.", ex);
             }
         });
 
