@@ -1,21 +1,20 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.tools;
 
-import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
 import java.io.IOException;
-import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.Reader;
 import java.io.Writer;
+import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
@@ -23,16 +22,15 @@ import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Types;
 import java.util.ArrayList;
-
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
-import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
+import org.h2.mvstore.DataUtils;
 import org.h2.store.fs.FileUtils;
 import org.h2.util.IOUtils;
 import org.h2.util.JdbcUtils;
-import org.h2.util.New;
 import org.h2.util.StringUtils;
+import org.h2.util.Utils;
 
 /**
  * A facility to read from and write to CSV (comma separated values) files. When
@@ -45,7 +43,7 @@ public class Csv implements SimpleRowSource {
 
     private String[] columnNames;
 
-    private String characterSet = SysProperties.FILE_ENCODING;
+    private String characterSet;
     private char escapeCharacter = '\"';
     private char fieldDelimiter = '\"';
     private char fieldSeparatorRead = ',';
@@ -54,11 +52,11 @@ public class Csv implements SimpleRowSource {
     private boolean preserveWhitespace;
     private boolean writeColumnHeader = true;
     private char lineComment;
-    private String lineSeparator = SysProperties.LINE_SEPARATOR;
+    private String lineSeparator = System.lineSeparator();
     private String nullString = "";
 
     private String fileName;
-    private Reader input;
+    private BufferedReader input;
     private char[] inputBuffer;
     private int inputBufferPos;
     private int inputBufferStart = -1;
@@ -72,31 +70,15 @@ public class Csv implements SimpleRowSource {
             ResultSetMetaData meta = rs.getMetaData();
             int columnCount = meta.getColumnCount();
             String[] row = new String[columnCount];
-            int[] sqlTypes = new int[columnCount];
             for (int i = 0; i < columnCount; i++) {
                 row[i] = meta.getColumnLabel(i + 1);
-                sqlTypes[i] = meta.getColumnType(i + 1);
             }
             if (writeColumnHeader) {
                 writeRow(row);
             }
             while (rs.next()) {
                 for (int i = 0; i < columnCount; i++) {
-                    Object o;
-                    switch (sqlTypes[i]) {
-                    case Types.DATE:
-                        o = rs.getDate(i + 1);
-                        break;
-                    case Types.TIME:
-                        o = rs.getTime(i + 1);
-                        break;
-                    case Types.TIMESTAMP:
-                        o = rs.getTimestamp(i + 1);
-                        break;
-                    default:
-                        o = rs.getString(i + 1);
-                    }
-                    row[i] = o == null ? null : o.toString();
+                    row[i] = rs.getString(i + 1);
                 }
                 writeRow(row);
                 rows++;
@@ -117,6 +99,7 @@ public class Csv implements SimpleRowSource {
      * @param writer the writer
      * @param rs the result set
      * @return the number of rows written
+     * @throws SQLException on failure
      */
     public int write(Writer writer, ResultSet rs) throws SQLException {
         this.output = writer;
@@ -137,8 +120,8 @@ public class Csv implements SimpleRowSource {
      * @param rs the result set - the result set must be positioned before the
      *          first row.
      * @param charset the charset or null to use the system default charset
-     *          (see system property file.encoding)
      * @return the number of rows written
+     * @throws SQLException on failure
      */
     public int write(String outputFileName, ResultSet rs, String charset)
             throws SQLException {
@@ -160,6 +143,7 @@ public class Csv implements SimpleRowSource {
      * @param charset the charset or null to use the system default charset
      *          (see system property file.encoding)
      * @return the number of rows written
+     * @throws SQLException on failure
      */
     public int write(Connection conn, String outputFileName, String sql,
             String charset) throws SQLException {
@@ -174,7 +158,7 @@ public class Csv implements SimpleRowSource {
      * Reads from the CSV file and returns a result set. The rows in the result
      * set are created on demand, that means the file is kept open until all
      * rows are read or the result set is closed.
-     * <br />
+     *
      * If the columns are read from the CSV file, then the following rules are
      * used: columns names that start with a letter or '_', and only
      * contain letters, '_', and digits, are considered case insensitive
@@ -185,8 +169,8 @@ public class Csv implements SimpleRowSource {
      * @param colNames or null if the column names should be read from the CSV
      *          file
      * @param charset the charset or null to use the system default charset
-     *          (see system property file.encoding)
      * @return the result set
+     * @throws SQLException on failure
      */
     public ResultSet read(String inputFileName, String[] colNames,
             String charset) throws SQLException {
@@ -207,10 +191,12 @@ public class Csv implements SimpleRowSource {
      * @param colNames or null if the column names should be read from the CSV
      *            file
      * @return the result set
+     * @throws IOException on failure
      */
     public ResultSet read(Reader reader, String[] colNames) throws IOException {
         init(null, null);
-        this.input = reader;
+        this.input = reader instanceof BufferedReader ? (BufferedReader) reader
+                : new BufferedReader(reader, Constants.IO_BUFFER_SIZE);
         return readResultSet(colNames);
     }
 
@@ -229,7 +215,7 @@ public class Csv implements SimpleRowSource {
         for (int i = 0; i < columnNames.length; i++) {
             StringBuilder buff = new StringBuilder();
             String n = columnNames[i];
-            if (n == null || n.length() == 0) {
+            if (n == null || n.isEmpty()) {
                 buff.append('C').append(i + 1);
             } else {
                 buff.append(n);
@@ -247,9 +233,7 @@ public class Csv implements SimpleRowSource {
 
     private void init(String newFileName, String charset) {
         this.fileName = newFileName;
-        if (charset != null) {
-            this.characterSet = charset;
-        }
+        this.characterSet = charset;
     }
 
     private void initWrite() throws IOException {
@@ -257,10 +241,11 @@ public class Csv implements SimpleRowSource {
             try {
                 OutputStream out = FileUtils.newOutputStream(fileName, false);
                 out = new BufferedOutputStream(out, Constants.IO_BUFFER_SIZE);
-                output = new BufferedWriter(new OutputStreamWriter(out, characterSet));
+                output = new BufferedWriter(characterSet != null ?
+                        new OutputStreamWriter(out, characterSet) : new OutputStreamWriter(out));
             } catch (Exception e) {
                 close();
-                throw DbException.convertToIOException(e);
+                throw DataUtils.convertToIOException(e);
             }
         }
     }
@@ -313,17 +298,13 @@ public class Csv implements SimpleRowSource {
     private void initRead() throws IOException {
         if (input == null) {
             try {
-                InputStream in = FileUtils.newInputStream(fileName);
-                in = new BufferedInputStream(in, Constants.IO_BUFFER_SIZE);
-                input = new InputStreamReader(in, characterSet);
+                input = FileUtils.newBufferedReader(fileName,
+                        characterSet != null ? Charset.forName(characterSet) : StandardCharsets.UTF_8);
             } catch (IOException e) {
                 close();
                 throw e;
             }
         }
-        if (!input.markSupported()) {
-            input = new BufferedReader(input);
-        }
         input.mark(1);
         int bom = input.read();
         if (bom != 0xfeff) {
@@ -338,12 +319,12 @@ public class Csv implements SimpleRowSource {
     }
 
     private void readHeader() throws IOException {
-        ArrayList<String> list = New.arrayList();
+        ArrayList<String> list = new ArrayList<>();
         while (true) {
             String v = readValue();
             if (v == null) {
                 if (endOfLine) {
-                    if (endOfFile || list.size() > 0) {
+                    if (endOfFile || !list.isEmpty()) {
                         break;
                     }
                 } else {
@@ -351,10 +332,10 @@ public class Csv implements SimpleRowSource {
                     list.add(v);
                 }
             } else {
-                if (v.length() == 0) {
+                if (v.isEmpty()) {
                     v = "COLUMN" + list.size();
                 } else if (!caseSensitiveColumnNames && isSimpleColumnName(v)) {
-                    v = v.toUpperCase();
+                    v = StringUtils.toUpperEnglish(v);
                 }
                 list.add(v);
                 if (endOfLine) {
@@ -362,8 +343,7 @@ public class Csv implements SimpleRowSource {
                 }
             }
         }
-        columnNames = new String[list.size()];
-        list.toArray(columnNames);
+        columnNames = list.toArray(new String[0]);
     }
 
     private static boolean isSimpleColumnName(String columnName) {
@@ -379,10 +359,7 @@ public class Csv implements SimpleRowSource {
                 }
             }
         }
-        if (columnName.length() == 0) {
-            return false;
-        }
-        return true;
+        return columnName.length() != 0;
     }
 
     private void pushBack() {
@@ -490,16 +467,12 @@ public class Csv implements SimpleRowSource {
                 return null;
             } else if (ch <= ' ') {
                 // ignore spaces
-                continue;
             } else if (lineComment != 0 && ch == lineComment) {
                 // comment until end of line
                 inputBufferStart = -1;
-                while (true) {
+                do {
                     ch = readChar();
-                    if (ch == '\n' || ch < 0 || ch == '\r') {
-                        break;
-                    }
-                }
+                } while (ch != '\n' && ch >= 0 && ch != '\r');
                 endOfLine = true;
                 return null;
             } else {
@@ -552,7 +525,7 @@ public class Csv implements SimpleRowSource {
             buff.append(chars[idx + 1]);
             start = idx + 2;
         }
-        buff.append(s.substring(start));
+        buff.append(s, start, s.length());
         return buff.toString();
     }
 
@@ -595,7 +568,7 @@ public class Csv implements SimpleRowSource {
     }
 
     private static SQLException convertException(String message, Exception e) {
-        return DbException.get(ErrorCode.IO_EXCEPTION_1, e, message).getSQLException();
+        return DbException.getJdbcSQLException(ErrorCode.IO_EXCEPTION_1, e, message);
     }
 
     /**
@@ -829,20 +802,20 @@ public class Csv implements SimpleRowSource {
      * INTERNAL.
      * Parse and set the CSV options.
      *
-     * @param options the the options
+     * @param options the options
      * @return the character set
      */
     public String setOptions(String options) {
         String charset = null;
         String[] keyValuePairs = StringUtils.arraySplit(options, ' ', false);
         for (String pair : keyValuePairs) {
-            if (pair.length() == 0) {
+            if (pair.isEmpty()) {
                 continue;
             }
             int index = pair.indexOf('=');
             String key = StringUtils.trim(pair.substring(0, index), true, true, " ");
             String value = pair.substring(index + 1);
-            char ch = value.length() == 0 ? 0 : value.charAt(0);
+            char ch = value.isEmpty() ? 0 : value.charAt(0);
             if (isParam(key, "escape", "esc", "escapeCharacter")) {
                 setEscapeCharacter(ch);
             } else if (isParam(key, "fieldDelimiter", "fieldDelim")) {
@@ -859,11 +832,11 @@ public class Csv implements SimpleRowSource {
             } else if (isParam(key, "charset", "characterSet")) {
                 charset = value;
             } else if (isParam(key, "preserveWhitespace")) {
-                setPreserveWhitespace(Boolean.parseBoolean(value));
+                setPreserveWhitespace(Utils.parseBoolean(value, false, false));
             } else if (isParam(key, "writeColumnHeader")) {
-                setWriteColumnHeader(Boolean.parseBoolean(value));
+                setWriteColumnHeader(Utils.parseBoolean(value, true, false));
             } else if (isParam(key, "caseSensitiveColumnNames")) {
-                setCaseSensitiveColumnNames(Boolean.parseBoolean(value));
+                setCaseSensitiveColumnNames(Utils.parseBoolean(value, false, false));
             } else {
                 throw DbException.getUnsupportedException(key);
             }
