@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.tools;
@@ -23,7 +23,6 @@ import org.h2.util.Utils;
 
 /**
  * Starts the H2 Console (web-) server, TCP, and PG server.
- * @h2.resource
  */
 public class Server extends Tool implements Runnable, ShutdownHandler {
 
@@ -42,6 +41,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      *
      * @param service the service
      * @param args the command line arguments
+     * @throws SQLException on failure
      */
     public Server(Service service, String... args) throws SQLException {
         verifyArgs(args);
@@ -55,21 +55,27 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
 
     /**
      * When running without options, -tcp, -web, -browser and -pg are started.
-     * <br />
-     * Options are case sensitive. Supported options are:
+     *
+     * Options are case sensitive.
      * <table>
+     * <caption>Supported options</caption>
      * <tr><td>[-help] or [-?]</td>
      * <td>Print the list of options</td></tr>
      * <tr><td>[-web]</td>
      * <td>Start the web server with the H2 Console</td></tr>
      * <tr><td>[-webAllowOthers]</td>
      * <td>Allow other computers to connect - see below</td></tr>
+     * <tr><td>[-webExternalNames &lt;names&gt;]</td>
+     * <td>The comma-separated list of external names and IP addresses of this server,
+     * used together with -webAllowOthers</td></tr>
      * <tr><td>[-webDaemon]</td>
      * <td>Use a daemon thread</td></tr>
      * <tr><td>[-webPort &lt;port&gt;]</td>
      * <td>The port (default: 8082)</td></tr>
      * <tr><td>[-webSSL]</td>
      * <td>Use encrypted (HTTPS) connections</td></tr>
+     * <tr><td>[-webAdminPassword]</td>
+     * <td>Password of DB Console administrator</td></tr>
      * <tr><td>[-browser]</td>
      * <td>Start a browser connecting to the web server</td></tr>
      * <tr><td>[-tcp]</td>
@@ -102,17 +108,19 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * <td>The base directory for H2 databases (all servers)</td></tr>
      * <tr><td>[-ifExists]</td>
      * <td>Only existing databases may be opened (all servers)</td></tr>
+     * <tr><td>[-ifNotExists]</td>
+     * <td>Databases are created when accessed</td></tr>
      * <tr><td>[-trace]</td>
      * <td>Print additional trace information (all servers)</td></tr>
      * <tr><td>[-key &lt;from&gt; &lt;to&gt;]</td>
      * <td>Allows to map a database name to another (all servers)</td></tr>
      * </table>
      * The options -xAllowOthers are potentially risky.
-     * <br />
+     *
      * For details, see Advanced Topics / Protection against Remote Access.
-     * @h2.resource
      *
      * @param args the command line arguments
+     * @throws SQLException on failure
      */
     public static void main(String... args) throws SQLException {
         new Server().runTool(args);
@@ -129,12 +137,16 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
                     // ok
                 } else if ("-webAllowOthers".equals(arg)) {
                     // no parameters
-                } else if ("-webDaemon".equals(arg)) {
+                } else if ("-webExternalNames".equals(arg)) {
+                    i++;
+                }  else if ("-webDaemon".equals(arg)) {
                     // no parameters
                 } else if ("-webSSL".equals(arg)) {
                     // no parameters
                 } else if ("-webPort".equals(arg)) {
                     i++;
+                } else if ("-webAdminPassword".equals(arg)) {
+                    i++;
                 } else {
                     throwUnsupportedOption(arg);
                 }
@@ -194,6 +206,8 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
                 // no parameters
             } else if ("-ifExists".equals(arg)) {
                 // no parameters
+            } else if ("-ifNotExists".equals(arg)) {
+                // no parameters
             } else if ("-baseDir".equals(arg)) {
                 i++;
             } else if ("-key".equals(arg)) {
@@ -232,6 +246,8 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
                     // no parameters
                 } else if ("-webPort".equals(arg)) {
                     i++;
+                } else if ("-webAdminPassword".equals(arg)) {
+                    i++;
                 } else {
                     showUsageAndThrowUnsupportedOption(arg);
                 }
@@ -280,6 +296,8 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
                 // no parameters
             } else if ("-ifExists".equals(arg)) {
                 // no parameters
+            } else if ("-ifNotExists".equals(arg)) {
+                // no parameters
             } else if ("-baseDir".equals(arg)) {
                 i++;
             } else if ("-key".equals(arg)) {
@@ -363,6 +381,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * @param force the shutdown (don't wait)
      * @param all whether all TCP servers that are running in the JVM should be
      *            stopped
+     * @throws SQLException on failure
      */
     public static void shutdownTcpServer(String url, String password,
             boolean force, boolean all) throws SQLException {
@@ -407,14 +426,32 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * </pre>
      * Supported options are:
      * -webPort, -webSSL, -webAllowOthers, -webDaemon,
-     * -trace, -ifExists, -baseDir, -properties.
+     * -trace, -ifExists, -ifNotExists, -baseDir, -properties.
      * See the main method for details.
      *
      * @param args the argument list
      * @return the server
+     * @throws SQLException on failure
      */
     public static Server createWebServer(String... args) throws SQLException {
+        return createWebServer(args, null, false);
+    }
+
+    /**
+     * Create a new web server, but does not start it yet.
+     *
+     * @param args
+     *            the argument list
+     * @param key
+     *            key, or null
+     * @param allowSecureCreation
+     *            whether creation of databases using the key should be allowed
+     * @return the server
+     */
+    static Server createWebServer(String[] args, String key, boolean allowSecureCreation) throws SQLException {
         WebServer service = new WebServer();
+        service.setKey(key);
+        service.setAllowSecureCreation(allowSecureCreation);
         Server server = new Server(service, args);
         service.setShutdownHandler(server);
         return server;
@@ -429,7 +466,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * </pre>
      * Supported options are:
      * -tcpPort, -tcpSSL, -tcpPassword, -tcpAllowOthers, -tcpDaemon,
-     * -trace, -ifExists, -baseDir, -key.
+     * -trace, -ifExists, -ifNotExists, -baseDir, -key.
      * See the main method for details.
      * <p>
      * If no port is specified, the default port is used if possible,
@@ -439,6 +476,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      *
      * @param args the argument list
      * @return the server
+     * @throws SQLException on failure
      */
     public static Server createTcpServer(String... args) throws SQLException {
         TcpServer service = new TcpServer();
@@ -456,7 +494,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * </pre>
      * Supported options are:
      * -pgPort, -pgAllowOthers, -pgDaemon,
-     * -trace, -ifExists, -baseDir, -key.
+     * -trace, -ifExists, -ifNotExists, -baseDir, -key.
      * See the main method for details.
      * <p>
      * If no port is specified, the default port is used if possible,
@@ -466,6 +504,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      *
      * @param args the argument list
      * @return the server
+     * @throws SQLException on failure
      */
     public static Server createPgServer(String... args) throws SQLException {
         return new Server(new PgServer(), args);
@@ -480,7 +519,12 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
         try {
             started = true;
             service.start();
-            String name = service.getName() + " (" + service.getURL() + ")";
+            String url = service.getURL();
+            int idx = url.indexOf('?');
+            if (idx >= 0) {
+                url = url.substring(0, idx);
+            }
+            String name = service.getName() + " (" + url + ')';
             Thread t = new Thread(this, name);
             t.setDaemon(service.isDaemon());
             t.start();
@@ -581,6 +625,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
 
     /**
      * INTERNAL
+     * @param shutdownHandler to set
      */
     public void setShutdownHandler(ShutdownHandler shutdownHandler) {
         this.shutdownHandler = shutdownHandler;
@@ -611,6 +656,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * Open a new browser tab or window with the given URL.
      *
      * @param url the URL to open
+     * @throws Exception on failure
      */
     public static void openBrowser(String url) throws Exception {
         try {
@@ -684,8 +730,8 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
                 if (!ok) {
                     // No success in detection.
                     throw new Exception(
-                            "Browser detection failed and system property " +
-                            SysProperties.H2_BROWSER + " not set");
+                            "Browser detection failed, and java property 'h2.browser' " +
+                            "and environment variable BROWSER are not set to a browser executable.");
                 }
             }
         } catch (Exception e) {
@@ -702,6 +748,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * user has disconnected.
      *
      * @param conn the database connection (the database must be open)
+     * @throws SQLException on failure
      */
     public static void startWebServer(Connection conn) throws SQLException {
         startWebServer(conn, false);
@@ -716,6 +763,7 @@ public class Server extends Tool implements Runnable, ShutdownHandler {
      * @param conn the database connection (the database must be open)
      * @param ignoreProperties if {@code true} properties from
      *         {@code .h2.server.properties} will be ignored
+     * @throws SQLException on failure
      */
     public static void startWebServer(Connection conn, boolean ignoreProperties) throws SQLException {
         WebServer webServer = new WebServer();
