@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.tools;
@@ -12,7 +12,6 @@ import java.io.InputStreamReader;
 import java.io.PrintStream;
 import java.io.StringReader;
 import java.sql.Connection;
-import java.sql.DriverManager;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
@@ -21,10 +20,10 @@ import java.util.ArrayList;
 import java.util.Properties;
 import java.util.concurrent.TimeUnit;
 
+import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
 import org.h2.server.web.ConnectionInfo;
 import org.h2.util.JdbcUtils;
-import org.h2.util.New;
 import org.h2.util.ScriptReader;
 import org.h2.util.SortedProperties;
 import org.h2.util.StringUtils;
@@ -33,7 +32,6 @@ import org.h2.util.Utils;
 
 /**
  * Interactive command line tool to access a database using JDBC.
- * @h2.resource
  */
 public class Shell extends Tool implements Runnable {
 
@@ -49,13 +47,14 @@ public class Shell extends Tool implements Runnable {
     private Statement stat;
     private boolean listMode;
     private int maxColumnSize = 100;
-    private final ArrayList<String> history = New.arrayList();
+    private final ArrayList<String> history = new ArrayList<>();
     private boolean stopHide;
     private String serverPropertiesDir = Constants.SERVER_PROPERTIES_DIR;
 
     /**
-     * Options are case sensitive. Supported options are:
+     * Options are case sensitive.
      * <table>
+     * <caption>Supported options</caption>
      * <tr><td>[-help] or [-?]</td>
      * <td>Print the list of options</td></tr>
      * <tr><td>[-url "&lt;url&gt;"]</td>
@@ -73,9 +72,9 @@ public class Shell extends Tool implements Runnable {
      * </table>
      * If special characters don't work as expected, you may need to use
      * -Dfile.encoding=UTF-8 (Mac OS X) or CP850 (Windows).
-     * @h2.resource
      *
      * @param args the command line arguments
+     * @throws SQLException on failure
      */
     public static void main(String... args) throws SQLException {
         new Shell().runTool(args);
@@ -115,6 +114,7 @@ public class Shell extends Tool implements Runnable {
      */
     @Override
     public void runTool(String... args) throws SQLException {
+        String driver = null;
         String url = null;
         String user = "";
         String password = "";
@@ -128,8 +128,7 @@ public class Shell extends Tool implements Runnable {
             } else if (arg.equals("-password")) {
                 password = args[++i];
             } else if (arg.equals("-driver")) {
-                String driver = args[++i];
-                JdbcUtils.loadUserClass(driver);
+                driver = args[++i];
             } else if (arg.equals("-sql")) {
                 sql = args[++i];
             } else if (arg.equals("-properties")) {
@@ -144,8 +143,7 @@ public class Shell extends Tool implements Runnable {
             }
         }
         if (url != null) {
-            org.h2.Driver.load();
-            conn = DriverManager.getConnection(url, user, password);
+            conn = JdbcUtils.getConnection(driver, url, user, password);
             stat = conn.createStatement();
         }
         if (sql == null) {
@@ -176,6 +174,7 @@ public class Shell extends Tool implements Runnable {
      *
      * @param conn the connection
      * @param args the command line settings
+     * @throws SQLException on failure
      */
     public void runTool(Connection conn, String... args) throws SQLException {
         this.conn = conn;
@@ -196,7 +195,7 @@ public class Shell extends Tool implements Runnable {
 
     private void promptLoop() {
         println("");
-        println("Welcome to H2 Shell " + Constants.getFullVersion());
+        println("Welcome to H2 Shell " + Constants.FULL_VERSION);
         println("Exit with Ctrl+C");
         if (conn != null) {
             showHelp();
@@ -221,7 +220,7 @@ public class Shell extends Tool implements Runnable {
                     break;
                 }
                 String trimmed = line.trim();
-                if (trimmed.length() == 0) {
+                if (trimmed.isEmpty()) {
                     continue;
                 }
                 boolean end = trimmed.endsWith(";");
@@ -249,7 +248,7 @@ public class Shell extends Tool implements Runnable {
                         println("No history");
                     }
                 } else if (lower.startsWith("autocommit")) {
-                    lower = lower.substring("autocommit".length()).trim();
+                    lower = StringUtils.trimSubstring(lower, "autocommit".length());
                     if ("true".equals(lower)) {
                         conn.setAutoCommit(true);
                     } else if ("false".equals(lower)) {
@@ -259,7 +258,7 @@ public class Shell extends Tool implements Runnable {
                     }
                     println("Autocommit is now " + conn.getAutoCommit());
                 } else if (lower.startsWith("maxwidth")) {
-                    lower = lower.substring("maxwidth".length()).trim();
+                    lower = StringUtils.trimSubstring(lower, "maxwidth".length());
                     try {
                         maxColumnSize = Integer.parseInt(lower);
                     } catch (NumberFormatException e) {
@@ -334,7 +333,7 @@ public class Shell extends Tool implements Runnable {
             String data = null;
             boolean found = false;
             for (int i = 0;; i++) {
-                String d = prop.getProperty(String.valueOf(i));
+                String d = prop.getProperty(Integer.toString(i));
                 if (d == null) {
                     break;
                 }
@@ -364,17 +363,33 @@ public class Shell extends Tool implements Runnable {
         println("[Enter]   " + user);
         print("User      ");
         user = readLine(user);
-        println("[Enter]   Hide");
-        print("Password  ");
-        String password = readLine();
-        if (password.length() == 0) {
-            password = readPassword();
-        }
-        conn = JdbcUtils.getConnection(driver, url, user, password);
+        conn = url.startsWith(Constants.START_URL) ? connectH2(driver, url, user)
+                : JdbcUtils.getConnection(driver, url, user, readPassword());
         stat = conn.createStatement();
         println("Connected");
     }
 
+    private Connection connectH2(String driver, String url, String user) throws IOException, SQLException {
+        for (;;) {
+            String password = readPassword();
+            try {
+                return JdbcUtils.getConnection(driver, url + ";IFEXISTS=TRUE", user, password);
+            } catch (SQLException ex) {
+                if (ex.getErrorCode() == ErrorCode.DATABASE_NOT_FOUND_WITH_IF_EXISTS_1) {
+                    println("Type the same password again to confirm database creation.");
+                    String password2 = readPassword();
+                    if (password.equals(password2)) {
+                        return JdbcUtils.getConnection(driver, url, user, password);
+                    } else {
+                        println("Passwords don't match. Try again.");
+                    }
+                } else {
+                    throw ex;
+                }
+            }
+        }
+    }
+
     /**
      * Print the string without newline, and flush.
      *
@@ -434,7 +449,7 @@ public class Shell extends Tool implements Runnable {
 
     private String readLine(String defaultValue) throws IOException {
         String s = readLine();
-        return s.length() == 0 ? defaultValue : s;
+        return s.isEmpty() ? defaultValue : s;
     }
 
     private String readLine() throws IOException {
@@ -446,21 +461,29 @@ public class Shell extends Tool implements Runnable {
     }
 
     private void execute(String sql) {
-        if (sql.trim().length() == 0) {
+        if (StringUtils.isWhitespaceOrEmpty(sql)) {
             return;
         }
         long time = System.nanoTime();
         try {
             ResultSet rs = null;
             try {
-                if (stat.execute(sql)) {
+                if (sql.startsWith("@")) {
+                    rs = JdbcUtils.getMetaResultSet(conn, sql);
+                    printResult(rs, listMode);
+                } else if (stat.execute(sql)) {
                     rs = stat.getResultSet();
                     int rowCount = printResult(rs, listMode);
                     time = System.nanoTime() - time;
                     println("(" + rowCount + (rowCount == 1 ?
                             " row, " : " rows, ") + TimeUnit.NANOSECONDS.toMillis(time) + " ms)");
                 } else {
-                    int updateCount = stat.getUpdateCount();
+                    long updateCount;
+                    try {
+                        updateCount = stat.getLargeUpdateCount();
+                    } catch (UnsupportedOperationException e) {
+                        updateCount = stat.getUpdateCount();
+                    }
                     time = System.nanoTime() - time;
                     println("(Update count: " + updateCount + ", " +
                             TimeUnit.NANOSECONDS.toMillis(time) + " ms)");
@@ -487,7 +510,7 @@ public class Shell extends Tool implements Runnable {
         ResultSetMetaData meta = rs.getMetaData();
         int len = meta.getColumnCount();
         boolean truncated = false;
-        ArrayList<String[]> rows = New.arrayList();
+        ArrayList<String[]> rows = new ArrayList<>();
         // buffer the header
         String[] columns = new String[len];
         for (int i = 0; i < len; i++) {
@@ -540,7 +563,7 @@ public class Shell extends Tool implements Runnable {
                 max = Math.max(max, row[i].length());
             }
             if (len > 1) {
-                Math.min(maxColumnSize, max);
+                max = Math.min(maxColumnSize, max);
             }
             columnSizes[i] = max;
         }
