@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.tools;
@@ -30,16 +30,16 @@ import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Map;
 import java.util.UUID;
-
 import org.h2.api.ErrorCode;
-import org.h2.jdbc.JdbcResultSetBackwardsCompat;
 import org.h2.message.DbException;
 import org.h2.util.Bits;
 import org.h2.util.JdbcUtils;
 import org.h2.util.MathUtils;
-import org.h2.util.New;
+import org.h2.util.SimpleColumnInfo;
 import org.h2.util.Utils;
 import org.h2.value.DataType;
+import org.h2.value.Value;
+import org.h2.value.ValueToObjectConverter;
 
 /**
  * This class is a simple result set and meta data implementation.
@@ -59,15 +59,14 @@ import org.h2.value.DataType;
  * </pre>
  *
  */
-public class SimpleResultSet implements ResultSet, ResultSetMetaData,
-        JdbcResultSetBackwardsCompat {
+public class SimpleResultSet implements ResultSet, ResultSetMetaData {
 
     private ArrayList<Object[]> rows;
     private Object[] currentRow;
     private int rowId = -1;
     private boolean wasNull;
     private SimpleRowSource source;
-    private ArrayList<Column> columns = New.arrayList();
+    private ArrayList<SimpleColumnInfo> columns = Utils.newSmallArrayList();
     private boolean autoClose = true;
 
     /**
@@ -75,7 +74,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      * addRow.
      */
     public SimpleResultSet() {
-        rows = New.arrayList();
+        rows = Utils.newSmallArrayList();
     }
 
     /**
@@ -100,8 +99,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      */
     public void addColumn(String name, int sqlType, int precision, int scale) {
         int valueType = DataType.convertSQLTypeToValueType(sqlType);
-        addColumn(name, sqlType, DataType.getDataType(valueType).name,
-                precision, scale);
+        addColumn(name, sqlType, Value.getTypeName(valueType), precision, scale);
     }
 
     /**
@@ -123,13 +121,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
         if (name == null) {
             name = "C" + (columns.size() + 1);
         }
-        Column column = new Column();
-        column.name = name;
-        column.sqlType = sqlType;
-        column.precision = precision;
-        column.scale = scale;
-        column.sqlTypeName = sqlTypeName;
-        columns.add(column);
+        columns.add(new SimpleColumnInfo(name, sqlType, sqlTypeName, precision, scale));
     }
 
     /**
@@ -249,7 +241,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     @Override
     public void beforeFirst() throws SQLException {
         if (autoClose) {
-            throw DbException.get(ErrorCode.RESULT_SET_NOT_SCROLLABLE);
+            throw DbException.getJdbcSQLException(ErrorCode.RESULT_SET_NOT_SCROLLABLE);
         }
         rowId = -1;
         if (source != null) {
@@ -285,8 +277,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
                 }
             }
         }
-        throw DbException.get(ErrorCode.COLUMN_NOT_FOUND_1, columnLabel)
-                .getSQLException();
+        throw DbException.getJdbcSQLException(ErrorCode.COLUMN_NOT_FOUND_1, columnLabel);
     }
 
     /**
@@ -613,8 +604,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      */
     @Override
     public Clob getClob(int columnIndex) throws SQLException {
-        Clob c = (Clob) get(columnIndex);
-        return c == null ? null : c;
+        return (Clob) get(columnIndex);
     }
 
     /**
@@ -849,48 +839,48 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      * @param type the class of the returned value
      * @return the value
      */
+    @SuppressWarnings("unchecked")
     @Override
     public <T> T getObject(int columnIndex, Class<T> type) throws SQLException {
-        if (wasNull()) {
+        if (get(columnIndex) == null) {
             return null;
         }
-
         if (type == BigDecimal.class) {
-            return type.cast(getBigDecimal(columnIndex));
+            return (T) getBigDecimal(columnIndex);
         } else if (type == BigInteger.class) {
-            return type.cast(getBigDecimal(columnIndex).toBigInteger());
+            return (T) getBigDecimal(columnIndex).toBigInteger();
         } else if (type == String.class) {
-            return type.cast(getString(columnIndex));
+            return (T) getString(columnIndex);
         } else if (type == Boolean.class) {
-            return type.cast(getBoolean(columnIndex));
+            return (T) (Boolean) getBoolean(columnIndex);
         } else if (type == Byte.class) {
-            return type.cast(getByte(columnIndex));
+            return (T) (Byte) getByte(columnIndex);
         } else if (type == Short.class) {
-            return type.cast(getShort(columnIndex));
+            return (T) (Short) getShort(columnIndex);
         } else if (type == Integer.class) {
-            return type.cast(getInt(columnIndex));
+            return (T) (Integer) getInt(columnIndex);
         } else if (type == Long.class) {
-            return type.cast(getLong(columnIndex));
+            return (T) (Long) getLong(columnIndex);
         } else if (type == Float.class) {
-            return type.cast(getFloat(columnIndex));
+            return (T) (Float) getFloat(columnIndex);
         } else if (type == Double.class) {
-            return type.cast(getDouble(columnIndex));
+            return (T) (Double) getDouble(columnIndex);
         } else if (type == Date.class) {
-            return type.cast(getDate(columnIndex));
+            return (T) getDate(columnIndex);
         } else if (type == Time.class) {
-            return type.cast(getTime(columnIndex));
+            return (T) getTime(columnIndex);
         } else if (type == Timestamp.class) {
-            return type.cast(getTimestamp(columnIndex));
+            return (T) getTimestamp(columnIndex);
         } else if (type == UUID.class) {
-            return type.cast(getObject(columnIndex));
+            return (T) getObject(columnIndex);
         } else if (type == byte[].class) {
-            return type.cast(getBytes(columnIndex));
+            return (T) getBytes(columnIndex);
         } else if (type == java.sql.Array.class) {
-            return type.cast(getArray(columnIndex));
+            return (T) getArray(columnIndex);
         } else if (type == Blob.class) {
-            return type.cast(getBlob(columnIndex));
+            return (T) getBlob(columnIndex);
         } else if (type == Clob.class) {
-            return type.cast(getClob(columnIndex));
+            return (T) getClob(columnIndex);
         } else {
             throw getUnsupportedException();
         }
@@ -1012,7 +1002,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
         if (o == null) {
             return null;
         }
-        switch (columns.get(columnIndex - 1).sqlType) {
+        switch (columns.get(columnIndex - 1).type) {
         case Types.CLOB:
             Clob c = (Clob) o;
             return c.getSubString(1, MathUtils.convertLongToInt(c.length()));
@@ -1868,7 +1858,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      */
     @Override
     public int getColumnType(int columnIndex) throws SQLException {
-        return getColumn(columnIndex - 1).sqlType;
+        return getColumn(columnIndex - 1).type;
     }
 
     /**
@@ -1993,14 +1983,14 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     }
 
     /**
-     * Returns null.
+     * Returns empty string.
      *
      * @param columnIndex (1,2,...)
-     * @return null
+     * @return empty string
      */
     @Override
     public String getCatalogName(int columnIndex) {
-        return null;
+        return "";
     }
 
     /**
@@ -2012,7 +2002,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     @Override
     public String getColumnClassName(int columnIndex) throws SQLException {
         int type = DataType.getValueTypeFromResultSet(this, columnIndex);
-        return DataType.getTypeClassName(type);
+        return ValueToObjectConverter.getDefaultClass(type, true).getName();
     }
 
     /**
@@ -2045,29 +2035,29 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      */
     @Override
     public String getColumnTypeName(int columnIndex) throws SQLException {
-        return getColumn(columnIndex - 1).sqlTypeName;
+        return getColumn(columnIndex - 1).typeName;
     }
 
     /**
-     * Returns null.
+     * Returns empty string.
      *
      * @param columnIndex (1,2,...)
-     * @return null
+     * @return empty string
      */
     @Override
     public String getSchemaName(int columnIndex) {
-        return null;
+        return "";
     }
 
     /**
-     * Returns null.
+     * Returns empty string.
      *
      * @param columnIndex (1,2,...)
-     * @return null
+     * @return empty string
      */
     @Override
     public String getTableName(int columnIndex) {
-        return null;
+        return "";
     }
 
     // ---- unsupported / result set -----------------------------------
@@ -2213,7 +2203,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      */
     @Override
     public boolean rowUpdated() throws SQLException {
-        throw getUnsupportedException();
+        return true;
     }
 
     /**
@@ -2259,6 +2249,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     // --- private -----------------------------
 
     private void update(int columnIndex, Object obj) throws SQLException {
+        checkClosed();
         checkColumnIndex(columnIndex);
         this.currentRow[columnIndex - 1] = obj;
     }
@@ -2271,8 +2262,13 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
      * INTERNAL
      */
     static SQLException getUnsupportedException() {
-        return DbException.get(ErrorCode.FEATURE_NOT_SUPPORTED_1).
-                getSQLException();
+        return DbException.getJdbcSQLException(ErrorCode.FEATURE_NOT_SUPPORTED_1);
+    }
+
+    private void checkClosed() throws SQLException {
+        if (columns == null) {
+            throw DbException.getJdbcSQLException(ErrorCode.OBJECT_CLOSED);
+        }
     }
 
     private void checkColumnIndex(int columnIndex) throws SQLException {
@@ -2284,8 +2280,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
 
     private Object get(int columnIndex) throws SQLException {
         if (currentRow == null) {
-            throw DbException.get(ErrorCode.NO_DATA_AVAILABLE).
-                    getSQLException();
+            throw DbException.getJdbcSQLException(ErrorCode.NO_DATA_AVAILABLE);
         }
         checkColumnIndex(columnIndex);
         columnIndex--;
@@ -2295,7 +2290,7 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
         return o;
     }
 
-    private Column getColumn(int i) throws SQLException {
+    private SimpleColumnInfo getColumn(int i) throws SQLException {
         checkColumnIndex(i + 1);
         return columns.get(i);
     }
@@ -2321,19 +2316,33 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
     }
 
     /**
-     * INTERNAL
+     * Return an object of this class if possible.
+     *
+     * @param iface the class
+     * @return this
      */
     @Override
+    @SuppressWarnings("unchecked")
     public <T> T unwrap(Class<T> iface) throws SQLException {
-        throw getUnsupportedException();
+        try {
+            if (isWrapperFor(iface)) {
+                return (T) this;
+            }
+            throw DbException.getInvalidValueException("iface", iface);
+        } catch (Exception e) {
+            throw DbException.toSQLException(e);
+        }
     }
 
     /**
-     * INTERNAL
+     * Checks if unwrap can return an object of this class.
+     *
+     * @param iface the class
+     * @return whether or not the interface is assignable from this class
      */
     @Override
     public boolean isWrapperFor(Class<?> iface) throws SQLException {
-        throw getUnsupportedException();
+        return iface != null && iface.isAssignableFrom(getClass());
     }
 
     /**
@@ -2355,37 +2364,6 @@ public class SimpleResultSet implements ResultSet, ResultSetMetaData,
         return autoClose;
     }
 
-    /**
-     * This class holds the data of a result column.
-     */
-    static class Column {
-
-        /**
-         * The column label.
-         */
-        String name;
-
-        /**
-         * The column type Name
-         */
-        String sqlTypeName;
-
-        /**
-         * The SQL type.
-         */
-        int sqlType;
-
-        /**
-         * The precision.
-         */
-        int precision;
-
-        /**
-         * The scale.
-         */
-        int scale;
-    }
-
     /**
      * A simple array implementation,
      * backed by an object array
