@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.util;
@@ -19,12 +19,7 @@ public class AbbaDetector {
 
     private static final boolean TRACE = false;
 
-    private static final ThreadLocal<Deque<Object>> STACK =
-            new ThreadLocal<Deque<Object>>() {
-                @Override protected Deque<Object> initialValue() {
-                    return new ArrayDeque<>();
-            }
-        };
+    private static final ThreadLocal<Deque<Object>> STACK = ThreadLocal.withInitial(ArrayDeque::new);
 
     /**
      * Map of (object A) -> (
