@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  *
  * According to a mail from Alan Tucker to Chris H Miller from IBM,
@@ -41,7 +41,7 @@ public class Permutations<T> {
         this.n = in.length;
         this.m = m;
         if (n < m || m < 0) {
-            DbException.throwInternalError("n < m or m < 0");
+            throw DbException.getInternalError("n < m or m < 0");
         }
         this.in = in;
         this.out = out;
