@@ -1,20 +1,22 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.util;
 
 import java.io.ByteArrayOutputStream;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.io.LineNumberReader;
 import java.io.OutputStream;
 import java.io.Reader;
 import java.io.StringReader;
 import java.lang.instrument.Instrumentation;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -81,14 +83,14 @@ public class Profiler implements Runnable {
 
     private volatile boolean stop;
     private final HashMap<String, Integer> counts =
-            new HashMap<String, Integer>();
+            new HashMap<>();
 
     /**
      * The summary (usually one entry per package, unless sumClasses is enabled,
      * in which case it's one entry per class).
      */
     private final HashMap<String, Integer> summary =
-            new HashMap<String, Integer>();
+            new HashMap<>();
     private int minCount = 1;
     private int total;
     private Thread thread;
@@ -165,26 +167,18 @@ public class Profiler implements Runnable {
                     }
                     continue;
                 }
-                String file = arg;
-                Reader reader;
-                LineNumberReader r;
-                reader = new InputStreamReader(
-                        new FileInputStream(file), "CP1252");
-                r = new LineNumberReader(reader);
-                while (true) {
-                    String line = r.readLine();
-                    if (line == null) {
-                        break;
-                    } else if (line.startsWith("Full thread dump")) {
-                        threadDumps++;
+                Path file = Paths.get(arg);
+                try (Reader reader = Files.newBufferedReader(file)) {
+                    LineNumberReader r = new LineNumberReader(reader);
+                    for (String line; (line = r.readLine()) != null;) {
+                        if (line.startsWith("Full thread dump")) {
+                            threadDumps++;
+                        }
                     }
                 }
-                reader.close();
-                reader = new InputStreamReader(
-                        new FileInputStream(file), "CP1252");
-                r = new LineNumberReader(reader);
-                processList(readStackTrace(r));
-                reader.close();
+                try (Reader reader = Files.newBufferedReader(file)) {
+                    processList(readStackTrace(new LineNumberReader(reader)));
+                }
             }
             System.out.println(getTopTraces(5));
         } catch (IOException e) {
@@ -193,7 +187,7 @@ public class Profiler implements Runnable {
     }
 
     private static List<Object[]> getRunnableStackTraces() {
-        ArrayList<Object[]> list = new ArrayList<Object[]>();
+        ArrayList<Object[]> list = new ArrayList<>();
         Map<Thread, StackTraceElement[]> map = Thread.getAllStackTraces();
         for (Map.Entry<Thread, StackTraceElement[]> entry : map.entrySet()) {
             Thread t = entry.getKey();
@@ -211,7 +205,7 @@ public class Profiler implements Runnable {
 
     private static List<Object[]> readRunnableStackTraces(int pid) {
         try {
-            String jstack = exec("jstack", "" + pid);
+            String jstack = exec("jstack", Integer.toString(pid));
             LineNumberReader r = new LineNumberReader(
                     new StringReader(jstack));
             return readStackTrace(r);
@@ -222,7 +216,7 @@ public class Profiler implements Runnable {
 
     private static List<Object[]> readStackTrace(LineNumberReader r)
             throws IOException {
-        ArrayList<Object[]> list = new ArrayList<Object[]>();
+        ArrayList<Object[]> list = new ArrayList<>();
         while (true) {
             String line = r.readLine();
             if (line == null) {
@@ -240,7 +234,7 @@ public class Profiler implements Runnable {
             if (!line.startsWith("java.lang.Thread.State: RUNNABLE")) {
                 continue;
             }
-            ArrayList<String> stack = new ArrayList<String>();
+            ArrayList<String> stack = new ArrayList<>();
             while (true) {
                 line = r.readLine();
                 if (line == null) {
@@ -253,11 +247,11 @@ public class Profiler implements Runnable {
                 if (!line.startsWith("at ")) {
                     break;
                 }
-                line = line.substring(3).trim();
+                line = StringUtils.trimSubstring(line, 3);
                 stack.add(line);
             }
-            if (stack.size() > 0) {
-                String[] s = stack.toArray(new String[stack.size()]);
+            if (!stack.isEmpty()) {
+                String[] s = stack.toArray(new String[0]);
                 list.add(s);
             }
         }
@@ -272,12 +266,11 @@ public class Profiler implements Runnable {
             copyInThread(p.getInputStream(), out);
             copyInThread(p.getErrorStream(), err);
             p.waitFor();
-            String e = new String(err.toByteArray(), "UTF-8");
+            String e = Utils10.byteArrayOutputStreamToString(err, StandardCharsets.UTF_8);
             if (e.length() > 0) {
                 throw new RuntimeException(e);
             }
-            String output = new String(out.toByteArray(), "UTF-8");
-            return output;
+            return Utils10.byteArrayOutputStreamToString(out, StandardCharsets.UTF_8);
         } catch (Exception e) {
             throw new RuntimeException(e);
         }
@@ -353,7 +346,7 @@ public class Profiler implements Runnable {
                 return;
             }
             try {
-                Thread.sleep(interval);
+                Thread.sleep(interval, 0);
             } catch (Exception e) {
                 // ignore
             }
@@ -472,10 +465,10 @@ public class Profiler implements Runnable {
         if (counts.size() == 0) {
             buff.append("(none)").append(LINE_SEPARATOR);
         }
-        HashMap<String, Integer> copy = new HashMap<String, Integer>(counts);
+        HashMap<String, Integer> copy = new HashMap<>(counts);
         appendTop(buff, copy, count, total, false);
         buff.append("summary:").append(LINE_SEPARATOR);
-        copy = new HashMap<String, Integer>(summary);
+        copy = new HashMap<>(summary);
         appendTop(buff, copy, count, total, true);
         buff.append('.');
         return buff.toString();
