@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.util;
@@ -44,7 +44,7 @@ public class SmallMap {
             Iterator<Integer> it = map.keySet().iterator();
             while (it.hasNext()) {
                 Integer k = it.next();
-                if (k.intValue() + maxElements < lastId) {
+                if (k + maxElements < lastId) {
                     it.remove();
                 }
             }
