@@ -1,29 +1,35 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.util;
 
 import java.io.BufferedReader;
 import java.io.ByteArrayOutputStream;
-import java.io.DataInputStream;
-import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PrintStream;
 import java.io.StringReader;
 import java.io.StringWriter;
-import java.io.Writer;
 import java.lang.reflect.Array;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.net.URI;
+import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.security.SecureClassLoader;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.Map;
+
+import javax.script.Compilable;
+import javax.script.CompiledScript;
+import javax.script.ScriptEngineManager;
+import javax.script.ScriptException;
 import javax.tools.FileObject;
 import javax.tools.ForwardingJavaFileManager;
 import javax.tools.JavaCompiler;
@@ -33,11 +39,10 @@ import javax.tools.JavaFileObject.Kind;
 import javax.tools.SimpleJavaFileObject;
 import javax.tools.StandardJavaFileManager;
 import javax.tools.ToolProvider;
+
 import org.h2.api.ErrorCode;
-import org.h2.engine.Constants;
 import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
-import org.h2.store.fs.FileUtils;
 
 /**
  * This class allows to convert source code to a class. It uses one class loader
@@ -58,12 +63,17 @@ public class SourceCompiler {
     /**
      * The class name to source code map.
      */
-    final HashMap<String, String> sources = New.hashMap();
+    final HashMap<String, String> sources = new HashMap<>();
 
     /**
      * The class name to byte code map.
      */
-    final HashMap<String, Class<?>> compiled = New.hashMap();
+    final HashMap<String, Class<?>> compiled = new HashMap<>();
+
+    /**
+     * The class name to compiled scripts map.
+     */
+    final Map<String, CompiledScript> compiledScripts = new HashMap<>();
 
     /**
      * Whether to use the ToolProvider.getSystemJavaCompiler().
@@ -114,6 +124,7 @@ public class SourceCompiler {
      *
      * @param packageAndClassName the class name
      * @return the class
+     * @throws ClassNotFoundException on failure
      */
     public Class<?> getClass(String packageAndClassName)
             throws ClassNotFoundException {
@@ -168,11 +179,57 @@ public class SourceCompiler {
         return source.startsWith("//groovy") || source.startsWith("@groovy");
     }
 
+    private static boolean isJavascriptSource(String source) {
+        return source.startsWith("//javascript");
+    }
+
+    private static boolean isRubySource(String source) {
+        return source.startsWith("#ruby");
+    }
+
+    /**
+     * Whether the passed source can be compiled using {@link javax.script.ScriptEngineManager}.
+     *
+     * @param source the source to test.
+     * @return <code>true</code> if {@link #getCompiledScript(String)} can be called.
+     */
+    public static boolean isJavaxScriptSource(String source) {
+        return isJavascriptSource(source) || isRubySource(source);
+    }
+
+    /**
+     * Get the compiled script.
+     *
+     * @param packageAndClassName the package and class name
+     * @return the compiled script
+     * @throws ScriptException on failure
+     */
+    public CompiledScript getCompiledScript(String packageAndClassName) throws ScriptException {
+        CompiledScript compiledScript = compiledScripts.get(packageAndClassName);
+        if (compiledScript == null) {
+            String source = sources.get(packageAndClassName);
+            final String lang;
+            if (isJavascriptSource(source)) {
+                lang = "javascript";
+            } else if (isRubySource(source)) {
+                lang = "ruby";
+            } else {
+                throw new IllegalStateException("Unknown language for " + source);
+            }
+
+            final Compilable jsEngine = (Compilable) new ScriptEngineManager().getEngineByName(lang);
+            compiledScript = jsEngine.compile(source);
+            compiledScripts.put(packageAndClassName, compiledScript);
+        }
+        return compiledScript;
+    }
+
     /**
      * Get the first public static method of the given class.
      *
      * @param className the class name
      * @return the method name
+     * @throws ClassNotFoundException on failure
      */
     public Method getMethod(String className) throws ClassNotFoundException {
         Class<?> clazz = getClass(className);
@@ -200,34 +257,37 @@ public class SourceCompiler {
      * @return the class file
      */
     byte[] javacCompile(String packageName, String className, String source) {
-        File dir = new File(COMPILE_DIR);
+        Path dir = Paths.get(COMPILE_DIR);
         if (packageName != null) {
-            dir = new File(dir, packageName.replace('.', '/'));
-            FileUtils.createDirectories(dir.getAbsolutePath());
+            dir = dir.resolve(packageName.replace('.', '/'));
+            try {
+                Files.createDirectories(dir);
+            } catch (Exception e) {
+                throw DbException.convert(e);
+            }
         }
-        File javaFile = new File(dir, className + ".java");
-        File classFile = new File(dir, className + ".class");
+        Path javaFile = dir.resolve(className + ".java");
+        Path classFile = dir.resolve(className + ".class");
         try {
-            OutputStream f = FileUtils.newOutputStream(javaFile.getAbsolutePath(), false);
-            Writer out = IOUtils.getBufferedWriter(f);
-            classFile.delete();
-            out.write(source);
-            out.close();
+            Files.write(javaFile, source.getBytes(StandardCharsets.UTF_8));
+            Files.deleteIfExists(classFile);
             if (JAVAC_SUN != null) {
                 javacSun(javaFile);
             } else {
                 javacProcess(javaFile);
             }
-            byte[] data = new byte[(int) classFile.length()];
-            DataInputStream in = new DataInputStream(new FileInputStream(classFile));
-            in.readFully(data);
-            in.close();
-            return data;
+            return Files.readAllBytes(classFile);
         } catch (Exception e) {
             throw DbException.convert(e);
         } finally {
-            javaFile.delete();
-            classFile.delete();
+            try {
+                Files.deleteIfExists(javaFile);
+            } catch (IOException e) {
+            }
+            try {
+                Files.deleteIfExists(classFile);
+            } catch (IOException e) {
+            }
         }
     }
 
@@ -277,31 +337,31 @@ public class SourceCompiler {
     Class<?> javaxToolsJavac(String packageName, String className, String source) {
         String fullClassName = packageName + "." + className;
         StringWriter writer = new StringWriter();
-        JavaFileManager fileManager = new
+        try (JavaFileManager fileManager = new
                 ClassFileManager(JAVA_COMPILER
-                    .getStandardFileManager(null, null, null));
-        ArrayList<JavaFileObject> compilationUnits = new ArrayList<JavaFileObject>();
-        compilationUnits.add(new StringJavaFileObject(fullClassName, source));
-        // cannot concurrently compile
-        synchronized (JAVA_COMPILER) {
-            JAVA_COMPILER.getTask(writer, fileManager, null, null,
-                null, compilationUnits).call();
-        }
-        String output = writer.toString();
-        handleSyntaxError(output);
-        try {
+                    .getStandardFileManager(null, null, null))) {
+            ArrayList<JavaFileObject> compilationUnits = new ArrayList<>();
+            compilationUnits.add(new StringJavaFileObject(fullClassName, source));
+            // cannot concurrently compile
+            final boolean ok;
+            synchronized (JAVA_COMPILER) {
+                ok = JAVA_COMPILER.getTask(writer, fileManager, null, null,
+                        null, compilationUnits).call();
+            }
+            String output = writer.toString();
+            handleSyntaxError(output, (ok? 0: 1));
             return fileManager.getClassLoader(null).loadClass(fullClassName);
-        } catch (ClassNotFoundException e) {
+        } catch (ClassNotFoundException | IOException e) {
             throw DbException.convert(e);
         }
     }
 
-    private static void javacProcess(File javaFile) {
+    private static void javacProcess(Path javaFile) {
         exec("javac",
                 "-sourcepath", COMPILE_DIR,
                 "-d", COMPILE_DIR,
                 "-encoding", "UTF-8",
-                javaFile.getAbsolutePath());
+                javaFile.toAbsolutePath().toString());
     }
 
     private static int exec(String... args) {
@@ -319,8 +379,8 @@ public class SourceCompiler {
             copyInThread(p.getInputStream(), buff);
             copyInThread(p.getErrorStream(), buff);
             p.waitFor();
-            String output = new String(buff.toByteArray(), Constants.UTF8);
-            handleSyntaxError(output);
+            String output = Utils10.byteArrayOutputStreamToString(buff, StandardCharsets.UTF_8);
+            handleSyntaxError(output, p.exitValue());
             return p.exitValue();
         } catch (Exception e) {
             throw DbException.convert(e);
@@ -336,23 +396,25 @@ public class SourceCompiler {
         }.execute();
     }
 
-    private static synchronized void javacSun(File javaFile) {
+    private static synchronized void javacSun(Path javaFile) {
         PrintStream old = System.err;
         ByteArrayOutputStream buff = new ByteArrayOutputStream();
-        PrintStream temp = new PrintStream(buff);
         try {
-            System.setErr(temp);
+            System.setErr(new PrintStream(buff, false, "UTF-8"));
             Method compile;
             compile = JAVAC_SUN.getMethod("compile", String[].class);
-            Object javac = JAVAC_SUN.newInstance();
-            compile.invoke(javac, (Object) new String[] {
+            Object javac = JAVAC_SUN.getDeclaredConstructor().newInstance();
+            // Bugfix: Here we should check exit status value instead of parsing javac output text.
+            // Because of the output text is different in different locale environment.
+            // @since 2018-07-20 little-pan
+            final Integer status = (Integer)compile.invoke(javac, (Object) new String[] {
                     "-sourcepath", COMPILE_DIR,
                     // "-Xlint:unchecked",
                     "-d", COMPILE_DIR,
                     "-encoding", "UTF-8",
-                    javaFile.getAbsolutePath() });
-            String output = new String(buff.toByteArray(), Constants.UTF8);
-            handleSyntaxError(output);
+                    javaFile.toAbsolutePath().toString() });
+            String output = Utils10.byteArrayOutputStreamToString(buff, StandardCharsets.UTF_8);
+            handleSyntaxError(output, status);
         } catch (Exception e) {
             throw DbException.convert(e);
         } finally {
@@ -360,12 +422,15 @@ public class SourceCompiler {
         }
     }
 
-    private static void handleSyntaxError(String output) {
+    private static void handleSyntaxError(String output, int exitStatus) {
+        if(0 == exitStatus){
+            return;
+        }
         boolean syntaxError = false;
         final BufferedReader reader = new BufferedReader(new StringReader(output));
         try {
             for (String line; (line = reader.readLine()) != null;) {
-                if (line.endsWith("warning")) {
+                if (line.endsWith("warning") || line.endsWith("warnings")) {
                     // ignore summary line
                 } else if (line.startsWith("Note:")
                         || line.startsWith("warning:")) {
@@ -405,7 +470,7 @@ public class SourceCompiler {
                 Object importCustomizer = Utils.newInstance(
                         "org.codehaus.groovy.control.customizers.ImportCustomizer");
                 // Call the method ImportCustomizer.addImports(String[])
-                String[] importsArray = new String[] {
+                String[] importsArray = {
                         "java.sql.Connection",
                         "java.sql.Types",
                         "java.sql.ResultSet",
@@ -422,7 +487,7 @@ public class SourceCompiler {
                 Object configuration = Utils.newInstance(
                         "org.codehaus.groovy.control.CompilerConfiguration");
                 Utils.callMethod(configuration,
-                        "addCompilationCustomizers", new Object[] { importCustomizerArray });
+                        "addCompilationCustomizers", importCustomizerArray);
 
                 ClassLoader parent = GroovyCompiler.class.getClassLoader();
                 loader = Utils.newInstance(
@@ -444,9 +509,8 @@ public class SourceCompiler {
                 Object codeSource = Utils.newInstance("groovy.lang.GroovyCodeSource",
                         source, packageAndClassName + ".groovy", "UTF-8");
                 Utils.callMethod(codeSource, "setCachable", false);
-                Class<?> clazz = (Class<?>) Utils.callMethod(
+                return (Class<?>) Utils.callMethod(
                         LOADER, "parseClass", codeSource);
-                return clazz;
             } catch (Exception e) {
                 throw new RuntimeException(e);
             }
@@ -502,9 +566,20 @@ public class SourceCompiler {
             ForwardingJavaFileManager<StandardJavaFileManager> {
 
         /**
-         * The class (only one class is kept).
+         * We use map because there can be nested, anonymous etc classes.
          */
-        JavaClassObject classObject;
+        Map<String, JavaClassObject> classObjectsByName = new HashMap<>();
+
+        private SecureClassLoader classLoader = new SecureClassLoader() {
+
+            @Override
+            protected Class<?> findClass(String name)
+                    throws ClassNotFoundException {
+                byte[] bytes = classObjectsByName.get(name).getBytes();
+                return super.defineClass(name, bytes, 0,
+                        bytes.length);
+            }
+        };
 
         public ClassFileManager(StandardJavaFileManager standardManager) {
             super(standardManager);
@@ -512,21 +587,14 @@ public class SourceCompiler {
 
         @Override
         public ClassLoader getClassLoader(Location location) {
-            return new SecureClassLoader() {
-                @Override
-                protected Class<?> findClass(String name)
-                        throws ClassNotFoundException {
-                    byte[] bytes = classObject.getBytes();
-                    return super.defineClass(name, bytes, 0,
-                            bytes.length);
-                }
-            };
+            return this.classLoader;
         }
 
         @Override
         public JavaFileObject getJavaFileForOutput(Location location,
                 String className, Kind kind, FileObject sibling) throws IOException {
-            classObject = new JavaClassObject(className, kind);
+            JavaClassObject classObject = new JavaClassObject(className, kind);
+            classObjectsByName.put(className, classObject);
             return classObject;
         }
     }
