@@ -6,10 +6,8 @@
 package org.h2.value;
 
 import org.h2.api.ErrorCode;
-import org.h2.engine.Constants;
 import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
-import org.h2.util.StringUtils;
 import org.h2.util.Utils;
 
 /**
@@ -21,11 +19,6 @@ public final class ValueJavaObject extends ValueBytesBase {
 
     private ValueJavaObject(byte[] v) {
         super(v);
-        int length = value.length;
-        if (length > Constants.MAX_STRING_LENGTH) {
-            throw DbException.getValueTooLongException(getTypeName(getValueType()),
-                    StringUtils.convertBytesToHex(value, 41), length);
-        }
     }
 
     /**
