@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.samples;
@@ -25,12 +25,13 @@ public class DirectInsert {
      * command line.
      *
      * @param args the command line parameters
+     * @throws Exception on failure
      */
     public static void main(String... args) throws Exception {
         Class.forName("org.h2.Driver");
         DeleteDbFiles.execute("~", "test", true);
         String url = "jdbc:h2:~/test";
-        initialInsert(url, 200000);
+        initialInsert(url, 200_000);
         for (int i = 0; i < 3; i++) {
             createAsSelect(url, true);
             createAsSelect(url, false);
