@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.samples;
@@ -23,6 +23,7 @@ public class MixedMode {
      * command line.
      *
      * @param args the command line parameters
+     * @throws Exception on failure
      */
     public static void main(String... args) throws Exception {
 
@@ -50,7 +51,7 @@ public class MixedMode {
         try {
             while (true) {
                 // runs forever, except if you drop the table remotely
-                stat.execute("MERGE INTO TIMER VALUES(1, NOW())");
+                stat.execute("MERGE INTO TIMER VALUES(1, LOCALTIME)");
                 Thread.sleep(1000);
             }
         } catch (SQLException e) {
