@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.samples;
@@ -13,6 +13,7 @@ import java.sql.Statement;
 import java.util.concurrent.TimeUnit;
 
 import org.h2.api.DatabaseEventListener;
+import org.h2.engine.SessionLocal;
 import org.h2.jdbc.JdbcConnection;
 
 /**
@@ -37,6 +38,7 @@ public class ShowProgress implements DatabaseEventListener {
      * command line.
      *
      * @param args the command line parameters
+     * @throws Exception on failure
      */
     public static void main(String... args) throws Exception {
         new ShowProgress().test();
@@ -47,7 +49,7 @@ public class ShowProgress implements DatabaseEventListener {
      */
     void test() throws Exception {
         Class.forName("org.h2.Driver");
-        Connection conn = DriverManager.getConnection("jdbc:h2:test", "sa", "");
+        Connection conn = DriverManager.getConnection("jdbc:h2:./test", "sa", "");
         Statement stat = conn.createStatement();
         stat.execute("DROP TABLE IF EXISTS TEST");
         stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, NAME VARCHAR)");
@@ -67,7 +69,7 @@ public class ShowProgress implements DatabaseEventListener {
         }
         boolean abnormalTermination = true;
         if (abnormalTermination) {
-            ((JdbcConnection) conn).setPowerOffCount(1);
+            ((SessionLocal) ((JdbcConnection) conn).getSession()).getDatabase().setPowerOffCount(1);
             try {
                 stat.execute("INSERT INTO TEST VALUES(-1, 'Test' || SPACE(100))");
             } catch (SQLException e) {
@@ -80,7 +82,7 @@ public class ShowProgress implements DatabaseEventListener {
         System.out.println("Open connection...");
         time = System.nanoTime();
         conn = DriverManager.getConnection(
-                "jdbc:h2:test;DATABASE_EVENT_LISTENER='" +
+                "jdbc:h2:./test;DATABASE_EVENT_LISTENER='" +
                 getClass().getName() + "'", "sa", "");
         time = System.nanoTime() - time;
         System.out.println("Done after " + TimeUnit.NANOSECONDS.toMillis(time) + " ms");
@@ -112,7 +114,7 @@ public class ShowProgress implements DatabaseEventListener {
      * @param max the 100% mark
      */
     @Override
-    public void setProgress(int state, String name, int current, int max) {
+    public void setProgress(int state, String name, long current, long max) {
         long time = System.nanoTime();
         if (time < lastNs + TimeUnit.SECONDS.toNanos(5)) {
             return;
