@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test;
@@ -8,16 +8,22 @@ package org.h2.test;
 import java.lang.management.ManagementFactory;
 import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Map.Entry;
 import java.util.Properties;
 import java.util.TimerTask;
 import java.util.concurrent.TimeUnit;
 import org.h2.Driver;
 import org.h2.engine.Constants;
-import org.h2.store.fs.FilePathRec;
 import org.h2.store.fs.FileUtils;
+import org.h2.store.fs.rec.FilePathRec;
+import org.h2.test.auth.TestAuthentication;
 import org.h2.test.bench.TestPerformance;
 import org.h2.test.db.TestAlter;
 import org.h2.test.db.TestAlterSchemaRename;
+import org.h2.test.db.TestAlterTableNotFound;
+import org.h2.test.db.TestAnalyzeTableTx;
 import org.h2.test.db.TestAutoRecompile;
 import org.h2.test.db.TestBackup;
 import org.h2.test.db.TestBigDb;
@@ -27,10 +33,10 @@ import org.h2.test.db.TestCheckpoint;
 import org.h2.test.db.TestCluster;
 import org.h2.test.db.TestCompatibility;
 import org.h2.test.db.TestCompatibilityOracle;
+import org.h2.test.db.TestCompatibilitySQLServer;
 import org.h2.test.db.TestCsv;
 import org.h2.test.db.TestDateStorage;
 import org.h2.test.db.TestDeadlock;
-import org.h2.test.db.TestDrop;
 import org.h2.test.db.TestDuplicateKeyUpdate;
 import org.h2.test.db.TestEncryptedDb;
 import org.h2.test.db.TestExclusive;
@@ -38,6 +44,7 @@ import org.h2.test.db.TestFullText;
 import org.h2.test.db.TestFunctionOverload;
 import org.h2.test.db.TestFunctions;
 import org.h2.test.db.TestGeneralCommonTableQueries;
+import org.h2.test.db.TestIgnoreCatalogs;
 import org.h2.test.db.TestIndex;
 import org.h2.test.db.TestIndexHints;
 import org.h2.test.db.TestLargeBlob;
@@ -52,61 +59,48 @@ import org.h2.test.db.TestMultiThread;
 import org.h2.test.db.TestMultiThreadedKernel;
 import org.h2.test.db.TestOpenClose;
 import org.h2.test.db.TestOptimizations;
-import org.h2.test.db.TestOptimizerHints;
 import org.h2.test.db.TestOutOfMemory;
 import org.h2.test.db.TestPersistentCommonTableExpressions;
 import org.h2.test.db.TestPowerOff;
 import org.h2.test.db.TestQueryCache;
 import org.h2.test.db.TestReadOnly;
 import org.h2.test.db.TestRecursiveQueries;
-import org.h2.test.db.TestReplace;
 import org.h2.test.db.TestRights;
-import org.h2.test.db.TestRowFactory;
 import org.h2.test.db.TestRunscript;
 import org.h2.test.db.TestSQLInjection;
-import org.h2.test.db.TestSelectCountNonNullColumn;
+import org.h2.test.db.TestSelectTableNotFound;
 import org.h2.test.db.TestSequence;
 import org.h2.test.db.TestSessionsLocks;
 import org.h2.test.db.TestSetCollation;
-import org.h2.test.db.TestShow;
 import org.h2.test.db.TestSpaceReuse;
 import org.h2.test.db.TestSpatial;
 import org.h2.test.db.TestSpeed;
+import org.h2.test.db.TestSubqueryPerformanceOnLazyExecutionMode;
 import org.h2.test.db.TestSynonymForTable;
 import org.h2.test.db.TestTableEngines;
 import org.h2.test.db.TestTempTables;
 import org.h2.test.db.TestTransaction;
 import org.h2.test.db.TestTriggersConstraints;
 import org.h2.test.db.TestTwoPhaseCommit;
-import org.h2.test.db.TestUpgrade;
-import org.h2.test.db.TestUsingIndex;
 import org.h2.test.db.TestView;
 import org.h2.test.db.TestViewAlterTable;
 import org.h2.test.db.TestViewDropView;
-import org.h2.test.jaqu.AliasMapTest;
-import org.h2.test.jaqu.AnnotationsTest;
-import org.h2.test.jaqu.ClobTest;
-import org.h2.test.jaqu.ModelsTest;
-import org.h2.test.jaqu.SamplesTest;
-import org.h2.test.jaqu.UpdateTest;
 import org.h2.test.jdbc.TestBatchUpdates;
 import org.h2.test.jdbc.TestCallableStatement;
 import org.h2.test.jdbc.TestCancel;
 import org.h2.test.jdbc.TestConcurrentConnectionUsage;
 import org.h2.test.jdbc.TestConnection;
-import org.h2.test.jdbc.TestCustomDataTypesHandler;
 import org.h2.test.jdbc.TestDatabaseEventListener;
 import org.h2.test.jdbc.TestDriver;
 import org.h2.test.jdbc.TestGetGeneratedKeys;
-import org.h2.test.jdbc.TestJavaObject;
 import org.h2.test.jdbc.TestJavaObjectSerializer;
-import org.h2.test.jdbc.TestLimitUpdates;
 import org.h2.test.jdbc.TestLobApi;
 import org.h2.test.jdbc.TestManyJdbcObjects;
 import org.h2.test.jdbc.TestMetaData;
 import org.h2.test.jdbc.TestNativeSQL;
 import org.h2.test.jdbc.TestPreparedStatement;
 import org.h2.test.jdbc.TestResultSet;
+import org.h2.test.jdbc.TestSQLXML;
 import org.h2.test.jdbc.TestStatement;
 import org.h2.test.jdbc.TestTransactionIsolation;
 import org.h2.test.jdbc.TestUpdatableResultSet;
@@ -126,22 +120,22 @@ import org.h2.test.poweroff.TestReorderWrites;
 import org.h2.test.recover.RecoverLobTest;
 import org.h2.test.rowlock.TestRowLocks;
 import org.h2.test.scripts.TestScript;
-import org.h2.test.scripts.TestScriptSimple;
 import org.h2.test.server.TestAutoServer;
 import org.h2.test.server.TestInit;
+import org.h2.test.server.TestJakartaWeb;
 import org.h2.test.server.TestNestedLoop;
 import org.h2.test.server.TestWeb;
 import org.h2.test.store.TestCacheConcurrentLIRS;
 import org.h2.test.store.TestCacheLIRS;
 import org.h2.test.store.TestCacheLongKeyLIRS;
-import org.h2.test.store.TestConcurrent;
-import org.h2.test.store.TestConcurrentLinkedList;
 import org.h2.test.store.TestDataUtils;
+import org.h2.test.store.TestDefrag;
 import org.h2.test.store.TestFreeSpace;
 import org.h2.test.store.TestKillProcessWhileWriting;
 import org.h2.test.store.TestMVRTree;
 import org.h2.test.store.TestMVStore;
 import org.h2.test.store.TestMVStoreBenchmark;
+import org.h2.test.store.TestMVStoreConcurrent;
 import org.h2.test.store.TestMVStoreStopCompact;
 import org.h2.test.store.TestMVStoreTool;
 import org.h2.test.store.TestMVTableEngine;
@@ -166,46 +160,46 @@ import org.h2.test.synth.TestNestedJoins;
 import org.h2.test.synth.TestOuterJoins;
 import org.h2.test.synth.TestRandomCompare;
 import org.h2.test.synth.TestRandomSQL;
-import org.h2.test.synth.TestStringAggCompatibility;
 import org.h2.test.synth.TestTimer;
 import org.h2.test.synth.sql.TestSynth;
 import org.h2.test.synth.thread.TestMulti;
 import org.h2.test.unit.TestAnsCompression;
 import org.h2.test.unit.TestAutoReconnect;
 import org.h2.test.unit.TestBinaryArithmeticStream;
-import org.h2.test.unit.TestBitField;
+import org.h2.test.unit.TestBinaryOperation;
 import org.h2.test.unit.TestBitStream;
 import org.h2.test.unit.TestBnf;
 import org.h2.test.unit.TestCache;
 import org.h2.test.unit.TestCharsetCollator;
-import org.h2.test.unit.TestClearReferences;
 import org.h2.test.unit.TestCollation;
 import org.h2.test.unit.TestCompress;
+import org.h2.test.unit.TestConcurrentJdbc;
 import org.h2.test.unit.TestConnectionInfo;
-import org.h2.test.unit.TestDataPage;
 import org.h2.test.unit.TestDate;
 import org.h2.test.unit.TestDateIso8601;
 import org.h2.test.unit.TestDateTimeUtils;
+import org.h2.test.unit.TestDbException;
 import org.h2.test.unit.TestExit;
 import org.h2.test.unit.TestFile;
 import org.h2.test.unit.TestFileLock;
 import org.h2.test.unit.TestFileLockProcess;
-import org.h2.test.unit.TestFileLockSerialized;
 import org.h2.test.unit.TestFileSystem;
 import org.h2.test.unit.TestFtp;
+import org.h2.test.unit.TestGeometryUtils;
 import org.h2.test.unit.TestIntArray;
-import org.h2.test.unit.TestIntIntHashMap;
 import org.h2.test.unit.TestIntPerfectHash;
+import org.h2.test.unit.TestInterval;
 import org.h2.test.unit.TestJmx;
+import org.h2.test.unit.TestJsonUtils;
+import org.h2.test.unit.TestKeywords;
 import org.h2.test.unit.TestLocale;
+import org.h2.test.unit.TestMVTempResult;
 import org.h2.test.unit.TestMathUtils;
+import org.h2.test.unit.TestMemoryUnmapper;
 import org.h2.test.unit.TestMode;
-import org.h2.test.unit.TestModifyOnWrite;
 import org.h2.test.unit.TestNetUtils;
 import org.h2.test.unit.TestObjectDeserialization;
-import org.h2.test.unit.TestOldVersion;
 import org.h2.test.unit.TestOverflow;
-import org.h2.test.unit.TestPageStore;
 import org.h2.test.unit.TestPageStoreCoverage;
 import org.h2.test.unit.TestPattern;
 import org.h2.test.unit.TestPerfectHash;
@@ -224,17 +218,15 @@ import org.h2.test.unit.TestStringUtils;
 import org.h2.test.unit.TestTimeStampWithTimeZone;
 import org.h2.test.unit.TestTools;
 import org.h2.test.unit.TestTraceSystem;
+import org.h2.test.unit.TestUpgrade;
 import org.h2.test.unit.TestUtils;
 import org.h2.test.unit.TestValue;
-import org.h2.test.unit.TestValueHashMap;
 import org.h2.test.unit.TestValueMemory;
 import org.h2.test.utils.OutputCatcher;
 import org.h2.test.utils.SelfDestructor;
-import org.h2.test.utils.TestColumnNamer;
 import org.h2.tools.DeleteDbFiles;
 import org.h2.tools.Server;
 import org.h2.util.AbbaLockingDetector;
-import org.h2.util.New;
 import org.h2.util.Profiler;
 import org.h2.util.StringUtils;
 import org.h2.util.Task;
@@ -285,11 +277,6 @@ java org.h2.test.TestAll timer
      */
     static boolean atLeastOneTestFailed;
 
-    /**
-     * Whether the MVStore storage is used.
-     */
-    public boolean mvStore = Constants.VERSION_MINOR >= 4;
-
     /**
      * If the test should run with many rows.
      */
@@ -310,16 +297,6 @@ java org.h2.test.TestAll timer
      */
     public boolean codeCoverage;
 
-    /**
-     * If the multi version concurrency control mode should be used.
-     */
-    public boolean mvcc = mvStore;
-
-    /**
-     * If the multi-threaded mode should be used.
-     */
-    public boolean multiThreaded;
-
     /**
      * If lazy queries should be used.
      */
@@ -361,9 +338,9 @@ java org.h2.test.TestAll timer
     public boolean splitFileSystem;
 
     /**
-     * If only fast/CI/Jenkins/Travis tests should be run.
+     * If only fast CI tests should be run.
      */
-    public boolean travis;
+    public boolean ci;
 
     /**
      * the vmlens.com race condition tool
@@ -404,7 +381,7 @@ java org.h2.test.TestAll timer
     /**
      * The THROTTLE value to use.
      */
-    int throttle;
+    public int throttle;
 
     /**
      * The THROTTLE value to use by default.
@@ -416,11 +393,6 @@ java org.h2.test.TestAll timer
      */
     boolean stopOnError;
 
-    /**
-     * If the database should always be defragmented when closing.
-     */
-    boolean defrag;
-
     /**
      * The cache type.
      */
@@ -438,10 +410,17 @@ java org.h2.test.TestAll timer
     /**
      * The list of tests.
      */
-    ArrayList<TestBase> tests = New.arrayList();
+    ArrayList<TestBase> tests = new ArrayList<>();
 
     private Server server;
 
+    HashSet<String> excludedTests = new HashSet<>();
+
+    /**
+     * The map of executed tests to detect not executed tests.
+     * Boolean value is 'false' for a disabled test.
+     */
+    HashMap<Class<? extends TestBase>, Boolean> executedTests = new HashMap<>();
 
     /**
      * Run all tests.
@@ -468,7 +447,6 @@ java org.h2.test.TestAll timer
 
         System.setProperty("h2.maxMemoryRows", "100");
 
-        System.setProperty("h2.check2", "true");
         System.setProperty("h2.delayWrongPasswordMin", "0");
         System.setProperty("h2.delayWrongPasswordMax", "0");
         System.setProperty("h2.useThreadContextClassLoader", "true");
@@ -486,7 +464,6 @@ reopen org.h2.test.unit.TestPageStore
 -Xmx1500m -D reopenOffset=3 -D reopenShift=1
 
 power failure test
-power failure test: MULTI_THREADED=TRUE
 power failure test: larger binaries and additional index.
 power failure test with randomly generating / dropping indexes and tables.
 
@@ -500,23 +477,20 @@ documentation: rolling review at main.html
 
 -------------
 
-remove old TODO, move to roadmap
-
 kill a test:
 kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
 
 */
         TestAll test = new TestAll();
         if (args.length > 0) {
-            if ("travis".equals(args[0])) {
-                test.travis = true;
-                test.testAll();
+            if ("ci".equals(args[0])) {
+                test.ci = true;
+                test.testAll(args, 1);
             } else if ("vmlens".equals(args[0])) {
                 test.vmlens = true;
-                test.testAll();
+                test.testAll(args, 1);
             } else if ("reopen".equals(args[0])) {
                 System.setProperty("h2.delayWrongPasswordMin", "0");
-                System.setProperty("h2.check2", "false");
                 System.setProperty("h2.analyzeAuto", "100");
                 System.setProperty("h2.pageSize", "64");
                 System.setProperty("h2.reopenShift", "5");
@@ -554,15 +528,24 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
                 new TestTimer().runTest(test);
             }
         } else {
-            test.testAll();
+            test.testAll(args, 0);
         }
-        System.out.println(TestBase.formatTime(
-                TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - time)) + " total");
+        System.out.println(TestBase.formatTime(new StringBuilder(),
+                TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - time)).append(" total").toString());
     }
 
-    private void testAll() throws Exception {
+    private void testAll(String[] args, int offset) throws Exception {
+        int l = args.length;
+        while (l > offset + 1) {
+            if ("-exclude".equals(args[offset])) {
+                excludedTests.add(args[offset + 1]);
+                offset += 2;
+            } else {
+                break;
+            }
+        }
         runTests();
-        if (!travis && !vmlens) {
+        if (!ci && !vmlens) {
             Profiler prof = new Profiler();
             prof.depth = 16;
             prof.interval = 1;
@@ -608,49 +591,49 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             abbaLockingDetector = new AbbaLockingDetector().startCollecting();
         }
 
-        smallLog = big = networked = memory = ssl = false;
+        smallLog = big = networked = memory = lazy = ssl = false;
         diskResult = traceSystemOut = diskUndo = false;
         traceTest = stopOnError = false;
-        defrag = false;
         traceLevelFile = throttle = 0;
         cipher = null;
 
         // memory is a good match for multi-threaded, makes things happen
         // faster, more chance of exposing race conditions
         memory = true;
-        multiThreaded = true;
         test();
         if (vmlens) {
             return;
         }
-        testUnit();
+        testAdditional();
+
+        // test utilities
+        big = !ci;
+        testUtils();
+        big = false;
 
         // lazy
         lazy = true;
         memory = true;
-        multiThreaded = true;
         test();
         lazy = false;
 
         // but sometimes race conditions need bigger windows
         memory = false;
-        multiThreaded = true;
         test();
-        testUnit();
+        testAdditional();
 
-        // a more normal setup
-        memory = false;
-        multiThreaded = false;
-        test();
-        testUnit();
+        networked = true;
 
         memory = true;
-        multiThreaded = false;
-        networked = true;
         test();
-
         memory = false;
+
+        lazy = true;
+        test();
+        lazy = false;
+
         networked = false;
+
         diskUndo = true;
         diskResult = true;
         traceLevelFile = 3;
@@ -665,14 +648,11 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         throttle = 0;
         cacheType = null;
         cipher = null;
-        defrag = true;
-        test();
 
-        if (!travis) {
+        if (!ci) {
             traceLevelFile = 0;
             smallLog = true;
             networked = true;
-            defrag = false;
             ssl = true;
             test();
 
@@ -682,7 +662,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             ssl = false;
             traceLevelFile = 0;
             test();
-            testUnit();
+            testAdditional();
 
             big = false;
             cipher = "AES";
@@ -690,24 +670,26 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             cipher = null;
             test();
         }
+
+        for (Entry<Class<? extends TestBase>, Boolean> entry : executedTests.entrySet()) {
+            if (!entry.getValue()) {
+                System.out.println("Warning: test " + entry.getKey().getName() + " was not executed.");
+            }
+        }
     }
 
     private void runCoverage() throws SQLException {
         smallLog = big = networked = memory = ssl = false;
         diskResult = traceSystemOut = diskUndo = false;
         traceTest = stopOnError = false;
-        defrag = false;
         traceLevelFile = throttle = 0;
         cipher = null;
 
         memory = true;
-        multiThreaded = true;
         test();
-        testUnit();
+        testAdditional();
+        testUtils();
 
-        multiThreaded = false;
-        mvStore = false;
-        mvcc = false;
         test();
         // testUnit();
     }
@@ -720,170 +702,205 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         System.out.println("Test " + toString() +
                 " (" + Utils.getMemoryUsed() + " KB used)");
         beforeTest();
+        try {
+            // db
+            addTest(new TestScript());
+            addTest(new TestAlter());
+            addTest(new TestAlterSchemaRename());
+            addTest(new TestAutoRecompile());
+            addTest(new TestBackup());
+            addTest(new TestBigDb());
+            addTest(new TestBigResult());
+            addTest(new TestCases());
+            addTest(new TestCheckpoint());
+            addTest(new TestCompatibility());
+            addTest(new TestCompatibilityOracle());
+            addTest(new TestCompatibilitySQLServer());
+            addTest(new TestCsv());
+            addTest(new TestDeadlock());
+            if (vmlens) {
+                return;
+            }
+            addTest(new TestDuplicateKeyUpdate());
+            addTest(new TestEncryptedDb());
+            addTest(new TestExclusive());
+            addTest(new TestFullText());
+            addTest(new TestFunctionOverload());
+            addTest(new TestFunctions());
+            addTest(new TestInit());
+            addTest(new TestIndex());
+            addTest(new TestIndexHints());
+            addTest(new TestLargeBlob());
+            addTest(new TestLinkedTable());
+            addTest(new TestListener());
+            addTest(new TestLob());
+            addTest(new TestMergeUsing());
+            addTest(new TestMultiConn());
+            addTest(new TestMultiDimension());
+            addTest(new TestMultiThreadedKernel());
+            addTest(new TestOpenClose());
+            addTest(new TestReadOnly());
+            addTest(new TestRecursiveQueries());
+            addTest(new TestGeneralCommonTableQueries());
+            addTest(new TestAlterTableNotFound());
+            addTest(new TestSelectTableNotFound());
+            if (!memory) {
+                // requires persistent store for reconnection tests
+                addTest(new TestPersistentCommonTableExpressions());
+            }
+            addTest(new TestRights());
+            addTest(new TestRunscript());
+            addTest(new TestSQLInjection());
+            addTest(new TestSessionsLocks());
+            addTest(new TestSequence());
+            addTest(new TestSpaceReuse());
+            addTest(new TestSpatial());
+            addTest(new TestSpeed());
+            addTest(new TestTableEngines());
+            addTest(new TestTempTables());
+            addTest(new TestTransaction());
+            addTest(new TestTriggersConstraints());
+            addTest(new TestTwoPhaseCommit());
+            addTest(new TestView());
+            addTest(new TestViewAlterTable());
+            addTest(new TestViewDropView());
+            addTest(new TestSynonymForTable());
 
-        // db
-        addTest(new TestScriptSimple());
-        addTest(new TestScript());
-        addTest(new TestAlter());
-        addTest(new TestAlterSchemaRename());
-        addTest(new TestAutoRecompile());
-        addTest(new TestBackup());
-        addTest(new TestBigDb());
-        addTest(new TestBigResult());
-        addTest(new TestCases());
-        addTest(new TestCheckpoint());
-        addTest(new TestCompatibility());
-        addTest(new TestCompatibilityOracle());
-        addTest(new TestCsv());
-        addTest(new TestDeadlock());
-        if (vmlens) {
-            return;
+            // jdbc
+            addTest(new TestBatchUpdates());
+            addTest(new TestCallableStatement());
+            addTest(new TestCancel());
+            addTest(new TestConcurrentConnectionUsage());
+            addTest(new TestConnection());
+            addTest(new TestDatabaseEventListener());
+            addTest(new TestLobApi());
+            addTest(new TestSQLXML());
+            addTest(new TestManyJdbcObjects());
+            addTest(new TestMetaData());
+            addTest(new TestNativeSQL());
+            addTest(new TestPreparedStatement());
+            addTest(new TestResultSet());
+            addTest(new TestStatement());
+            addTest(new TestGetGeneratedKeys());
+            addTest(new TestTransactionIsolation());
+            addTest(new TestUpdatableResultSet());
+            addTest(new TestZloty());
+            addTest(new TestSetCollation());
+
+            // jdbcx
+            addTest(new TestConnectionPool());
+            addTest(new TestDataSource());
+            addTest(new TestXA());
+            addTest(new TestXASimple());
+
+            // server
+            addTest(new TestAutoServer());
+            addTest(new TestNestedLoop());
+
+            // mvcc & row level locking
+            addTest(new TestMvcc1());
+            addTest(new TestMvcc2());
+            addTest(new TestMvcc3());
+            addTest(new TestMvcc4());
+            addTest(new TestMvccMultiThreaded());
+            addTest(new TestMvccMultiThreaded2());
+            addTest(new TestRowLocks());
+            addTest(new TestAnalyzeTableTx());
+
+            // synth
+            addTest(new TestBtreeIndex());
+            addTest(new TestConcurrentUpdate());
+            addTest(new TestDiskFull());
+            addTest(new TestCrashAPI());
+            addTest(new TestFuzzOptimizations());
+            addTest(new TestLimit());
+            addTest(new TestRandomCompare());
+            addTest(new TestKillRestart());
+            addTest(new TestKillRestartMulti());
+            addTest(new TestMultiThreaded());
+            addTest(new TestOuterJoins());
+            addTest(new TestNestedJoins());
+
+            runAddedTests();
+
+            // serial
+            addTest(new TestDateStorage());
+            addTest(new TestDriver());
+            addTest(new TestJavaObjectSerializer());
+            addTest(new TestLocale());
+            addTest(new TestMemoryUsage());
+            addTest(new TestMultiThread());
+            addTest(new TestPowerOff());
+            addTest(new TestReorderWrites());
+            addTest(new TestRandomSQL());
+            addTest(new TestQueryCache());
+            addTest(new TestUrlJavaObjectSerializer());
+            addTest(new TestWeb());
+            addTest(new TestJakartaWeb());
+
+            // other unsafe
+            addTest(new TestOptimizations());
+            addTest(new TestOutOfMemory());
+            addTest(new TestIgnoreCatalogs());
+
+
+            runAddedTests(1);
+        } finally {
+            afterTest();
         }
-        addTest(new TestDrop());
-        addTest(new TestDuplicateKeyUpdate());
-        addTest(new TestEncryptedDb());
-        addTest(new TestExclusive());
-        addTest(new TestFullText());
-        addTest(new TestFunctionOverload());
-        addTest(new TestFunctions());
-        addTest(new TestInit());
-        addTest(new TestIndex());
-        addTest(new TestIndexHints());
-        addTest(new TestLargeBlob());
-        addTest(new TestLinkedTable());
-        addTest(new TestListener());
-        addTest(new TestLob());
-        addTest(new TestMergeUsing());
-        addTest(new TestMultiConn());
-        addTest(new TestMultiDimension());
+    }
+
+    /**
+     * Run additional tests.
+     */
+    private void testAdditional() {
+        if (networked) {
+            throw new RuntimeException("testAdditional() is not allowed in networked mode");
+        }
+
+        addTest(new TestMVTableEngine());
+        addTest(new TestAutoReconnect());
+        addTest(new TestBnf());
+        addTest(new TestCache());
+        addTest(new TestCollation());
+        addTest(new TestCompress());
+        addTest(new TestConnectionInfo());
+        addTest(new TestExit());
+        addTest(new TestFileLock());
+        addTest(new TestJmx());
         addTest(new TestMultiThreadedKernel());
-        addTest(new TestOpenClose());
-        addTest(new TestOptimizations());
-        addTest(new TestOptimizerHints());
-        addTest(new TestOutOfMemory());
-        addTest(new TestReadOnly());
-        addTest(new TestRecursiveQueries());
-        addTest(new TestGeneralCommonTableQueries());
-        if (!memory) {
-            // requires persistent store for reconnection tests
-            addTest(new TestPersistentCommonTableExpressions());
-        }
-        addTest(new TestRights());
-        addTest(new TestRunscript());
-        addTest(new TestSQLInjection());
-        addTest(new TestSessionsLocks());
-        addTest(new TestSelectCountNonNullColumn());
-        addTest(new TestSequence());
-        addTest(new TestShow());
-        addTest(new TestSpaceReuse());
-        addTest(new TestSpatial());
-        addTest(new TestSpeed());
-        addTest(new TestTableEngines());
-        addTest(new TestRowFactory());
-        addTest(new TestTempTables());
-        addTest(new TestTransaction());
-        addTest(new TestTriggersConstraints());
-        addTest(new TestTwoPhaseCommit());
-        addTest(new TestView());
-        addTest(new TestViewAlterTable());
-        addTest(new TestViewDropView());
-        addTest(new TestReplace());
-        addTest(new TestSynonymForTable());
-        addTest(new TestColumnNamer());
-
-
-        // jaqu
-        addTest(new AliasMapTest());
-        addTest(new AnnotationsTest());
-        addTest(new ClobTest());
-        addTest(new ModelsTest());
-        addTest(new SamplesTest());
-        addTest(new UpdateTest());
-
-        // jdbc
-        addTest(new TestBatchUpdates());
-        addTest(new TestCallableStatement());
-        addTest(new TestCancel());
-        addTest(new TestConcurrentConnectionUsage());
-        addTest(new TestConnection());
-        addTest(new TestDatabaseEventListener());
-        addTest(new TestJavaObject());
-        addTest(new TestLimitUpdates());
-        addTest(new TestLobApi());
-        addTest(new TestManyJdbcObjects());
-        addTest(new TestMetaData());
-        addTest(new TestNativeSQL());
-        addTest(new TestPreparedStatement());
-        addTest(new TestResultSet());
-        addTest(new TestStatement());
-        addTest(new TestGetGeneratedKeys());
-        addTest(new TestTransactionIsolation());
-        addTest(new TestUpdatableResultSet());
-        addTest(new TestZloty());
-        addTest(new TestCustomDataTypesHandler());
-        addTest(new TestSetCollation());
-
-        // jdbcx
-        addTest(new TestConnectionPool());
-        addTest(new TestDataSource());
-        addTest(new TestXA());
-        addTest(new TestXASimple());
-
-        // server
-        addTest(new TestAutoServer());
-        addTest(new TestNestedLoop());
-
-        // mvcc & row level locking
-        addTest(new TestMvcc1());
-        addTest(new TestMvcc2());
-        addTest(new TestMvcc3());
-        addTest(new TestMvcc4());
-        addTest(new TestMvccMultiThreaded());
-        addTest(new TestMvccMultiThreaded2());
-        addTest(new TestRowLocks());
-
-        // synth
-        addTest(new TestBtreeIndex());
-        addTest(new TestConcurrentUpdate());
-        addTest(new TestDiskFull());
-        addTest(new TestCrashAPI());
-        addTest(new TestFuzzOptimizations());
-        addTest(new TestLimit());
-        addTest(new TestRandomCompare());
-        addTest(new TestKillRestart());
-        addTest(new TestKillRestartMulti());
-        addTest(new TestMultiThreaded());
-        addTest(new TestOuterJoins());
-        addTest(new TestNestedJoins());
-        addTest(new TestStringAggCompatibility());
+        addTest(new TestPageStoreCoverage());
+        addTest(new TestPgServer());
+        addTest(new TestRecovery());
+        addTest(new RecoverLobTest());
+        addTest(createTest("org.h2.test.unit.TestServlet"));
+        addTest(createTest("org.h2.test.unit.TestJakartaServlet"));
+        addTest(new TestTimeStampWithTimeZone());
+        addTest(new TestValue());
 
         runAddedTests();
 
-        // serial
-        addTest(new TestDateStorage());
-        addTest(new TestDriver());
-        addTest(new TestJavaObjectSerializer());
-        addTest(new TestLocale());
-        addTest(new TestMemoryUsage());
-        addTest(new TestMultiThread());
-        addTest(new TestPowerOff());
-        addTest(new TestReorderWrites());
-        addTest(new TestRandomSQL());
-        addTest(new TestQueryCache());
-        addTest(new TestUrlJavaObjectSerializer());
-        addTest(new TestWeb());
+        addTest(new TestCluster());
+        addTest(new TestFileLockProcess());
+        addTest(new TestDefrag());
+        addTest(new TestTools());
+        addTest(new TestSampleApps());
+        addTest(new TestSubqueryPerformanceOnLazyExecutionMode());
 
         runAddedTests(1);
-
-        afterTest();
     }
 
-    private void testUnit() {
+    /**
+     * Run tests for utilities.
+     */
+    private void testUtils() {
+        System.out.println();
+        System.out.println("Test utilities (" + Utils.getMemoryUsed() + " KB used)");
+
         // mv store
         addTest(new TestCacheConcurrentLIRS());
         addTest(new TestCacheLIRS());
         addTest(new TestCacheLongKeyLIRS());
-        addTest(new TestConcurrentLinkedList());
         addTest(new TestDataUtils());
         addTest(new TestFreeSpace());
         addTest(new TestKillProcessWhileWriting());
@@ -892,89 +909,67 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestMVStoreBenchmark());
         addTest(new TestMVStoreStopCompact());
         addTest(new TestMVStoreTool());
-        addTest(new TestMVTableEngine());
         addTest(new TestObjectDataType());
         addTest(new TestRandomMapOps());
         addTest(new TestSpinLock());
         addTest(new TestStreamStore());
         addTest(new TestTransactionStore());
+        addTest(new TestMVTempResult());
 
         // unit
+        addTest(new TestConcurrentJdbc());
         addTest(new TestAnsCompression());
-        addTest(new TestAutoReconnect());
         addTest(new TestBinaryArithmeticStream());
-        addTest(new TestBitField());
+        addTest(new TestBinaryOperation());
         addTest(new TestBitStream());
-        addTest(new TestBnf());
-        addTest(new TestCache());
         addTest(new TestCharsetCollator());
-        addTest(new TestClearReferences());
-        addTest(new TestCollation());
-        addTest(new TestCompress());
-        addTest(new TestConnectionInfo());
-        addTest(new TestDataPage());
         addTest(new TestDateIso8601());
-        addTest(new TestExit());
+        addTest(new TestDbException());
         addTest(new TestFile());
-        addTest(new TestFileLock());
+        addTest(new TestFileSystem());
         addTest(new TestFtp());
+        addTest(new TestGeometryUtils());
+        addTest(new TestInterval());
         addTest(new TestIntArray());
-        addTest(new TestIntIntHashMap());
         addTest(new TestIntPerfectHash());
-        addTest(new TestJmx());
+        addTest(new TestJsonUtils());
+        addTest(new TestKeywords());
         addTest(new TestMathUtils());
+        addTest(new TestMemoryUnmapper());
         addTest(new TestMode());
-        addTest(new TestModifyOnWrite());
-        addTest(new TestOldVersion());
         addTest(new TestObjectDeserialization());
-        addTest(new TestMultiThreadedKernel());
         addTest(new TestOverflow());
-        addTest(new TestPageStore());
-        addTest(new TestPageStoreCoverage());
         addTest(new TestPerfectHash());
-        addTest(new TestPgServer());
         addTest(new TestReader());
-        addTest(new TestRecovery());
         addTest(new TestScriptReader());
-        addTest(new RecoverLobTest());
-        addTest(createTest("org.h2.test.unit.TestServlet"));
         addTest(new TestSecurity());
         addTest(new TestShell());
         addTest(new TestSort());
         addTest(new TestStreams());
         addTest(new TestStringUtils());
-        addTest(new TestTimeStampWithTimeZone());
         addTest(new TestTraceSystem());
-        addTest(new TestUpgrade());
-        addTest(new TestUsingIndex());
         addTest(new TestUtils());
-        addTest(new TestValue());
-        addTest(new TestValueHashMap());
-        addTest(new TestWeb());
-
+        addTest(new TestUpgrade());
 
         runAddedTests();
 
         // serial
         addTest(new TestDate());
         addTest(new TestDateTimeUtils());
-        addTest(new TestCluster());
-        addTest(new TestConcurrent());
-        addTest(new TestFileLockSerialized());
-        addTest(new TestFileLockProcess());
-        addTest(new TestFileSystem());
+        addTest(new TestMVStoreConcurrent());
         addTest(new TestNetUtils());
         addTest(new TestPattern());
-        addTest(new TestTools());
-        addTest(new TestSampleApps());
         addTest(new TestStringCache());
         addTest(new TestValueMemory());
+        addTest(new TestAuthentication());
 
         runAddedTests(1);
-
     }
 
     private void addTest(TestBase test) {
+        if (excludedTests.contains(test.getClass().getName())) {
+            return;
+        }
         // tests.add(test);
         // run directly for now, because concurrently running tests
         // fails on Raspberry Pi quite often (seems to be a JVM problem)
@@ -1015,7 +1010,9 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
                             }
                             test = tests.remove(0);
                         }
-                        test.runTest(TestAll.this);
+                        if (!excludedTests.contains(test.getClass().getName())) {
+                            test.runTest(TestAll.this);
+                        }
                     }
                 }
             };
@@ -1030,11 +1027,8 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
     private static TestBase createTest(String className) {
         try {
             Class<?> clazz = Class.forName(className);
-            return (TestBase) clazz.newInstance();
-        } catch (Exception e) {
-            // ignore
-            TestBase.printlnWithTime(0, className + " class not found");
-        } catch (NoClassDefFoundError e) {
+            return (TestBase) clazz.getDeclaredConstructor().newInstance();
+        } catch (Exception | NoClassDefFoundError e) {
             // ignore
             TestBase.printlnWithTime(0, className + " class not found");
         }
@@ -1059,7 +1053,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         DeleteDbFiles.execute(TestBase.BASE_TEST_DIR, null, true);
         FileUtils.deleteRecursive("trace.db", false);
         if (networked) {
-            String[] args = ssl ? new String[] { "-tcpSSL" } : new String[0];
+            String[] args = ssl ? new String[] { "-ifNotExists", "-tcpSSL" } : new String[] { "-ifNotExists" };
             server = Server.createTcpServer(args);
             try {
                 server.start();
@@ -1090,7 +1084,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
      */
     public static void printSystemInfo() {
         Properties prop = System.getProperties();
-        System.out.println("H2 " + Constants.getFullVersion() +
+        System.out.println("H2 " + Constants.FULL_VERSION +
                 " @ " + new java.sql.Timestamp(System.currentTimeMillis()).toString());
         System.out.println("Java " +
                 prop.getProperty("java.runtime.version") + ", " +
@@ -1116,13 +1110,10 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
     public String toString() {
         StringBuilder buff = new StringBuilder();
         appendIf(buff, lazy, "lazy");
-        appendIf(buff, mvStore, "mvStore");
         appendIf(buff, big, "big");
         appendIf(buff, networked, "net");
         appendIf(buff, memory, "memory");
         appendIf(buff, codeCoverage, "codeCoverage");
-        appendIf(buff, mvcc, "mvcc");
-        appendIf(buff, multiThreaded, "multiThreaded");
         appendIf(buff, cipher != null, cipher);
         appendIf(buff, cacheType != null, cacheType);
         appendIf(buff, smallLog, "smallLog");
@@ -1135,7 +1126,6 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         appendIf(buff, throttle > 0, "throttle:" + throttle);
         appendIf(buff, traceTest, "traceTest");
         appendIf(buff, stopOnError, "stopOnError");
-        appendIf(buff, defrag, "defrag");
         appendIf(buff, splitFileSystem, "split");
         appendIf(buff, collation != null, collation);
         return buff.toString();
