@@ -1,11 +1,11 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test;
 
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
 
 /**
  * This class is a bridge between JUnit and the custom test framework
@@ -17,7 +17,7 @@ public class TestAllJunit {
      * Run all the fast tests.
      */
     @Test
-    public void testFast() throws Exception {
-        TestAll.main("fast");
+    public void testCI() throws Exception {
+        TestAll.main("ci");
     }
 }
