@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.bench;
@@ -545,18 +545,13 @@ public class BenchC implements Bench {
 
     @Override
     public void runTest() throws SQLException {
-        database.start(this, "Transactions");
         database.openConnection();
+        database.start(this, "Transactions");
         for (int i = 0; i < 70; i++) {
             BenchCThread process = new BenchCThread(database, this, random, i);
             process.process();
         }
-        database.closeConnection();
         database.end();
-
-        database.openConnection();
-        BenchCThread process = new BenchCThread(database, this, random, 0);
-        process.process();
         database.logMemory(this, "Memory Usage");
         database.closeConnection();
     }
