@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.bench;
@@ -157,7 +157,7 @@ public class BenchCRandom {
      * @return the big decimal object
      */
     BigDecimal getBigDecimal(int value, int scale) {
-        return new BigDecimal(new BigInteger(String.valueOf(value)), scale);
+        return new BigDecimal(BigInteger.valueOf(value), scale);
     }
 
     /**
