@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.bench;
@@ -36,7 +36,7 @@ public class BenchSimple implements Bench {
             prep.setInt(1, i);
             prep.setString(2, "Hello World " + i);
             db.update(prep, "insertTest");
-            if (i % commitEvery == 0) {
+            if ((i+1) % commitEvery == 0) {
                 db.commit();
             }
         }
@@ -78,7 +78,7 @@ public class BenchSimple implements Bench {
         db.start(this, "Update (sequential)");
         prep = db.prepare("UPDATE TEST SET NAME=? WHERE ID=?");
         for (int i = 0; i < records; i += 3) {
-            prep.setString(1, "Hallo Welt");
+            prep.setString(1, "Hallo Welt " + i);
             prep.setInt(2, i);
             db.update(prep, "updateTest");
         }
@@ -92,18 +92,8 @@ public class BenchSimple implements Bench {
             db.update(prep, "deleteTest");
         }
         db.end();
-
-        db.closeConnection();
-
-        db.openConnection();
-        prep = db.prepare("SELECT * FROM TEST WHERE ID=?");
-        for (int i = 0; i < records; i++) {
-            prep.setInt(1, random.nextInt(records));
-            db.queryReadResult(prep);
-        }
         db.logMemory(this, "Memory Usage");
         db.closeConnection();
-
     }
 
     @Override
