@@ -1,12 +1,11 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.bench;
 
 import java.io.FileWriter;
-import java.io.InputStream;
 import java.io.PrintWriter;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
@@ -19,7 +18,6 @@ import java.util.ArrayList;
 import java.util.Properties;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
-import org.h2.util.IOUtils;
 import org.h2.util.JdbcUtils;
 
 /**
@@ -72,10 +70,7 @@ public class TestPerformance implements Database.DatabaseTest {
         int dbId = -1;
         boolean exit = false;
         String out = "benchmark.html";
-        Properties prop = new Properties();
-        InputStream in = getClass().getResourceAsStream("test.properties");
-        prop.load(in);
-        in.close();
+        Properties prop = loadProperties();
         int size = Integer.parseInt(prop.getProperty("size"));
         for (int i = 0; i < args.length; i++) {
             String arg = args[i];
@@ -100,9 +95,8 @@ public class TestPerformance implements Database.DatabaseTest {
             }
             String dbString = prop.getProperty("db" + i);
             if (dbString != null) {
-                Database db = Database.parse(this, i, dbString, 1);
+                Database db = Database.parse(this, i, dbString, prop);
                 if (db != null) {
-                    db.setTranslations(prop);
                     dbs.add(db);
                 }
             }
@@ -111,43 +105,39 @@ public class TestPerformance implements Database.DatabaseTest {
         for (int i = 0; i < 100; i++) {
             String testString = prop.getProperty("test" + i);
             if (testString != null) {
-                Bench bench = (Bench) Class.forName(testString).newInstance();
+                Bench bench = (Bench) Class.forName(testString).getDeclaredConstructor().newInstance();
                 tests.add(bench);
             }
         }
         testAll(dbs, tests, size);
         collect = false;
-        if (dbs.size() == 0) {
+        if (dbs.isEmpty()) {
             return;
         }
-        ArrayList<Object[]> results = dbs.get(0).getResults();
-        Connection conn = null;
-        PreparedStatement prep = null;
-        Statement stat = null;
-        PrintWriter writer = null;
-        try {
+        ArrayList<Database.Measurement> results = dbs.get(0).getResults();
+        try (Connection conn = getResultConnection()) {
             openResults();
-            conn = getResultConnection();
-            stat = conn.createStatement();
-            prep = conn.prepareStatement(
+            try (PreparedStatement prep = conn.prepareStatement(
                         "INSERT INTO RESULTS(TESTID, TEST, " +
-                        "UNIT, DBID, DB, RESULT) VALUES(?, ?, ?, ?, ?, ?)");
-            for (int i = 0; i < results.size(); i++) {
-                Object[] res = results.get(i);
-                prep.setInt(1, i);
-                prep.setString(2, res[0].toString());
-                prep.setString(3, res[1].toString());
-                for (Database db : dbs) {
-                    prep.setInt(4, db.getId());
-                    prep.setString(5, db.getName());
-                    Object[] v = db.getResults().get(i);
-                    prep.setString(6, v[2].toString());
-                    prep.execute();
+                        "UNIT, DBID, DB, RESULT) VALUES(?, ?, ?, ?, ?, ?)")) {
+                for (int i = 0; i < results.size(); i++) {
+                    Database.Measurement res = results.get(i);
+                    prep.setInt(1, i);
+                    prep.setString(2, res.name);
+                    prep.setString(3, res.unit);
+                    for (Database db : dbs) {
+                        prep.setInt(4, db.getId());
+                        prep.setString(5, db.getName());
+                        Database.Measurement measurement = db.getResults().get(i);
+                        prep.setString(6, String.valueOf(measurement.value));
+                        prep.execute();
+                    }
                 }
             }
 
-            writer = new PrintWriter(new FileWriter(out));
-            ResultSet rs = stat.executeQuery(
+            try (Statement stat = conn.createStatement();
+                PrintWriter writer = new PrintWriter(new FileWriter(out));
+                ResultSet rs = stat.executeQuery(
                     "CALL '<table><tr><th>Test Case</th><th>Unit</th>' " +
                     "|| (SELECT GROUP_CONCAT('<th>' || DB || '</th>' " +
                     "ORDER BY DBID SEPARATOR '') FROM " +
@@ -160,58 +150,13 @@ public class TestPerformance implements Database.DatabaseTest {
                     "R2.TESTID = R1.TESTID) || '</tr>' " +
                     "ORDER BY TESTID SEPARATOR CHAR(10)) FROM " +
                     "(SELECT DISTINCT TESTID, TEST, UNIT FROM RESULTS) R1)" +
-                    "|| '</table>'"
-            );
-            rs.next();
-            String result = rs.getString(1);
-            writer.println(result);
-        } finally {
-            JdbcUtils.closeSilently(prep);
-            JdbcUtils.closeSilently(stat);
-            JdbcUtils.closeSilently(conn);
-            IOUtils.closeSilently(writer);
+                    "|| '</table>'")) {
+                rs.next();
+                String result = rs.getString(1);
+                writer.println(result);
+            }
         }
 
-//        ResultSet rsDbs = conn.createStatement().executeQuery(
-//                "SELECT DB RESULTS GROUP BY DBID, DB ORDER BY DBID");
-//        while(rsDbs.next()) {
-//            writer.println("<th>" + rsDbs.getString(1) + "</th>");
-//        }
-//        ResultSet rs = conn.createStatement().executeQuery(
-//                "SELECT TEST, UNIT FROM RESULTS " +
-//                "GROUP BY TESTID, TEST, UNIT ORDER BY TESTID");
-//        while(rs.next()) {
-//            writer.println("<tr><td>" + rs.getString(1) + "</td>");
-//            writer.println("<td>" + rs.getString(2) + "</td>");
-//            ResultSet rsRes = conn.createStatement().executeQuery(
-//                "SELECT RESULT FROM RESULTS WHERE TESTID=? ORDER BY DBID");
-//
-//
-//        }
-
-//        PrintWriter writer =
-//            new PrintWriter(new FileWriter("benchmark.html"));
-//        writer.println("<table><tr><th>Test Case</th><th>Unit</th>");
-//        for(int j=0; j<dbs.size(); j++) {
-//            Database db = (Database)dbs.get(j);
-//            writer.println("<th>" + db.getName() + "</th>");
-//        }
-//        writer.println("</tr>");
-//        for(int i=0; i<results.size(); i++) {
-//            Object[] res = (Object[])results.get(i);
-//            writer.println("<tr><td>" + res[0] + "</td>");
-//            writer.println("<td>" + res[1] + "</td>");
-//            for(int j=0; j<dbs.size(); j++) {
-//                Database db  = (Database)dbs.get(j);
-//                ArrayList r = db.getResults();
-//                Object[] v = (Object[])r.get(i);
-//                writer.println(
-//                    "<td  style=\"text-align: right\">" + v[2] + "</td>");
-//            }
-//            writer.println("</tr>");
-//        }
-//        writer.println("</table>");
-
         if (exit) {
             System.exit(0);
         }
@@ -231,17 +176,19 @@ public class TestPerformance implements Database.DatabaseTest {
             db.startServer();
             Connection conn = db.openNewConnection();
             DatabaseMetaData meta = conn.getMetaData();
-            System.out.println(" " + meta.getDatabaseProductName() + " " +
-                    meta.getDatabaseProductVersion());
+            System.out.println("Database: " + meta.getDatabaseProductName() + " " + meta.getDatabaseProductVersion());
+            System.out.println("Driver: " + meta.getDriverName() + " " + meta.getDriverVersion());
             runDatabase(db, tests, 1);
             runDatabase(db, tests, 1);
+            db.reset();
             collect = true;
             runDatabase(db, tests, size);
             conn.close();
             db.log("Executed statements", "#", db.getExecutedStatements());
             db.log("Total time", "ms", db.getTotalTime());
+            System.out.println("Total time: " + db.getTotalTime() + " ms");
             int statPerSec = (int) (db.getExecutedStatements() * 1000L / db.getTotalTime());
-            db.log("Statements per second", "#", statPerSec);
+            db.log("Statements per second", "#/s", statPerSec);
             System.out.println("Statements per second: " + statPerSec);
             System.out.println("GC overhead: " + (100 * db.getTotalGCTime() / db.getTotalTime()) + "%");
             collect = false;
