@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -15,6 +15,8 @@ import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.util.ArrayList;
 import java.util.Arrays;
+
+import org.h2.test.TestBase;
 import org.h2.test.utils.SelfDestructor;
 import org.h2.util.StringUtils;
 import org.h2.util.Task;
@@ -48,7 +50,7 @@ public class TaskProcess {
         try {
             String selfDestruct = SelfDestructor.getPropertyString(60);
             ArrayList<String> list = new ArrayList<>();
-            list.add("java");
+            list.add(TestBase.getJVM());
             list.add(selfDestruct);
             list.add("-cp");
             list.add("bin" + File.pathSeparator + ".");
