@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -15,6 +15,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import org.h2.api.DatabaseEventListener;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.Backup;
 import org.h2.tools.Restore;
 import org.h2.util.Task;
@@ -22,7 +23,7 @@ import org.h2.util.Task;
 /**
  * Test for the BACKUP SQL statement.
  */
-public class TestBackup extends TestBase {
+public class TestBackup extends TestDb {
 
     /**
      * Run just this test.
@@ -30,14 +31,19 @@ public class TestBackup extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.memory) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws SQLException {
-        if (config.memory) {
-            return;
-        }
         testConcurrentBackup();
         testBackupRestoreLobStatement();
         testBackupRestoreLob();
@@ -51,7 +57,7 @@ public class TestBackup extends TestBase {
             return;
         }
         deleteDb("backup");
-        String url = getURL("backup;multi_threaded=true", true);
+        String url = getURL("backup", true);
         Connection conn = getConnection(url);
         final Statement stat = conn.createStatement();
         stat.execute("create table test(id int primary key, name varchar)");
@@ -109,27 +115,7 @@ public class TestBackup extends TestBase {
     public static class BackupListener implements DatabaseEventListener {
 
         @Override
-        public void closingDatabase() {
-            // ignore
-        }
-
-        @Override
-        public void exceptionThrown(SQLException e, String sql) {
-            // ignore
-        }
-
-        @Override
-        public void init(String url) {
-            // ignore
-        }
-
-        @Override
-        public void opened() {
-            // ignore
-        }
-
-        @Override
-        public void setProgress(int state, String name, int x, int max) {
+        public void setProgress(int state, String name, long x, long max) {
             try {
                 Thread.sleep(1);
             } catch (InterruptedException e) {
@@ -183,7 +169,7 @@ public class TestBackup extends TestBase {
         stat1.execute("create table testlob" +
                 "(id int primary key, b blob, c clob)");
         stat1.execute("insert into testlob values" +
-                "(1, space(10000), repeat('00', 10000))");
+                "(1, repeat(char(0), 10000), space(10000))");
         conn2 = getConnection("backup");
         stat2 = conn2.createStatement();
         stat2.execute("insert into test values(3, 'third')");
