@@ -1,10 +1,12 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
 
+import java.math.BigDecimal;
+import java.nio.charset.StandardCharsets;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
 import java.sql.PreparedStatement;
@@ -12,13 +14,15 @@ import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.util.Locale;
 import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests the compatibility with other databases.
  */
-public class TestCompatibility extends TestBase {
+public class TestCompatibility extends TestDb {
 
     private Connection conn;
 
@@ -28,16 +32,14 @@ public class TestCompatibility extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
     public void test() throws SQLException {
         deleteDb("compatibility");
 
-        testOnDuplicateKey();
         testCaseSensitiveIdentifiers();
-        testKeyAsColumnInMySQLMode();
 
         conn = getConnection("compatibility");
         testDomain();
@@ -50,35 +52,21 @@ public class TestCompatibility extends TestBase {
         testDB2();
         testDerby();
         testSybaseAndMSSQLServer();
-        testIgnite();
+
+        testUnknownSet();
 
         conn.close();
+        testIdentifiers();
+        testIdentifiersCaseInResultSet();
+        testDatabaseToLowerParser();
+        testOldInformationSchema();
         deleteDb("compatibility");
-    }
 
-    private void testOnDuplicateKey() throws SQLException {
-        Connection c = getConnection("compatibility;MODE=MYSQL");
-        Statement stat = c.createStatement();
-        stat.execute("set mode mysql");
-        stat.execute("create schema s2");
-        stat.execute("create table s2.test(id int primary key, name varchar(255))");
-        stat.execute("insert into s2.test(id, name) values(1, 'a')");
-        stat.execute("insert into s2.test(id, name) values(1, 'b') " +
-                "on duplicate key update name = values(name)");
-        stat.execute("drop schema s2");
-        c.close();
-    }
-
-    private void testKeyAsColumnInMySQLMode() throws SQLException {
-        Connection c = getConnection("compatibility;MODE=MYSQL");
-        Statement stat = c.createStatement();
-        stat.execute("create table test(id int primary key, key varchar)");
-        stat.execute("drop table test");
-        c.close();
+        testUnknownURL();
     }
 
     private void testCaseSensitiveIdentifiers() throws SQLException {
-        Connection c = getConnection("compatibility;DATABASE_TO_UPPER=FALSE");
+        Connection c = getConnection("compatibility;DATABASE_TO_UPPER=FALSE;CASE_INSENSITIVE_IDENTIFIERS=TRUE");
         Statement stat = c.createStatement();
         stat.execute("create table test(id int primary key, name varchar) " +
                 "as select 1, 'hello'");
@@ -117,6 +105,20 @@ public class TestCompatibility extends TestBase {
         stat.execute("select id from test t group by T.ID");
 
         stat.execute("drop table test");
+
+        rs = stat.executeQuery("select 1e10, 1000000000000000000000e10, 0xfAfBl");
+        assertTrue(rs.next());
+        assertEquals(1e10, rs.getDouble(1));
+        assertEquals(1000000000000000000000e10, rs.getDouble(2));
+        assertEquals(0xfafbL, rs.getLong(3));
+        assertFalse(rs.next());
+
+        stat.execute("create table \"t 1\" (a int, b int)");
+        stat.execute("create view v as select * from \"t 1\"");
+        stat.executeQuery("select * from v").close();
+        stat.execute("drop view v");
+        stat.execute("drop table \"t 1\"");
+
         c.close();
     }
 
@@ -126,7 +128,7 @@ public class TestCompatibility extends TestBase {
         }
         Statement stat = conn.createStatement();
         stat.execute("create table test(id int primary key) as select 1");
-        assertThrows(ErrorCode.USER_DATA_TYPE_ALREADY_EXISTS_1, stat).
+        assertThrows(ErrorCode.DOMAIN_ALREADY_EXISTS_1, stat).
                 execute("create domain int as varchar");
         conn.close();
         conn = getConnection("compatibility");
@@ -140,22 +142,24 @@ public class TestCompatibility extends TestBase {
         String[] modes = { "PostgreSQL", "MySQL", "HSQLDB", "MSSQLServer",
                 "Derby", "Oracle", "Regular" };
         String columnAlias;
-        columnAlias = "MySQL,Regular";
+        columnAlias = "HSQLDB,MySQL,Regular";
         stat.execute("CREATE TABLE TEST(ID INT)");
         for (String mode : modes) {
             stat.execute("SET MODE " + mode);
             ResultSet rs = stat.executeQuery("SELECT ID I FROM TEST");
             ResultSetMetaData meta = rs.getMetaData();
+            assertEquals(mode + " mode", "I", meta.getColumnLabel(1));
             String columnName = meta.getColumnName(1);
             String tableName = meta.getTableName(1);
-            if ("ID".equals(columnName) && "TEST".equals(tableName)) {
-                assertTrue(mode + " mode should not support columnAlias",
-                        columnAlias.contains(mode));
-            } else if ("I".equals(columnName) && tableName.equals("")) {
-                assertTrue(mode + " mode should support columnAlias",
-                        columnAlias.indexOf(mode) < 0);
+            String schemaName = meta.getSchemaName(1);
+            if (columnAlias.contains(mode)) {
+                assertEquals(mode + " mode", "ID", columnName);
+                assertEquals(mode + " mode", "TEST", tableName);
+                assertEquals(mode + " mode", "PUBLIC", schemaName);
             } else {
-                fail();
+                assertEquals(mode + " mode", "I", columnName);
+                assertEquals(mode + " mode", "", tableName);
+                assertEquals(mode + " mode", "", schemaName);
             }
         }
         stat.execute("DROP TABLE TEST");
@@ -165,7 +169,7 @@ public class TestCompatibility extends TestBase {
         Statement stat = conn.createStatement();
         String[] modes = { "PostgreSQL", "MySQL", "HSQLDB", "MSSQLServer",
                 "Derby", "Oracle", "Regular" };
-        String multiNull = "PostgreSQL,MySQL,Oracle,Regular";
+        String multiNull = "PostgreSQL,MySQL,HSQLDB,Oracle,Regular";
         for (String mode : modes) {
             stat.execute("SET MODE " + mode);
             stat.execute("CREATE TABLE TEST(ID INT)");
@@ -212,18 +216,6 @@ public class TestCompatibility extends TestBase {
         stat.execute("CALL TODAY");
 
         stat.execute("DROP TABLE TEST IF EXISTS");
-        stat.execute("CREATE TABLE TEST(ID INT)");
-        stat.execute("INSERT INTO TEST VALUES(1)");
-        PreparedStatement prep = conn.prepareStatement(
-                "SELECT LIMIT ? 1 ID FROM TEST");
-        prep.setInt(1, 2);
-        prep.executeQuery();
-        stat.execute("DROP TABLE TEST IF EXISTS");
-
-        stat.execute("DROP TABLE TEST IF EXISTS");
-        stat.execute("CREATE TABLE TEST(ID INT)");
-        stat.executeQuery("SELECT * FROM TEST WHERE ID IN ()");
-        stat.execute("DROP TABLE TEST IF EXISTS");
     }
 
     private void testLog(double expected, Statement stat) throws SQLException {
@@ -245,52 +237,150 @@ public class TestCompatibility extends TestBase {
 
         assertResult("ABC", stat, "SELECT SUBSTRING('ABCDEF' FOR 3)");
         assertResult("ABCD", stat, "SELECT SUBSTRING('0ABCDEF' FROM 2 FOR 4)");
+
+        /* --------- Behaviour of CHAR(N) --------- */
+
+        /* Test right-padding of CHAR(N) at INSERT */
+        stat.execute("CREATE TABLE TEST(CH CHAR(10))");
+        stat.execute("INSERT INTO TEST (CH) VALUES ('Hello')");
+        assertResult("Hello     ", stat, "SELECT CH FROM TEST");
+
+        /* Test that WHERE clauses accept unpadded values and will pad before comparison */
+        assertResult("Hello     ", stat, "SELECT CH FROM TEST WHERE CH = 'Hello'");
+
+        /* Test CHAR which is identical to CHAR(1) */
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("CREATE TABLE TEST(CH CHAR)");
+        stat.execute("INSERT INTO TEST (CH) VALUES ('')");
+        assertResult(" ", stat, "SELECT CH FROM TEST");
+        assertResult(" ", stat, "SELECT CH FROM TEST WHERE CH = ''");
+
+        /* Test that excessive spaces are trimmed */
+        stat.execute("DELETE FROM TEST");
+        stat.execute("INSERT INTO TEST (CH) VALUES ('1   ')");
+        assertResult("1", stat, "SELECT CH FROM TEST");
+        assertResult("1", stat, "SELECT CH FROM TEST WHERE CH = '1      '");
+
+        /* Test that we do not trim too far */
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("CREATE TABLE TEST(CH CHAR(2))");
+        stat.execute("INSERT INTO TEST (CH) VALUES ('1   ')");
+        assertResult("1 ", stat, "SELECT CH FROM TEST");
+        assertResult("1 ", stat, "SELECT CH FROM TEST WHERE CH = '1      '");
+
+        /* --------- Disallowed column types --------- */
+
+        String[] DISALLOWED_TYPES = {"NUMBER", "IDENTITY", "TINYINT", "BLOB"};
+        for (String type : DISALLOWED_TYPES) {
+            stat.execute("DROP TABLE IF EXISTS TEST");
+            assertThrows(ErrorCode.UNKNOWN_DATA_TYPE_1, stat).execute("CREATE TABLE TEST(COL " + type + ")");
+        }
+
+        /* Test MONEY data type */
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("CREATE TABLE TEST(M MONEY)");
+        stat.execute("INSERT INTO TEST(M) VALUES (-92233720368547758.08)");
+        stat.execute("INSERT INTO TEST(M) VALUES (0.11111)");
+        stat.execute("INSERT INTO TEST(M) VALUES (92233720368547758.07)");
+        ResultSet rs = stat.executeQuery("SELECT M FROM TEST ORDER BY M");
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("-92233720368547758.08"), rs.getBigDecimal(1));
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("0.11"), rs.getBigDecimal(1));
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("92233720368547758.07"), rs.getBigDecimal(1));
+        assertFalse(rs.next());
+
+        /* Test SET STATEMENT_TIMEOUT */
+        assertEquals(0, stat.getQueryTimeout());
+        conn.close();
+        deleteDb("compatibility");
+        // `stat.getQueryTimeout()` caches the result, so create another connection
+        conn = getConnection("compatibility;MODE=PostgreSQL");
+        stat = conn.createStatement();
+        // `STATEMENT_TIMEOUT` uses milliseconds
+        stat.execute("SET STATEMENT_TIMEOUT TO 30000");
+        // `stat.getQueryTimeout()` returns seconds
+        assertEquals(30, stat.getQueryTimeout());
     }
 
     private void testMySQL() throws SQLException {
+        // need to reconnect to change DATABASE_TO_LOWER
+        conn.close();
+        deleteDb("compatibility");
+        conn = getConnection("compatibility;MODE=MYSQL;DATABASE_TO_LOWER=TRUE");
         Statement stat = conn.createStatement();
         stat.execute("create schema test_schema");
         stat.execute("use test_schema");
-        assertResult("TEST_SCHEMA", stat, "select schema()");
+        assertResult("test_schema", stat, "select schema()");
         stat.execute("use public");
-        assertResult("PUBLIC", stat, "select schema()");
+        assertResult("public", stat, "select schema()");
 
         stat.execute("SELECT 1");
         stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, NAME VARCHAR)");
+        stat.execute("CREATE TABLE `TEST`(ID INT PRIMARY KEY, NAME VARCHAR)");
         stat.execute("INSERT INTO TEST VALUES(1, 'Hello'), (2, 'World')");
-        org.h2.mode.FunctionsMySQL.register(conn);
         assertResult("0", stat, "SELECT UNIX_TIMESTAMP('1970-01-01 00:00:00Z')");
-        assertResult("1196418619", stat,
-                "SELECT UNIX_TIMESTAMP('2007-11-30 10:30:19Z')");
-        assertResult("1196418619", stat,
-                "SELECT UNIX_TIMESTAMP(FROM_UNIXTIME(1196418619))");
-        assertResult("2007 November", stat,
-                "SELECT FROM_UNIXTIME(1196300000, '%Y %M')");
-        assertResult("2003-12-31", stat,
-                "SELECT DATE('2003-12-31 11:02:03')");
+        assertResult("1196418619", stat, "SELECT UNIX_TIMESTAMP('2007-11-30 10:30:19Z')");
+        assertResult("1196418619", stat, "SELECT UNIX_TIMESTAMP(FROM_UNIXTIME(1196418619))");
+        assertResult("2007 November", stat, "SELECT FROM_UNIXTIME(1196300000, '%Y %M')");
+        assertResult("2003-12-31", stat, "SELECT DATE('2003-12-31 11:02:03')");
+        assertResult("2003-12-31", stat, "SELECT DATE('2003-12-31 11:02:03')");
+        assertResult(null, stat, "SELECT DATE('100')");
+        // check the weird MySQL variant of DELETE
+        stat.execute("DELETE TEST FROM TEST WHERE 1=2");
+
+        // Check conversion between VARCHAR and VARBINARY
+        String string = "ABCD\u1234";
+        byte[] bytes = string.getBytes(StandardCharsets.UTF_8);
+        stat.execute("CREATE TABLE TEST2(C VARCHAR, B VARBINARY)");
+        stat.execute("INSERT INTO TEST2(C) VALUES ('" + string + "')");
+        assertEquals(1, stat.executeUpdate("UPDATE TEST2 SET B = C"));
+        ResultSet rs = stat.executeQuery("SELECT B FROM TEST2");
+        assertTrue(rs.next());
+        assertEquals(bytes, rs.getBytes(1));
+        assertEquals(bytes, rs.getBytes("B"));
+        assertEquals(1, stat.executeUpdate("UPDATE TEST2 SET C = B"));
+        testMySQLBytesCheck(stat, string, bytes);
+        PreparedStatement prep = conn.prepareStatement("UPDATE TEST2 SET C = ?");
+        prep.setBytes(1, bytes);
+        assertEquals(1, prep.executeUpdate());
+        testMySQLBytesCheck(stat, string, bytes);
+        stat.execute("DELETE FROM TEST2");
+        prep = conn.prepareStatement("INSERT INTO TEST2(C) VALUES (?)");
+        prep.setBytes(1, bytes);
+        assertEquals(1, prep.executeUpdate());
+        testMySQLBytesCheck(stat, string, bytes);
+        prep = conn.prepareStatement("SELECT C FROM TEST2 WHERE C = ?");
+        prep.setBytes(1, bytes);
+        testMySQLBytesCheck(prep.executeQuery(), string, bytes);
+        stat.execute("CREATE INDEX TEST2_C ON TEST2(C)");
+        prep = conn.prepareStatement("SELECT C FROM TEST2 WHERE C = ?");
+        prep.setBytes(1, bytes);
+        testMySQLBytesCheck(prep.executeQuery(), string, bytes);
+        stat.execute("DROP TABLE TEST2");
 
         if (config.memory) {
             return;
         }
         // need to reconnect, because meta data tables may be initialized
         conn.close();
-        conn = getConnection("compatibility;MODE=MYSQL");
+        conn = getConnection("compatibility;MODE=MYSQL;DATABASE_TO_LOWER=TRUE");
         stat = conn.createStatement();
         testLog(Math.log(10), stat);
 
         DatabaseMetaData meta = conn.getMetaData();
         assertTrue(meta.storesLowerCaseIdentifiers());
-        assertTrue(meta.storesLowerCaseQuotedIdentifiers());
+        assertFalse(meta.storesLowerCaseQuotedIdentifiers());
         assertFalse(meta.storesMixedCaseIdentifiers());
         assertFalse(meta.storesMixedCaseQuotedIdentifiers());
         assertFalse(meta.storesUpperCaseIdentifiers());
-        assertTrue(meta.storesUpperCaseQuotedIdentifiers());
+        assertFalse(meta.storesUpperCaseQuotedIdentifiers());
 
         stat = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
                 ResultSet.CONCUR_UPDATABLE);
         assertResult("test", stat, "SHOW TABLES");
-        ResultSet rs = stat.executeQuery("SELECT * FROM TEST");
+        rs = stat.executeQuery("SELECT * FROM TEST");
         rs.next();
         rs.updateString(2, "Hallo");
         rs.updateRow();
@@ -311,13 +401,29 @@ public class TestCompatibility extends TestBase {
         stat.execute("CREATE TABLE TEST_4" +
                 "(ID INT PRIMARY KEY) charset=UTF8");
         stat.execute("CREATE TABLE TEST_5" +
-                "(ID INT PRIMARY KEY) ENGINE=InnoDb auto_increment=3 default charset=UTF8");
+                "(ID INT AUTO_INCREMENT PRIMARY KEY) ENGINE=InnoDb auto_increment=3 default charset=UTF8");
         stat.execute("CREATE TABLE TEST_6" +
-                "(ID INT PRIMARY KEY) ENGINE=InnoDb auto_increment=3 charset=UTF8");
+                "(ID INT AUTO_INCREMENT PRIMARY KEY) ENGINE=MyISAM default character set UTF8MB4, auto_increment 3");
         stat.execute("CREATE TABLE TEST_7" +
-                "(ID INT, KEY TEST_7_IDX(ID) USING BTREE)");
+                "(ID INT AUTO_INCREMENT PRIMARY KEY) ENGINE=InnoDb auto_increment=3 charset=UTF8 comment 'text'");
         stat.execute("CREATE TABLE TEST_8" +
-                "(ID INT, UNIQUE KEY TEST_8_IDX(ID) USING BTREE)");
+                "(ID INT AUTO_INCREMENT PRIMARY KEY) ENGINE=InnoDb auto_increment=3 character set=UTF8");
+        stat.execute("CREATE TABLE TEST_9" +
+                "(ID INT, KEY TEST_7_IDX(ID) USING BTREE)");
+        stat.execute("CREATE TABLE TEST_10" +
+                "(ID INT, UNIQUE KEY TEST_10_IDX(ID) USING BTREE)");
+        stat.execute("CREATE TABLE TEST_11(ID INT) COLLATE UTF8");
+        stat.execute("CREATE TABLE TEST_12(ID INT) DEFAULT COLLATE UTF8");
+        stat.execute("CREATE TABLE TEST_13(a VARCHAR(10) COLLATE UTF8MB4)");
+        stat.execute("CREATE TABLE TEST_14(a VARCHAR(10) NULL CHARACTER SET UTF8MB4 COLLATE UTF8MB4_BIN)");
+        stat.execute("ALTER TABLE TEST_14 CONVERT TO CHARACTER SET UTF8MB4 COLLATE UTF8MB4_UNICODE_CI");
+        stat.execute("ALTER TABLE TEST_14 MODIFY a VARCHAR(10) NOT NULL CHARACTER SET UTF8MB4 COLLATE UTF8");
+        assertThrows(ErrorCode.SYNTAX_ERROR_2, stat).execute("CREATE TABLE TEST_99" +
+                "(ID INT PRIMARY KEY) CHARSET UTF8,");
+        assertThrows(ErrorCode.COLUMN_NOT_FOUND_1, stat).execute("CREATE TABLE TEST_99" +
+                "(ID INT PRIMARY KEY) AUTO_INCREMENT 100");
+        assertThrows(ErrorCode.COLUMN_NOT_FOUND_1, stat).execute("CREATE TABLE TEST_99" +
+                "(ID INT) AUTO_INCREMENT 100");
 
         // this maps to SET REFERENTIAL_INTEGRITY TRUE/FALSE
         stat.execute("SET foreign_key_checks = 0");
@@ -325,7 +431,7 @@ public class TestCompatibility extends TestBase {
 
         // Check if mysql comments are supported, ensure clean connection
         conn.close();
-        conn = getConnection("compatibility;MODE=MYSQL");
+        conn = getConnection("compatibility;MODE=MYSQL;DATABASE_TO_LOWER=TRUE");
         stat = conn.createStatement();
         stat.execute("DROP TABLE IF EXISTS TEST_NO_COMMENT");
         stat.execute("CREATE table TEST_NO_COMMENT " +
@@ -347,10 +453,30 @@ public class TestCompatibility extends TestBase {
 
         stat.execute("CREATE TABLE TEST2(ID INT) ROW_FORMAT=DYNAMIC");
 
+        // check the MySQL index dropping syntax
+        stat.execute("ALTER TABLE TEST_COMMENT_ENGINE ADD CONSTRAINT CommentUnique UNIQUE (SOME_ITEM_ID)");
+        stat.execute("ALTER TABLE TEST_COMMENT_ENGINE DROP INDEX CommentUnique");
+        stat.execute("CREATE INDEX IDX_ATTACHMENT_ID ON TEST_COMMENT_ENGINE (ATTACHMENT_ID)");
+        stat.execute("DROP INDEX IDX_ATTACHMENT_ID ON TEST_COMMENT_ENGINE");
+
+        stat.execute("DROP ALL OBJECTS");
+
         conn.close();
+        deleteDb("compatibility");
         conn = getConnection("compatibility");
     }
 
+    private void testMySQLBytesCheck(Statement stat, String string, byte[] bytes) throws SQLException {
+        testMySQLBytesCheck(stat.executeQuery("SELECT C FROM TEST2"), string, bytes);
+    }
+
+    private void testMySQLBytesCheck(ResultSet rs, String string, byte[] bytes) throws SQLException {
+        assertTrue(rs.next());
+        assertEquals(string, rs.getString(1));
+        assertEquals(bytes, rs.getBytes(1));
+        assertEquals(bytes, rs.getBytes("C"));
+    }
+
     private void testSybaseAndMSSQLServer() throws SQLException {
         Statement stat = conn.createStatement();
         stat.execute("SET MODE MSSQLServer");
@@ -415,6 +541,36 @@ public class TestCompatibility extends TestBase {
 
         // UNIQUEIDENTIFIER is MSSQL's equivalent of UUID
         stat.execute("create table test3 (id UNIQUEIDENTIFIER)");
+
+        /* Test MONEY data type */
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("CREATE TABLE TEST(M MONEY)");
+        stat.execute("INSERT INTO TEST(M) VALUES (-922337203685477.5808)");
+        stat.execute("INSERT INTO TEST(M) VALUES (0.11111)");
+        stat.execute("INSERT INTO TEST(M) VALUES (922337203685477.5807)");
+        rs = stat.executeQuery("SELECT M FROM TEST ORDER BY M");
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("-922337203685477.5808"), rs.getBigDecimal(1));
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("0.1111"), rs.getBigDecimal(1));
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("922337203685477.5807"), rs.getBigDecimal(1));
+        assertFalse(rs.next());
+
+        /* Test SMALLMONEY data type */
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("CREATE TABLE TEST(M SMALLMONEY)");
+        stat.execute("INSERT INTO TEST(M) VALUES (-214748.3648)");
+        stat.execute("INSERT INTO TEST(M) VALUES (0.11111)");
+        stat.execute("INSERT INTO TEST(M) VALUES (214748.3647)");
+        rs = stat.executeQuery("SELECT M FROM TEST ORDER BY M");
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("-214748.3648"), rs.getBigDecimal(1));
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("0.1111"), rs.getBigDecimal(1));
+        assertTrue(rs.next());
+        assertEquals(new BigDecimal("214748.3647"), rs.getBigDecimal(1));
+        assertFalse(rs.next());
     }
 
     private void testDB2() throws SQLException {
@@ -469,6 +625,30 @@ public class TestCompatibility extends TestBase {
                 "fetch next 2 rows only with rs use and keep update locks");
         res = stat.executeQuery("select * from test order by id " +
                 "fetch next 2 rows only with rr use and keep exclusive locks");
+
+        // Test DB2 TIMESTAMP format with dash separating date and time
+        stat.execute("drop table test if exists");
+        stat.execute("create table test(date TIMESTAMP)");
+        stat.executeUpdate("insert into test (date) values ('2014-04-05-09.48.28.020005')");
+        assertResult("2014-04-05 09:48:28.020005", stat,
+                "select date from test"); // <- result is always H2 format timestamp!
+        assertResult("2014-04-05 09:48:28.020005", stat,
+                "select date from test where date = '2014-04-05-09.48.28.020005'");
+        assertResult("2014-04-05 09:48:28.020005", stat,
+                "select date from test where date = '2014-04-05 09:48:28.020005'");
+
+        // Test limited support for DB2's special registers
+
+        // Standard SQL functions like LOCALTIMESTAMP, CURRENT_TIMESTAMP and
+        // others are used to compare values, their implementation in H2 is
+        // compatible with standard, but may be not really compatible with DB2.
+        assertResult("TRUE", stat, "SELECT LOCALTIMESTAMP = CURRENT TIMESTAMP");
+        assertResult("TRUE", stat, "SELECT CAST(LOCALTIMESTAMP AS VARCHAR) = CAST(CURRENT TIMESTAMP AS VARCHAR)");
+        assertResult("TRUE", stat, "SELECT CURRENT_TIMESTAMP = CURRENT TIMESTAMP WITH TIME ZONE");
+        assertResult("TRUE", stat,
+                "SELECT CAST(CURRENT_TIMESTAMP AS VARCHAR) = CAST(CURRENT TIMESTAMP WITH TIME ZONE AS VARCHAR)");
+        assertResult("TRUE", stat, "SELECT CURRENT_TIME = CURRENT TIME");
+        assertResult("TRUE", stat, "SELECT CURRENT_DATE = CURRENT DATE");
     }
 
     private void testDerby() throws SQLException {
@@ -489,25 +669,120 @@ public class TestCompatibility extends TestBase {
         conn = getConnection("compatibility");
     }
 
-    private void testIgnite() throws SQLException {
+    private void testUnknownSet() throws SQLException {
         Statement stat = conn.createStatement();
-        stat.execute("SET MODE Ignite");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int affinity key)");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int affinity primary key)");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int, v1 varchar, v2 long affinity key, primary key(v1, id))");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int, v1 varchar, v2 long, primary key(v1, id), affinity key (id))");
-
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int shard key)");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int shard primary key)");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int, v1 varchar, v2 long shard key, primary key(v1, id))");
-        stat.execute("DROP TABLE IF EXISTS TEST");
-        stat.execute("create table test(id int, v1 varchar, v2 long, primary key(v1, id), shard key (id))");
+        assertThrows(ErrorCode.UNKNOWN_MODE_1, stat).execute("SET MODE UnknownMode");
     }
+
+    private void testIdentifiers() throws SQLException {
+        deleteDb("compatibility");
+        testIdentifiers(false, false, false);
+        testIdentifiers(false, false, true);
+        testIdentifiers(true, false, false);
+        testIdentifiers(true, false, true);
+        testIdentifiers(false, true, false);
+        testIdentifiers(false, true, true);
+    }
+
+    private void testIdentifiers(boolean upper, boolean lower, boolean caseInsensitiveIdentifiers) //
+            throws SQLException {
+        try (Connection conn = getConnection("compatibility;DATABASE_TO_UPPER=" + upper + ";DATABASE_TO_LOWER=" + lower
+                + ";CASE_INSENSITIVE_IDENTIFIERS=" + caseInsensitiveIdentifiers)) {
+            Statement stat = conn.createStatement();
+            stat.execute("CREATE TABLE Test(Id INT) AS VALUES 2");
+            String schema = "PUBLIC", table = "Test", column = "Id";
+            if (upper) {
+                table = table.toUpperCase(Locale.ROOT);
+                column = column.toUpperCase(Locale.ROOT);
+            } else if (lower) {
+                schema = schema.toLowerCase(Locale.ROOT);
+                table = table.toLowerCase(Locale.ROOT);
+                column = column.toLowerCase(Locale.ROOT);
+            }
+            try (ResultSet rs = stat.executeQuery("SELECT TABLE_SCHEMA, TABLE_NAME, COLUMN_NAME"
+                    + " FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME ILIKE 'Test'")) {
+                assertTrue(rs.next());
+                assertEquals(schema, rs.getString(1));
+                assertEquals(table, rs.getString(2));
+                assertEquals(column, rs.getString(3));
+            }
+            testIdentifiers(stat, "Test", "Id", true);
+            testIdentifiers(stat, "`Test`", "`Id`", true);
+            boolean ok = upper || lower || caseInsensitiveIdentifiers;
+            testIdentifiers(stat, "TEST", "ID", ok);
+            testIdentifiers(stat, "`TEST`", "`ID`", ok);
+            testIdentifiers(stat, "test", "id", ok);
+            testIdentifiers(stat, "`test`", "`id`", ok);
+            testIdentifiers(stat, '"' + table + '"', '"' + column + '"', true);
+            testIdentifiers(stat, "\"TeSt\"", "\"iD\"", caseInsensitiveIdentifiers);
+            stat.execute("CREATE TABLE T2(\"`\" INT, `\"'\"` INT) AS VALUES (1, 2)");
+            try (ResultSet rs = stat.executeQuery("SELECT ````, \"\"\"'\"\"\" FROM T2")) {
+                assertTrue(rs.next());
+                assertEquals(1, rs.getInt(1));
+                assertEquals(2, rs.getInt(2));
+            }
+        } finally {
+            deleteDb("compatibility");
+        }
+    }
+
+    private void testIdentifiers(Statement stat, String table, String column, boolean ok) throws SQLException {
+        String query = "SELECT _ROWID_, " + column + " FROM " + table;
+        if (ok) {
+            try (ResultSet rs = stat.executeQuery(query)) {
+                assertTrue(rs.next());
+                assertEquals(1L, rs.getLong(1));
+                assertEquals(2, rs.getInt(2));
+            }
+        } else {
+            assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_WITH_CANDIDATES_2, stat).executeQuery(query);
+        }
+    }
+
+    private void testUnknownURL() {
+        assertThrows(ErrorCode.UNKNOWN_MODE_1, () -> {
+            getConnection("compatibility;MODE=Unknown").close();
+            deleteDb("compatibility");
+        });
+    }
+
+    private void testIdentifiersCaseInResultSet() throws SQLException {
+        try (Connection conn = getConnection(
+                "compatibility;DATABASE_TO_UPPER=FALSE;CASE_INSENSITIVE_IDENTIFIERS=TRUE")) {
+            Statement stat = conn.createStatement();
+            stat.execute("CREATE TABLE TEST(A INT)");
+            ResultSet rs = stat.executeQuery("SELECT a from test");
+            ResultSetMetaData md = rs.getMetaData();
+            assertEquals("A", md.getColumnName(1));
+            rs = stat.executeQuery("SELECT a FROM (SELECT 1) t(A)");
+            md = rs.getMetaData();
+            assertEquals("A", md.getColumnName(1));
+        } finally {
+            deleteDb("compatibility");
+        }
+    }
+
+    private void testDatabaseToLowerParser() throws SQLException {
+        try (Connection conn = getConnection("compatibility;DATABASE_TO_LOWER=TRUE")) {
+            Statement stat = conn.createStatement();
+            ResultSet rs = stat.executeQuery("SELECT 0x1234567890AbCdEf");
+            rs.next();
+            assertEquals(0x1234567890ABCDEFL, rs.getLong(1));
+        } finally {
+            deleteDb("compatibility");
+        }
+    }
+
+    private void testOldInformationSchema() throws SQLException {
+        try (Connection conn = getConnection(
+                "compatibility;OLD_INFORMATION_SCHEMA=TRUE")) {
+            Statement stat = conn.createStatement();
+            ResultSet rs = stat.executeQuery("TABLE INFORMATION_SCHEMA.TABLE_TYPES");
+            rs.next();
+            assertEquals("TABLE", rs.getString(1));
+        } finally {
+            deleteDb("compatibility");
+        }
+    }
+
 }
