@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -13,6 +13,7 @@ import java.io.OutputStream;
 import java.io.Reader;
 import java.io.StringReader;
 import java.io.StringWriter;
+import java.nio.charset.StandardCharsets;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -24,12 +25,11 @@ import java.util.Random;
 import java.util.concurrent.TimeUnit;
 
 import org.h2.api.ErrorCode;
-import org.h2.engine.SysProperties;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.Csv;
 import org.h2.util.IOUtils;
-import org.h2.util.New;
 import org.h2.util.StringUtils;
 
 /**
@@ -38,7 +38,7 @@ import org.h2.util.StringUtils;
  * @author Thomas Mueller
  * @author Sylvain Cuaz (testNull)
  */
-public class TestCsv extends TestBase {
+public class TestCsv extends TestDb {
 
     /**
      * Run just this test.
@@ -48,7 +48,7 @@ public class TestCsv extends TestBase {
     public static void main(String... a) throws Exception {
         TestBase test = TestBase.createCaller().init();
         test.config.traceTest = true;
-        test.test();
+        test.testFromMain();
     }
 
     @Override
@@ -77,12 +77,12 @@ public class TestCsv extends TestBase {
         Connection conn = getConnection("csv");
         Statement stat = conn.createStatement();
         stat.execute("call csvwrite('" + getBaseDir() +
-                "/test.tsv', 'select x from dual', 'writeColumnHeader=false')");
+                "/test.tsv', 'select x from system_range(1, 1)', 'writeColumnHeader=false')");
         String x = IOUtils.readStringAndClose(IOUtils.getReader(
                 FileUtils.newInputStream(getBaseDir() + "/test.tsv")), -1);
         assertEquals("\"1\"", x.trim());
         stat.execute("call csvwrite('" + getBaseDir() +
-                "/test.tsv', 'select x from dual', 'writeColumnHeader=true')");
+                "/test.tsv', 'select x from system_range(1, 1)', 'writeColumnHeader=true')");
         x = IOUtils.readStringAndClose(IOUtils.getReader(
                 FileUtils.newInputStream(getBaseDir() + "/test.tsv")), -1);
         x = x.trim();
@@ -106,9 +106,7 @@ public class TestCsv extends TestBase {
         csv.setLineSeparator(";");
         csv.write(writer, rs);
         conn.close();
-        // getTimestamp().getString() needs to be used (not for H2, but for
-        // Oracle)
-        assertEquals("TS,N;0101-01-01 12:00:00.0,;", writer.toString());
+        assertEquals("TS,N;-0100-01-01 12:00:00,;", writer.toString());
     }
 
     private void testCaseSensitiveColumnNames() throws Exception {
@@ -183,7 +181,7 @@ public class TestCsv extends TestBase {
     private void testOptions() {
         Csv csv = new Csv();
         assertEquals(",", csv.getFieldSeparatorWrite());
-        assertEquals(SysProperties.LINE_SEPARATOR, csv.getLineSeparator());
+        assertEquals(System.lineSeparator(), csv.getLineSeparator());
         assertEquals("", csv.getNullString());
         assertEquals('\"', csv.getEscapeCharacter());
         assertEquals('"', csv.getFieldDelimiter());
@@ -232,9 +230,7 @@ public class TestCsv extends TestBase {
         assertEquals("\0", csv.getNullString());
         assertEquals("", charset);
 
-        createClassProxy(Csv.class);
-        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, csv).
-            setOptions("escape=a error=b");
+        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, () -> csv.setOptions("escape=a error=b"));
         assertEquals('a', csv.getEscapeCharacter());
     }
 
@@ -242,9 +238,9 @@ public class TestCsv extends TestBase {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         // UTF-8 "BOM" / marker
         out.write(StringUtils.convertHexToBytes("ef" + "bb" + "bf"));
-        out.write("\"ID\", \"NAME\"\n1, Hello".getBytes("UTF-8"));
+        out.write("\"ID\", \"NAME\"\n1, Hello".getBytes(StandardCharsets.UTF_8));
         byte[] buff = out.toByteArray();
-        Reader r = new InputStreamReader(new ByteArrayInputStream(buff), "UTF-8");
+        Reader r = new InputStreamReader(new ByteArrayInputStream(buff), StandardCharsets.UTF_8);
         ResultSet rs = new Csv().read(r, null);
         assertEquals("ID", rs.getMetaData().getColumnLabel(1));
         assertEquals("NAME", rs.getMetaData().getColumnLabel(2));
@@ -306,7 +302,7 @@ public class TestCsv extends TestBase {
 
         OutputStream out = FileUtils.newOutputStream(fileName, false);
         String csvContent = "\"A\",\"B\",\"C\",\"D\"\n\\N,\"\",\"\\N\",";
-        byte[] b = csvContent.getBytes("UTF-8");
+        byte[] b = csvContent.getBytes(StandardCharsets.UTF_8);
         out.write(b, 0, b.length);
         out.close();
         Csv csv = new Csv();
@@ -352,7 +348,7 @@ public class TestCsv extends TestBase {
         int len = getSize(1000, 10000);
         PreparedStatement prep = conn.prepareStatement(
                 "insert into test(a, b) values(?, ?)");
-        ArrayList<String[]> list = New.arrayList();
+        ArrayList<String[]> list = new ArrayList<>(len);
         Random random = new Random(1);
         for (int i = 0; i < len; i++) {
             String a = randomData(random), b = randomData(random);
@@ -401,7 +397,7 @@ public class TestCsv extends TestBase {
         InputStreamReader reader = new InputStreamReader(
                 FileUtils.newInputStream(fileName));
         String text = IOUtils.readStringAndClose(reader, -1).trim();
-        text = StringUtils.replaceAll(text, "\n", " ");
+        text = text.replace('\n', ' ');
         assertEquals("ID|NAME 1|Hello", text);
         ResultSet rs = stat.executeQuery("select * from csvread('" +
                 fileName + "', null, null, '|', '')");
@@ -491,7 +487,7 @@ public class TestCsv extends TestBase {
         assertTrue(rs.next());
         assertEquals("Hello", rs.getString(1));
         assertFalse(rs.next());
-        rs = stat.executeQuery("call csvread('" + getBaseDir() + "/test.csv')");
+        rs = stat.executeQuery("select * from csvread('" + getBaseDir() + "/test.csv')");
         assertTrue(rs.next());
         assertEquals(1, rs.getInt(1));
         assertEquals("Hello", rs.getString(2));
@@ -572,7 +568,7 @@ public class TestCsv extends TestBase {
         }
         trace("read: " + TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - time));
         rs = new Csv().read(getBaseDir() + "/testRW.csv", null, "UTF8");
-        // stat.execute("CREATE ALIAS CSVREAD FOR \"org.h2.tools.Csv.read\"");
+        // stat.execute("CREATE ALIAS CSVREAD FOR 'org.h2.tools.Csv.read'");
         ResultSetMetaData meta = rs.getMetaData();
         assertEquals(2, meta.getColumnCount());
         for (int i = 0; i < len; i++) {
