@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -10,12 +10,15 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+
+import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests for the ON DUPLICATE KEY UPDATE in the Insert class.
  */
-public class TestDuplicateKeyUpdate extends TestBase {
+public class TestDuplicateKeyUpdate extends TestDb {
 
     /**
      * Run just this test.
@@ -23,7 +26,7 @@ public class TestDuplicateKeyUpdate extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -37,6 +40,8 @@ public class TestDuplicateKeyUpdate extends TestBase {
         testOnDuplicateKeyInsertBatch(conn);
         testOnDuplicateKeyInsertMultiValue(conn);
         testPrimaryKeyAndUniqueKey(conn);
+        testUpdateCountAndQualifiedNames(conn);
+        testEnum(conn);
         conn.close();
         deleteDb("duplicateKeyUpdate");
     }
@@ -109,12 +114,14 @@ public class TestDuplicateKeyUpdate extends TestBase {
         assertEquals("UPDATE", rs.getNString(1));
 
         stat.execute("INSERT INTO table_test2 (a_text, some_text, updatable_text ) " +
-                "VALUES ('b', 'b', 'test') " +
+                "VALUES ('b', 'b', 'test'), ('c', 'c', 'test2') " +
                 "ON DUPLICATE KEY UPDATE updatable_text=values(updatable_text)");
         rs = stat.executeQuery("SELECT updatable_text " +
-                "FROM table_test2 where a_text = 'b'");
+                "FROM table_test2 where a_text in ('b', 'c') order by a_text");
         rs.next();
         assertEquals("test", rs.getNString(1));
+        rs.next();
+        assertEquals("test2", rs.getNString(1));
     }
 
     private void testDuplicateCache(Connection conn) throws SQLException {
@@ -186,12 +193,12 @@ public class TestDuplicateKeyUpdate extends TestBase {
             throws SQLException {
         Statement stat = conn.createStatement();
         stat.execute("create table test " +
-                "(key varchar(1) primary key, count int not null)");
+                "(id varchar(1) primary key, count int not null)");
 
         // Insert multiple values as a batch
         for (int i = 0; i <= 2; ++i) {
             PreparedStatement prep = conn.prepareStatement(
-                    "insert into test(key, count) values(?, ?) " +
+                    "insert into test(id, count) values(?, ?) " +
                     "on duplicate key update count = count + 1");
             prep.setString(1, "a");
             prep.setInt(2, 1);
@@ -207,7 +214,7 @@ public class TestDuplicateKeyUpdate extends TestBase {
 
         // Check result
         ResultSet rs = stat.executeQuery(
-                "select count from test where key = 'a'");
+                "select count from test where id = 'a'");
         rs.next();
         assertEquals(3, rs.getInt(1));
 
@@ -218,12 +225,12 @@ public class TestDuplicateKeyUpdate extends TestBase {
             throws SQLException {
         Statement stat = conn.createStatement();
         stat.execute("create table test" +
-                "(key varchar(1) primary key, count int not null)");
+                "(id varchar(1) primary key, count int not null)");
 
         // Insert multiple values in single insert operation
         for (int i = 0; i <= 2; ++i) {
             PreparedStatement prep = conn.prepareStatement(
-                    "insert into test(key, count) values(?, ?), (?, ?), (?, ?) " +
+                    "insert into test(id, count) values(?, ?), (?, ?), (?, ?) " +
                     "on duplicate key update count = count + 1");
             prep.setString(1, "a");
             prep.setInt(2, 1);
@@ -236,15 +243,14 @@ public class TestDuplicateKeyUpdate extends TestBase {
         conn.commit();
 
         // Check result
-        ResultSet rs = stat.executeQuery("select count from test where key = 'a'");
+        ResultSet rs = stat.executeQuery("select count from test where id = 'a'");
         rs.next();
         assertEquals(3, rs.getInt(1));
 
         stat.execute("drop table test");
     }
 
-    private void testPrimaryKeyAndUniqueKey(Connection conn) throws SQLException
-    {
+    private void testPrimaryKeyAndUniqueKey(Connection conn) throws SQLException {
         Statement stat = conn.createStatement();
         stat.execute("CREATE TABLE test (id INT, dup INT, " +
                 "counter INT, PRIMARY KEY(id), UNIQUE(dup))");
@@ -260,4 +266,49 @@ public class TestDuplicateKeyUpdate extends TestBase {
 
         stat.execute("drop table test");
     }
+
+    private void testUpdateCountAndQualifiedNames(Connection conn) throws SQLException {
+        Statement stat = conn.createStatement();
+        stat.execute("set mode mysql");
+        stat.execute("create schema s2");
+        stat.execute("create table s2.test(id int primary key, name varchar(255))");
+        stat.execute("insert into s2.test(id, name) values(1, 'a')");
+        assertEquals(2, stat.executeUpdate("insert into s2.test(id, name) values(1, 'b') " +
+                "on duplicate key update name = values(name)"));
+        assertEquals(0, stat.executeUpdate("insert into s2.test(id, name) values(1, 'b') " +
+                "on duplicate key update name = values(name)"));
+        assertEquals(1, stat.executeUpdate("insert into s2.test(id, name) values(2, 'c') " +
+                "on duplicate key update name = values(name)"));
+        ResultSet rs = stat.executeQuery("select id, name from s2.test order by id");
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertEquals("b", rs.getString(2));
+        assertTrue(rs.next());
+        assertEquals(2, rs.getInt(1));
+        assertEquals("c", rs.getString(2));
+        assertFalse(rs.next());
+        // Check qualified names in ON UPDATE case
+        assertEquals(2, stat.executeUpdate("insert into s2.test(id, name) values(2, 'd') " +
+                "on duplicate key update test.name = values(name)"));
+        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_1, stat)
+                .executeUpdate("insert into s2.test(id, name) values(2, 'd') " +
+                        "on duplicate key update test2.name = values(name)");
+        assertEquals(2, stat.executeUpdate("insert into s2.test(id, name) values(2, 'e') " +
+                "on duplicate key update s2.test.name = values(name)"));
+        assertThrows(ErrorCode.SCHEMA_NAME_MUST_MATCH, stat)
+                .executeUpdate("insert into s2.test(id, name) values(2, 'd') " +
+                        "on duplicate key update s3.test.name = values(name)");
+        stat.execute("drop schema s2 cascade");
+    }
+
+    private void testEnum(Connection conn) throws SQLException {
+        Statement stat = conn.createStatement();
+        stat.execute("create table test(e enum('a', 'b') unique)");
+        PreparedStatement ps = conn.prepareStatement("insert into test(e) values (?) on duplicate key update e = e");
+        ps.setString(1, "a");
+        assertEquals(1, ps.executeUpdate());
+        assertEquals(0, ps.executeUpdate());
+        stat.execute("drop table test");
+    }
+
 }
