@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -23,12 +23,14 @@ import java.util.concurrent.TimeUnit;
 import org.h2.fulltext.FullText;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
+import org.h2.util.IOUtils;
 import org.h2.util.Task;
 
 /**
  * Fulltext search tests.
  */
-public class TestFullText extends TestBase {
+public class TestFullText extends TestDb {
 
     /**
      * The words used in this test.
@@ -44,15 +46,11 @@ public class TestFullText extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
     public void test() throws Exception {
-        if (config.multiThreaded) {
-            // It is even mentioned in the docs that this is not supported
-            return;
-        }
         testUuidPrimaryKey(false);
         testAutoAnalyze();
         testNativeFeatures();
@@ -77,10 +75,7 @@ public class TestFullText extends TestBase {
                 testPerformance(true);
                 testReopen(true);
                 testDropIndex(true);
-            } catch (ClassNotFoundException e) {
-                println("Class not found, not tested: " + LUCENE_FULLTEXT_CLASS_NAME);
-                // ok
-            } catch (NoClassDefFoundError e) {
+            } catch (ClassNotFoundException | NoClassDefFoundError e) {
                 println("Class not found, not tested: " + LUCENE_FULLTEXT_CLASS_NAME);
                 // ok
             }
@@ -90,15 +85,15 @@ public class TestFullText extends TestBase {
         deleteDb("fullTextReopen");
     }
 
-    private static void close(Collection<Connection> list) throws SQLException {
+    private static void close(Collection<Connection> list) {
         for (Connection conn : list) {
-            conn.close();
+            IOUtils.closeSilently(conn);
         }
     }
 
     private Connection getConnection(String name, Collection<Connection> list)
             throws SQLException {
-        Connection conn = getConnection(name);
+        Connection conn = getConnection(name + ";MODE=STRICT");
         list.add(conn);
         return conn;
     }
@@ -108,12 +103,11 @@ public class TestFullText extends TestBase {
         Connection conn;
         Statement stat;
 
-        ArrayList<Connection> connList = new ArrayList<Connection>();
+        ArrayList<Connection> connList = new ArrayList<>();
 
         conn = getConnection("fullTextNative", connList);
         stat = conn.createStatement();
-        stat.execute("create alias if not exists ft_init " +
-                "for \"org.h2.fulltext.FullText.init\"");
+        stat.execute("create alias if not exists ft_init for 'org.h2.fulltext.FullText.init'");
         stat.execute("call ft_init()");
         stat.execute("create table test(id int primary key, name varchar)");
         stat.execute("call ft_create_index('PUBLIC', 'TEST', 'NAME')");
@@ -130,11 +124,10 @@ public class TestFullText extends TestBase {
 
     private void testNativeFeatures() throws SQLException {
         deleteDb("fullTextNative");
-        ArrayList<Connection> connList = new ArrayList<Connection>();
+        ArrayList<Connection> connList = new ArrayList<>();
         Connection conn = getConnection("fullTextNative", connList);
         Statement stat = conn.createStatement();
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_INIT " +
-                "FOR \"org.h2.fulltext.FullText.init\"");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_INIT FOR 'org.h2.fulltext.FullText.init'");
         stat.execute("CALL FT_INIT()");
         FullText.setIgnoreList(conn, "to,this");
         FullText.setWhitespaceChars(conn, " ,.-");
@@ -160,8 +153,8 @@ public class TestFullText extends TestBase {
         assertEquals("KEYS", rs.getMetaData().getColumnLabel(4));
         assertEquals("PUBLIC", rs.getString(1));
         assertEquals("TEST", rs.getString(2));
-        assertEquals("(ID)", rs.getString(3));
-        assertEquals("(1)", rs.getString(4));
+        assertEquals("[ID]", rs.getString(3));
+        assertEquals("[1]", rs.getString(4));
 
         rs = stat.executeQuery("SELECT * FROM FT_SEARCH('this', 0, 0)");
         assertFalse(rs.next());
@@ -214,7 +207,7 @@ public class TestFullText extends TestBase {
         String prefix = lucene ? "FTL" : "FT";
         deleteDb("fullTextTransaction");
         FileUtils.deleteRecursive(getBaseDir() + "/fullTextTransaction", false);
-        ArrayList<Connection> connList = new ArrayList<Connection>();
+        ArrayList<Connection> connList = new ArrayList<>();
         Connection conn = getConnection("fullTextTransaction", connList);
         Statement stat = conn.createStatement();
         initFullText(stat, lucene);
@@ -250,12 +243,12 @@ public class TestFullText extends TestBase {
         final String prefix = lucene ? "FTL" : "FT";
         trace("Testing multithreaded " + prefix);
         deleteDb("fullText");
-        ArrayList<Connection> connList = new ArrayList<Connection>();
+        ArrayList<Connection> connList = new ArrayList<>();
         try {
             int len = 2;
             Task[] task = new Task[len];
             for (int i = 0; i < len; i++) {
-                final Connection conn = getConnection("fullText", connList);
+                final Connection conn = getConnection("fullText;LOCK_TIMEOUT=60000", connList);
                 Statement stat = conn.createStatement();
                 initFullText(stat, lucene);
                 initFullText(stat, lucene);
@@ -323,8 +316,7 @@ public class TestFullText extends TestBase {
         deleteDb("fullText");
         Connection conn = getConnection("fullText");
         Statement stat = conn.createStatement();
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_INIT " +
-                "FOR \"org.h2.fulltext.FullText.init\"");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_INIT FOR 'org.h2.fulltext.FullText.init'");
         stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, DATA CLOB)");
         FullText.createIndex(conn, "PUBLIC", "TEST", null);
         conn.setAutoCommit(false);
@@ -369,8 +361,7 @@ public class TestFullText extends TestBase {
         FileUtils.deleteRecursive(getBaseDir() + "/fullText", false);
         Connection conn = getConnection("fullText");
         Statement stat = conn.createStatement();
-        stat.execute("CREATE ALIAS IF NOT EXISTS FT_INIT " +
-                "FOR \"org.h2.fulltext.FullText.init\"");
+        stat.execute("CREATE ALIAS IF NOT EXISTS FT_INIT FOR 'org.h2.fulltext.FullText.init'");
         stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, NAME VARCHAR)");
         for (int i = 0; i < 10; i++) {
             FullText.createIndex(conn, "PUBLIC", "TEST", null);
@@ -451,9 +442,19 @@ public class TestFullText extends TestBase {
         initFullText(stat, lucene);
         stat.execute("DROP TABLE IF EXISTS TEST");
         stat.execute(
-                "CREATE TABLE TEST AS SELECT * FROM INFORMATION_SCHEMA.HELP");
-        stat.execute("ALTER TABLE TEST ALTER COLUMN ID INT NOT NULL");
-        stat.execute("CREATE PRIMARY KEY ON TEST(ID)");
+                "CREATE TABLE TEST(ID BIGINT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,"
+                + " SECTION VARCHAR, TOPIC VARCHAR, SYNTAX VARCHAR, TEXT VARCHAR)");
+        PreparedStatement ps = conn.prepareStatement(
+                "INSERT INTO TEST(SECTION, TOPIC, SYNTAX, TEXT) VALUES (?, ?, ?, ?)");
+        try (ResultSet rs = stat.executeQuery("HELP \"\"")) {
+            while (rs.next()) {
+                for (int i = 1; i <= 4; i++) {
+                    ps.setString(i, rs.getString(i));
+                }
+                ps.addBatch();
+            }
+        }
+        ps.executeUpdate();
         long time = System.nanoTime();
         stat.execute("CALL " + prefix + "_CREATE_INDEX('PUBLIC', 'TEST', NULL)");
         println("create " + prefix + ": " +
@@ -491,13 +492,11 @@ public class TestFullText extends TestBase {
             return;
         }
         deleteDb("fullText");
-        ArrayList<Connection> connList = new ArrayList<Connection>();
-        Connection conn = getConnection("fullText", connList);
+        Connection conn = getConnection("fullText");
         String prefix = lucene ? "FTL_" : "FT_";
         Statement stat = conn.createStatement();
         String className = lucene ? "FullTextLucene" : "FullText";
-        stat.execute("CREATE ALIAS IF NOT EXISTS " +
-                prefix + "INIT FOR \"org.h2.fulltext." + className + ".init\"");
+        stat.execute("CREATE ALIAS IF NOT EXISTS " + prefix + "INIT FOR 'org.h2.fulltext." + className + ".init'");
         stat.execute("CALL " + prefix + "INIT()");
         stat.execute("DROP TABLE IF EXISTS TEST");
         stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, NAME " + dataType + ")");
@@ -590,15 +589,15 @@ public class TestFullText extends TestBase {
 
         if (!config.memory) {
             conn.close();
+            conn = getConnection("fullText");
         }
 
-        conn = getConnection("fullText", connList);
         stat = conn.createStatement();
         stat.executeQuery("SELECT * FROM " + prefix + "SEARCH('World', 0, 0)");
 
         stat.execute("CALL " + prefix + "DROP_ALL()");
 
-        close(connList);
+        conn.close();
     }
 
     private void testDropIndex(boolean lucene) throws SQLException {
@@ -625,7 +624,6 @@ public class TestFullText extends TestBase {
                 "_CREATE_INDEX('PUBLIC', 'TEST', 'NAME1, NAME2')");
         stat.execute("UPDATE TEST SET NAME2=NULL WHERE ID=1");
         stat.execute("UPDATE TEST SET NAME2='Hello World' WHERE ID=1");
-        conn.close();
 
         conn.close();
         FileUtils.deleteRecursive(getBaseDir() + "/fullTextDropIndex", false);
@@ -635,8 +633,7 @@ public class TestFullText extends TestBase {
             throws SQLException {
         String prefix = lucene ? "FTL" : "FT";
         String className = lucene ? "FullTextLucene" : "FullText";
-        stat.execute("CREATE ALIAS IF NOT EXISTS " + prefix +
-                "_INIT FOR \"org.h2.fulltext." + className + ".init\"");
+        stat.execute("CREATE ALIAS IF NOT EXISTS " + prefix + "_INIT FOR 'org.h2.fulltext." + className + ".init'");
         stat.execute("CALL " + prefix + "_INIT()");
     }
 }
