@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -13,13 +13,14 @@ import java.sql.Statement;
 
 import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests for overloaded user defined functions.
  *
  * @author Gary Tong
  */
-public class TestFunctionOverload extends TestBase {
+public class TestFunctionOverload extends TestDb {
 
     private static final String ME = TestFunctionOverload.class.getName();
     private Connection conn;
@@ -31,7 +32,7 @@ public class TestFunctionOverload extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -51,12 +52,12 @@ public class TestFunctionOverload extends TestBase {
     private void testOverloadError() throws SQLException {
         Statement stat = conn.createStatement();
         assertThrows(ErrorCode.METHODS_MUST_HAVE_DIFFERENT_PARAMETER_COUNTS_2, stat).
-                execute("create alias overloadError for \"" + ME + ".overloadError\"");
+                execute("create alias overloadError for '" + ME + ".overloadError'");
     }
 
     private void testControl() throws SQLException {
         Statement stat = conn.createStatement();
-        stat.execute("create alias overload0 for \"" + ME + ".overload0\"");
+        stat.execute("create alias overload0 for '" + ME + ".overload0'");
         ResultSet rs = stat.executeQuery("select overload0() from dual");
         assertTrue(rs.next());
         assertEquals("0 args", 0, rs.getInt(1));
@@ -68,7 +69,7 @@ public class TestFunctionOverload extends TestBase {
 
     private void testOverload() throws SQLException {
         Statement stat = conn.createStatement();
-        stat.execute("create alias overload1or2 for \"" + ME + ".overload1or2\"");
+        stat.execute("create alias overload1or2 for '" + ME + ".overload1or2'");
         ResultSet rs = stat.executeQuery("select overload1or2(1) from dual");
         rs.next();
         assertEquals("1 arg", 1, rs.getInt(1));
@@ -79,17 +80,16 @@ public class TestFunctionOverload extends TestBase {
         assertFalse(rs.next());
         rs = meta.getProcedures(null, null, "OVERLOAD1OR2");
         rs.next();
-        assertEquals(1, rs.getInt("NUM_INPUT_PARAMS"));
+        assertEquals("OVERLOAD1OR2_1", rs.getString("SPECIFIC_NAME"));
         rs.next();
-        assertEquals(2, rs.getInt("NUM_INPUT_PARAMS"));
+        assertEquals("OVERLOAD1OR2_2", rs.getString("SPECIFIC_NAME"));
         assertFalse(rs.next());
     }
 
     private void testOverloadNamedArgs() throws SQLException {
         Statement stat = conn.createStatement();
 
-        stat.execute("create alias overload1or2Named for \"" + ME +
-                ".overload1or2(int)\"");
+        stat.execute("create alias overload1or2Named for '" + ME + ".overload1or2(int)'");
 
         ResultSet rs = stat.executeQuery("select overload1or2Named(1) from dual");
         assertTrue("First Row", rs.next());
@@ -104,8 +104,7 @@ public class TestFunctionOverload extends TestBase {
     private void testOverloadWithConnection() throws SQLException {
         Statement stat = conn.createStatement();
 
-        stat.execute("create alias overload1or2WithConn for \"" + ME +
-                ".overload1or2WithConn\"");
+        stat.execute("create alias overload1or2WithConn for '" + ME + ".overload1or2WithConn'");
 
         ResultSet rs = stat.executeQuery("select overload1or2WithConn(1) from dual");
         rs.next();
