@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -8,8 +8,8 @@ package org.h2.test.db;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
+import java.sql.SQLException;
 import java.sql.Statement;
-import org.h2.jdbc.JdbcSQLException;
 import org.h2.test.TestAll;
 import org.h2.test.TestBase;
 
@@ -24,7 +24,7 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -235,9 +235,9 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
             rs = prep.executeQuery();
             fail("Temp view T1 was accessible after previous WITH statement finished "+
                     "- but should not have been.");
-        } catch (JdbcSQLException e) {
+        } catch (SQLException e) {
             // ensure the T1 table has been removed even without auto commit
-            assertContains(e.getMessage(), "Table \"T1\" not found;");
+            assertContains(e.getMessage(), "Table \"T1\" not found (this database is empty);");
         }
 
         conn.close();
@@ -253,7 +253,7 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
         int rowCount;
 
         stat = conn.createStatement();
-        stat.execute("CREATE TABLE T1 ( ID INT IDENTITY,  X INT NULL, Y VARCHAR(100) NULL )");
+        stat.execute("CREATE TABLE T1 ( ID INT GENERATED BY DEFAULT AS IDENTITY,  X INT NULL, Y VARCHAR(100) NULL )");
 
         prep = conn.prepareStatement("WITH v1 AS ("
                 + "     SELECT R.X, 'X1' AS Y FROM SYSTEM_RANGE(?1,?2) R"
@@ -492,7 +492,7 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
         int expectedNumberOfRows = expectedRowData.length;
 
         testRepeatedQueryWithSetup(maxRetries, expectedRowData, expectedColumnNames, expectedNumberOfRows, setupSQL,
-                withQuery, maxRetries - 1, expectedColumnTypes);
+                withQuery, maxRetries - 1, expectedColumnTypes, false);
 
     }
 
@@ -512,11 +512,14 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
         int expectedNumberOfRows = expectedRowData.length;
 
         testRepeatedQueryWithSetup(maxRetries, expectedRowData, expectedColumnNames, expectedNumberOfRows, setupSQL,
-                withQuery, maxRetries - 1, expectedColumnTypes);
+                withQuery, maxRetries - 1, expectedColumnTypes, false);
 
     }
 
     private void testSimple3RowRecursiveQueryWithLazyEval() throws Exception {
+        if (config.lazy && config.networked) {
+            return;
+        }
 
         String[] expectedRowData = new String[]{"|6"};
         String[] expectedColumnTypes = new String[]{"BIGINT"};
@@ -527,15 +530,12 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
         config = new TestAll();
 
         try {
-            // Test with settings: lazy mvStore memory mvcc multiThreaded
+            // Test with settings: lazy mvStore memory multiThreaded
             // connection url is
-            // mem:script;MV_STORE=true;LOG=1;LOCK_TIMEOUT=50;MVCC=TRUE;
-            // MULTI_THREADED=TRUE;LAZY_QUERY_EXECUTION=1
+            // mem:script;MV_STORE=true;LOG=1;LOCK_TIMEOUT=50;
+            // LAZY_QUERY_EXECUTION=1
             config.lazy = true;
-            config.mvStore = true;
             config.memory = true;
-            config.mvcc = true;
-            config.multiThreaded = true;
 
             String setupSQL = "--no config set";
             String withQuery = "select sum(n) from (\n"
@@ -549,7 +549,7 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
             int expectedNumberOfRows = expectedRowData.length;
 
             testRepeatedQueryWithSetup(maxRetries, expectedRowData, expectedColumnNames, expectedNumberOfRows,
-                    setupSQL, withQuery, maxRetries - 1, expectedColumnTypes);
+                    setupSQL, withQuery, maxRetries - 1, expectedColumnTypes, false);
         } finally {
             config = backupConfig;
         }
@@ -576,6 +576,6 @@ public class TestGeneralCommonTableQueries extends AbstractBaseForCommonTableExp
         int expectedNumberOfRows = expectedRowData.length;
 
         testRepeatedQueryWithSetup(maxRetries, expectedRowData, expectedColumnNames, expectedNumberOfRows, setupSQL,
-                withQuery, maxRetries - 1, expectedColumnTypes);
+                withQuery, maxRetries - 1, expectedColumnTypes, false);
     }
 }
