@@ -1,27 +1,34 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
 
 import java.sql.Connection;
+import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Types;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Random;
-
+import java.util.concurrent.atomic.AtomicInteger;
 import org.h2.api.ErrorCode;
-import org.h2.result.SortOrder;
+import org.h2.command.query.Select;
 import org.h2.test.TestBase;
-import org.h2.util.New;
+import org.h2.test.TestDb;
+import org.h2.tools.SimpleResultSet;
+import org.h2.value.ValueInteger;
 
 /**
  * Index tests.
  */
-public class TestIndex extends TestBase {
+public class TestIndex extends TestDb {
+
+    private static int testFunctionIndexCounter;
 
     private Connection conn;
     private Statement stat;
@@ -33,7 +40,7 @@ public class TestIndex extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -44,6 +51,13 @@ public class TestIndex extends TestBase {
         testHashIndexOnMemoryTable();
         testErrorMessage();
         testDuplicateKeyException();
+        int to = config.lockTimeout;
+        config.lockTimeout = 50000;
+        try {
+            testConcurrentUpdate();
+        } finally {
+            config.lockTimeout = to;
+        }
         testNonUniqueHashIndex();
         testRenamePrimaryKey();
         testRandomized();
@@ -88,8 +102,13 @@ public class TestIndex extends TestBase {
 
         testMultiColumnHashIndex();
 
+        testFunctionIndex();
+
         conn.close();
         deleteDb("index");
+
+        // This test uses own connection
+        testEnumIndex();
     }
 
     private void testOrderIndex() throws SQLException {
@@ -145,10 +164,10 @@ public class TestIndex extends TestBase {
         stat.execute("create table test(id int, name int primary key)");
         testErrorMessage("PRIMARY", "KEY", " ON PUBLIC.TEST(NAME)");
         stat.execute("create table test(id int, name int, unique(name))");
-        testErrorMessage("CONSTRAINT_INDEX_2 ON PUBLIC.TEST(NAME)");
+        testErrorMessage("CONSTRAINT_INDEX_2 ON PUBLIC.TEST(NAME NULLS FIRST)");
         stat.execute("create table test(id int, name int, " +
                 "constraint abc unique(name, id))");
-        testErrorMessage("ABC_INDEX_2 ON PUBLIC.TEST(NAME, ID)");
+        testErrorMessage("ABC_INDEX_2 ON PUBLIC.TEST(NAME NULLS FIRST, ID NULLS FIRST)");
     }
 
     private void testErrorMessage(String... expected) throws SQLException {
@@ -158,7 +177,7 @@ public class TestIndex extends TestBase {
             fail();
         } catch (SQLException e) {
             String m = e.getMessage();
-            int start = m.indexOf('\"'), end = m.indexOf('\"', start + 1);
+            int start = m.indexOf('"'), end = m.lastIndexOf('"');
             String s = m.substring(start + 1, end);
             for (String t : expected) {
                 assertContains(s, t);
@@ -181,12 +200,110 @@ public class TestIndex extends TestBase {
             // The format of the VALUES clause varies a little depending on the
             // type of the index, so just test that we're getting useful info
             // back.
-            assertContains(m, "IDX_TEST_NAME ON PUBLIC.TEST(NAME)");
+            assertContains(m, "IDX_TEST_NAME ON PUBLIC.TEST(NAME NULLS FIRST)");
             assertContains(m, "'Hello'");
         }
         stat.execute("drop table test");
     }
 
+    private static class ConcurrentUpdateThread extends Thread {
+        private final AtomicInteger concurrentUpdateId, concurrentUpdateValue;
+
+        private final PreparedStatement psInsert, psDelete;
+
+        boolean haveDuplicateKeyException;
+
+        ConcurrentUpdateThread(Connection c, AtomicInteger concurrentUpdateId,
+                AtomicInteger concurrentUpdateValue) throws SQLException {
+            this.concurrentUpdateId = concurrentUpdateId;
+            this.concurrentUpdateValue = concurrentUpdateValue;
+            psInsert = c.prepareStatement("insert into test(id, v) values (?, ?)");
+            psDelete = c.prepareStatement("delete from test where v = ?");
+        }
+
+        @Override
+        public void run() {
+            for (int i = 0; i < 10000; i++) {
+                try {
+                    if (Math.random() > 0.05) {
+                        psInsert.setInt(1, concurrentUpdateId.incrementAndGet());
+                        psInsert.setInt(2, concurrentUpdateValue.get());
+                        psInsert.executeUpdate();
+                    } else {
+                        psDelete.setInt(1, concurrentUpdateValue.get());
+                        psDelete.executeUpdate();
+                    }
+                } catch (SQLException ex) {
+                    switch (ex.getErrorCode()) {
+                    case 23505:
+                        haveDuplicateKeyException = true;
+                        break;
+                    case 90131:
+                        // Unlikely but possible
+                        break;
+                    default:
+                        ex.printStackTrace();
+                    }
+                }
+                if (Math.random() > 0.95)
+                    concurrentUpdateValue.incrementAndGet();
+            }
+        }
+    }
+
+    private void testConcurrentUpdate() throws SQLException {
+        Connection c = getConnection("index");
+        Statement stat = c.createStatement();
+        stat.execute("create table test(id int primary key, v int)");
+        stat.execute("create unique index idx_value_name on test(v)");
+        PreparedStatement check = c.prepareStatement("select v from test");
+        ConcurrentUpdateThread[] threads = new ConcurrentUpdateThread[4];
+        AtomicInteger concurrentUpdateId = new AtomicInteger(), concurrentUpdateValue = new AtomicInteger();
+
+        // The same connection
+        for (int i = 0; i < threads.length; i++) {
+            threads[i] = new ConcurrentUpdateThread(c, concurrentUpdateId, concurrentUpdateValue);
+        }
+        testConcurrentUpdateRun(threads, check);
+        // Different connections
+        Connection[] connections = new Connection[threads.length];
+        for (int i = 0; i < threads.length; i++) {
+            Connection c2 = getConnection("index");
+            connections[i] = c2;
+            threads[i] = new ConcurrentUpdateThread(c2, concurrentUpdateId, concurrentUpdateValue);
+        }
+        testConcurrentUpdateRun(threads, check);
+        for (Connection c2 : connections) {
+            c2.close();
+        }
+        stat.execute("drop table test");
+        c.close();
+    }
+
+    private void testConcurrentUpdateRun(ConcurrentUpdateThread[] threads, PreparedStatement check)
+            throws SQLException {
+        for (ConcurrentUpdateThread t : threads) {
+            t.start();
+        }
+        boolean haveDuplicateKeyException = false;
+        for (ConcurrentUpdateThread t : threads) {
+            try {
+                t.join();
+                haveDuplicateKeyException |= t.haveDuplicateKeyException;
+            } catch (InterruptedException e) {
+            }
+        }
+        assertTrue("haveDuplicateKeys", haveDuplicateKeyException);
+        HashSet<Integer> set = new HashSet<>();
+        try (ResultSet rs = check.executeQuery()) {
+            while (rs.next()) {
+                if (!set.add(rs.getInt(1))) {
+                    fail("unique index violation");
+                }
+            }
+        }
+    }
+
     private void testNonUniqueHashIndex() throws SQLException {
         reconnect();
         stat.execute("create memory table test(id bigint, data bigint)");
@@ -198,7 +315,7 @@ public class TestIndex extends TestBase {
                 "delete from test where id=?");
         PreparedStatement prepSelect = conn.prepareStatement(
                 "select count(*) from test where id=?");
-        HashMap<Long, Integer> map = New.hashMap();
+        HashMap<Long, Integer> map = new HashMap<>();
         for (int i = 0; i < 1000; i++) {
             long key = rand.nextInt(10) * 1000000000L;
             Integer r = map.get(key);
@@ -252,7 +369,7 @@ public class TestIndex extends TestBase {
         Random rand = new Random(1);
         reconnect();
         stat.execute("drop all objects");
-        stat.execute("CREATE TABLE TEST(ID identity)");
+        stat.execute("CREATE TABLE TEST(ID identity default on null)");
         int len = getSize(100, 1000);
         for (int i = 0; i < len; i++) {
             switch (rand.nextInt(4)) {
@@ -343,7 +460,6 @@ public class TestIndex extends TestBase {
         rs = conn.getMetaData().getIndexInfo(null, null, "TEST", false, false);
         rs.next();
         assertEquals("D", rs.getString("ASC_OR_DESC"));
-        assertEquals(SortOrder.DESCENDING, rs.getInt("SORT_TYPE"));
         stat.execute("INSERT INTO TEST SELECT X FROM SYSTEM_RANGE(1, 30)");
         rs = stat.executeQuery(
                 "SELECT COUNT(*) FROM TEST WHERE ID BETWEEN 10 AND 20");
@@ -353,7 +469,6 @@ public class TestIndex extends TestBase {
         rs = conn.getMetaData().getIndexInfo(null, null, "TEST", false, false);
         rs.next();
         assertEquals("D", rs.getString("ASC_OR_DESC"));
-        assertEquals(SortOrder.DESCENDING, rs.getInt("SORT_TYPE"));
         rs = stat.executeQuery(
                 "SELECT COUNT(*) FROM TEST WHERE ID BETWEEN 10 AND 20");
         rs.next();
@@ -423,8 +538,7 @@ public class TestIndex extends TestBase {
         stat.execute("CREATE TABLE CHILD(ID INT PRIMARY KEY, " +
                 "PID INT, FOREIGN KEY(PID) REFERENCES PARENT(ID))");
         reconnect();
-        stat.execute("DROP TABLE PARENT");
-        stat.execute("DROP TABLE CHILD");
+        stat.execute("DROP TABLE PARENT, CHILD");
     }
 
     private void testLargeIndex() throws SQLException {
@@ -603,4 +717,67 @@ public class TestIndex extends TestBase {
         trace("---done---");
     }
 
+    /**
+     * This method is called from the database.
+     *
+     * @return the result set
+     */
+    public static ResultSet testFunctionIndexFunction() {
+        // There are additional callers like JdbcConnection.prepareCommand() and
+        // CommandContainer.recompileIfRequired()
+        for (StackTraceElement element : Thread.currentThread().getStackTrace()) {
+            if (element.getClassName().startsWith(Select.class.getName())) {
+                testFunctionIndexCounter++;
+                break;
+            }
+        }
+        SimpleResultSet rs = new SimpleResultSet();
+        rs.addColumn("ID", Types.INTEGER, ValueInteger.PRECISION, 0);
+        rs.addColumn("VALUE", Types.INTEGER, ValueInteger.PRECISION, 0);
+        rs.addRow(1, 10);
+        rs.addRow(2, 20);
+        rs.addRow(3, 30);
+        return rs;
+    }
+
+    private void testFunctionIndex() throws SQLException {
+        testFunctionIndexCounter = 0;
+        stat.execute("CREATE ALIAS TEST_INDEX FOR '" + TestIndex.class.getName() + ".testFunctionIndexFunction'");
+        try (ResultSet rs = stat.executeQuery("SELECT * FROM TEST_INDEX() WHERE ID = 1 OR ID = 3")) {
+            assertTrue(rs.next());
+            assertEquals(1, rs.getInt(1));
+            assertEquals(10, rs.getInt(2));
+            assertTrue(rs.next());
+            assertEquals(3, rs.getInt(1));
+            assertEquals(30, rs.getInt(2));
+            assertFalse(rs.next());
+        } finally {
+            stat.execute("DROP ALIAS TEST_INDEX");
+        }
+        assertEquals(1, testFunctionIndexCounter);
+    }
+
+    private void testEnumIndex() throws SQLException {
+        if (config.memory || config.networked) {
+            return;
+        }
+        deleteDb("index");
+        String url = "jdbc:h2:" + getBaseDir() + "/index;DB_CLOSE_DELAY=0";
+        Connection conn = DriverManager.getConnection(url);
+        Statement stat = conn.createStatement();
+
+        stat.execute("CREATE TABLE TEST(ID INT, V ENUM('A', 'B'), CONSTRAINT PK PRIMARY KEY(ID, V))");
+        stat.execute("INSERT INTO TEST VALUES (1, 'A'), (2, 'B')");
+
+        conn.close();
+        conn = DriverManager.getConnection(url);
+        stat = conn.createStatement();
+
+        stat.execute("DELETE FROM TEST WHERE V = 'A'");
+        stat.execute("DROP TABLE TEST");
+
+        conn.close();
+        deleteDb("index");
+    }
+
 }
