@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -17,12 +17,12 @@ import java.sql.Timestamp;
 import org.h2.api.ErrorCode;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
-import org.h2.value.DataType;
+import org.h2.test.TestDb;
 
 /**
  * Tests the linked table feature (CREATE LINKED TABLE).
  */
-public class TestLinkedTable extends TestBase {
+public class TestLinkedTable extends TestDb {
 
     /**
      * Run just this test.
@@ -30,7 +30,7 @@ public class TestLinkedTable extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -52,6 +52,9 @@ public class TestLinkedTable extends TestBase {
         testCachingResults();
         testLinkedTableInReadOnlyDb();
         testGeometry();
+        testFetchSize();
+        testFetchSizeWithAutoCommit();
+        testQuotedIdentifiers();
         deleteDb("linkedTable");
     }
 
@@ -236,7 +239,7 @@ public class TestLinkedTable extends TestBase {
         assertSingleValue(sb, "SELECT * FROM T2", 2);
         sa.execute("DROP ALL OBJECTS");
         sb.execute("DROP ALL OBJECTS");
-        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_1, sa).
+        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_DATABASE_EMPTY_1, sa).
                 execute("SELECT * FROM TEST");
         ca.close();
         cb.close();
@@ -287,9 +290,9 @@ public class TestLinkedTable extends TestBase {
         sa.execute("CREATE TABLE GOOD (X NUMBER)");
         sa.execute("CREATE SCHEMA S");
         sa.execute("CREATE TABLE S.BAD (X NUMBER)");
-        sb.execute("CALL LINK_SCHEMA('G', '', " +
+        sb.execute("SELECT * FROM LINK_SCHEMA('G', '', " +
                 "'jdbc:h2:mem:one', 'sa', 'sa', 'PUBLIC'); ");
-        sb.execute("CALL LINK_SCHEMA('B', '', " +
+        sb.execute("SELECT * FROM LINK_SCHEMA('B', '', " +
                 "'jdbc:h2:mem:one', 'sa', 'sa', 'S'); ");
         // OK
         sb.executeQuery("SELECT * FROM G.GOOD");
@@ -427,7 +430,7 @@ public class TestLinkedTable extends TestBase {
 
         Connection conn2 = DriverManager.getConnection(url2, "sa2", "def def");
         Statement stat2 = conn2.createStatement();
-        String link = "CALL LINK_SCHEMA('LINKED', '', '" + url1 +
+        String link = "SELECT * FROM LINK_SCHEMA('LINKED', '', '" + url1 +
                 "', 'sa1', 'abc abc', 'PUBLIC')";
         stat2.execute(link);
         stat2.executeQuery("SELECT * FROM LINKED.TEST1");
@@ -458,7 +461,7 @@ public class TestLinkedTable extends TestBase {
         stat.execute("CREATE TEMP TABLE TEST_TEMP(ID INT PRIMARY KEY)");
         stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, " +
                 "NAME VARCHAR(200), XT TINYINT, XD DECIMAL(10,2), " +
-                "XTS TIMESTAMP, XBY BINARY(255), XBO BIT, XSM SMALLINT, " +
+                "XTS TIMESTAMP, XBY VARBINARY(255), XBO BIT, XSM SMALLINT, " +
                 "XBI BIGINT, XBL BLOB, XDA DATE, XTI TIME, XCL CLOB, XDO DOUBLE)");
         stat.execute("CREATE INDEX IDXNAME ON TEST(NAME)");
         stat.execute("INSERT INTO TEST VALUES(0, NULL, NULL, NULL, NULL, " +
@@ -494,7 +497,7 @@ public class TestLinkedTable extends TestBase {
         testRow(stat, "LINK_TEST");
         ResultSet rs = stat.executeQuery("SELECT * FROM LINK_TEST");
         ResultSetMetaData meta = rs.getMetaData();
-        assertEquals(10, meta.getPrecision(1));
+        assertEquals(32, meta.getPrecision(1));
         assertEquals(200, meta.getPrecision(2));
 
         conn.close();
@@ -524,7 +527,7 @@ public class TestLinkedTable extends TestBase {
         rs = stat.executeQuery("SELECT * FROM " +
                 "INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='LINK_TEST'");
         rs.next();
-        assertEquals("TABLE LINK", rs.getString("TABLE_TYPE"));
+        assertEquals("TABLE LINK", rs.getString("STORAGE_TYPE"));
 
         rs.next();
         rs = stat.executeQuery("SELECT * FROM LINK_TEST WHERE ID=0");
@@ -575,7 +578,7 @@ public class TestLinkedTable extends TestBase {
         assertTrue(rs.getBoolean("XBO"));
         assertEquals(3000, rs.getShort("XSM"));
         assertEquals(1234567890123456789L, rs.getLong("XBI"));
-        assertEquals("1122aa", rs.getString("XBL"));
+        assertEquals(new byte[] {0x11, 0x22, (byte) 0xAA }, rs.getBytes("XBL"));
         assertEquals("0002-01-01", rs.getString("XDA"));
         assertEquals("00:00:00", rs.getString("XTI"));
         assertEquals("J\u00fcrg", rs.getString("XCL"));
@@ -693,13 +696,7 @@ public class TestLinkedTable extends TestBase {
     }
 
     private void testGeometry() throws SQLException {
-        if (!config.mvStore && config.mvcc) {
-            return;
-        }
-        if (config.memory && config.mvcc) {
-            return;
-        }
-        if (DataType.GEOMETRY_CLASS == null) {
+        if (config.memory) {
             return;
         }
         org.h2.Driver.load();
@@ -707,17 +704,99 @@ public class TestLinkedTable extends TestBase {
         Connection cb = DriverManager.getConnection("jdbc:h2:mem:two", "sa", "sa");
         Statement sa = ca.createStatement();
         Statement sb = cb.createStatement();
-        sa.execute("CREATE TABLE TEST(ID SERIAL, the_geom geometry)");
-        sa.execute("INSERT INTO TEST(THE_GEOM) VALUES('POINT (1 1)')");
+        sa.execute("CREATE TABLE TEST(ID BIGINT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,"
+                + " THE_GEOM GEOMETRY, THE_GEOM_2 GEOMETRY(POINT, 4326))");
+        sa.execute("INSERT INTO TEST(THE_GEOM, THE_GEOM_2) VALUES"
+                + " (GEOMETRY 'POINT (1 1)', GEOMETRY 'SRID=4326;POINT(2 2)')");
         String sql = "CREATE LINKED TABLE T(NULL, " +
                 "'jdbc:h2:mem:one', 'sa', 'sa', 'TEST') READONLY";
         sb.execute(sql);
         try (ResultSet rs = sb.executeQuery("SELECT * FROM T")) {
             assertTrue(rs.next());
             assertEquals("POINT (1 1)", rs.getString("THE_GEOM"));
+            assertEquals("SRID=4326;POINT (2 2)", rs.getString("THE_GEOM_2"));
         }
         sb.execute("DROP TABLE T");
         ca.close();
         cb.close();
     }
+
+    private void testFetchSize() throws SQLException {
+        if (config.memory) {
+            return;
+        }
+        org.h2.Driver.load();
+        Connection ca = DriverManager.getConnection("jdbc:h2:mem:one", "sa", "sa");
+        Connection cb = DriverManager.getConnection("jdbc:h2:mem:two", "sa", "sa");
+        Statement sa = ca.createStatement();
+        Statement sb = cb.createStatement();
+        sa.execute("DROP TABLE IF EXISTS TEST; "
+                + "CREATE TABLE TEST as select * from SYSTEM_RANGE(1,1000) as n;");
+        String sql = "CREATE LINKED TABLE T(NULL, " +
+                "'jdbc:h2:mem:one', 'sa', 'sa', 'TEST') FETCH_SIZE 10";
+        sb.execute(sql);
+        try (ResultSet rs = sb.executeQuery("SELECT count(*) FROM T")) {
+            assertTrue(rs.next());
+            assertEquals(1000, rs.getInt(1));
+        }
+        ResultSet res = sb.executeQuery("CALL DB_OBJECT_SQL('TABLE', 'PUBLIC', 'T')");
+        res.next();
+        assertEquals("CREATE FORCE LINKED TABLE \"PUBLIC\".\"T\"(NULL, 'jdbc:h2:mem:one', 'sa', 'sa', 'TEST')"
+                + " FETCH_SIZE 10 /*--hide--*/", res.getString(1));
+        sb.execute("DROP TABLE T");
+        ca.close();
+        cb.close();
+    }
+
+    private void testFetchSizeWithAutoCommit() throws SQLException {
+        if (config.memory) {
+            return;
+        }
+        org.h2.Driver.load();
+        Connection ca = DriverManager.getConnection("jdbc:h2:mem:one", "sa", "sa");
+        Connection cb = DriverManager.getConnection("jdbc:h2:mem:two", "sa", "sa");
+        Statement sa = ca.createStatement();
+        Statement sb = cb.createStatement();
+        sa.execute("DROP TABLE IF EXISTS TEST; "
+                + "CREATE TABLE TEST as select * from SYSTEM_RANGE(1,1000) as n;");
+        String sql = "CREATE LINKED TABLE T(NULL, " +
+                "'jdbc:h2:mem:one', 'sa', 'sa', 'TEST') FETCH_SIZE 10 AUTOCOMMIT OFF";
+        sb.execute(sql);
+        try (ResultSet rs = sb.executeQuery("SELECT count(*) FROM T")) {
+            assertTrue(rs.next());
+            assertEquals(1000, rs.getInt(1));
+        }
+        ResultSet res = sb.executeQuery("CALL DB_OBJECT_SQL('TABLE', 'PUBLIC', 'T')");
+        res.next();
+        assertEquals("CREATE FORCE LINKED TABLE \"PUBLIC\".\"T\"(NULL, 'jdbc:h2:mem:one', 'sa', 'sa', 'TEST')"
+                + " FETCH_SIZE 10 AUTOCOMMIT OFF /*--hide--*/", res.getString(1));
+        sb.execute("DROP TABLE T");
+        ca.close();
+        cb.close();
+    }
+
+    private void testQuotedIdentifiers() throws SQLException {
+        if (config.memory) {
+            return;
+        }
+        org.h2.Driver.load();
+        Connection ca = DriverManager.getConnection("jdbc:h2:mem:one", "sa", "sa");
+        Connection cb = DriverManager.getConnection("jdbc:h2:mem:two", "sa", "sa");
+        Statement sa = ca.createStatement();
+        Statement sb = cb.createStatement();
+        sa.execute("CREATE TABLE \"Test\" AS SELECT X \"Num\", X \"Column \"\"1\"\"\" FROM SYSTEM_RANGE(1, 100)");
+        sb.execute("CREATE LINKED TABLE T(NULL, 'jdbc:h2:mem:one', 'sa', 'sa', '\"Test\"')");
+        try (ResultSet rs = sb.executeQuery("SELECT SUM(\"Num\") FROM T WHERE \"Num\" > 1")) {
+            assertTrue(rs.next());
+            assertEquals(5049, rs.getInt(1));
+        }
+        try (ResultSet rs = sb.executeQuery(
+                "SELECT SUM(\"Column \"\"1\"\"\") FROM T WHERE \"Column \"\"1\"\"\" > 1")) {
+            assertTrue(rs.next());
+            assertEquals(5049, rs.getInt(1));
+        }
+        ca.close();
+        cb.close();
+    }
+
 }
