@@ -171,6 +171,7 @@ public class TestMergeUsing extends TestDb implements Trigger {
                 "SELECT 2 AS ID, 'Marcy22-updated2' AS NAME UNION ALL " +
                 "SELECT X AS ID, 'Marcy'||X||'-inserted'||X AS NAME FROM SYSTEM_RANGE(3,4)",
                 4);
+        testDataChangeDeltaTable();
     }
 
     /**
@@ -298,4 +299,28 @@ public class TestMergeUsing extends TestDb implements Trigger {
         return buf.toString();
     }
 
+    private void testDataChangeDeltaTable() throws SQLException {
+        deleteDb("mergeUsingQueries");
+        try (Connection conn = getConnection("mergeUsingQueries")) {
+            Statement stat = conn.createStatement();
+            stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, C INTEGER) AS (VALUES (1, 2), (2, 3), (3, 4))");
+            PreparedStatement prep = conn.prepareStatement("SELECT TEST.ID FROM FINAL TABLE ( "
+                    + "MERGE INTO TEST USING ( "
+                    + "SELECT T.ID, T.C FROM (SELECT 1, 3) T(ID, C) "
+                    + ") T ON TEST.ID = T.ID "
+                    + "WHEN MATCHED AND TEST.ID = 1 THEN "
+                    + "UPDATE SET C = T.C "
+                    + "WHEN NOT MATCHED THEN INSERT(ID, C) VALUES (T.ID, T.C) "
+                    + ") TEST");
+            ResultSet rs = prep.executeQuery();
+            rs.next();
+            assertEquals(1L, rs.getLong(1));
+            rs = prep.executeQuery();
+            rs.next();
+            assertEquals(1L, rs.getLong(1));
+        } finally {
+            deleteDb("mergeUsingQueries");
+        }
+    }
+
 }
