@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -16,16 +16,19 @@ import java.util.Random;
 import java.util.concurrent.atomic.AtomicReference;
 import org.h2.api.ErrorCode;
 import org.h2.mvstore.MVStore;
+import org.h2.mvstore.MVStoreException;
 import org.h2.store.fs.FilePath;
-import org.h2.store.fs.FilePathMem;
 import org.h2.store.fs.FileUtils;
+import org.h2.store.fs.mem.FilePathMem;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
+import org.h2.util.Utils;
 
 /**
  * Tests out of memory situations. The database must not get corrupted, and
  * transactions must stay atomic.
  */
-public class TestOutOfMemory extends TestBase {
+public class TestOutOfMemory extends TestDb {
 
     private static final String DB_NAME = "outOfMemory";
 
@@ -35,24 +38,31 @@ public class TestOutOfMemory extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.vmlens) {
+            // running out of memory will cause the vmlens agent to stop working
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws Exception {
-        if (config.vmlens) {
-            // running out of memory will cause the vmlens agent to stop working
-            return;
-        }
         try {
-            if (!config.travis) {
+            if (!config.ci) {
                 System.gc();
                 testMVStoreUsingInMemoryFileSystem();
                 System.gc();
                 testDatabaseUsingInMemoryFileSystem();
             }
             System.gc();
-            testUpdateWhenNearlyOutOfMemory();
+            if (!config.networked) {  // for some unknown reason it fails
+                testUpdateWhenNearlyOutOfMemory();
+            }
         } finally {
             System.gc();
         }
@@ -61,15 +71,10 @@ public class TestOutOfMemory extends TestBase {
     private void testMVStoreUsingInMemoryFileSystem() {
         FilePath.register(new FilePathMem());
         String fileName = "memFS:" + getTestName();
-        final AtomicReference<Throwable> exRef = new AtomicReference<>();
+        AtomicReference<Throwable> exRef = new AtomicReference<>();
         MVStore store = new MVStore.Builder()
                 .fileName(fileName)
-                .backgroundExceptionHandler(new Thread.UncaughtExceptionHandler() {
-                    @Override
-                    public void uncaughtException(Thread t, Throwable e) {
-                        exRef.compareAndSet(null, e);
-                    }
-                })
+                .backgroundExceptionHandler((t, e) -> exRef.compareAndSet(null, e))
                 .open();
         try {
             Map<Integer, byte[]> map = store.openMap("test");
@@ -82,14 +87,14 @@ public class TestOutOfMemory extends TestBase {
                 }
                 Throwable throwable = exRef.get();
                 if(throwable instanceof OutOfMemoryError) throw (OutOfMemoryError)throwable;
-                if(throwable instanceof IllegalStateException) throw (IllegalStateException)throwable;
+                if(throwable instanceof MVStoreException) throw (MVStoreException)throwable;
                 fail();
-            } catch (OutOfMemoryError | IllegalStateException e) {
+            } catch (OutOfMemoryError | MVStoreException e) {
                 // expected
             }
             try {
                 store.close();
-            } catch (IllegalStateException e) {
+            } catch (MVStoreException e) {
                 // expected
             }
             store.closeImmediately();
@@ -110,9 +115,10 @@ public class TestOutOfMemory extends TestBase {
         try {
             Connection conn = DriverManager.getConnection(url);
             Statement stat = conn.createStatement();
+            long memoryFree = Utils.getMemoryFree();
             try {
                 stat.execute("create table test(id int, name varchar) as " +
-                        "select x, space(10000000+x) from system_range(1, 1000)");
+                        "select x, space(1000000+x) from system_range(1, 10000)");
                 fail();
             } catch (SQLException e) {
                 assertTrue("Unexpected error code: " + e.getErrorCode(),
@@ -121,7 +127,7 @@ public class TestOutOfMemory extends TestBase {
                         ErrorCode.DATABASE_IS_CLOSED == e.getErrorCode() ||
                         ErrorCode.GENERAL_ERROR_1 == e.getErrorCode());
             }
-            recoverAfterOOM();
+            recoverAfterOOM(memoryFree * 3 / 4);
             try {
                 conn.close();
                 fail();
@@ -132,7 +138,7 @@ public class TestOutOfMemory extends TestBase {
                         ErrorCode.DATABASE_IS_CLOSED == e.getErrorCode() ||
                         ErrorCode.GENERAL_ERROR_1 == e.getErrorCode());
             }
-            recoverAfterOOM();
+            recoverAfterOOM(memoryFree * 3 / 4);
             conn = DriverManager.getConnection(url);
             stat = conn.createStatement();
             stat.execute("SELECT 1");
@@ -143,9 +149,11 @@ public class TestOutOfMemory extends TestBase {
         }
     }
 
-    private static void recoverAfterOOM() throws InterruptedException {
-        for (int i = 0; i < 5; i++) {
-            System.gc();
+    private static void recoverAfterOOM(long expectedFreeMemory) throws InterruptedException {
+        for (int i = 0; i < 50; i++) {
+            if (Utils.getMemoryFree() > expectedFreeMemory) {
+                break;
+            }
             Thread.sleep(20);
         }
     }
@@ -203,8 +211,7 @@ public class TestOutOfMemory extends TestBase {
         }
     }
 
-    public static final class MyChild extends TestBase.Child
-    {
+    public static final class MyChild extends TestDb.Child {
 
         /**
          * Run just this test.
