@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -14,14 +14,14 @@ import java.util.Random;
 
 import org.h2.api.ErrorCode;
 import org.h2.engine.Database;
-import org.h2.jdbc.JdbcConnection;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.util.JdbcUtils;
 
 /**
  * Tests simulated power off conditions.
  */
-public class TestPowerOff extends TestBase {
+public class TestPowerOff extends TestDb {
 
     private static final String DB_NAME = "powerOff";
     private String dir, url;
@@ -34,14 +34,19 @@ public class TestPowerOff extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.memory) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws SQLException {
-        if (config.memory) {
-            return;
-        }
         if (config.big || config.googleAppEngine) {
             dir = getBaseDir();
             url = DB_NAME;
@@ -71,18 +76,18 @@ public class TestPowerOff extends TestBase {
         conn = getConnection(url);
         stat = conn.createStatement();
         stat.execute("set write_delay 0");
-        ((JdbcConnection) conn).setPowerOffCount(Integer.MAX_VALUE);
-        stat.execute("insert into test values(null, space(11000))");
-        int max = Integer.MAX_VALUE - ((JdbcConnection) conn).getPowerOffCount();
+        setPowerOffCount(conn, Integer.MAX_VALUE);
+        stat.execute("insert into test(data) values space(11000)");
+        int max = Integer.MAX_VALUE - getPowerOffCount(conn);
         for (int i = 0; i < max + 10; i++) {
             conn.close();
             conn = getConnection(url);
             stat = conn.createStatement();
-            stat.execute("insert into test values(null, space(11000))");
+            stat.execute("insert into test(data) values space(11000)");
             stat.execute("set write_delay 0");
-            ((JdbcConnection) conn).setPowerOffCount(i);
+            setPowerOffCount(conn, i);
             try {
-                stat.execute("insert into test values(null, space(11000))");
+                stat.execute("insert into test(data) values space(11000)");
             } catch (SQLException e) {
                 // ignore
             }
@@ -150,7 +155,7 @@ public class TestPowerOff extends TestBase {
             conn = getConnection(url);
             Statement stat = conn.createStatement();
             stat.execute("SET WRITE_DELAY 0");
-            ((JdbcConnection) conn).setPowerOffCount(random.nextInt(100));
+            setPowerOffCount(conn, random.nextInt(100));
             try {
                 stat.execute("DROP TABLE IF EXISTS TEST");
                 stat.execute("CREATE TABLE TEST" +
@@ -208,7 +213,7 @@ public class TestPowerOff extends TestBase {
                 "(ID INT PRIMARY KEY, NAME VARCHAR(255))");
         stat.execute("INSERT INTO TEST VALUES(1, 'Hello')");
         stat.execute("CHECKPOINT");
-        ((JdbcConnection) conn).setPowerOffCount(1);
+        setPowerOffCount(conn, 1);
         try {
             stat.execute("INSERT INTO TEST VALUES(2, 'Hello')");
             stat.execute("INSERT INTO TEST VALUES(3, 'Hello')");
@@ -218,7 +223,7 @@ public class TestPowerOff extends TestBase {
             assertKnownException(e);
         }
 
-        ((JdbcConnection) conn).setPowerOffCount(0);
+        setPowerOffCount(conn, 0);
         try {
             conn.close();
         } catch (SQLException e) {
@@ -298,8 +303,7 @@ public class TestPowerOff extends TestBase {
             stat.execute("DROP TABLE TEST");
             state = 0;
             if (init) {
-                maxPowerOffCount = Integer.MAX_VALUE -
-                        ((JdbcConnection) conn).getPowerOffCount();
+                maxPowerOffCount = Integer.MAX_VALUE - getPowerOffCount(conn);
             }
             conn.close();
         } catch (SQLException e) {
@@ -317,7 +321,7 @@ public class TestPowerOff extends TestBase {
         int state;
         Database.setInitialPowerOffCount(0);
         Connection conn = getConnection(url);
-        assertEquals(0, ((JdbcConnection) conn).getPowerOffCount());
+        assertEquals(0, getPowerOffCount(conn));
         Statement stat = conn.createStatement();
         DatabaseMetaData meta = conn.getMetaData();
         ResultSet rs = meta.getTables(null, null, "TEST", null);
