@@ -1,20 +1,27 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
 
+import java.nio.file.Files;
+import java.nio.file.Paths;
+import java.nio.file.StandardOpenOption;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.sql.Types;
+import java.util.Collections;
 
 import org.h2.api.ErrorCode;
 import org.h2.api.Trigger;
+import org.h2.engine.Constants;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.ChangeFileEncryption;
 import org.h2.tools.Recover;
 import org.h2.util.Task;
@@ -22,7 +29,7 @@ import org.h2.util.Task;
 /**
  * Tests the RUNSCRIPT SQL statement.
  */
-public class TestRunscript extends TestBase implements Trigger {
+public class TestRunscript extends TestDb implements Trigger {
 
     /**
      * Run just this test.
@@ -30,7 +37,13 @@ public class TestRunscript extends TestBase implements Trigger {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+        org.h2.test.TestAll config = new org.h2.test.TestAll();
+        config.traceLevelFile = 1;
+        System.out.println(config);
+        TestBase test = createCaller();
+        test.runTest(config);
+//        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -51,6 +64,8 @@ public class TestRunscript extends TestBase implements Trigger {
         testCancelScript();
         testEncoding();
         testClobPrimaryKey();
+        testTruncateLargeLength();
+        testVariableBinary();
         deleteDb("runscript");
     }
 
@@ -59,7 +74,7 @@ public class TestRunscript extends TestBase implements Trigger {
         Connection conn;
         conn = getConnection("runscript");
         Statement stat = conn.createStatement();
-        stat.execute("create alias int_decode for \"java.lang.Integer.decode\"");
+        stat.execute("create alias int_decode for 'java.lang.Integer.decode'");
         stat.execute("create table test(x varchar, y int as int_decode(x))");
         stat.execute("script simple drop to '" +
                 getBaseDir() + "/backup.sql'");
@@ -93,15 +108,15 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script schema include_schema1");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The schema 'exclude_schema1' should not be present in the script",
-                    rs.getString(1).indexOf("exclude_schema1".toUpperCase()) == -1);
+            assertFalse("The schema 'exclude_schema1' should not be present in the script",
+                    rs.getString(1).contains("exclude_schema1".toUpperCase()));
         }
         rs.close();
         stat.execute("create schema include_schema2");
         stat.execute("script nosettings schema include_schema1, include_schema2");
         rs = stat.getResultSet();
-        // user and one row per schema = 3
-        assertResultRowCount(3, rs);
+        // version, user, and one row per schema = 4
+        assertResultRowCount(4, rs);
         rs.close();
         conn.close();
     }
@@ -122,29 +137,29 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script table a.test1");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The table 'a.test2' should not be present in the script",
-                    rs.getString(1).indexOf("a.test2".toUpperCase()) == -1);
-            assertTrue("The table 'b.test1' should not be present in the script",
-                    rs.getString(1).indexOf("b.test1".toUpperCase()) == -1);
-            assertTrue("The table 'b.test2' should not be present in the script",
-                    rs.getString(1).indexOf("b.test2".toUpperCase()) == -1);
+            assertFalse("The table 'a.test2' should not be present in the script",
+                    rs.getString(1).contains("a.test2".toUpperCase()));
+            assertFalse("The table 'b.test1' should not be present in the script",
+                    rs.getString(1).contains("b.test1".toUpperCase()));
+            assertFalse("The table 'b.test2' should not be present in the script",
+                    rs.getString(1).contains("b.test2".toUpperCase()));
         }
         rs.close();
         stat.execute("set schema b");
         stat.execute("script table test1");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The table 'a.test1' should not be present in the script",
-                    rs.getString(1).indexOf("a.test1".toUpperCase()) == -1);
-            assertTrue("The table 'a.test2' should not be present in the script",
-                    rs.getString(1).indexOf("a.test2".toUpperCase()) == -1);
-            assertTrue("The table 'b.test2' should not be present in the script",
-                    rs.getString(1).indexOf("b.test2".toUpperCase()) == -1);
+            assertFalse("The table 'a.test1' should not be present in the script",
+                    rs.getString(1).contains("a.test1".toUpperCase()));
+            assertFalse("The table 'a.test2' should not be present in the script",
+                    rs.getString(1).contains("a.test2".toUpperCase()));
+            assertFalse("The table 'b.test2' should not be present in the script",
+                    rs.getString(1).contains("b.test2".toUpperCase()));
         }
         stat.execute("script nosettings table a.test1, test2");
         rs = stat.getResultSet();
-        // user, schemas 'a' & 'b' and 2 rows per table = 7
-        assertResultRowCount(7, rs);
+        // version, user, schemas 'a' & 'b', and 2 rows per table = 7
+        assertResultRowCount(8, rs);
         rs.close();
         conn.close();
     }
@@ -158,14 +173,14 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("create schema a");
         stat.execute("create schema b");
         stat.execute("create schema c");
-        stat.execute("create alias a.int_decode for \"java.lang.Integer.decode\"");
+        stat.execute("create alias a.int_decode for 'java.lang.Integer.decode'");
         stat.execute("create table a.test(x varchar, y int as a.int_decode(x))");
         stat.execute("script schema b");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The function alias 'int_decode' " +
+            assertFalse("The function alias 'int_decode' " +
                     "should not be present in the script",
-                    rs.getString(1).indexOf("int_decode".toUpperCase()) == -1);
+                    rs.getString(1).contains("int_decode".toUpperCase()));
         }
         rs.close();
         conn.close();
@@ -186,9 +201,9 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script schema b");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The constant 'default_email' " +
+            assertFalse("The constant 'default_email' " +
                     "should not be present in the script",
-                    rs.getString(1).indexOf("default_email".toUpperCase()) == -1);
+                    rs.getString(1).contains("default_email".toUpperCase()));
         }
         rs.close();
         conn.close();
@@ -207,8 +222,8 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script schema b");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The sequence 'seq_id' should not be present in the script",
-                    rs.getString(1).indexOf("seq_id".toUpperCase()) == -1);
+            assertFalse("The sequence 'seq_id' should not be present in the script",
+                    rs.getString(1).contains("seq_id".toUpperCase()));
         }
         rs.close();
         conn.close();
@@ -229,18 +244,18 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script schema b");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The sequence 'unique_constraint' " +
+            assertFalse("The sequence 'unique_constraint' " +
                     "should not be present in the script",
-                    rs.getString(1).indexOf("unique_constraint".toUpperCase()) == -1);
+                    rs.getString(1).contains("unique_constraint".toUpperCase()));
         }
         rs.close();
         stat.execute("create table a.test2(x varchar, y int)");
         stat.execute("script table a.test2");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The sequence 'unique_constraint' " +
+            assertFalse("The sequence 'unique_constraint' " +
                     "should not be present in the script",
-                    rs.getString(1).indexOf("unique_constraint".toUpperCase()) == -1);
+                    rs.getString(1).contains("unique_constraint".toUpperCase()));
         }
         rs.close();
         conn.close();
@@ -261,16 +276,16 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script schema b");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The trigger 'trigger_insert' should not be present in the script",
-                    rs.getString(1).indexOf("trigger_insert".toUpperCase()) == -1);
+            assertFalse("The trigger 'trigger_insert' should not be present in the script",
+                    rs.getString(1).contains("trigger_insert".toUpperCase()));
         }
         rs.close();
         stat.execute("create table a.test2(x varchar, y int)");
         stat.execute("script table a.test2");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The trigger 'trigger_insert' should not be present in the script",
-                    rs.getString(1).indexOf("trigger_insert".toUpperCase()) == -1);
+            assertFalse("The trigger 'trigger_insert' should not be present in the script",
+                    rs.getString(1).contains("trigger_insert".toUpperCase()));
         }
         rs.close();
         conn.close();
@@ -294,8 +309,8 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script schema b");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The grant to 'USER_A1' should not be present in the script",
-                    rs.getString(1).indexOf("to USER_A1".toUpperCase()) == -1);
+            assertFalse("The grant to 'USER_A1' should not be present in the script",
+                    rs.getString(1).contains("to USER_A1".toUpperCase()));
         }
         rs.close();
         stat.execute("create user USER_A2 password 'test'");
@@ -304,10 +319,10 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("script table a.test2");
         rs = stat.getResultSet();
         while (rs.next()) {
-            assertTrue("The grant to 'USER_A1' should not be present in the script",
-                    rs.getString(1).indexOf("to USER_A1".toUpperCase()) == -1);
-            assertTrue("The grant to 'USER_B1' should not be present in the script",
-                    rs.getString(1).indexOf("to USER_B1".toUpperCase()) == -1);
+            assertFalse("The grant to 'USER_A1' should not be present in the script",
+                    rs.getString(1).contains("to USER_A1".toUpperCase()));
+            assertFalse("The grant to 'USER_B1' should not be present in the script",
+                    rs.getString(1).contains("to USER_B1".toUpperCase()));
         }
         rs.close();
         conn.close();
@@ -324,6 +339,10 @@ public class TestRunscript extends TestBase implements Trigger {
     }
 
     private void testCancelScript() throws Exception {
+        if (config.ci) {
+            // fails regularly under Travis, not sure why
+            return;
+        }
         deleteDb("runscript");
         Connection conn;
         conn = getConnection("runscript");
@@ -347,7 +366,7 @@ public class TestRunscript extends TestBase implements Trigger {
         Thread.sleep(200);
         stat.cancel();
         SQLException e = (SQLException) task.getException();
-        assertTrue(e != null);
+        assertNotNull(e);
         assertEquals(ErrorCode.STATEMENT_WAS_CANCELED, e.getErrorCode());
 
         stat.execute("set throttle 1000");
@@ -361,10 +380,10 @@ public class TestRunscript extends TestBase implements Trigger {
             }
         };
         task.execute();
-        Thread.sleep(100);
+        Thread.sleep(200);
         stat.cancel();
         e = (SQLException) task.getException();
-        assertTrue(e != null);
+        assertNotNull(e);
         assertEquals(ErrorCode.STATEMENT_WAS_CANCELED, e.getErrorCode());
 
         conn.close();
@@ -414,7 +433,7 @@ public class TestRunscript extends TestBase implements Trigger {
         stat.execute("create table test(id int not null, data clob) " +
                 "as select 1, space(4100)");
         // the primary key for SYSTEM_LOB_STREAM used to be named like this
-        stat.execute("create primary key primary_key_e on test(id)");
+        stat.execute("alter table test add constraint primary_key_e primary key(id)");
         stat.execute("script to '" + getBaseDir() + "/backup.sql'");
         conn.close();
         deleteDb("runscript");
@@ -437,8 +456,7 @@ public class TestRunscript extends TestBase implements Trigger {
         stat1.execute("create table test2(id int primary key) as " +
                 "select x from system_range(1, 5000)");
         stat1.execute("create sequence testSeq start with 100 increment by 10");
-        stat1.execute("create alias myTest for \"" +
-                getClass().getName() + ".test\"");
+        stat1.execute("create alias myTest for '" + getClass().getName() + ".test'");
         stat1.execute("create trigger myTrigger before insert " +
                 "on test nowait call \"" + getClass().getName() + "\"");
         stat1.execute("create view testView as select * " +
@@ -457,7 +475,7 @@ public class TestRunscript extends TestBase implements Trigger {
         stat1.execute("grant all on testSchema.child to testUser");
         stat1.execute("grant select, insert on testSchema.parent to testRole");
         stat1.execute("grant testRole to testUser");
-        stat1.execute("create table blob (value blob)");
+        stat1.execute("create table blob (v blob)");
         PreparedStatement prep = conn1.prepareStatement(
                 "insert into blob values (?)");
         prep.setBytes(1, new byte[65536]);
@@ -530,9 +548,54 @@ public class TestRunscript extends TestBase implements Trigger {
         deleteDb("runscriptRestoreRecover");
         FileUtils.delete(getBaseDir() + "/backup.2.sql");
         FileUtils.delete(getBaseDir() + "/backup.3.sql");
+        FileUtils.delete(getBaseDir() + "/runscript.h2.sql");
 
     }
 
+    private void testTruncateLargeLength() throws Exception {
+        deleteDb("runscript");
+        Connection conn;
+        Statement stat;
+        Files.write(Paths.get(getBaseDir() + "/backup.sql"),
+                Collections.singleton("CREATE TABLE TEST(V VARCHAR(2147483647))"), //
+                StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING);
+        conn = getConnection("runscript");
+        stat = conn.createStatement();
+        assertThrows(ErrorCode.INVALID_VALUE_PRECISION, stat)
+                .execute("RUNSCRIPT FROM '" + getBaseDir() + "/backup.sql'");
+        stat.execute("RUNSCRIPT FROM '" + getBaseDir() + "/backup.sql' QUIRKS_MODE");
+        assertEquals(Constants.MAX_STRING_LENGTH, stat.executeQuery("TABLE TEST").getMetaData().getPrecision(1));
+        conn.close();
+        deleteDb("runscript");
+        FileUtils.delete(getBaseDir() + "/backup.sql");
+    }
+
+    private void testVariableBinary() throws SQLException {
+        deleteDb("runscript");
+        Connection conn;
+        Statement stat;
+        conn = getConnection("runscript");
+        stat = conn.createStatement();
+        stat.execute("CREATE TABLE TEST(B BINARY)");
+        assertEquals(Types.BINARY, stat.executeQuery("TABLE TEST").getMetaData().getColumnType(1));
+        stat.execute("SCRIPT TO '" + getBaseDir() + "/backup.sql'");
+        conn.close();
+        deleteDb("runscript");
+        conn = getConnection("runscript");
+        stat = conn.createStatement();
+        stat.execute("RUNSCRIPT FROM '" + getBaseDir() + "/backup.sql'");
+        assertEquals(Types.BINARY, stat.executeQuery("TABLE TEST").getMetaData().getColumnType(1));
+        conn.close();
+        deleteDb("runscript");
+        conn = getConnection("runscript");
+        stat = conn.createStatement();
+        stat.execute("RUNSCRIPT FROM '" + getBaseDir() + "/backup.sql' VARIABLE_BINARY");
+        assertEquals(Types.VARBINARY, stat.executeQuery("TABLE TEST").getMetaData().getColumnType(1));
+        conn.close();
+        deleteDb("runscript");
+        FileUtils.delete(getBaseDir() + "/backup.sql");
+    }
+
     @Override
     public void init(Connection conn, String schemaName, String triggerName,
             String tableName, boolean before, int type) {
