@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -14,13 +14,15 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import org.h2.api.Trigger;
+import org.h2.engine.Constants;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.util.Task;
 
 /**
  * Tests the sequence feature of this database.
  */
-public class TestSequence extends TestBase {
+public class TestSequence extends TestDb {
 
     /**
      * Run just this test.
@@ -28,12 +30,13 @@ public class TestSequence extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
     public void test() throws Exception {
         testConcurrentCreate();
+        testConcurrentNextAndCurrentValue();
         testSchemaSearchPath();
         testAlterSequenceColumn();
         testAlterSequence();
@@ -50,7 +53,7 @@ public class TestSequence extends TestBase {
 
     private void testConcurrentCreate() throws Exception {
         deleteDb("sequence");
-        final String url = getURL("sequence;MULTI_THREADED=1;LOCK_TIMEOUT=2000", true);
+        final String url = getURL("sequence;LOCK_TIMEOUT=2000", true);
         Connection conn = getConnection(url);
         Task[] tasks = new Task[2];
         try {
@@ -103,6 +106,66 @@ public class TestSequence extends TestBase {
         }
     }
 
+    private void testConcurrentNextAndCurrentValue() throws Exception {
+        deleteDb("sequence");
+        final String url = getURL("sequence", true);
+        Connection conn = getConnection(url);
+        Task[] tasks = new Task[2];
+        try {
+            Statement stat = conn.createStatement();
+            stat.execute("CREATE SEQUENCE SEQ1");
+            stat.execute("CREATE SEQUENCE SEQ2");
+            for (int i = 0; i < tasks.length; i++) {
+                tasks[i] = new Task() {
+                    @Override
+                    public void call() throws Exception {
+                        try (Connection conn = getConnection(url)) {
+                            PreparedStatement next1 = conn.prepareStatement("CALL NEXT VALUE FOR SEQ1");
+                            PreparedStatement next2 = conn.prepareStatement("CALL NEXT VALUE FOR SEQ2");
+                            PreparedStatement current1 = conn.prepareStatement("CALL CURRENT VALUE FOR SEQ1");
+                            PreparedStatement current2 = conn.prepareStatement("CALL CURRENT VALUE FOR SEQ2");
+                            while (!stop) {
+                                long v1, v2;
+                                try (ResultSet rs = next1.executeQuery()) {
+                                    rs.next();
+                                    v1 = rs.getLong(1);
+                                }
+                                try (ResultSet rs = next2.executeQuery()) {
+                                    rs.next();
+                                    v2 = rs.getLong(1);
+                                }
+                                try (ResultSet rs = current1.executeQuery()) {
+                                    rs.next();
+                                    if (v1 != rs.getLong(1)) {
+                                        throw new RuntimeException("Unexpected CURRENT VALUE FOR SEQ1");
+                                    }
+                                }
+                                try (ResultSet rs = current2.executeQuery()) {
+                                    rs.next();
+                                    if (v2 != rs.getLong(1)) {
+                                        throw new RuntimeException("Unexpected CURRENT VALUE FOR SEQ2");
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }.execute();
+            }
+            Thread.sleep(1000);
+            for (Task t : tasks) {
+                Exception e = t.getException();
+                if (e != null) {
+                    throw new AssertionError(e.getMessage());
+                }
+            }
+        } finally {
+            for (Task t : tasks) {
+                t.join();
+            }
+            conn.close();
+        }
+    }
+
     private void testSchemaSearchPath() throws SQLException {
         deleteDb("sequence");
         Connection conn = getConnection("sequence");
@@ -110,8 +173,8 @@ public class TestSequence extends TestBase {
         stat.execute("CREATE SCHEMA TEST");
         stat.execute("CREATE SEQUENCE TEST.TEST_SEQ");
         stat.execute("SET SCHEMA_SEARCH_PATH PUBLIC, TEST");
-        stat.execute("CALL TEST_SEQ.NEXTVAL");
-        stat.execute("CALL TEST_SEQ.CURRVAL");
+        stat.execute("CALL NEXT VALUE FOR TEST_SEQ");
+        stat.execute("CALL CURRENT VALUE FOR TEST_SEQ");
         conn.close();
     }
 
@@ -121,7 +184,7 @@ public class TestSequence extends TestBase {
         Statement stat = conn.createStatement();
         stat.execute("CREATE TABLE TEST(ID INT , NAME VARCHAR(255))");
         stat.execute("INSERT INTO TEST VALUES(1, 'Hello')");
-        stat.execute("ALTER TABLE TEST ALTER COLUMN ID INT IDENTITY");
+        stat.execute("ALTER TABLE TEST ALTER COLUMN ID INT GENERATED BY DEFAULT AS IDENTITY");
         stat.execute("ALTER TABLE test ALTER COLUMN ID RESTART WITH 3");
         stat.execute("INSERT INTO TEST (name) VALUES('Other World')");
         conn.close();
@@ -130,8 +193,8 @@ public class TestSequence extends TestBase {
     private void testAlterSequence() throws SQLException {
         test("create sequence s; alter sequence s restart with 2", null, 2, 3, 4);
         test("create sequence s; alter sequence s restart with 7", null, 7, 8, 9, 10);
-        test("create sequence s; alter sequence s restart with 11 " +
-        "minvalue 3 maxvalue 12 cycle", null, 11, 12, 3, 4);
+        test("create sequence s; alter sequence s start with 3 restart with 11 minvalue 3 maxvalue 12 cycle",
+                null, 11, 12, 3, 4);
         test("create sequence s; alter sequence s restart with 5 cache 2",
                 null, 5, 6, 7, 8);
         test("create sequence s; alter sequence s restart with 9 " +
@@ -187,38 +250,35 @@ public class TestSequence extends TestBase {
         assertEquals("SEQUENCE", rs.getString("SEQUENCE_CATALOG"));
         assertEquals("PUBLIC", rs.getString("SEQUENCE_SCHEMA"));
         assertEquals("A", rs.getString("SEQUENCE_NAME"));
-        assertEquals(0, rs.getLong("CURRENT_VALUE"));
+        assertEquals(1, rs.getLong("BASE_VALUE"));
         assertEquals(1, rs.getLong("INCREMENT"));
-        assertEquals(false, rs.getBoolean("IS_GENERATED"));
-        assertEquals("", rs.getString("REMARKS"));
+        assertNull(rs.getString("REMARKS"));
         assertEquals(32, rs.getLong("CACHE"));
-        assertEquals(1, rs.getLong("MIN_VALUE"));
-        assertEquals(Long.MAX_VALUE, rs.getLong("MAX_VALUE"));
-        assertEquals(false, rs.getBoolean("IS_CYCLE"));
+        assertEquals(1, rs.getLong("MINIMUM_VALUE"));
+        assertEquals(Long.MAX_VALUE, rs.getLong("MAXIMUM_VALUE"));
+        assertEquals("NO", rs.getString("CYCLE_OPTION"));
         rs.next();
         assertEquals("SEQUENCE", rs.getString("SEQUENCE_CATALOG"));
         assertEquals("PUBLIC", rs.getString("SEQUENCE_SCHEMA"));
         assertEquals("B", rs.getString("SEQUENCE_NAME"));
-        assertEquals(5, rs.getLong("CURRENT_VALUE"));
+        assertEquals(7, rs.getLong("BASE_VALUE"));
         assertEquals(2, rs.getLong("INCREMENT"));
-        assertEquals(false, rs.getBoolean("IS_GENERATED"));
-        assertEquals("", rs.getString("REMARKS"));
+        assertNull(rs.getString("REMARKS"));
         assertEquals(1, rs.getLong("CACHE"));
-        assertEquals(5, rs.getLong("MIN_VALUE"));
-        assertEquals(9, rs.getLong("MAX_VALUE"));
-        assertEquals(true, rs.getBoolean("IS_CYCLE"));
+        assertEquals(5, rs.getLong("MINIMUM_VALUE"));
+        assertEquals(9, rs.getLong("MAXIMUM_VALUE"));
+        assertEquals("YES", rs.getString("CYCLE_OPTION"));
         rs.next();
         assertEquals("SEQUENCE", rs.getString("SEQUENCE_CATALOG"));
         assertEquals("PUBLIC", rs.getString("SEQUENCE_SCHEMA"));
         assertEquals("C", rs.getString("SEQUENCE_NAME"));
-        assertEquals(-2, rs.getLong("CURRENT_VALUE"));
+        assertEquals(-4, rs.getLong("BASE_VALUE"));
         assertEquals(-2, rs.getLong("INCREMENT"));
-        assertEquals(false, rs.getBoolean("IS_GENERATED"));
-        assertEquals("", rs.getString("REMARKS"));
+        assertNull(rs.getString("REMARKS"));
         assertEquals(3, rs.getLong("CACHE"));
-        assertEquals(-9, rs.getLong("MIN_VALUE"));
-        assertEquals(-3, rs.getLong("MAX_VALUE"));
-        assertEquals(false, rs.getBoolean("IS_CYCLE"));
+        assertEquals(-9, rs.getLong("MINIMUM_VALUE"));
+        assertEquals(-3, rs.getLong("MAXIMUM_VALUE"));
+        assertEquals("NO", rs.getString("CYCLE_OPTION"));
         assertFalse(rs.next());
         conn.close();
     }
@@ -271,33 +331,33 @@ public class TestSequence extends TestBase {
                 stat,
                 "create sequence a minvalue 5 start with 2",
                 "Unable to create or alter sequence \"A\" because of " +
-                "invalid attributes (start value \"2\", " +
+                "invalid attributes (base value \"2\", start value \"2\", " +
                 "min value \"5\", max value \"" + Long.MAX_VALUE +
-                "\", increment \"1\")");
+                "\", increment \"1\", cache size \"32\")");
         expectError(
                 stat,
                 "create sequence b maxvalue 5 start with 7",
                 "Unable to create or alter sequence \"B\" because of " +
-                "invalid attributes (start value \"7\", " +
-                        "min value \"1\", max value \"5\", increment \"1\")");
+                "invalid attributes (base value \"7\", start value \"7\", " +
+                        "min value \"1\", max value \"5\", increment \"1\", cache size \"32\")");
         expectError(
                 stat,
                 "create sequence c minvalue 5 maxvalue 2",
                 "Unable to create or alter sequence \"C\" because of " +
-                "invalid attributes (start value \"5\", " +
-                "min value \"5\", max value \"2\", increment \"1\")");
+                "invalid attributes (base value \"5\", start value \"5\", " +
+                "min value \"5\", max value \"2\", increment \"1\", cache size \"32\")");
         expectError(
                 stat,
                 "create sequence d increment by 0",
                 "Unable to create or alter sequence \"D\" because of " +
-                "invalid attributes (start value \"1\", " +
+                "invalid attributes (base value \"1\", start value \"1\", " +
                 "min value \"1\", max value \"" +
-                Long.MAX_VALUE + "\", increment \"0\")");
+                Long.MAX_VALUE + "\", increment \"0\", cache size \"32\")");
         expectError(stat,
                 "create sequence e minvalue 1 maxvalue 5 increment 99",
                 "Unable to create or alter sequence \"E\" because of " +
-                "invalid attributes (start value \"1\", " +
-                "min value \"1\", max value \"5\", increment \"99\")");
+                "invalid attributes (base value \"1\", start value \"1\", " +
+                "min value \"1\", max value \"5\", increment \"99\", cache size \"32\")");
         conn.close();
     }
 
@@ -318,17 +378,18 @@ public class TestSequence extends TestBase {
             script.add(rs.getString(1));
         }
         Collections.sort(script);
-        assertEquals("CREATE SEQUENCE PUBLIC.A START WITH 1;", script.get(0));
-        assertEquals("CREATE SEQUENCE PUBLIC.B START " +
+        assertEquals("-- H2 " + Constants.VERSION + ";", script.get(0));
+        assertEquals("CREATE SEQUENCE \"PUBLIC\".\"A\" START WITH 1;", script.get(1));
+        assertEquals("CREATE SEQUENCE \"PUBLIC\".\"B\" START " +
                 "WITH 5 INCREMENT BY 2 " +
-                "MINVALUE 3 MAXVALUE 7 CYCLE CACHE 1;", script.get(1));
-        assertEquals("CREATE SEQUENCE PUBLIC.C START " +
+                "MINVALUE 3 MAXVALUE 7 CYCLE NO CACHE;", script.get(2));
+        assertEquals("CREATE SEQUENCE \"PUBLIC\".\"C\" START " +
                 "WITH 3 MINVALUE 2 MAXVALUE 9 CACHE 2;",
-                script.get(2));
-        assertEquals("CREATE SEQUENCE PUBLIC.D START " +
-                "WITH 1 CACHE 1;", script.get(3));
-        assertEquals("CREATE SEQUENCE PUBLIC.E START " +
-                "WITH 1 CACHE 1;", script.get(4));
+                script.get(3));
+        assertEquals("CREATE SEQUENCE \"PUBLIC\".\"D\" START " +
+                "WITH 1 NO CACHE;", script.get(4));
+        assertEquals("CREATE SEQUENCE \"PUBLIC\".\"E\" START " +
+                "WITH 1 NO CACHE;", script.get(5));
         conn.close();
     }
 
@@ -434,16 +495,6 @@ public class TestSequence extends TestBase {
             // ignore
         }
 
-        @Override
-        public void close() throws SQLException {
-            // ignore
-        }
-
-        @Override
-        public void remove() throws SQLException {
-            // ignore
-        }
-
     }
 
 }
