@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -13,21 +13,33 @@ import java.sql.Statement;
 import java.sql.Types;
 import java.util.Random;
 import org.h2.api.Aggregate;
+import org.h2.api.ErrorCode;
+import org.h2.message.DbException;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.SimpleResultSet;
 import org.h2.tools.SimpleRowSource;
-import org.h2.value.DataType;
+import org.h2.util.HasSQL;
+import org.h2.value.TypeInfo;
 import org.h2.value.Value;
 import org.h2.value.ValueGeometry;
-import com.vividsolutions.jts.geom.Coordinate;
-import com.vividsolutions.jts.geom.Envelope;
-import com.vividsolutions.jts.geom.Geometry;
-import com.vividsolutions.jts.geom.GeometryFactory;
-import com.vividsolutions.jts.geom.Point;
-import com.vividsolutions.jts.geom.Polygon;
-import com.vividsolutions.jts.geom.util.AffineTransformation;
-import com.vividsolutions.jts.io.ParseException;
-import com.vividsolutions.jts.io.WKTReader;
+import org.h2.value.ValueToObjectConverter;
+import org.h2.value.ValueToObjectConverter2;
+import org.locationtech.jts.geom.Coordinate;
+import org.locationtech.jts.geom.CoordinateSequence;
+import org.locationtech.jts.geom.Envelope;
+import org.locationtech.jts.geom.Geometry;
+import org.locationtech.jts.geom.GeometryFactory;
+import org.locationtech.jts.geom.MultiPoint;
+import org.locationtech.jts.geom.Point;
+import org.locationtech.jts.geom.Polygon;
+import org.locationtech.jts.geom.PrecisionModel;
+import org.locationtech.jts.geom.impl.CoordinateArraySequenceFactory;
+import org.locationtech.jts.geom.util.AffineTransformation;
+import org.locationtech.jts.io.ByteOrderValues;
+import org.locationtech.jts.io.ParseException;
+import org.locationtech.jts.io.WKBWriter;
+import org.locationtech.jts.io.WKTReader;
 
 /**
  * Spatial datatype and index tests.
@@ -36,7 +48,7 @@ import com.vividsolutions.jts.io.WKTReader;
  * @author Noel Grandin
  * @author Nicolas Fortin, Atelier SIG, IRSTV FR CNRS 24888
  */
-public class TestSpatial extends TestBase {
+public class TestSpatial extends TestDb {
 
     private static final String URL = "spatial";
 
@@ -46,25 +58,29 @@ public class TestSpatial extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.memory) {
+            return false;
+        }
+        if (ValueToObjectConverter.GEOMETRY_CLASS == null) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws SQLException {
-        if (!config.mvStore && config.mvcc) {
-            return;
-        }
-        if (config.memory && config.mvcc) {
-            return;
-        }
-        if (DataType.GEOMETRY_CLASS != null) {
-            deleteDb("spatial");
-            testSpatial();
-            deleteDb("spatial");
-        }
+        deleteDb("spatial");
+        testSpatial();
+        deleteDb("spatial");
     }
 
     private void testSpatial() throws SQLException {
+        testNaNs();
         testBug1();
         testSpatialValues();
         testOverlap();
@@ -80,7 +96,6 @@ public class TestSpatial extends TestBase {
         testValueConversion();
         testEquals();
         testTableFunctionGeometry();
-        testHashCode();
         testAggregateWithGeometry();
         testTableViewSpatialPredicate();
         testValueGeometryScript();
@@ -97,6 +112,26 @@ public class TestSpatial extends TestBase {
         testSpatialIndexWithOrder();
     }
 
+    private void testNaNs() {
+        GeometryFactory factory = new GeometryFactory(new PrecisionModel(), 0,
+                CoordinateArraySequenceFactory.instance());
+        CoordinateSequence c2 = factory.getCoordinateSequenceFactory().create(1, 2, 0);
+        c2.setOrdinate(0, 0, 1d);
+        c2.setOrdinate(0, 1, 1d);
+        CoordinateSequence c3 = factory.getCoordinateSequenceFactory().create(1, 3, 0);
+        c3.setOrdinate(0, 0, 1d);
+        c3.setOrdinate(0, 1, 2d);
+        c3.setOrdinate(0, 2, 3d);
+        Point p2 = factory.createPoint(c2);
+        Point p3 = factory.createPoint(c3);
+        try {
+            ValueGeometry.getFromGeometry(new MultiPoint(new Point[] { p2, p3 }, factory));
+            fail("Expected exception");
+        } catch (DbException e) {
+            assertEquals(ErrorCode.DATA_CONVERSION_ERROR_1, e.getErrorCode());
+        }
+    }
+
     private void testBug1() throws SQLException {
         deleteDb("spatial");
         Connection conn = getConnection(URL);
@@ -112,17 +147,6 @@ public class TestSpatial extends TestBase {
         deleteDb("spatial");
     }
 
-    private void testHashCode() {
-        ValueGeometry geomA = ValueGeometry
-                .get("POLYGON ((67 13 6, 67 18 5, 59 18 4, 59 13 6,  67 13 6))");
-        ValueGeometry geomB = ValueGeometry
-                .get("POLYGON ((67 13 6, 67 18 5, 59 18 4, 59 13 6,  67 13 6))");
-        ValueGeometry geomC = ValueGeometry
-                .get("POLYGON ((67 13 6, 67 18 5, 59 18 4, 59 13 5,  67 13 6))");
-        assertEquals(geomA.hashCode(), geomB.hashCode());
-        assertFalse(geomA.hashCode() == geomC.hashCode());
-    }
-
     private void testSpatialValues() throws SQLException {
         deleteDb("spatial");
         Connection conn = getConnection(URL);
@@ -143,6 +167,13 @@ public class TestSpatial extends TestBase {
                 new Coordinate(2, 2),
                 new Coordinate(1, 1) });
         assertTrue(polygon.equals(rs.getObject(2)));
+        rs.close();
+        rs = stat.executeQuery("select id, cast(polygon as varchar) from test");
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt(1));
+        assertEquals("POLYGON ((1 1, 1 2, 2 2, 1 1))", rs.getObject(2));
+        assertTrue(polygon.equals(rs.getObject(2, Geometry.class)));
+        rs.close();
 
         rs = stat.executeQuery("select * from test where polygon = " +
                 "'POLYGON ((1 1, 1 2, 2 2, 1 1))'");
@@ -438,9 +469,7 @@ public class TestSpatial extends TestBase {
                 "explain select * from test " +
                 "where polygon && 'POLYGON ((1 1, 1 2, 2 2, 1 1))'::Geometry");
         rs.next();
-        if (config.mvStore) {
-            assertContains(rs.getString(1), "/* PUBLIC.IDX_TEST_POLYGON: POLYGON &&");
-        }
+        assertContains(rs.getString(1), "/* PUBLIC.IDX_TEST_POLYGON: POLYGON &&");
 
         // TODO equality should probably also use the spatial index
         // rs = stat.executeQuery("explain select * from test " +
@@ -484,8 +513,7 @@ public class TestSpatial extends TestBase {
         deleteDb("spatial");
         try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
-            stat.execute("CREATE ALIAS T_GEOM_FROM_TEXT FOR \"" +
-                    TestSpatial.class.getName() + ".geomFromText\"");
+            stat.execute("CREATE ALIAS T_GEOM_FROM_TEXT FOR '" + TestSpatial.class.getName() + ".geomFromText'");
             stat.execute("create table test(id int primary key " +
                     "auto_increment, the_geom geometry)");
             stat.execute("insert into test(the_geom) values(" +
@@ -507,8 +535,8 @@ public class TestSpatial extends TestBase {
         deleteDb("spatial");
         try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
-            stat.execute("CREATE ALIAS T_RANDOM_GEOM_TABLE FOR \"" +
-                    TestSpatial.class.getName() + ".getRandomGeometryTable\"");
+            stat.execute("CREATE ALIAS T_RANDOM_GEOM_TABLE FOR '" +
+                    TestSpatial.class.getName() + ".getRandomGeometryTable'");
             stat.execute(
                     "create table test as " +
                     "select * from T_RANDOM_GEOM_TABLE(42,20,-100,100,-100,100,4)");
@@ -576,6 +604,7 @@ public class TestSpatial extends TestBase {
      */
     public static Geometry geomFromText(String text, int srid) throws SQLException {
         WKTReader wktReader = new WKTReader();
+        wktReader.setIsOldJtsCoordinateSyntaxAllowed(false);
         try {
             Geometry geom = wktReader.read(text);
             geom.setSRID(srid);
@@ -588,22 +617,51 @@ public class TestSpatial extends TestBase {
     private void testGeometryDataType() {
         GeometryFactory geometryFactory = new GeometryFactory();
         Geometry geometry = geometryFactory.createPoint(new Coordinate(0, 0));
-        assertEquals(Value.GEOMETRY, DataType.getTypeFromClass(geometry.getClass()));
+        assertEquals(TypeInfo.TYPE_GEOMETRY, ValueToObjectConverter2.classToType(geometry.getClass()));
     }
 
     /**
      * Test serialization of Z and SRID values.
      */
     private void testWKB() {
-        ValueGeometry geom3d = ValueGeometry.get(
-                "POLYGON ((67 13 6, 67 18 5, 59 18 4, 59 13 6,  67 13 6))", 27572);
+        String ewkt = "SRID=27572;POLYGON Z ((67 13 6, 67 18 5, 59 18 4, 59 13 6, 67 13 6))";
+        ValueGeometry geom3d = ValueGeometry.get(ewkt);
+        assertEquals(ewkt, geom3d.getString());
         ValueGeometry copy = ValueGeometry.get(geom3d.getBytes());
-        assertEquals(6, copy.getGeometry().getCoordinates()[0].z);
-        assertEquals(5, copy.getGeometry().getCoordinates()[1].z);
-        assertEquals(4, copy.getGeometry().getCoordinates()[2].z);
+        Geometry g = copy.getGeometry();
+        assertEquals(6, g.getCoordinates()[0].getZ());
+        assertEquals(5, g.getCoordinates()[1].getZ());
+        assertEquals(4, g.getCoordinates()[2].getZ());
         // Test SRID
         copy = ValueGeometry.get(geom3d.getBytes());
-        assertEquals(27572, copy.getGeometry().getSRID());
+        assertEquals(27572, g.getSRID());
+
+        Point point = new GeometryFactory().createPoint((new Coordinate(1.1d, 1.2d)));
+        // SRID 0
+        checkSRID(ValueGeometry.getFromGeometry(point).getBytes(), 0);
+        checkSRID(new WKBWriter(2, ByteOrderValues.BIG_ENDIAN, false).write(point), 0);
+        checkSRID(new WKBWriter(2, ByteOrderValues.BIG_ENDIAN, true).write(point), 0);
+        checkSRID(new WKBWriter(2, ByteOrderValues.LITTLE_ENDIAN, false).write(point), 0);
+        checkSRID(new WKBWriter(2, ByteOrderValues.LITTLE_ENDIAN, true).write(point), 0);
+        ewkt = "POINT (1.1 1.2)";
+        assertEquals(ewkt, ValueGeometry.getFromGeometry(point).getString());
+        assertEquals(ewkt, ValueGeometry.get(ewkt).getString());
+        // SRID 1,000,000,000
+        point.setSRID(1_000_000_000);
+        checkSRID(ValueGeometry.getFromGeometry(point).getBytes(), 1_000_000_000);
+        checkSRID(new WKBWriter(2, ByteOrderValues.BIG_ENDIAN, true).write(point), 1_000_000_000);
+        checkSRID(new WKBWriter(2, ByteOrderValues.LITTLE_ENDIAN, true).write(point), 1_000_000_000);
+        ewkt = "SRID=1000000000;POINT (1.1 1.2)";
+        assertEquals(ewkt, ValueGeometry.getFromGeometry(point).getString());
+        assertEquals(ewkt, ValueGeometry.get(ewkt).getString());
+    }
+
+    private void checkSRID(byte[] bytes, int srid) {
+        Point point = (Point) ValueGeometry.getFromEWKB(bytes).getGeometry();
+        assertEquals(1.1, point.getX());
+        assertEquals(1.2, point.getY());
+        assertEquals(srid, point.getSRID());
+        assertEquals(srid, point.getFactory().getSRID());
     }
 
     /**
@@ -613,9 +671,7 @@ public class TestSpatial extends TestBase {
         deleteDb("spatial");
         Connection conn = getConnection(URL);
         Statement stat = conn.createStatement();
-        stat.execute("CREATE ALIAS OBJ_STRING FOR \"" +
-                TestSpatial.class.getName() +
-                ".getObjectString\"");
+        stat.execute("CREATE ALIAS OBJ_STRING FOR '" + TestSpatial.class.getName() + ".getObjectString'");
         ResultSet rs = stat.executeQuery(
                 "select OBJ_STRING('POINT( 15 25 )'::geometry)");
         assertTrue(rs.next());
@@ -630,7 +686,7 @@ public class TestSpatial extends TestBase {
      * @param object the object
      * @return the string representation
      */
-    public static String getObjectString(Object object) {
+    public static String getObjectString(Geometry object) {
         return object.toString();
     }
 
@@ -640,7 +696,7 @@ public class TestSpatial extends TestBase {
     private void testEquals() {
         // 3d equality test
         ValueGeometry geom3d = ValueGeometry.get(
-                "POLYGON ((67 13 6, 67 18 5, 59 18 4, 59 13 6,  67 13 6))");
+                "POLYGON Z((67 13 6, 67 18 5, 59 18 4, 59 13 6,  67 13 6))");
         ValueGeometry geom2d = ValueGeometry.get(
                 "POLYGON ((67 13, 67 18, 59 18, 59 13,  67 13))");
         assertFalse(geom3d.equals(geom2d));
@@ -648,20 +704,14 @@ public class TestSpatial extends TestBase {
         GeometryFactory geometryFactory = new GeometryFactory();
         Geometry geometry = geometryFactory.createPoint(new Coordinate(0, 0));
         geometry.setSRID(27572);
-        ValueGeometry valueGeometry =
-                ValueGeometry.getFromGeometry(geometry);
+        ValueGeometry valueGeometry = ValueGeometry.getFromGeometry(geometry);
         Geometry geometry2 = geometryFactory.createPoint(new Coordinate(0, 0));
         geometry2.setSRID(5326);
-        ValueGeometry valueGeometry2 =
-                ValueGeometry.getFromGeometry(geometry2);
+        ValueGeometry valueGeometry2 = ValueGeometry.getFromGeometry(geometry2);
         assertFalse(valueGeometry.equals(valueGeometry2));
-        // Check illegal geometry (no WKB representation)
-        try {
-            ValueGeometry.get("POINT EMPTY");
-            fail("expected this to throw IllegalArgumentException");
-        } catch (IllegalArgumentException ex) {
-            // expected
-        }
+        ValueGeometry valueGeometry3 = ValueGeometry.getFromGeometry(geometry);
+        assertEquals(valueGeometry, valueGeometry3);
+        assertEquals(geometry.getSRID(), valueGeometry3.getGeometry().getSRID());
     }
 
     /**
@@ -671,8 +721,7 @@ public class TestSpatial extends TestBase {
         deleteDb("spatial");
         try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
-            stat.execute("CREATE ALIAS POINT_TABLE FOR \"" +
-                    TestSpatial.class.getName() + ".pointTable\"");
+            stat.execute("CREATE ALIAS POINT_TABLE FOR '" + TestSpatial.class.getName() + ".pointTable'");
             stat.execute("create table test as select * from point_table(1, 1)");
             // Read column type
             ResultSet columnMeta = conn.getMetaData().
@@ -695,7 +744,7 @@ public class TestSpatial extends TestBase {
     public static ResultSet pointTable(double x, double y) {
         GeometryFactory factory = new GeometryFactory();
         SimpleResultSet rs = new SimpleResultSet();
-        rs.addColumn("THE_GEOM", Types.JAVA_OBJECT, "GEOMETRY", 0, 0);
+        rs.addColumn("THE_GEOM", Types.OTHER, "GEOMETRY", 0, 0);
         rs.addRow(factory.createPoint(new Coordinate(x, y)));
         return rs;
     }
@@ -704,8 +753,7 @@ public class TestSpatial extends TestBase {
         deleteDb("spatialIndex");
         try (Connection conn = getConnection("spatialIndex")) {
             Statement st = conn.createStatement();
-            st.execute("CREATE AGGREGATE TABLE_ENVELOPE FOR \""+
-                    TableEnvelope.class.getName()+"\"");
+            st.execute("CREATE AGGREGATE TABLE_ENVELOPE FOR '" + TableEnvelope.class.getName() + '\'');
             st.execute("CREATE TABLE test(the_geom GEOMETRY)");
             st.execute("INSERT INTO test VALUES ('POINT(1 1)'), (null), (null), ('POINT(10 5)')");
             ResultSet rs = st.executeQuery("select TABLE_ENVELOPE(the_geom) from test");
@@ -793,10 +841,10 @@ public class TestSpatial extends TestBase {
      * Check ValueGeometry conversion into SQL script
      */
     private void testValueGeometryScript() throws SQLException {
-        ValueGeometry valueGeometry = ValueGeometry.get("POINT(1 1 5)");
+        ValueGeometry valueGeometry = ValueGeometry.get("POINT Z(1 1 5)");
         try (Connection conn = getConnection(URL)) {
             ResultSet rs = conn.createStatement().executeQuery(
-                    "SELECT " + valueGeometry.getSQL());
+                    "SELECT " + valueGeometry.getSQL(HasSQL.DEFAULT_SQL_FLAGS));
             assertTrue(rs.next());
             Object obj = rs.getObject(1);
             ValueGeometry g = ValueGeometry.getFromGeometry(obj);
@@ -833,7 +881,7 @@ public class TestSpatial extends TestBase {
             Statement stat = conn.createStatement();
             stat.execute("drop table if exists test");
             stat.execute("create table test(id serial primary key, " +
-                    "value double, the_geom geometry)");
+                    "v double, the_geom geometry)");
             stat.execute("create spatial index spatial on test(the_geom)");
             ResultSet rs = stat.executeQuery("explain select * from test where _ROWID_ = 5");
             assertTrue(rs.next());
@@ -873,19 +921,17 @@ public class TestSpatial extends TestBase {
         try (Connection conn = getConnection(URL)) {
             Statement stat = conn.createStatement();
             stat.execute("drop table if exists pt_cloud;");
-            stat.execute("CREATE TABLE PT_CLOUD(id serial, the_geom geometry) AS " +
-                "SELECT null, CONCAT('POINT(',A.X,' ',B.X,')')::geometry the_geom " +
+            stat.execute("CREATE TABLE PT_CLOUD(id serial, the_geom geometry)");
+            stat.execute("INSERT INTO PT_CLOUD(the_geom) " +
+                "SELECT 'POINT(' || A.X || ' ' || B.X || ')' " +
                 "from system_range(0,120) A,system_range(0,10) B;");
             stat.execute("create spatial index on pt_cloud(the_geom);");
-            ResultSet rs = stat.executeQuery(
+            try (ResultSet rs = stat.executeQuery(
                     "explain select * from  PT_CLOUD " +
-                    "where the_geom && 'POINT(1 1)'");
-            try {
+                            "where the_geom && 'POINT(1 1)'")) {
                 assertTrue(rs.next());
                 assertFalse("H2 should use spatial index got this explain:\n" +
                         rs.getString(1), rs.getString(1).contains("tableScan"));
-            } finally {
-                rs.close();
             }
         }
         deleteDb("spatial");
@@ -917,7 +963,7 @@ public class TestSpatial extends TestBase {
             count++;
             int id = rs.getInt(1);
             if (id == 3 || id == 6) {
-                assertTrue(rs.getObject(2) != null);
+                assertNotNull(rs.getObject(2));
             } else {
                 assertNull(rs.getObject(2));
             }
@@ -936,7 +982,7 @@ public class TestSpatial extends TestBase {
         count = 0;
         while (rs.next()) {
             count++;
-            assertTrue(rs.getObject(2) != null);
+            assertNotNull(rs.getObject(2));
         }
         assertEquals(2, count);
 
@@ -993,7 +1039,7 @@ public class TestSpatial extends TestBase {
                 + "(id identity, the_geom geometry)");
         stat.execute("create spatial index on test(the_geom)");
         for (int i = 0; i < 1000; i++) {
-            stat.execute("insert into test values(null, null)");
+            stat.execute("insert into test(the_geom) values null");
         }
         ResultSet rs = stat.executeQuery("select * from test");
         while (rs.next()) {
@@ -1064,96 +1110,100 @@ public class TestSpatial extends TestBase {
 
     private void testInsertNull() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        Statement stat = conn.createStatement();
-        stat.execute("\n" +
-            "drop table if exists PUBLIC.DUMMY_12;\n" +
-            "CREATE TABLE PUBLIC.DUMMY_12 (\n" +
-            "    \"fid\" serial,\n" +
-            "    Z_ID INTEGER,\n" +
-            "    GEOM GEOMETRY,\n" +
-            "    CONSTRAINT CONSTRAINT_DUMMY_12 PRIMARY KEY (\"fid\")\n" +
-            ");\n" +
-            "CREATE INDEX PRIMARY_KEY_DUMMY_12 ON PUBLIC.DUMMY_12 (\"fid\");\n" +
-            "CREATE spatial INDEX PUBLIC_DUMMY_12_SPATIAL_INDEX_ ON PUBLIC.DUMMY_12 (GEOM);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (123,3125163,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (124,3125164,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (125,3125173,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (126,3125174,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (127,3125175,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (128,3125176,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (129,3125177,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (130,3125178,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (131,3125179,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (132,3125180,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (133,3125335,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (134,3125336,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (135,3125165,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (136,3125337,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (137,3125338,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (138,3125339,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (139,3125340,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (140,3125341,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (141,3125342,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (142,3125343,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (143,3125344,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (144,3125345,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (145,3125346,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (146,3125166,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (147,3125347,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (148,3125348,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (149,3125349,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (150,3125350,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (151,3125351,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (152,3125352,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (153,3125353,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (154,3125354,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (155,3125355,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (156,3125356,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (157,3125167,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (158,3125357,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (159,3125358,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (160,3125359,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (161,3125360,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (162,3125361,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (163,3125362,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (164,3125363,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (165,3125364,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (166,3125365,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (167,3125366,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (168,3125168,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (169,3125367,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (170,3125368,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (171,3125369,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (172,3125370,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (173,3125169,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (174,3125170,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (175,3125171,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (176,3125172,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (177,-2,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (178,-1,NULL);\n" +
-            "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (179," +
-            "-1,NULL);");
-        try (ResultSet rs = stat.executeQuery("select * from DUMMY_12")) {
-            assertTrue(rs.next());
+        try (Connection conn = getConnection(URL)) {
+            Statement stat = conn.createStatement();
+            stat.execute("\n" +
+                "drop table if exists PUBLIC.DUMMY_12;\n" +
+                "CREATE TABLE PUBLIC.DUMMY_12 (\n" +
+                "    \"fid\" serial,\n" +
+                "    Z_ID INTEGER,\n" +
+                "    GEOM GEOMETRY,\n" +
+                "    CONSTRAINT CONSTRAINT_DUMMY_12 PRIMARY KEY (\"fid\")\n" +
+                ");\n" +
+                "CREATE INDEX PRIMARY_KEY_DUMMY_12 ON PUBLIC.DUMMY_12 (\"fid\");\n" +
+                "CREATE spatial INDEX PUBLIC_DUMMY_12_SPATIAL_INDEX_ ON PUBLIC.DUMMY_12 (GEOM);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (123,3125163,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (124,3125164,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (125,3125173,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (126,3125174,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (127,3125175,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (128,3125176,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (129,3125177,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (130,3125178,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (131,3125179,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (132,3125180,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (133,3125335,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (134,3125336,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (135,3125165,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (136,3125337,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (137,3125338,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (138,3125339,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (139,3125340,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (140,3125341,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (141,3125342,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (142,3125343,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (143,3125344,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (144,3125345,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (145,3125346,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (146,3125166,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (147,3125347,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (148,3125348,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (149,3125349,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (150,3125350,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (151,3125351,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (152,3125352,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (153,3125353,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (154,3125354,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (155,3125355,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (156,3125356,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (157,3125167,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (158,3125357,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (159,3125358,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (160,3125359,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (161,3125360,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (162,3125361,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (163,3125362,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (164,3125363,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (165,3125364,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (166,3125365,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (167,3125366,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (168,3125168,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (169,3125367,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (170,3125368,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (171,3125369,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (172,3125370,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (173,3125169,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (174,3125170,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (175,3125171,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (176,3125172,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (177,-2,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (178,-1,NULL);\n" +
+                "INSERT INTO PUBLIC.DUMMY_12 (\"fid\",Z_ID,GEOM) VALUES (179," +
+                "-1,NULL);");
+            try (ResultSet rs = stat.executeQuery("select * from DUMMY_12")) {
+                assertTrue(rs.next());
+            }
         }
-
+        deleteDb("spatial");
     }
 
     private void testSpatialIndexWithOrder() throws SQLException {
         deleteDb("spatial");
-        Connection conn = getConnection(URL);
-        Statement stat = conn.createStatement();
-        stat.execute("DROP TABLE IF EXISTS BUILDINGS;" +
-                "CREATE TABLE BUILDINGS (PK serial, THE_GEOM geometry);" +
-                "insert into buildings(the_geom) SELECT 'POINT(1 1)" +
-                "'::geometry from SYSTEM_RANGE(1,10000);\n" +
-                "CREATE SPATIAL INDEX ON PUBLIC.BUILDINGS(THE_GEOM);\n");
+        try (Connection conn = getConnection(URL)) {
+            Statement stat = conn.createStatement();
+            stat.execute("DROP TABLE IF EXISTS BUILDINGS;" +
+                    "CREATE TABLE BUILDINGS (PK BIGINT GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, " +
+                    "THE_GEOM geometry);" +
+                    "insert into buildings(the_geom) SELECT 'POINT(1 1)" +
+                    "'::geometry from SYSTEM_RANGE(1,10000);\n" +
+                    "CREATE SPATIAL INDEX ON PUBLIC.BUILDINGS(THE_GEOM);\n");
 
-        try (ResultSet rs = stat.executeQuery("EXPLAIN SELECT * FROM " +
-                "BUILDINGS ORDER BY PK LIMIT 51;")) {
-            assertTrue(rs.next());
-            assertTrue(rs.getString(1).contains("PRIMARY_KEY"));
+            try (ResultSet rs = stat.executeQuery("EXPLAIN SELECT * FROM " +
+                    "BUILDINGS ORDER BY PK LIMIT 51;")) {
+                assertTrue(rs.next());
+                assertTrue(rs.getString(1).contains("PRIMARY_KEY"));
+            }
         }
+        deleteDb("spatial");
     }
 }
