@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -11,14 +11,16 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import org.h2.api.ErrorCode;
-import org.h2.engine.Constants;
-import org.h2.store.fs.FileUtils;
+import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
+import org.h2.jdbc.JdbcConnection;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Temporary table tests.
  */
-public class TestTempTables extends TestBase {
+public class TestTempTables extends TestDb {
 
     /**
      * Run just this test.
@@ -26,7 +28,7 @@ public class TestTempTables extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -37,7 +39,6 @@ public class TestTempTables extends TestBase {
         testTempFileResultSet();
         testTempTableResultSet();
         testTransactionalTemp();
-        testDeleteGlobalTempTableWhenClosing();
         Connection c1 = getConnection("tempTables");
         testAlter(c1);
         Connection c2 = getConnection("tempTables");
@@ -57,7 +58,7 @@ public class TestTempTables extends TestBase {
         Statement stat = conn.createStatement();
         stat.execute("create local temporary table test(id identity)");
         PreparedStatement prep = conn
-                .prepareStatement("insert into test values(null)");
+                .prepareStatement("insert into test default values");
         for (int i = 0; i < 10000; i++) {
             prep.execute();
         }
@@ -71,21 +72,18 @@ public class TestTempTables extends TestBase {
         Connection conn = getConnection("tempTables");
         Statement stat = conn.createStatement();
         stat.execute("create local temporary table test(id identity)");
-        ResultSet rs = stat.executeQuery("script");
-        boolean foundSequence = false;
-        while (rs.next()) {
-            if (rs.getString(1).startsWith("CREATE SEQUENCE")) {
-                foundSequence = true;
-            }
+        Session iface = ((JdbcConnection) conn).getSession();
+        if ((iface instanceof SessionLocal)) {
+            assertEquals(1, ((SessionLocal) iface).getDatabase().getMainSchema().getAllSequences().size());
         }
-        assertTrue(foundSequence);
-        stat.execute("insert into test values(null)");
+        stat.execute("insert into test default values");
         stat.execute("shutdown");
         conn.close();
         conn = getConnection("tempTables");
-        rs = conn.createStatement().executeQuery(
-                "select * from information_schema.sequences");
-        assertFalse(rs.next());
+        iface = ((JdbcConnection) conn).getSession();
+        if ((iface instanceof SessionLocal)) {
+            assertEquals(0, ((SessionLocal) iface).getDatabase().getMainSchema().getAllSequences().size());
+        }
         conn.close();
     }
 
@@ -197,7 +195,7 @@ public class TestTempTables extends TestBase {
         stat.execute("commit");
         stat.execute("insert into test values(2)");
         stat.execute("create local temporary table temp(" +
-                "id int primary key, name varchar, constraint x index(name)) transactional");
+                "id int primary key, name varchar, constraint x unique(name)) transactional");
         stat.execute("insert into temp values(3, 'test')");
         stat.execute("rollback");
         rs = stat.executeQuery("select * from test");
@@ -208,34 +206,6 @@ public class TestTempTables extends TestBase {
         conn.close();
     }
 
-    private void testDeleteGlobalTempTableWhenClosing() throws SQLException {
-        if (config.memory) {
-            return;
-        }
-        if (config.mvStore) {
-            return;
-        }
-        deleteDb("tempTables");
-        Connection conn = getConnection("tempTables");
-        Statement stat = conn.createStatement();
-        stat.execute("create global temporary table test(id int, data varchar)");
-        stat.execute("insert into test " +
-                    "select x, space(1000) from system_range(1, 1000)");
-        stat.execute("shutdown compact");
-        try {
-            conn.close();
-        } catch (SQLException e) {
-            // expected
-        }
-        String dbName = getBaseDir() + "/tempTables" + Constants.SUFFIX_PAGE_FILE;
-        long before = FileUtils.size(dbName);
-        assertTrue(before > 0);
-        conn = getConnection("tempTables");
-        conn.close();
-        long after = FileUtils.size(dbName);
-        assertEquals(after, before);
-    }
-
     private void testAlter(Connection conn) throws SQLException {
         Statement stat;
         stat = conn.createStatement();
@@ -318,7 +288,7 @@ public class TestTempTables extends TestBase {
         assertResultRowCount(1, rs);
         c1.commit();
         // test_temp should have been dropped automatically
-        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_1, s1).
+        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_DATABASE_EMPTY_1, s1).
                 executeQuery("select * from test_temp");
     }
 
@@ -327,6 +297,9 @@ public class TestTempTables extends TestBase {
      * transaction table in the MVStore
      */
     private void testLotsOfTables() throws SQLException {
+        if (config.networked || config.throttle > 0) {
+            return; // just to save some testing time
+        }
         deleteDb("tempTables");
         Connection conn = getConnection("tempTables");
         Statement stat = conn.createStatement();
