@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.jdbc;
@@ -13,12 +13,15 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+
+import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Test for batch updates.
  */
-public class TestBatchUpdates extends TestBase {
+public class TestBatchUpdates extends TestDb {
 
     private static final String COFFEE_UPDATE =
             "UPDATE TEST SET PRICE=PRICE*20 WHERE TYPE_ID=?";
@@ -54,7 +57,7 @@ public class TestBatchUpdates extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -75,15 +78,15 @@ public class TestBatchUpdates extends TestBase {
         try {
             stat.executeBatch();
         } catch (SQLException e) {
-            assertContains(e.toString(), "TEST_Y");
-            e = e.getNextException();
-            assertTrue(e != null);
-            assertContains(e.toString(), "TEST_Y");
-            e = e.getNextException();
-            assertTrue(e != null);
             assertContains(e.toString(), "TEST_X");
             e = e.getNextException();
-            assertTrue(e == null);
+            assertNotNull(e);
+            assertContains(e.toString(), "TEST_X");
+            e = e.getNextException();
+            assertNotNull(e);
+            assertContains(e.toString(), "TEST_Y");
+            e = e.getNextException();
+            assertNull(e);
         }
         stat.execute("create table test(id int)");
         PreparedStatement prep = conn.prepareStatement("insert into test values(?)");
@@ -94,15 +97,15 @@ public class TestBatchUpdates extends TestBase {
         try {
             prep.executeBatch();
         } catch (SQLException e) {
-            assertContains(e.toString(), "TEST_Y");
-            e = e.getNextException();
-            assertTrue(e != null);
-            assertContains(e.toString(), "TEST_Y");
-            e = e.getNextException();
-            assertTrue(e != null);
             assertContains(e.toString(), "TEST_X");
             e = e.getNextException();
-            assertTrue(e == null);
+            assertNotNull(e);
+            assertContains(e.toString(), "TEST_X");
+            e = e.getNextException();
+            assertNotNull(e);
+            assertContains(e.toString(), "TEST_Y");
+            e = e.getNextException();
+            assertNull(e);
         }
         stat.execute("drop table test");
         conn.close();
@@ -112,8 +115,7 @@ public class TestBatchUpdates extends TestBase {
         deleteDb("batchUpdates");
         conn = getConnection("batchUpdates");
         stat = conn.createStatement();
-        stat.execute("CREATE ALIAS updatePrices FOR \"" +
-                getClass().getName() + ".updatePrices\"");
+        stat.execute("CREATE ALIAS updatePrices FOR '" + getClass().getName() + ".updatePrices'");
         CallableStatement call = conn.prepareCall("{call updatePrices(?, ?)}");
         call.setString(1, "Hello");
         call.setFloat(2, 1.4f);
@@ -151,12 +153,7 @@ public class TestBatchUpdates extends TestBase {
             prep.setString(1, "x");
             prep.addBatch();
         }
-        try {
-            prep.executeBatch();
-            fail();
-        } catch (BatchUpdateException e) {
-            // expected
-        }
+        assertThrows(BatchUpdateException.class, prep).executeBatch();
         conn.close();
     }
 
@@ -210,6 +207,7 @@ public class TestBatchUpdates extends TestBase {
         String s = COFFEE_UPDATE;
         trace("Prepared Statement String:" + s);
         prep = conn.prepareStatement(s);
+        assertThrows(ErrorCode.PARAMETER_NOT_SET_1, prep).addBatch();
         prep.setInt(1, 2);
         prep.addBatch();
         prep.setInt(1, 3);
@@ -539,7 +537,7 @@ public class TestBatchUpdates extends TestBase {
             trace("Count val is: " + count);
             // make sure that we have the correct error code for
             // the failed update.
-            if (!(batchUpdates[1] == -3 && count == 1)) {
+            if (batchUpdates[1] != -3 || count != 1) {
                 fail("insert failed");
             }
         }
