@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.jdbc;
@@ -10,6 +10,7 @@ import java.io.Reader;
 import java.io.StringReader;
 import java.math.BigDecimal;
 import java.net.URL;
+import java.nio.charset.StandardCharsets;
 import java.sql.Array;
 import java.sql.CallableStatement;
 import java.sql.Connection;
@@ -21,20 +22,23 @@ import java.sql.SQLXML;
 import java.sql.Statement;
 import java.sql.Timestamp;
 import java.sql.Types;
+import java.time.LocalDate;
+import java.time.LocalDateTime;
+import java.time.LocalTime;
 import java.util.Collections;
 
 import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.SimpleResultSet;
 import org.h2.util.IOUtils;
 import org.h2.util.JdbcUtils;
-import org.h2.util.LocalDateTimeUtils;
 import org.h2.util.Utils;
 
 /**
  * Tests for the CallableStatement class.
  */
-public class TestCallableStatement extends TestBase {
+public class TestCallableStatement extends TestDb {
 
     /**
      * Run just this test.
@@ -42,7 +46,7 @@ public class TestCallableStatement extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -64,18 +68,16 @@ public class TestCallableStatement extends TestBase {
     }
 
     private void testOutParameter(Connection conn) throws SQLException {
-        conn.createStatement().execute(
-                "create table test(id identity) as select null");
+        conn.createStatement().execute("CREATE SEQUENCE SEQ");
         for (int i = 1; i < 20; i++) {
-            CallableStatement cs = conn.prepareCall("{ ? = call IDENTITY()}");
+            CallableStatement cs = conn.prepareCall("{ ? = CALL NEXT VALUE FOR SEQ}");
             cs.registerOutParameter(1, Types.BIGINT);
             cs.execute();
             long id = cs.getLong(1);
-            assertEquals(1, id);
+            assertEquals(i, id);
             cs.close();
         }
-        conn.createStatement().execute(
-                "drop table test");
+        conn.createStatement().execute("DROP SEQUENCE SEQ");
     }
 
     private void testUnsupportedOperations(Connection conn) throws SQLException {
@@ -84,24 +86,20 @@ public class TestCallableStatement extends TestBase {
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 getURL(1);
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
-                getObject(1, Collections.<String, Class<?>>emptyMap());
+                getObject(1, Collections.emptyMap());
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 getRef(1);
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 getRowId(1);
-        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
-                getSQLXML(1);
 
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 getURL("a");
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
-                getObject("a", Collections.<String, Class<?>>emptyMap());
+                getObject("a", Collections.emptyMap());
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 getRef("a");
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 getRowId("a");
-        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
-                getSQLXML("a");
 
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 setURL(1, (URL) null);
@@ -109,16 +107,11 @@ public class TestCallableStatement extends TestBase {
                 setRef(1, (Ref) null);
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 setRowId(1, (RowId) null);
-        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
-                setSQLXML(1, (SQLXML) null);
 
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 setURL("a", (URL) null);
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
                 setRowId("a", (RowId) null);
-        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, call).
-                setSQLXML("a", (SQLXML) null);
-
     }
 
     private void testCallWithResultSet(Connection conn) throws SQLException {
@@ -174,29 +167,20 @@ public class TestCallableStatement extends TestBase {
         call.registerOutParameter(1, Types.DATE);
         call.execute();
         assertEquals("2000-01-01", call.getDate(1).toString());
-        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
-            assertEquals("2000-01-01", call.getObject(1,
-                            LocalDateTimeUtils.getLocalDateClass()).toString());
-        }
+        assertEquals("2000-01-01", call.getObject(1, LocalDate.class).toString());
 
         call.setTime(2, java.sql.Time.valueOf("01:02:03"));
         call.registerOutParameter(1, Types.TIME);
         call.execute();
         assertEquals("01:02:03", call.getTime(1).toString());
-        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
-            assertEquals("01:02:03", call.getObject(1,
-                            LocalDateTimeUtils.getLocalTimeClass()).toString());
-        }
+        assertEquals("01:02:03", call.getObject(1, LocalTime.class).toString());
 
         call.setTimestamp(2, java.sql.Timestamp.valueOf(
                 "2001-02-03 04:05:06.789"));
         call.registerOutParameter(1, Types.TIMESTAMP);
         call.execute();
         assertEquals("2001-02-03 04:05:06.789", call.getTimestamp(1).toString());
-        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
-            assertEquals("2001-02-03T04:05:06.789", call.getObject(1,
-                            LocalDateTimeUtils.getLocalDateTimeClass()).toString());
-        }
+        assertEquals("2001-02-03T04:05:06.789", call.getObject(1, LocalDateTime.class).toString());
 
         call.setBoolean(2, true);
         call.registerOutParameter(1, Types.BIT);
@@ -254,9 +238,8 @@ public class TestCallableStatement extends TestBase {
         assertEquals(1, rs.getInt(1));
         assertEquals("Hello", rs.getString(2));
         assertFalse(rs.next());
-        stat.execute("CREATE ALIAS testCall FOR \"" +
-                    getClass().getName() + ".testCall\"");
-        call = conn.prepareCall("{CALL testCall(?, ?, ?, ?)}");
+        stat.execute("CREATE ALIAS testCall FOR '" + getClass().getName() + ".testCall'");
+        call = conn.prepareCall("{SELECT * FROM testCall(?, ?, ?, ?)}");
         call.setInt("A", 50);
         call.setString("B", "abc");
         long t = System.currentTimeMillis();
@@ -265,12 +248,7 @@ public class TestCallableStatement extends TestBase {
         call.registerOutParameter(1, Types.INTEGER);
         call.registerOutParameter("B", Types.VARCHAR);
         call.executeUpdate();
-        try {
-            call.getTimestamp("C");
-            fail("not registered out parameter accessible");
-        } catch (SQLException e) {
-            // expected exception
-        }
+        assertThrows(ErrorCode.INVALID_VALUE_2, call).getTimestamp("C");
         call.registerOutParameter(3, Types.TIMESTAMP);
         call.registerOutParameter(4, Types.TIMESTAMP);
         call.executeUpdate();
@@ -280,28 +258,16 @@ public class TestCallableStatement extends TestBase {
 
         assertEquals("2001-02-03 10:20:30.0", call.getTimestamp(4).toString());
         assertEquals("2001-02-03 10:20:30.0", call.getTimestamp("D").toString());
-        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
-            assertEquals("2001-02-03T10:20:30", call.getObject(4,
-                            LocalDateTimeUtils.getLocalDateTimeClass()).toString());
-            assertEquals("2001-02-03T10:20:30", call.getObject("D",
-                            LocalDateTimeUtils.getLocalDateTimeClass()).toString());
-        }
+        assertEquals("2001-02-03T10:20:30", call.getObject(4, LocalDateTime.class).toString());
+        assertEquals("2001-02-03T10:20:30", call.getObject("D", LocalDateTime.class).toString());
         assertEquals("10:20:30", call.getTime(4).toString());
         assertEquals("10:20:30", call.getTime("D").toString());
-        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
-            assertEquals("10:20:30", call.getObject(4,
-                            LocalDateTimeUtils.getLocalTimeClass()).toString());
-            assertEquals("10:20:30", call.getObject("D",
-                            LocalDateTimeUtils.getLocalTimeClass()).toString());
-        }
+        assertEquals("10:20:30", call.getObject(4, LocalTime.class).toString());
+        assertEquals("10:20:30", call.getObject("D", LocalTime.class).toString());
         assertEquals("2001-02-03", call.getDate(4).toString());
         assertEquals("2001-02-03", call.getDate("D").toString());
-        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
-            assertEquals("2001-02-03", call.getObject(4,
-                            LocalDateTimeUtils.getLocalDateClass()).toString());
-            assertEquals("2001-02-03", call.getObject("D",
-                            LocalDateTimeUtils.getLocalDateClass()).toString());
-        }
+        assertEquals("2001-02-03", call.getObject(4, LocalDate.class).toString());
+        assertEquals("2001-02-03", call.getObject("D", LocalDate.class).toString());
 
         assertEquals(100, call.getInt(1));
         assertEquals(100, call.getInt("A"));
@@ -332,25 +298,12 @@ public class TestCallableStatement extends TestBase {
         assertEquals("ABC", call.getClob("B").getSubString(1, 3));
         assertEquals("ABC", call.getNClob(2).getSubString(1, 3));
         assertEquals("ABC", call.getNClob("B").getSubString(1, 3));
+        assertEquals("ABC", call.getSQLXML(2).getString());
+        assertEquals("ABC", call.getSQLXML("B").getString());
 
-        try {
-            call.getString(100);
-            fail("incorrect parameter index value");
-        } catch (SQLException e) {
-            // expected exception
-        }
-        try {
-            call.getString(0);
-            fail("incorrect parameter index value");
-        } catch (SQLException e) {
-            // expected exception
-        }
-        try {
-            call.getBoolean("X");
-            fail("incorrect parameter name value");
-        } catch (SQLException e) {
-            // expected exception
-        }
+        assertThrows(ErrorCode.INVALID_VALUE_2, call).getString(100);
+        assertThrows(ErrorCode.INVALID_VALUE_2, call).getString(0);
+        assertThrows(ErrorCode.INVALID_VALUE_2, call).getBoolean("X");
 
         call.setCharacterStream("B",
                 new StringReader("xyz"));
@@ -365,15 +318,15 @@ public class TestCallableStatement extends TestBase {
         call.executeUpdate();
         assertEquals("XYZ", call.getString("B"));
         call.setAsciiStream("B",
-                new ByteArrayInputStream("xyz".getBytes("UTF-8")));
+                new ByteArrayInputStream("xyz".getBytes(StandardCharsets.UTF_8)));
         call.executeUpdate();
         assertEquals("XYZ", call.getString("B"));
         call.setAsciiStream("B",
-                new ByteArrayInputStream("xyz-".getBytes("UTF-8")), 3);
+                new ByteArrayInputStream("xyz-".getBytes(StandardCharsets.UTF_8)), 3);
         call.executeUpdate();
         assertEquals("XYZ", call.getString("B"));
         call.setAsciiStream("B",
-                new ByteArrayInputStream("xyz-".getBytes("UTF-8")), 3L);
+                new ByteArrayInputStream("xyz-".getBytes(StandardCharsets.UTF_8)), 3L);
         call.executeUpdate();
         assertEquals("XYZ", call.getString("B"));
 
@@ -397,6 +350,11 @@ public class TestCallableStatement extends TestBase {
         call.setNString("B", "xyz");
         call.executeUpdate();
         assertEquals("XYZ", call.getString("B"));
+        SQLXML xml = conn.createSQLXML();
+        xml.setString("<x>xyz</x>");
+        call.setSQLXML("B", xml);
+        call.executeUpdate();
+        assertEquals("<X>XYZ</X>", call.getString("B"));
 
         // test for exceptions after closing
         call.close();
@@ -413,7 +371,7 @@ public class TestCallableStatement extends TestBase {
         JdbcUtils.addClassFactory(myFactory);
         try {
             Statement stat = conn.createStatement();
-            stat.execute("CREATE ALIAS T_CLASSLOADER FOR \"TestClassFactory.testClassF\"");
+            stat.execute("CREATE ALIAS T_CLASSLOADER FOR 'TestClassFactory.testClassF'");
             ResultSet rs = stat.executeQuery("SELECT T_CLASSLOADER(true)");
             assertTrue(rs.next());
             assertEquals(false, rs.getBoolean(1));
@@ -425,8 +383,7 @@ public class TestCallableStatement extends TestBase {
     private void testArrayArgument(Connection connection) throws SQLException {
         Array array = connection.createArrayOf("Int", new Object[] {0, 1, 2});
         try (Statement statement = connection.createStatement()) {
-            statement.execute("CREATE ALIAS getArrayLength FOR \"" +
-                            getClass().getName() + ".getArrayLength\"");
+            statement.execute("CREATE ALIAS getArrayLength FOR '" + getClass().getName() + ".getArrayLength'");
 
             // test setArray
             try (CallableStatement callableStatement = connection
@@ -459,18 +416,16 @@ public class TestCallableStatement extends TestBase {
     }
 
     private void testArrayReturnValue(Connection connection) throws SQLException {
-        Object[][] arraysToTest = new Object[][] {
-            new Object[] {0, 1, 2},
-            new Object[] {0, "1", 2},
-            new Object[] {0, null, 2},
-            new Object[] {0, new Object[] {"s", 1}, new Object[] {null, 1L}},
+        Integer[][] arraysToTest = new Integer[][] {
+            {0, 1, 2},
+            {0, 1, 2},
+            {0, null, 2},
         };
         try (Statement statement = connection.createStatement()) {
-            statement.execute("CREATE ALIAS arrayIdentiy FOR \"" +
-                            getClass().getName() + ".arrayIdentiy\"");
+            statement.execute("CREATE ALIAS arrayIdentiy FOR '" + getClass().getName() + ".arrayIdentiy'");
 
-            for (Object[] arrayToTest : arraysToTest) {
-                Array sqlInputArray = connection.createArrayOf("ignored", arrayToTest);
+            for (Integer[] arrayToTest : arraysToTest) {
+                Array sqlInputArray = connection.createArrayOf("INTEGER", arrayToTest);
                 try {
                     try (CallableStatement callableStatement = connection
                             .prepareCall("{call arrayIdentiy(?)}")) {
@@ -526,7 +481,7 @@ public class TestCallableStatement extends TestBase {
      * @param array the array
      * @return the length of the array
      */
-    public static int getArrayLength(Object[] array) {
+    public static int getArrayLength(Integer[] array) {
         return array == null ? 0 : array.length;
     }
 
@@ -536,7 +491,7 @@ public class TestCallableStatement extends TestBase {
      * @param array the array
      * @return the array
      */
-    public static Object[] arrayIdentiy(Object[] array) {
+    public static Integer[] arrayIdentiy(Integer[] array) {
         return array;
     }
 
