@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.jdbc;
@@ -13,13 +13,13 @@ import java.util.Properties;
 
 import org.h2.Driver;
 import org.h2.api.DatabaseEventListener;
-import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests the DatabaseEventListener interface.
  */
-public class TestDatabaseEventListener extends TestBase {
+public class TestDatabaseEventListener extends TestDb {
 
     /**
      * A flag to mark that the given method was called.
@@ -34,7 +34,7 @@ public class TestDatabaseEventListener extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -78,21 +78,6 @@ public class TestDatabaseEventListener extends TestBase {
             }
         }
 
-        @Override
-        public void closingDatabase() {
-            // nothing to do
-        }
-
-        @Override
-        public void exceptionThrown(SQLException e, String sql) {
-            // nothing to do
-        }
-
-        @Override
-        public void setProgress(int state, String name, int x, int max) {
-            // nothing to do
-        }
-
     }
 
     private void testInit() throws SQLException {
@@ -118,32 +103,29 @@ public class TestDatabaseEventListener extends TestBase {
         Properties p = new Properties();
         p.setProperty("user", user);
         p.setProperty("password", password);
-        Connection conn;
         Statement stat;
-        conn = DriverManager.getConnection(url, p);
-        stat = conn.createStatement();
-        // the old.id index head is at position 0
-        stat.execute("create table old(id identity) as select 1");
-        // the test.id index head is at position 1
-        stat.execute("create table test(id identity) as select 1");
-        conn.close();
-        conn = DriverManager.getConnection(url, p);
-        stat = conn.createStatement();
-        // free up space at position 0
-        stat.execute("drop table old");
-        stat.execute("insert into test values(2)");
-        stat.execute("checkpoint sync");
-        stat.execute("shutdown immediately");
-        assertThrows(ErrorCode.DATABASE_IS_CLOSED, conn).close();
+        try (Connection conn = DriverManager.getConnection(url, p)) {
+            stat = conn.createStatement();
+            // the old.id index head is at position 0
+            stat.execute("create table old(id identity) as select 1");
+            // the test.id index head is at position 1
+            stat.execute("create table test(id identity) as select 1");
+        }
+        try (Connection conn = DriverManager.getConnection(url, p)) {
+            stat = conn.createStatement();
+            // free up space at position 0
+            stat.execute("drop table old");
+            stat.execute("insert into test values(2)");
+            stat.execute("checkpoint sync");
+            stat.execute("shutdown immediately");
+        }
         // now the index should be re-built
-        conn = DriverManager.getConnection(url, p);
-        conn.close();
+        try (Connection conn = DriverManager.getConnection(url, p)) {/**/}
         calledCreateIndex = false;
         p.put("DATABASE_EVENT_LISTENER",
                 MyDatabaseEventListener.class.getName());
-        conn = org.h2.Driver.load().connect(url, p);
-        conn.close();
-        assertTrue(!calledCreateIndex);
+        try (Connection conn = org.h2.Driver.load().connect(url, p)) {/**/}
+        assertFalse(calledCreateIndex);
     }
 
     private void testIndexNotRebuilt() throws SQLException {
@@ -176,7 +158,7 @@ public class TestDatabaseEventListener extends TestBase {
                 MyDatabaseEventListener.class.getName());
         conn = org.h2.Driver.load().connect(url, p);
         conn.close();
-        assertTrue(!calledCreateIndex);
+        assertFalse(calledCreateIndex);
     }
 
     private void testCloseLog0(boolean shutdown) throws SQLException {
@@ -205,7 +187,7 @@ public class TestDatabaseEventListener extends TestBase {
         conn = org.h2.Driver.load().connect(url, p);
         conn.close();
         if (calledOpened) {
-            assertTrue(!calledScan);
+            assertFalse(calledScan);
         }
     }
 
@@ -247,31 +229,20 @@ public class TestDatabaseEventListener extends TestBase {
     /**
      * The database event listener for this test.
      */
-    public static final class MyDatabaseEventListener implements
-            DatabaseEventListener {
+    public static final class MyDatabaseEventListener implements DatabaseEventListener {
 
         @Override
         public void closingDatabase() {
             calledClosingDatabase = true;
         }
 
-        @Override
-        public void exceptionThrown(SQLException e, String sql) {
-            // nothing to do
-        }
-
-        @Override
-        public void init(String url) {
-            // nothing to do
-        }
-
         @Override
         public void opened() {
             calledOpened = true;
         }
 
         @Override
-        public void setProgress(int state, String name, int x, int max) {
+        public void setProgress(int state, String name, long x, long max) {
             if (state == DatabaseEventListener.STATE_SCAN_FILE) {
                 calledScan = true;
             }
