@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.jdbc;
@@ -13,11 +13,12 @@ import java.sql.SQLException;
 import java.sql.Statement;
 
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests the server by creating many JDBC objects (result sets and so on).
  */
-public class TestManyJdbcObjects extends TestBase {
+public class TestManyJdbcObjects extends TestDb {
 
     /**
      * Run just this test.
@@ -25,7 +26,7 @@ public class TestManyJdbcObjects extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -45,8 +46,8 @@ public class TestManyJdbcObjects extends TestBase {
         DatabaseMetaData meta = conn.getMetaData();
         ResultSet rsTables = meta.getColumns(null, null, null, null);
         while (rsTables.next()) {
-            meta.getExportedKeys(null, null, null);
-            meta.getImportedKeys(null, null, null);
+            meta.getExportedKeys(null, null, "TEST");
+            meta.getImportedKeys(null, null, "TEST");
         }
         conn.close();
     }
