@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.jdbc;
@@ -12,13 +12,14 @@ import java.sql.Statement;
 import java.sql.Types;
 import org.h2.api.JavaObjectSerializer;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests per-db {@link JavaObjectSerializer} when set through the JDBC URL.
  *
  * @author Davide Cavestro
  */
-public class TestUrlJavaObjectSerializer extends TestBase {
+public class TestUrlJavaObjectSerializer extends TestDb {
 
     /**
      * Run just this test.
@@ -30,9 +31,7 @@ public class TestUrlJavaObjectSerializer extends TestBase {
         test.config.traceTest = true;
         test.config.memory = true;
         test.config.networked = true;
-        test.config.beforeTest();
-        test.test();
-        test.config.afterTest();
+        test.testFromMain();
     }
 
     @Override
