@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.mvcc;
@@ -11,14 +11,14 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Timestamp;
-import java.util.Map;
 import java.util.concurrent.CountDownLatch;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Additional MVCC (multi version concurrency) test cases.
  */
-public class TestMvcc4 extends TestBase {
+public class TestMvcc4 extends TestDb {
 
     /**
      * Run just this test.
@@ -27,21 +27,26 @@ public class TestMvcc4 extends TestBase {
      */
     public static void main(String... a) throws Exception {
         TestBase test = TestBase.createCaller().init();
-        test.config.mvcc = true;
         test.config.lockTimeout = 20000;
         test.config.memory = true;
-        test.test();
+        test.testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.networked) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws SQLException {
-        if (config.networked) {
-            return;
-        }
         testSelectForUpdateAndUpdateConcurrency();
     }
 
     private void testSelectForUpdateAndUpdateConcurrency() throws SQLException {
+        deleteDb("mvcc4");
         Connection setup = getConnection("mvcc4");
         setup.setAutoCommit(false);
 
@@ -67,7 +72,6 @@ public class TestMvcc4 extends TestBase {
         c1.setAutoCommit(false);
 
         //Fire off a concurrent update.
-        final Thread mainThread = Thread.currentThread();
         final CountDownLatch executedUpdate = new CountDownLatch(1);
         new Thread() {
             @Override
@@ -82,7 +86,14 @@ public class TestMvcc4 extends TestBase {
                     ps.executeQuery().next();
 
                     executedUpdate.countDown();
-                    waitForThreadToBlockOnDB(mainThread);
+                    // interrogate new "blocker_id" metatable field instead of
+                    // relying on stacktraces!? to determine when session is blocking
+                    PreparedStatement stmt = c2.prepareStatement(
+                            "SELECT * FROM INFORMATION_SCHEMA.SESSIONS WHERE BLOCKER_ID = SESSION_ID()");
+                    ResultSet resultSet;
+                    do {
+                        resultSet = stmt.executeQuery();
+                    } while(!resultSet.next());
 
                     c2.commit();
                     c2.close();
@@ -103,7 +114,7 @@ public class TestMvcc4 extends TestBase {
         // for lock case.
         PreparedStatement ps = c1.prepareStatement("UPDATE test SET lastUpdated = ?");
         ps.setTimestamp(1, new Timestamp(System.currentTimeMillis()));
-        ps.executeUpdate();
+        assertEquals(2, ps.executeUpdate());
 
         c1.commit();
         c1.close();
@@ -114,44 +125,12 @@ public class TestMvcc4 extends TestBase {
         ps = verify.prepareStatement("SELECT COUNT(*) FROM test");
         ResultSet rs = ps.executeQuery();
         assertTrue(rs.next());
-        assertTrue(rs.getInt(1) == 2);
+        assertEquals(2,rs.getInt(1));
         verify.commit();
         verify.close();
 
         setup.close();
     }
-
-    /**
-     * Wait for the given thread to block on synchronizing on the database
-     * object.
-     *
-     * @param t the thread
-     */
-    void waitForThreadToBlockOnDB(Thread t) {
-        while (true) {
-            // sleep the first time through the loop so we give the main thread
-            // a chance
-            try {
-                Thread.sleep(20);
-            } catch (InterruptedException e1) {
-                // ignore
-            }
-            // TODO must not use getAllStackTraces, as the method names are
-            // implementation details
-            Map<Thread, StackTraceElement[]> threadMap = Thread.getAllStackTraces();
-            StackTraceElement[] elements = threadMap.get(t);
-            if (elements != null
-                    &&
-                    elements.length > 1 &&
-                    (config.multiThreaded ? "sleep".equals(elements[0]
-                            .getMethodName()) : "wait".equals(elements[0]
-                            .getMethodName())) &&
-                    "filterConcurrentUpdate"
-                            .equals(elements[1].getMethodName())) {
-                return;
-            }
-        }
-    }
 }
 
 
