@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.poweroff;
@@ -20,15 +20,15 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-import java.text.SimpleDateFormat;
+import java.time.LocalDateTime;
+import java.time.format.DateTimeFormatter;
 import java.util.ArrayList;
-import java.util.Date;
 import java.util.List;
 import java.util.Random;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipOutputStream;
+
 import org.h2.util.IOUtils;
-import org.h2.util.New;
 
 /**
  * This standalone test checks if recovery of a database works after power
@@ -54,7 +54,7 @@ public class TestRecover {
     //         "jdbc:derby:/temp/derby/data/test;create=true");
     // private static final String DRIVER =
     //     System.getProperty("test.driver",
-    //         "org.apache.derby.jdbc.EmbeddedDriver");
+    //         "org.apache.derby.iapi.jdbc.AutoloadedDriver");
 
     /**
      * This method is called when executing this application from the command
@@ -103,10 +103,9 @@ public class TestRecover {
             }
             oldest.delete();
         }
-        SimpleDateFormat sd = new SimpleDateFormat("yyMMdd-HHmmss");
-        String date = sd.format(new Date());
+        String date = DateTimeFormatter.ofPattern("yyMMdd-HHmmss").format(LocalDateTime.now());
         File zipFile = new File(root, "backup-" + date + "-" + node + ".zip");
-        ArrayList<File> list = New.arrayList();
+        ArrayList<File> list = new ArrayList<>();
         File base = new File(sourcePath);
         listRecursive(list, base);
         if (list.size() == 0) {
@@ -205,7 +204,7 @@ public class TestRecover {
                 // ignore
             }
             try {
-                Driver driver = (Driver) Class.forName(DRIVER).newInstance();
+                Driver driver = (Driver) Class.forName(DRIVER).getDeclaredConstructor().newInstance();
                 DriverManager.registerDriver(driver);
             } catch (Exception e) {
                 e.printStackTrace();
