@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.poweroff;
@@ -39,12 +39,12 @@ public class TestWrite {
         testFile("rwd", true);
         testFile("rws", true);
         testDatabase("org.h2.Driver",
-                "jdbc:h2:test", "sa", "");
+                "jdbc:h2:./test", "sa", "");
         testDatabase("org.hsqldb.jdbcDriver",
                 "jdbc:hsqldb:test4", "sa", "");
-        testDatabase("org.apache.derby.jdbc.EmbeddedDriver",
+        testDatabase("org.apache.derby.iapi.jdbc.AutoloadedDriver",
                 "jdbc:derby:test;create=true", "sa", "");
-        testDatabase("com.mysql.jdbc.Driver",
+        testDatabase("com.mysql.cj.jdbc.Driver",
                 "jdbc:mysql://localhost/test", "sa", "sa");
         testDatabase("org.postgresql.Driver",
                 "jdbc:postgresql:test", "sa", "sa");
