@@ -40,3 +40,12 @@ EXPLAIN VALUES (TRUE, FALSE, UNKNOWN);
 
 EXPLAIN SELECT A IS TRUE OR B IS FALSE FROM (VALUES (TRUE, TRUE)) T(A, B);
 >> SELECT ("A" IS TRUE) OR ("B" IS FALSE) FROM (VALUES (TRUE, TRUE)) "T"("A", "B") /* table scan */
+
+SET MODE MySQL;
+> ok
+
+CREATE TABLE TEST(A BIT(1));
+> ok
+
+DROP TABLE TEST;
+> ok
