@@ -320,3 +320,30 @@ VALUES NULL UNION VALUES CAST('POINT(1 1)' AS GEOMETRY(POINT));
 > POINT (1 1)
 > null
 > rows: 2
+
+VALUES CAST(GEOMETRY 'POINT EMPTY' AS GEOMETRY)
+UNION
+VALUES CAST(GEOMETRY 'SRID=10;POINT EMPTY' AS GEOMETRY(GEOMETRY, 10));
+> C1
+> -------------------
+> POINT EMPTY
+> SRID=10;POINT EMPTY
+> rows: 2
+
+VALUES CAST(GEOMETRY 'POINT EMPTY' AS GEOMETRY(POINT))
+UNION
+VALUES CAST(GEOMETRY 'SRID=10;POINT EMPTY' AS GEOMETRY(POINT, 10));
+> C1
+> -------------------
+> POINT EMPTY
+> SRID=10;POINT EMPTY
+> rows: 2
+
+VALUES CAST(GEOMETRY 'POINT EMPTY' AS GEOMETRY(POINT))
+UNION
+VALUES CAST(GEOMETRY 'SRID=10;MULTIPOINT EMPTY' AS GEOMETRY(MULTIPOINT, 10));
+> C1
+> ------------------------
+> POINT EMPTY
+> SRID=10;MULTIPOINT EMPTY
+> rows: 2
