@@ -1100,3 +1100,6 @@ SELECT T1, T2, (T1 - T2) YEAR TO MONTH, (T2 - T1) YEAR TO MONTH FROM (VALUES
 
 SELECT (DATE '2010-01-02' - DATE '2000-01-01') YEAR;
 >> INTERVAL '10' YEAR
+
+VALUES INTERVAL '100' YEAR(2);
+> exception INVALID_DATETIME_CONSTANT_2
