@@ -12,7 +12,9 @@ SELECT N'A' 'b'
 >> Abc
 
 CREATE TABLE TEST(C1 VARCHAR, C2 CHARACTER VARYING, C3 VARCHAR2, C4 NVARCHAR, C5 NVARCHAR2, C6 VARCHAR_CASESENSITIVE,
-    C7 LONGVARCHAR, C8 TID, C9 CHAR VARYING, C10 NCHAR VARYING, C11 NATIONAL CHARACTER VARYING, C12 NATIONAL CHAR VARYING);
+    C7 LONGVARCHAR, C8 TID, C9 CHAR VARYING,
+    C10 NCHAR VARYING, C11 NATIONAL CHARACTER VARYING, C12 NATIONAL CHAR VARYING,
+    C13 TINYTEXT, C14 TEXT, C15 MEDIUMTEXT, C16 LONGTEXT, C17 NTEXT);
 > ok
 
 SELECT COLUMN_NAME, DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS
@@ -31,7 +33,12 @@ SELECT COLUMN_NAME, DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS
 > C10         CHARACTER VARYING
 > C11         CHARACTER VARYING
 > C12         CHARACTER VARYING
-> rows (ordered): 12
+> C13         CHARACTER VARYING
+> C14         CHARACTER VARYING
+> C15         CHARACTER VARYING
+> C16         CHARACTER VARYING
+> C17         CHARACTER VARYING
+> rows (ordered): 17
 
 DROP TABLE TEST;
 > ok
