@@ -29,3 +29,5 @@ SELECT * FROM (VALUES 1, 2) AS T1(X), (VALUES 1, 2) AS T2(X) WHERE ROWNUM = 1;
 > 1 1
 > rows: 1
 
+SELECT 1 ORDER BY ROWNUM;
+>> 1
