@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.server;
@@ -11,9 +11,10 @@ import java.io.IOException;
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
+import java.net.ConnectException;
+import java.nio.charset.StandardCharsets;
 import java.security.Principal;
 import java.sql.Connection;
-import java.sql.SQLException;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.Locale;
@@ -24,6 +25,8 @@ import javax.servlet.AsyncContext;
 import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletConfig;
+import javax.servlet.ServletContext;
+import javax.servlet.ServletException;
 import javax.servlet.ServletInputStream;
 import javax.servlet.ServletOutputStream;
 import javax.servlet.ServletRequest;
@@ -35,8 +38,6 @@ import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import javax.servlet.http.HttpUpgradeHandler;
 import javax.servlet.http.Part;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletException;
 
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
@@ -44,15 +45,16 @@ import org.h2.engine.SysProperties;
 import org.h2.server.web.WebServlet;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
-import org.h2.test.utils.AssertThrows;
+import org.h2.test.TestDb;
 import org.h2.tools.Server;
 import org.h2.util.StringUtils;
 import org.h2.util.Task;
+import org.h2.util.Utils10;
 
 /**
  * Tests the H2 Console application.
  */
-public class TestWeb extends TestBase {
+public class TestWeb extends TestDb {
 
     private static volatile String lastUrl;
 
@@ -62,7 +64,7 @@ public class TestWeb extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -79,11 +81,9 @@ public class TestWeb extends TestBase {
 
     private void testServlet() throws Exception {
         WebServlet servlet = new WebServlet();
-        final HashMap<String, String> configMap = new HashMap<String, String>();
+        final HashMap<String, String> configMap = new HashMap<>();
         configMap.put("ifExists", "");
         configMap.put("", "");
-        configMap.put("", "");
-        configMap.put("", "");
         ServletConfig config = new ServletConfig() {
 
             @Override
@@ -93,7 +93,7 @@ public class TestWeb extends TestBase {
 
             @Override
             public Enumeration<String> getInitParameterNames() {
-                return new Vector<String>(configMap.keySet()).elements();
+                return new Vector<>(configMap.keySet()).elements();
             }
 
             @Override
@@ -120,22 +120,10 @@ public class TestWeb extends TestBase {
         servlet.destroy();
     }
 
-    private static void testWrongParameters() {
-        new AssertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1) {
-            @Override
-            public void test() throws SQLException {
-                Server.createPgServer("-pgPort 8182");
-        }};
-        new AssertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1) {
-            @Override
-            public void test() throws SQLException {
-                Server.createTcpServer("-tcpPort 8182");
-        }};
-        new AssertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1) {
-            @Override
-            public void test() throws SQLException {
-                Server.createWebServer("-webPort=8182");
-        }};
+    private void testWrongParameters() {
+        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, () -> Server.createPgServer("-pgPort 8182"));
+        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, () -> Server.createTcpServer("-tcpPort 8182"));
+        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, () -> Server.createWebServer("-webPort=8182"));
     }
 
     private void testAlreadyRunning() throws Exception {
@@ -169,7 +157,7 @@ public class TestWeb extends TestBase {
         Server server = new Server();
         server.setOut(new PrintStream(new ByteArrayOutputStream()));
         server.runTool("-web", "-webPort", "8182",
-                "-properties", "null", "-tcp", "-tcpPort", "9101");
+                "-properties", "null", "-tcp", "-tcpPort", "9101", "-webAdminPassword", "123");
         try {
             String url = "http://localhost:8182";
             WebClient client;
@@ -177,6 +165,7 @@ public class TestWeb extends TestBase {
             client = new WebClient();
             result = client.get(url);
             client.readSessionId(result);
+            result = client.get(url, "adminLogin.do?password=123");
             result = client.get(url, "tools.jsp");
             FileUtils.delete(getBaseDir() + "/backup.zip");
             result = client.get(url, "tools.do?tool=Backup&args=-dir," +
@@ -187,12 +176,7 @@ public class TestWeb extends TestBase {
             result = client.get(url,
                     "tools.do?tool=DeleteDbFiles&args=-dir," +
                     getBaseDir() + ",-db," + getTestName());
-            String fn = getBaseDir() + "/" + getTestName();
-            if (config.mvStore) {
-                fn += Constants.SUFFIX_MV_FILE;
-            } else {
-                fn += Constants.SUFFIX_PAGE_FILE;
-            }
+            String fn = getBaseDir() + "/" + getTestName() + Constants.SUFFIX_MV_FILE;
             assertFalse(FileUtils.exists(fn));
             result = client.get(url, "tools.do?tool=Restore&args=-dir," +
                     getBaseDir() + ",-db," + getTestName() +",-file," + getBaseDir() +
@@ -261,7 +245,8 @@ public class TestWeb extends TestBase {
                 getUser(), getPassword());
         Server server = new Server();
         server.setOut(new PrintStream(new ByteArrayOutputStream()));
-        server.runTool("-ifExists", "-web", "-webPort", "8182",
+        // -ifExists is the default
+        server.runTool("-web", "-webPort", "8182",
                 "-properties", "null", "-tcp", "-tcpPort", "9101");
         try {
             String url = "http://localhost:8182";
@@ -285,12 +270,13 @@ public class TestWeb extends TestBase {
             server.shutdown();
             conn.close();
         }
+
     }
 
     private void testWebApp() throws Exception {
         Server server = new Server();
         server.setOut(new PrintStream(new ByteArrayOutputStream()));
-        server.runTool("-web", "-webPort", "8182",
+        server.runTool("-ifNotExists", "-web", "-webPort", "8182",
                 "-properties", "null", "-tcp", "-tcpPort", "9101");
         try {
             String url = "http://localhost:8182";
@@ -444,8 +430,23 @@ public class TestWeb extends TestBase {
             result = client.get(url, "query.do?sql=@cancel");
             assertContains(result, "There is currently no running statement");
             result = client.get(url,
-                    "query.do?sql=@generated insert into test(id) values(test_sequence.nextval)");
-            assertContains(result, "SCOPE_IDENTITY()");
+                    "query.do?sql=@generated insert into test(id) values(next value for test_sequence)");
+            assertContains(result, "<tr><th>ID</th></tr><tr><td>1</td></tr>");
+            result = client.get(url,
+                    "query.do?sql=@generated(1) insert into test(id) values(next value for test_sequence)");
+            assertContains(result, "<tr><th>ID</th></tr><tr><td>2</td></tr>");
+            result = client.get(url,
+                    "query.do?sql=@generated(1, 1) insert into test(id) values(next value for test_sequence)");
+            assertContains(result, "<tr><th>ID</th><th>ID</th></tr><tr><td>3</td><td>3</td></tr>");
+            result = client.get(url,
+                    "query.do?sql=@generated(id) insert into test(id) values(next value for test_sequence)");
+            assertContains(result, "<tr><th>ID</th></tr><tr><td>4</td></tr>");
+            result = client.get(url,
+                    "query.do?sql=@generated(id, id) insert into test(id) values(next value for test_sequence)");
+            assertContains(result, "<tr><th>ID</th><th>ID</th></tr><tr><td>5</td><td>5</td></tr>");
+            result = client.get(url,
+                    "query.do?sql=@generated() insert into test(id) values(next value for test_sequence)");
+            assertContains(result, "<table class=\"resultSet\" cellspacing=\"0\" cellpadding=\"0\"><tr></tr></table>");
             result = client.get(url, "query.do?sql=@maxrows 2000");
             assertContains(result, "Max rowcount is set");
             result = client.get(url, "query.do?sql=@password_hash user password");
@@ -455,20 +456,15 @@ public class TestWeb extends TestBase {
             assertContains(result, "Ok");
             result = client.get(url, "query.do?sql=@catalogs");
             assertContains(result, "PUBLIC");
-            result = client.get(url,
-                    "query.do?sql=@column_privileges null null null TEST null");
+            result = client.get(url, "query.do?sql=@column_privileges null null TEST null");
             assertContains(result, "PRIVILEGE");
-            result = client.get(url,
-                    "query.do?sql=@cross_references null null null TEST");
+            result = client.get(url, "query.do?sql=@cross_references null null TEST null null TEST");
             assertContains(result, "PKTABLE_NAME");
-            result = client.get(url,
-                    "query.do?sql=@exported_keys null null null TEST");
+            result = client.get(url, "query.do?sql=@exported_keys null null TEST");
             assertContains(result, "PKTABLE_NAME");
-            result = client.get(url,
-                    "query.do?sql=@imported_keys null null null TEST");
+            result = client.get(url, "query.do?sql=@imported_keys null null TEST");
             assertContains(result, "PKTABLE_NAME");
-            result = client.get(url,
-                    "query.do?sql=@primary_keys null null null TEST");
+            result = client.get(url, "query.do?sql=@primary_keys null null TEST");
             assertContains(result, "PK_NAME");
             result = client.get(url, "query.do?sql=@procedures null null null");
             assertContains(result, "PROCEDURE_NAME");
@@ -479,23 +475,22 @@ public class TestWeb extends TestBase {
             result = client.get(url, "query.do?sql=@table_privileges");
             assertContains(result, "PRIVILEGE");
             result = client.get(url, "query.do?sql=@table_types");
-            assertContains(result, "SYSTEM TABLE");
+            assertContains(result, "BASE TABLE");
             result = client.get(url, "query.do?sql=@type_info");
-            assertContains(result, "CLOB");
+            assertContains(result, "CHARACTER LARGE OBJECT");
             result = client.get(url, "query.do?sql=@version_columns");
             assertContains(result, "PSEUDO_COLUMN");
             result = client.get(url, "query.do?sql=@attributes");
-            assertContains(result, "Feature not supported: &quot;attributes&quot;");
+            assertContains(result, "ATTR_NAME");
             result = client.get(url, "query.do?sql=@super_tables");
             assertContains(result, "SUPERTABLE_NAME");
             result = client.get(url, "query.do?sql=@super_types");
-            assertContains(result, "Feature not supported: &quot;superTypes&quot;");
+            assertContains(result, "SUPERTYPE_NAME");
             result = client.get(url, "query.do?sql=@prof_start");
             assertContains(result, "Ok");
             result = client.get(url, "query.do?sql=@prof_stop");
             assertContains(result, "Top Stack Trace(s)");
-            result = client.get(url,
-                    "query.do?sql=@best_row_identifier null null TEST");
+            result = client.get(url, "query.do?sql=@best_row_identifier null null TEST");
             assertContains(result, "SCOPE");
             assertContains(result, "COLUMN_NAME");
             assertContains(result, "ID");
@@ -556,7 +551,7 @@ public class TestWeb extends TestBase {
             Task t = new Task() {
                 @Override
                 public void call() throws Exception {
-                    Server.startWebServer(conn);
+                    Server.startWebServer(conn, true);
                 }
             };
             t.execute();
@@ -572,7 +567,7 @@ public class TestWeb extends TestBase {
             url = client.getBaseUrl(url);
             try {
                 client.get(url, "logout.do");
-            } catch (Exception e) {
+            } catch (ConnectException e) {
                 // the server stops on logout
             }
             t.get();
@@ -719,7 +714,7 @@ public class TestWeb extends TestBase {
 
         @Override
         public String getScheme() {
-            return null;
+            return "http";
         }
 
         @Override
@@ -729,7 +724,7 @@ public class TestWeb extends TestBase {
 
         @Override
         public int getServerPort() {
-            return 0;
+            return 80;
         }
 
         @Override
@@ -1174,11 +1169,7 @@ public class TestWeb extends TestBase {
 
         @Override
         public String toString() {
-            try {
-                return new String(buff.toByteArray(), "UTF-8");
-            } catch (UnsupportedEncodingException e) {
-                return e.toString();
-            }
+            return Utils10.byteArrayOutputStreamToString(buff, StandardCharsets.UTF_8);
         }
 
         @Override
