@@ -1,17 +1,19 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.store;
 
 import java.util.AbstractSet;
 import java.util.Iterator;
+import java.util.Map;
 import java.util.Set;
 import org.h2.mvstore.MVMap;
+import org.h2.mvstore.type.DataType;
 
 /**
- * A custom map returning the keys and values values 1 .. 10.
+ * A custom map returning the keys and values 1 .. 10.
  */
 public class SequenceMap extends MVMap<Long, Long> {
 
@@ -25,8 +27,8 @@ public class SequenceMap extends MVMap<Long, Long> {
      */
     int max = 10;
 
-    public SequenceMap() {
-        super(null, null);
+    public SequenceMap(Map<String, Object> config, DataType<Long> keyType, DataType<Long> valueType) {
+        super(config, keyType, valueType);
     }
 
     @Override
@@ -49,11 +51,6 @@ public class SequenceMap extends MVMap<Long, Long> {
                         return Long.valueOf(x++);
                     }
 
-                    @Override
-                    public void remove() {
-                        throw new UnsupportedOperationException();
-                    }
-
                 };
             }
 
@@ -67,20 +64,11 @@ public class SequenceMap extends MVMap<Long, Long> {
     /**
      * A builder for this class.
      */
-    public static class Builder implements MapBuilder<SequenceMap, Long, Long> {
-
-        /**
-         * Create a new builder.
-         */
-        public Builder() {
-            // ignore
-        }
-
+    public static class Builder extends MVMap.Builder<Long, Long> {
         @Override
-        public SequenceMap create() {
-            return new SequenceMap();
+        public SequenceMap create(Map<String, Object> config) {
+            return new SequenceMap(config, getKeyType(), getValueType());
         }
 
     }
-
 }
