@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.store;
@@ -78,7 +78,7 @@ public class TestImmutableArray extends TestBase {
                 }
             }
         } else {
-            ArrayList<Integer> test = new ArrayList<Integer>();
+            ArrayList<Integer> test = new ArrayList<>();
             for (int i = 0; i < count; i++) {
                 if (i % 10 != 0) {
                     test.add(test.size(), x);
@@ -111,7 +111,7 @@ public class TestImmutableArray extends TestBase {
         for (int i = 0; i < 100; i++) {
             ImmutableArray2<Integer> test = ImmutableArray2.empty();
             // ConcurrentRing<Integer> test = new ConcurrentRing<Integer>();
-            ArrayList<Integer> x = new ArrayList<Integer>();
+            ArrayList<Integer> x = new ArrayList<>();
             StringBuilder buff = new StringBuilder();
             for (int j = 0; j < 1000; j++) {
                 buff.append("[" + j + "] ");
