@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.synth;
@@ -11,6 +11,7 @@ import java.io.StringWriter;
 import java.lang.reflect.Array;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.sql.BatchUpdateException;
 import java.sql.Blob;
 import java.sql.CallableStatement;
@@ -29,28 +30,27 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Comparator;
+import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.Map;
+
 import org.h2.api.ErrorCode;
-import org.h2.jdbc.JdbcConnection;
 import org.h2.store.FileLister;
 import org.h2.store.fs.FileUtils;
-import org.h2.test.TestAll;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.test.scripts.TestScript;
 import org.h2.test.synth.sql.RandomGen;
 import org.h2.tools.Backup;
 import org.h2.tools.DeleteDbFiles;
 import org.h2.tools.Restore;
-import org.h2.util.DateTimeUtils;
 import org.h2.util.MathUtils;
-import org.h2.util.New;
 
 /**
  * A test that calls random methods with random parameters from JDBC objects.
  * This is sometimes called 'Fuzz Testing'.
  */
-public class TestCrashAPI extends TestBase implements Runnable {
+public class TestCrashAPI extends TestDb implements Runnable {
 
     private static final boolean RECOVER_ALL = false;
 
@@ -61,11 +61,11 @@ public class TestCrashAPI extends TestBase implements Runnable {
 
     private static final String DIR = "synth";
 
-    private final ArrayList<Object> objects = New.arrayList();
+    private final ArrayList<Object> objects = new ArrayList<>();
     private final HashMap<Class <?>, ArrayList<Method>> classMethods =
             new HashMap<>();
     private RandomGen random = new RandomGen();
-    private final ArrayList<String> statements = New.arrayList();
+    private ArrayList<String> statements;
     private int openCount;
     private long callCount;
     private volatile long maxWait = 60;
@@ -81,11 +81,10 @@ public class TestCrashAPI extends TestBase implements Runnable {
     public static void main(String... a) throws Exception {
         System.setProperty("h2.delayWrongPasswordMin", "0");
         System.setProperty("h2.delayWrongPasswordMax", "0");
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
-    @SuppressWarnings("deprecation")
     public void run() {
         while (--maxWait > 0) {
             try {
@@ -102,11 +101,11 @@ public class TestCrashAPI extends TestBase implements Runnable {
         if (maxWait == 0 && running) {
             objects.clear();
             if (running) {
-                println("stopping (force)...");
+                println("stopping (trying to interrupt)...");
                 for (StackTraceElement e : mainThread.getStackTrace()) {
                     System.out.println(e.toString());
                 }
-                mainThread.stop(new SQLException("stop"));
+                mainThread.interrupt();
             }
         }
     }
@@ -114,12 +113,7 @@ public class TestCrashAPI extends TestBase implements Runnable {
     private static void recoverAll() {
         org.h2.Driver.load();
         File[] files = new File("temp/backup").listFiles();
-        Arrays.sort(files, new Comparator<File>() {
-            @Override
-            public int compare(File o1, File o2) {
-                return o1.getName().compareTo(o2.getName());
-            }
-        });
+        Arrays.sort(files, Comparator.comparing(File::getName));
         for (File f : files) {
             if (!f.getName().startsWith("db-")) {
                 continue;
@@ -147,15 +141,29 @@ public class TestCrashAPI extends TestBase implements Runnable {
         }
     }
 
+    @Override
+    public boolean isEnabled() {
+        if (config.networked) {
+            return false;
+        }
+        return true;
+    }
+
     @Override
     public void test() throws Exception {
         if (RECOVER_ALL) {
             recoverAll();
             return;
         }
-        if (config.mvcc || config.networked) {
+
+        if (config.networked) {
             return;
         }
+
+        TestScript script = new TestScript();
+        statements = script.getAllStatements(config);
+        initMethods();
+
         int len = getSize(2, 6);
         Thread t = new Thread(this);
         try {
@@ -327,7 +335,7 @@ public class TestCrashAPI extends TestBase implements Runnable {
                 continue;
             }
             if (random.getInt(2000) == 0 && conn != null) {
-                ((JdbcConnection) conn).setPowerOffCount(random.getInt(50));
+                setPowerOffCount(conn, random.getInt(50));
             }
             Object o = objects.get(objectId);
             if (o == null) {
@@ -370,6 +378,11 @@ public class TestCrashAPI extends TestBase implements Runnable {
     }
 
     private Object callRandom(int seed, int id, int objectId, Object o, Method m) {
+        // TODO m.isDefault() can be used on Java 8
+        boolean isDefault =
+                (m.getModifiers() & (Modifier.ABSTRACT | Modifier.PUBLIC | Modifier.STATIC)) == Modifier.PUBLIC
+                && m.getDeclaringClass().isInterface();
+        boolean allowNPE = isDefault || o instanceof Blob && "setBytes".equals(m.getName());
         Class<?>[] paramClasses = m.getParameterTypes();
         Object[] params = new Object[paramClasses.length];
         for (int i = 0; i < params.length; i++) {
@@ -379,13 +392,11 @@ public class TestCrashAPI extends TestBase implements Runnable {
         try {
             callCount++;
             result = m.invoke(o, params);
-        } catch (IllegalArgumentException e) {
-            TestBase.logError("error", e);
-        } catch (IllegalAccessException e) {
+        } catch (IllegalArgumentException | IllegalAccessException e) {
             TestBase.logError("error", e);
         } catch (InvocationTargetException e) {
             Throwable t = e.getTargetException();
-            printIfBad(seed, id, objectId, t);
+            printIfBad(seed, id, objectId, t, allowNPE);
         }
         if (result == null) {
             return null;
@@ -398,6 +409,10 @@ public class TestCrashAPI extends TestBase implements Runnable {
     }
 
     private void printIfBad(int seed, int id, int objectId, Throwable t) {
+        printIfBad(seed, id, objectId, t, false);
+    }
+
+    private void printIfBad(int seed, int id, int objectId, Throwable t, boolean allowNPE) {
         if (t instanceof BatchUpdateException) {
             // do nothing
         } else if (t.getClass().getName().contains("SQLClientInfoException")) {
@@ -421,6 +436,8 @@ public class TestCrashAPI extends TestBase implements Runnable {
                 // General error [HY000]
                 printError(seed, id, s);
             }
+        } else if (allowNPE && t instanceof NullPointerException) {
+            // do nothing, this methods may throw this exception
         } else {
             printError(seed, id, t);
         }
@@ -485,7 +502,7 @@ public class TestCrashAPI extends TestBase implements Runnable {
             // TODO should use generated savepoints
             return null;
         } else if (type == Calendar.class) {
-            return DateTimeUtils.createGregorianCalendar();
+            return new GregorianCalendar();
         } else if (type == java.net.URL.class) {
             return null;
         } else if (type == java.math.BigDecimal.class) {
@@ -507,29 +524,12 @@ public class TestCrashAPI extends TestBase implements Runnable {
 
     private void initMethods() {
         for (Class<?> inter : INTERFACES) {
-            classMethods.put(inter, new ArrayList<Method>());
-        }
-        for (Class<?> inter : INTERFACES) {
-            ArrayList<Method> list = classMethods.get(inter);
+            ArrayList<Method> list = new ArrayList<>();
             for (Method m : inter.getMethods()) {
                 list.add(m);
             }
+            classMethods.put(inter, list);
         }
     }
 
-    @Override
-    public TestBase init(TestAll conf) throws Exception {
-        super.init(conf);
-        if (config.mvcc || config.networked) {
-            return this;
-        }
-        startServerIfRequired();
-        TestScript script = new TestScript();
-        ArrayList<String> add = script.getAllStatements(config);
-        initMethods();
-        org.h2.Driver.load();
-        statements.addAll(add);
-        return this;
-    }
-
 }
