@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.synth;
@@ -14,6 +14,7 @@ import java.util.Random;
 import java.util.concurrent.TimeUnit;
 
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.Backup;
 import org.h2.tools.DeleteDbFiles;
 
@@ -22,7 +23,7 @@ import org.h2.tools.DeleteDbFiles;
  * then deletes everything and runs in an endless loop executing random
  * operations. This loop is usually stopped by switching off the computer.
  */
-public class TestTimer extends TestBase {
+public class TestTimer extends TestDb {
 
     /**
      * Run just this test.
@@ -30,7 +31,7 @@ public class TestTimer extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
