@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.synth.sql;
@@ -12,7 +12,6 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
-import org.h2.util.New;
 
 /**
  * Represents a connection to a real database.
@@ -45,7 +44,7 @@ class DbConnection implements DbInterface {
         log("reset;");
         DatabaseMetaData meta = conn.getMetaData();
         Statement stat = conn.createStatement();
-        ArrayList<String> tables = New.arrayList();
+        ArrayList<String> tables = new ArrayList<>();
         ResultSet rs = meta.getTables(null, null, null, new String[] { "TABLE" });
         while (rs.next()) {
             String schemaName = rs.getString("TABLE_SCHEM");
