@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  */
 /*
  * Licensed to the Apache Software Foundation (ASF) under one or more
@@ -80,9 +80,7 @@ class Statement {
                 player.assign(assignVariable, obj);
             }
             return obj;
-        } catch (IllegalArgumentException e) {
-            e.printStackTrace();
-        } catch (IllegalAccessException e) {
+        } catch (IllegalArgumentException | IllegalAccessException e) {
             e.printStackTrace();
         } catch (InvocationTargetException e) {
             Throwable t = e.getTargetException();
