@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
@@ -13,12 +13,13 @@ import java.sql.Statement;
 import org.h2.api.DatabaseEventListener;
 import org.h2.api.ErrorCode;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.Server;
 
 /**
  * Tests automatic embedded/server mode.
  */
-public class TestAutoReconnect extends TestBase {
+public class TestAutoReconnect extends TestDb {
 
     private String url;
     private boolean autoServer;
@@ -32,10 +33,10 @@ public class TestAutoReconnect extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
-    private void restart() throws SQLException {
+    private void restart() throws SQLException, InterruptedException {
         if (autoServer) {
             if (connServer != null) {
                 connServer.createStatement().execute("SHUTDOWN");
@@ -45,6 +46,7 @@ public class TestAutoReconnect extends TestBase {
             connServer = getConnection(url);
         } else {
             server.stop();
+            Thread.sleep(100); // try to prevent "port may be in use" error
             server.start();
         }
     }
@@ -63,21 +65,17 @@ public class TestAutoReconnect extends TestBase {
         deleteDb(getTestName());
         Server tcp = Server.createTcpServer().start();
         try {
-            conn = getConnection("jdbc:h2:" + getBaseDir() +
-                    "/" + getTestName() + ";AUTO_SERVER=TRUE");
-            assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1, this).
-                    getConnection("jdbc:h2:" + getBaseDir() +
-                            "/" + getTestName() + ";OPEN_NEW=TRUE");
-            assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1, this).
-                    getConnection("jdbc:h2:" + getBaseDir() +
-                            "/" + getTestName() + ";OPEN_NEW=TRUE");
+            conn = getConnection("jdbc:h2:" + getBaseDir() + '/' + getTestName() + ";AUTO_SERVER=TRUE");
+            assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1,
+                    () -> getConnection("jdbc:h2:" + getBaseDir() + '/' + getTestName() + ";OPEN_NEW=TRUE"));
+            assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1,
+                    () -> getConnection("jdbc:h2:" + getBaseDir() + '/' + getTestName() + ";OPEN_NEW=TRUE"));
             conn.close();
 
-            conn = getConnection("jdbc:h2:tcp://localhost/" + getBaseDir() +
-                    "/" + getTestName());
-            assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1, this).
-                    getConnection("jdbc:h2:" + getBaseDir() +
-                            "/" + getTestName() + ";AUTO_SERVER=TRUE;OPEN_NEW=TRUE");
+            conn = getConnection("jdbc:h2:tcp://localhost:" + tcp.getPort() + '/' + getBaseDir() + '/' //
+                    + getTestName());
+            assertThrows(ErrorCode.DATABASE_ALREADY_OPEN_1, () -> getConnection(
+                    "jdbc:h2:" + getBaseDir() + '/' + getTestName() + ";AUTO_SERVER=TRUE;OPEN_NEW=TRUE"));
             conn.close();
         } finally {
             tcp.stop();
@@ -92,8 +90,9 @@ public class TestAutoReconnect extends TestBase {
                 "AUTO_SERVER=TRUE;OPEN_NEW=TRUE";
             restart();
         } else {
-            server = Server.createTcpServer("-tcpPort", "8181").start();
-            url = "jdbc:h2:tcp://localhost:8181/" + getBaseDir() + "/" + getTestName() + ";" +
+            server = Server.createTcpServer("-ifNotExists").start();
+            int port = server.getPort();
+            url = "jdbc:h2:tcp://localhost:" + port + "/" + getBaseDir() + "/" + getTestName() + ";" +
                 "FILE_LOCK=SOCKET;AUTO_RECONNECT=TRUE";
         }
 
@@ -111,7 +110,7 @@ public class TestAutoReconnect extends TestBase {
         stat.execute("create table test(id identity, name varchar)");
         restart();
         PreparedStatement prep = conn.prepareStatement(
-                "insert into test values(null, ?)");
+                "insert into test(name) values(?)");
         restart();
         prep.setString(1, "Hello");
         restart();
@@ -163,6 +162,7 @@ public class TestAutoReconnect extends TestBase {
                 if (i < 10) {
                     throw e;
                 }
+                break;
             }
         }
         restart();
@@ -184,32 +184,6 @@ public class TestAutoReconnect extends TestBase {
     /**
      * A database event listener used in this test.
      */
-    public static final class MyDatabaseEventListener implements
-            DatabaseEventListener {
-
-        @Override
-        public void closingDatabase() {
-            // ignore
-        }
-
-        @Override
-        public void exceptionThrown(SQLException e, String sql) {
-            // ignore
-        }
-
-        @Override
-        public void init(String u) {
-            // ignore
-        }
-
-        @Override
-        public void opened() {
-            // ignore
-        }
-
-        @Override
-        public void setProgress(int state, String name, int x, int max) {
-            // ignore
-        }
+    public static final class MyDatabaseEventListener implements DatabaseEventListener {
     }
 }
