@@ -1,17 +1,16 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
 
 import java.io.File;
-import java.util.Properties;
 
 import org.h2.api.ErrorCode;
 import org.h2.engine.ConnectionInfo;
-import org.h2.engine.SysProperties;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.DeleteDbFiles;
 
 /**
@@ -20,7 +19,7 @@ import org.h2.tools.DeleteDbFiles;
  * @author Kerry Sainsbury
  * @author Thomas Mueller Graf
  */
-public class TestConnectionInfo extends TestBase {
+public class TestConnectionInfo extends TestDb {
 
     /**
      * Run just this test.
@@ -28,7 +27,7 @@ public class TestConnectionInfo extends TestBase {
      * @param a ignored
      */
     public static void main(String[] a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -40,40 +39,29 @@ public class TestConnectionInfo extends TestBase {
     }
 
     private void testImplicitRelativePath() throws Exception {
-        if (SysProperties.IMPLICIT_RELATIVE_PATH) {
-            return;
-        }
-        assertThrows(ErrorCode.URL_RELATIVE_TO_CWD, this).
-            getConnection("jdbc:h2:" + getTestName());
-        assertThrows(ErrorCode.URL_RELATIVE_TO_CWD, this).
-            getConnection("jdbc:h2:data/" + getTestName());
+        assertThrows(ErrorCode.URL_RELATIVE_TO_CWD, () -> getConnection("jdbc:h2:" + getTestName()));
+        assertThrows(ErrorCode.URL_RELATIVE_TO_CWD, () -> getConnection("jdbc:h2:data/" + getTestName()));
 
         getConnection("jdbc:h2:./data/" + getTestName()).close();
         DeleteDbFiles.execute("data", getTestName(), true);
     }
 
     private void testConnectInitError() throws Exception {
-        assertThrows(ErrorCode.SYNTAX_ERROR_2, this).
-                getConnection("jdbc:h2:mem:;init=error");
-        assertThrows(ErrorCode.IO_EXCEPTION_2, this).
-                getConnection("jdbc:h2:mem:;init=runscript from 'wrong.file'");
+        assertThrows(ErrorCode.SYNTAX_ERROR_2, () -> getConnection("jdbc:h2:mem:;init=error"));
+        assertThrows(ErrorCode.IO_EXCEPTION_2, () -> getConnection("jdbc:h2:mem:;init=runscript from 'wrong.file'"));
     }
 
     private void testConnectionInfo() {
-        Properties info = new Properties();
         ConnectionInfo connectionInfo = new ConnectionInfo(
                 "jdbc:h2:mem:" + getTestName() +
-                        ";LOG=2" +
                         ";ACCESS_MODE_DATA=rws" +
                         ";INIT=CREATE this...\\;INSERT that..." +
                         ";IFEXISTS=TRUE",
-                info);
+                null, null, null);
 
         assertEquals("jdbc:h2:mem:" + getTestName(),
                 connectionInfo.getURL());
 
-        assertEquals("2",
-                connectionInfo.getProperty("LOG", ""));
         assertEquals("rws",
                 connectionInfo.getProperty("ACCESS_MODE_DATA", ""));
         assertEquals("CREATE this...;INSERT that...",
