@@ -1,15 +1,14 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
 
 import java.util.Random;
-import org.h2.api.JavaObjectSerializer;
 import org.h2.store.DataHandler;
 import org.h2.store.FileStore;
-import org.h2.store.LobStorageBackend;
+import org.h2.store.LobStorageInterface;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
 import org.h2.util.SmallLRUCache;
@@ -27,7 +26,7 @@ public class TestFile extends TestBase implements DataHandler {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -151,11 +150,6 @@ public class TestFile extends TestBase implements DataHandler {
         return null;
     }
 
-    @Override
-    public String getLobCompressionAlgorithm(int type) {
-        return null;
-    }
-
     @Override
     public Object getLobSyncObject() {
         return null;
@@ -182,7 +176,7 @@ public class TestFile extends TestBase implements DataHandler {
     }
 
     @Override
-    public LobStorageBackend getLobStorage() {
+    public LobStorageInterface getLobStorage() {
         return null;
     }
 
@@ -192,11 +186,6 @@ public class TestFile extends TestBase implements DataHandler {
         return -1;
     }
 
-    @Override
-    public JavaObjectSerializer getJavaObjectSerializer() {
-        return null;
-    }
-
     @Override
     public CompareMode getCompareMode() {
         return CompareMode.getInstance(null, 0);
