@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
@@ -26,14 +26,19 @@ public class TestFtp extends TestBase implements FtpEventListener {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (getBaseDir().indexOf(':') > 0) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws Exception {
-        if (getBaseDir().indexOf(':') > 0) {
-            return;
-        }
         FileUtils.delete(getBaseDir() + "/ftp");
         test(getBaseDir());
         FileUtils.delete(getBaseDir() + "/ftp");
