@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
@@ -11,11 +11,12 @@ import java.sql.PreparedStatement;
 import java.sql.SQLException;
 
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 
 /**
  * Tests the multi-threaded kernel feature.
  */
-public class TestMultiThreadedKernel extends TestBase implements Runnable {
+public class TestMultiThreadedKernel extends TestDb implements Runnable {
 
     private String url, user, password;
     private int id;
@@ -28,20 +29,25 @@ public class TestMultiThreadedKernel extends TestBase implements Runnable {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.networked) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws Exception {
-        if (config.networked || config.mvcc) {
-            return;
-        }
         deleteDb("multiThreadedKernel");
         int count = getSize(2, 5);
         Thread[] list = new Thread[count];
         for (int i = 0; i < count; i++) {
             TestMultiThreadedKernel r = new TestMultiThreadedKernel();
-            r.url = getURL("multiThreadedKernel;MULTI_THREADED=1", true);
+            r.url = getURL("multiThreadedKernel", true);
             r.user = getUser();
             r.password = getPassword();
             r.master = this;
@@ -64,7 +70,7 @@ public class TestMultiThreadedKernel extends TestBase implements Runnable {
         try {
             org.h2.Driver.load();
             Connection conn = DriverManager.getConnection(url +
-                    ";MULTI_THREADED=1;LOCK_MODE=3;WRITE_DELAY=0",
+                    ";LOCK_MODE=3;WRITE_DELAY=0",
                     user, password);
             conn.createStatement().execute(
                     "CREATE TABLE TEST" + id +
