@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
@@ -25,7 +25,7 @@ public class TestReader extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -35,7 +35,7 @@ public class TestReader extends TestBase {
         InputStream in = new ReaderInputStream(r);
         byte[] buff = IOUtils.readBytesAndClose(in, 0);
         InputStream in2 = new ByteArrayInputStream(buff);
-        Reader r2 = IOUtils.getBufferedReader(in2);
+        Reader r2 = IOUtils.getReader(in2);
         String s2 = IOUtils.readStringAndClose(r2, Integer.MAX_VALUE);
         assertEquals(s, s2);
     }
