@@ -1,28 +1,26 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
 
 import java.sql.SQLException;
 import java.util.HashSet;
-import java.util.Properties;
 import java.util.concurrent.TimeUnit;
 
 import org.h2.api.ErrorCode;
 import org.h2.engine.ConnectionInfo;
 import org.h2.engine.Constants;
 import org.h2.engine.Database;
-import org.h2.engine.Session;
+import org.h2.engine.SessionLocal;
 import org.h2.message.DbException;
-import org.h2.store.fs.FilePathRec;
 import org.h2.store.fs.FileUtils;
 import org.h2.store.fs.Recorder;
+import org.h2.store.fs.rec.FilePathRec;
 import org.h2.test.TestBase;
 import org.h2.tools.Recover;
 import org.h2.util.IOUtils;
-import org.h2.util.New;
 import org.h2.util.Profiler;
 import org.h2.util.Utils;
 
@@ -40,7 +38,7 @@ public class TestReopen extends TestBase implements Recorder {
     private final long maxFileSize = Utils.getProperty("h2.reopenMaxFileSize",
             Integer.MAX_VALUE) * 1024L * 1024;
     private int verifyCount;
-    private final HashSet<String> knownErrors = New.hashSet();
+    private final HashSet<String> knownErrors = new HashSet<>();
     private volatile boolean testing;
 
     /**
@@ -49,7 +47,7 @@ public class TestReopen extends TestBase implements Recorder {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -73,8 +71,7 @@ public class TestReopen extends TestBase implements Recorder {
         if (op != Recorder.WRITE && op != Recorder.TRUNCATE) {
             return;
         }
-        if (!fileName.endsWith(Constants.SUFFIX_PAGE_FILE) &&
-                !fileName.endsWith(Constants.SUFFIX_MV_FILE)) {
+        if (!fileName.endsWith(Constants.SUFFIX_MV_FILE)) {
             return;
         }
         if (testing) {
@@ -101,25 +98,16 @@ public class TestReopen extends TestBase implements Recorder {
         System.out.println("+ write #" + writeCount + " verify #" + verifyCount);
 
         try {
-            if (fileName.endsWith(Constants.SUFFIX_PAGE_FILE)) {
-                IOUtils.copyFiles(fileName, testDatabase +
-                        Constants.SUFFIX_PAGE_FILE);
-            } else {
-                IOUtils.copyFiles(fileName, testDatabase +
-                        Constants.SUFFIX_MV_FILE);
-            }
+            IOUtils.copyFiles(fileName, testDatabase +
+                    Constants.SUFFIX_MV_FILE);
             verifyCount++;
             // avoid using the Engine class to avoid deadlocks
-            Properties p = new Properties();
-            String userName =  getUser();
-            p.setProperty("user", userName);
-            p.setProperty("password", getPassword());
             String url = "jdbc:h2:" + testDatabase +
                     ";FILE_LOCK=NO;TRACE_LEVEL_FILE=0";
-            ConnectionInfo ci = new ConnectionInfo(url, p);
+            ConnectionInfo ci = new ConnectionInfo(url, null, getUser(), getPassword());
             Database database = new Database(ci, null);
             // close the database
-            Session session = database.getSystemSession();
+            SessionLocal session = database.getSystemSession();
             session.prepare("script to '" + testDatabase + ".sql'").query(0);
             session.prepare("shutdown immediately").update();
             database.removeSession(null);
@@ -157,17 +145,11 @@ public class TestReopen extends TestBase implements Recorder {
         }
         testDatabase += "X";
         try {
-            if (fileName.endsWith(Constants.SUFFIX_PAGE_FILE)) {
-                IOUtils.copyFiles(fileName, testDatabase +
-                        Constants.SUFFIX_PAGE_FILE);
-            } else {
-                IOUtils.copyFiles(fileName, testDatabase +
-                        Constants.SUFFIX_MV_FILE);
-            }
+            IOUtils.copyFiles(fileName, testDatabase +
+                    Constants.SUFFIX_MV_FILE);
             // avoid using the Engine class to avoid deadlocks
-            Properties p = new Properties();
             String url = "jdbc:h2:" + testDatabase + ";FILE_LOCK=NO";
-            ConnectionInfo ci = new ConnectionInfo(url, p);
+            ConnectionInfo ci = new ConnectionInfo(url, null, null, null);
             Database database = new Database(ci, null);
             // close the database
             database.removeSession(null);
