@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
@@ -32,13 +32,13 @@ import javax.servlet.descriptor.JspConfigDescriptor;
 import org.h2.api.ErrorCode;
 import org.h2.server.web.DbStarter;
 import org.h2.test.TestBase;
-import org.h2.util.New;
+import org.h2.test.TestDb;
 
 /**
  * Tests the DbStarter servlet.
  * This test simulates a minimum servlet container environment.
  */
-public class TestServlet extends TestBase {
+public class TestServlet extends TestDb {
 
     /**
      * Run just this test.
@@ -46,7 +46,7 @@ public class TestServlet extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     /**
@@ -56,7 +56,7 @@ public class TestServlet extends TestBase {
     static class TestServletContext implements ServletContext {
 
         private final Properties initParams = new Properties();
-        private final HashMap<String, Object> attributes = New.hashMap();
+        private final HashMap<String, Object> attributes = new HashMap<>();
 
         @Override
         public void setAttribute(String key, Object value) {
@@ -342,13 +342,53 @@ public class TestServlet extends TestBase {
             throw new UnsupportedOperationException();
         }
 
+        @Override
+        public ServletRegistration.Dynamic addJspFile(String servletName, String jspFile) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public int getSessionTimeout() {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setSessionTimeout(int sessionTimeout) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public String getRequestCharacterEncoding() {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setRequestCharacterEncoding(String encoding) {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public String getResponseCharacterEncoding() {
+            throw new UnsupportedOperationException();
+        }
+
+        @Override
+        public void setResponseCharacterEncoding(String encoding) {
+            throw new UnsupportedOperationException();
+        }
+
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.networked || config.memory) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws SQLException {
-        if (config.networked || config.memory) {
-            return;
-        }
         DbStarter listener = new DbStarter();
 
         TestServletContext context = new TestServletContext();
@@ -375,16 +415,16 @@ public class TestServlet extends TestBase {
         stat2.execute("SELECT * FROM T");
         stat2.execute("DROP TABLE T");
 
-        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_1, stat1).
+        assertThrows(ErrorCode.TABLE_OR_VIEW_NOT_FOUND_DATABASE_EMPTY_1, stat1).
                 execute("SELECT * FROM T");
         conn2.close();
 
         listener.contextDestroyed(event);
 
         // listener must be stopped
-        assertThrows(ErrorCode.CONNECTION_BROKEN_1, this).getConnection(
-                "jdbc:h2:tcp://localhost:8888/" + getBaseDir() + "/servlet",
-                getUser(), getPassword());
+        assertThrows(ErrorCode.CONNECTION_BROKEN_1,
+                () -> getConnection("jdbc:h2:tcp://localhost:8888/" + getBaseDir() + "/servlet", getUser(),
+                        getPassword()));
 
         // connection must be closed
         assertThrows(ErrorCode.OBJECT_CLOSED, stat1).
