@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
@@ -38,19 +38,19 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Random;
 import java.util.UUID;
-
 import org.h2.api.ErrorCode;
 import org.h2.engine.SysProperties;
 import org.h2.store.FileLister;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.test.trace.Player;
-import org.h2.test.utils.AssertThrows;
 import org.h2.tools.Backup;
 import org.h2.tools.ChangeFileEncryption;
 import org.h2.tools.Console;
 import org.h2.tools.ConvertTraceFile;
 import org.h2.tools.DeleteDbFiles;
+import org.h2.tools.GUIConsole;
 import org.h2.tools.Recover;
 import org.h2.tools.Restore;
 import org.h2.tools.RunScript;
@@ -60,12 +60,13 @@ import org.h2.tools.SimpleResultSet;
 import org.h2.tools.SimpleResultSet.SimpleArray;
 import org.h2.util.JdbcUtils;
 import org.h2.util.Task;
+import org.h2.util.Utils10;
 import org.h2.value.ValueUuid;
 
 /**
  * Tests the database tools.
  */
-public class TestTools extends TestBase {
+public class TestTools extends TestDb {
 
     private static String lastUrl;
     private Server server;
@@ -77,14 +78,19 @@ public class TestTools extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.networked) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws Exception {
-        if (config.networked) {
-            return;
-        }
         DeleteDbFiles.execute(getBaseDir(), null, true);
         org.h2.Driver.load();
         testSimpleResultSet();
@@ -95,7 +101,6 @@ public class TestTools extends TestBase {
         testDeleteFiles();
         testScriptRunscriptLob();
         testServerMain();
-        testRemove();
         testConvertTraceFile();
         testManagementDb();
         testChangeFileEncryption(false);
@@ -120,15 +125,13 @@ public class TestTools extends TestBase {
         s2.stop();
         s1 = Server.createTcpServer("-tcpPort", "9123").start();
         assertEquals(9123, s1.getPort());
-        createClassProxy(Server.class);
-        assertThrows(ErrorCode.EXCEPTION_OPENING_PORT_2,
-                Server.createTcpServer("-tcpPort", "9123")).start();
+        assertThrows(ErrorCode.EXCEPTION_OPENING_PORT_2, () -> Server.createTcpServer("-tcpPort", "9123").start());
         s1.stop();
     }
 
     private void testConsole() throws Exception {
         String old = System.getProperty(SysProperties.H2_BROWSER);
-        Console c = new Console();
+        GUIConsole c = new GUIConsole();
         c.setOut(new PrintStream(new ByteArrayOutputStream()));
         try {
 
@@ -173,9 +176,8 @@ public class TestTools extends TestBase {
 
             // trying to use the same port for two services should fail,
             // but also stop the first service
-            createClassProxy(c.getClass());
-            assertThrows(ErrorCode.EXCEPTION_OPENING_PORT_2, c).runTool("-web",
-                    "-webPort", "9002", "-tcp", "-tcpPort", "9002");
+            assertThrows(ErrorCode.EXCEPTION_OPENING_PORT_2,
+                    () -> c.runTool("-web", "-webPort", "9002", "-tcp", "-tcpPort", "9002"));
             c.runTool("-web", "-webPort", "9002");
 
         } finally {
@@ -210,14 +212,12 @@ public class TestTools extends TestBase {
     }
 
     private void testSimpleResultSet() throws Exception {
-
         SimpleResultSet rs;
         rs = new SimpleResultSet();
         rs.addColumn(null, 0, 0, 0);
         rs.addRow(1);
-        createClassProxy(rs.getClass());
-        assertThrows(IllegalStateException.class, rs).
-                addColumn(null, 0, 0, 0);
+        SimpleResultSet r = rs;
+        assertThrows(IllegalStateException.class, () -> r.addColumn(null, 0, 0, 0));
         assertEquals(ResultSet.TYPE_FORWARD_ONLY, rs.getType());
 
         rs.next();
@@ -238,11 +238,11 @@ public class TestTools extends TestBase {
         assertTrue(rs.getMetaData().isSearchable(1));
         assertTrue(rs.getMetaData().isSigned(1));
         assertFalse(rs.getMetaData().isWritable(1));
-        assertEquals(null, rs.getMetaData().getCatalogName(1));
-        assertEquals(null, rs.getMetaData().getColumnClassName(1));
+        assertEquals("", rs.getMetaData().getCatalogName(1));
+        assertEquals(Void.class.getName(), rs.getMetaData().getColumnClassName(1));
         assertEquals("NULL", rs.getMetaData().getColumnTypeName(1));
-        assertEquals(null, rs.getMetaData().getSchemaName(1));
-        assertEquals(null, rs.getMetaData().getTableName(1));
+        assertEquals("", rs.getMetaData().getSchemaName(1));
+        assertEquals("", rs.getMetaData().getTableName(1));
         assertEquals(ResultSet.HOLD_CURSORS_OVER_COMMIT, rs.getHoldability());
         assertEquals(1, rs.getColumnCount());
 
@@ -273,6 +273,7 @@ public class TestTools extends TestBase {
         rs.addRow(BigInteger.ONE, null, true, null, BigDecimal.ONE, 1d, null, null, null, null, null);
         rs.addRow(BigInteger.ZERO, null, false, null, BigDecimal.ZERO, 0d, null, null, null, null, null);
         rs.addRow(null, null, null, null, null, null, null, null, null, null, null);
+        rs.addRow(null, null, true, null, null, null, null, null, null, null, null);
 
         rs.next();
 
@@ -390,6 +391,12 @@ public class TestTools extends TestBase {
         assertNull(rs.getBinaryStream(12));
         assertTrue(rs.wasNull());
 
+        assertTrue(rs.next());
+        assertTrue(rs.getBoolean(3));
+        assertFalse(rs.wasNull());
+        assertNull(rs.getObject(6, Float.class));
+        assertTrue(rs.wasNull());
+
         // all updateX methods
         for (Method m: rs.getClass().getMethods()) {
             if (m.getName().startsWith("update")) {
@@ -476,7 +483,7 @@ public class TestTools extends TestBase {
         assertEquals(ResultSet.FETCH_FORWARD, rs.getFetchDirection());
         assertEquals(0, rs.getFetchSize());
         assertEquals(ResultSet.TYPE_SCROLL_INSENSITIVE, rs.getType());
-        assertTrue(rs.getStatement() == null);
+        assertNull(rs.getStatement());
         assertFalse(rs.isClosed());
 
         rs.beforeFirst();
@@ -487,6 +494,7 @@ public class TestTools extends TestBase {
         assertTrue(rs.next());
         assertTrue(rs.next());
         assertTrue(rs.next());
+        assertTrue(rs.next());
         assertFalse(rs.next());
         assertThrows(ErrorCode.NO_DATA_AVAILABLE, (ResultSet) rs).
                 getInt(1);
@@ -500,22 +508,47 @@ public class TestTools extends TestBase {
         rs.addRow(uuid);
         rs.next();
         assertEquals(uuid, rs.getObject(1));
-        assertEquals(uuid, ValueUuid.get(rs.getBytes(1)).getObject());
+        assertEquals(uuid, ValueUuid.get(rs.getBytes(1)).getUuid());
+
+        assertTrue(rs.isWrapperFor(Object.class));
+        assertTrue(rs.isWrapperFor(ResultSet.class));
+        assertTrue(rs.isWrapperFor(rs.getClass()));
+        assertFalse(rs.isWrapperFor(Integer.class));
+        assertTrue(rs == rs.unwrap(Object.class));
+        assertTrue(rs == rs.unwrap(ResultSet.class));
+        assertTrue(rs == rs.unwrap(rs.getClass()));
+        SimpleResultSet rs2 = rs;
+        assertThrows(ErrorCode.INVALID_VALUE_2, () -> rs2.unwrap(Integer.class));
     }
 
     private void testJdbcDriverUtils() {
-        assertEquals("org.h2.Driver",
-                JdbcUtils.getDriver("jdbc:h2:~/test"));
-        assertEquals("org.postgresql.Driver",
-                JdbcUtils.getDriver("jdbc:postgresql:test"));
-        assertEquals(null,
-                JdbcUtils.getDriver("jdbc:unknown:test"));
+        assertEquals("org.h2.Driver", JdbcUtils.getDriver("jdbc:h2:~/test"));
+        assertEquals("org.postgresql.Driver", JdbcUtils.getDriver("jdbc:postgresql:test"));
+        assertEquals(null, JdbcUtils.getDriver("jdbc:unknown:test"));
+        try {
+            JdbcUtils.getConnection("org.h2.Driver", "jdbc:h2x:test", "sa", "");
+            fail("Expected SQLException: 08001");
+        } catch (SQLException e) {
+            assertEquals("08001", e.getSQLState());
+        }
+        try {
+            JdbcUtils.getConnection("javax.naming.InitialContext", "ldap://localhost/ds", "sa", "");
+            fail("Expected SQLException: 08001");
+        } catch (SQLException e) {
+            assertEquals("08001", e.getSQLState());
+            assertEquals("Only java scheme is supported for JNDI lookups", e.getMessage());
+        }
+        try {
+            JdbcUtils.getConnection("org.h2.Driver", "jdbc:h2:mem:", "sa", "", null, true);
+            fail("Expected SQLException: " + ErrorCode.REMOTE_DATABASE_NOT_FOUND_1);
+        } catch (SQLException e) {
+            assertEquals(ErrorCode.REMOTE_DATABASE_NOT_FOUND_1, e.getErrorCode());
+        }
     }
 
     private void testWrongServer() throws Exception {
         // try to connect when the server is not running
-        assertThrows(ErrorCode.CONNECTION_BROKEN_1, this).
-                getConnection("jdbc:h2:tcp://localhost:9001/test");
+        assertThrows(ErrorCode.CONNECTION_BROKEN_1, () -> getConnection("jdbc:h2:tcp://localhost:9001/test"));
         final ServerSocket serverSocket = new ServerSocket(9001);
         Task task = new Task() {
             @Override
@@ -534,12 +567,7 @@ public class TestTools extends TestBase {
         try {
             task.execute();
             Thread.sleep(100);
-            try {
-                getConnection("jdbc:h2:tcp://localhost:9001/test");
-                fail();
-            } catch (SQLException e) {
-                assertEquals(ErrorCode.CONNECTION_BROKEN_1, e.getErrorCode());
-            }
+            assertThrows(ErrorCode.CONNECTION_BROKEN_1, () -> getConnection("jdbc:h2:tcp://localhost:9001/test"));
         } finally {
             serverSocket.close();
             task.getException();
@@ -567,14 +595,14 @@ public class TestTools extends TestBase {
         deleteDb("testDeleteFiles");
     }
 
-    private void testServerMain() throws SQLException {
+    private void testServerMain() throws Exception {
         testNonSSL();
-        if (!config.travis) {
+        if (!config.ci) {
             testSSL();
         }
     }
 
-    private void testNonSSL() throws SQLException {
+    private void testNonSSL() throws Exception {
         String result;
         Connection conn;
 
@@ -586,7 +614,7 @@ public class TestTools extends TestBase {
             result = runServer(1, new String[]{"-xy"});
             assertContains(result, "Starts the H2 Console");
             assertContains(result, "Feature not supported");
-            result = runServer(0, new String[]{"-tcp",
+            result = runServer(0, new String[]{"-ifNotExists", "-tcp",
                     "-tcpPort", "9001", "-tcpPassword", "abc"});
             assertContains(result, "tcp://");
             assertContains(result, ":9001");
@@ -602,12 +630,12 @@ public class TestTools extends TestBase {
         }
     }
 
-    private void testSSL() throws SQLException {
+    private void testSSL() throws Exception {
         String result;
         Connection conn;
 
         try {
-            result = runServer(0, new String[]{"-tcp",
+            result = runServer(0, new String[]{"-ifNotExists", "-tcp",
                     "-tcpAllowOthers", "-tcpPort", "9001", "-tcpPassword", "abcdef", "-tcpSSL"});
             assertContains(result, "ssl://");
             assertContains(result, ":9001");
@@ -619,11 +647,11 @@ public class TestTools extends TestBase {
             result = runServer(0, new String[]{"-tcpShutdown",
                     "ssl://localhost:9001", "-tcpPassword", "abcdef"});
             assertContains(result, "Shutting down");
-            assertThrows(ErrorCode.CONNECTION_BROKEN_1, this).
-            getConnection("jdbc:h2:ssl://localhost:9001/mem:", "sa", "sa");
+            assertThrows(ErrorCode.CONNECTION_BROKEN_1,
+                    () -> getConnection("jdbc:h2:ssl://localhost:9001/mem:", "sa", "sa"));
 
             result = runServer(0, new String[]{
-                    "-web", "-webPort", "9002", "-webAllowOthers", "-webSSL",
+                    "-ifNotExists", "-web", "-webPort", "9002", "-webAllowOthers", "-webSSL",
                     "-pg", "-pgAllowOthers", "-pgPort", "9003",
                     "-tcp", "-tcpAllowOthers", "-tcpPort", "9006", "-tcpPassword", "abc"});
             Server stop = server;
@@ -643,16 +671,16 @@ public class TestTools extends TestBase {
                     "tcp://localhost:9006", "-tcpPassword", "abc", "-tcpShutdownForce"});
             assertContains(result, "Shutting down");
             stop.shutdown();
-            assertThrows(ErrorCode.CONNECTION_BROKEN_1, this).
-            getConnection("jdbc:h2:tcp://localhost:9006/mem:", "sa", "sa");
+            assertThrows(ErrorCode.CONNECTION_BROKEN_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:9006/mem:", "sa", "sa"));
         } finally {
             shutdownServers();
         }
     }
 
-    private String runServer(int exitCode, String... args) {
+    private String runServer(int exitCode, String... args) throws Exception {
         ByteArrayOutputStream buff = new ByteArrayOutputStream();
-        PrintStream ps = new PrintStream(buff);
+        PrintStream ps = new PrintStream(buff, false, "UTF-8");
         if (server != null) {
             remainingServers.add(server);
         }
@@ -667,8 +695,7 @@ public class TestTools extends TestBase {
         }
         assertEquals(exitCode, result);
         ps.flush();
-        String s = new String(buff.toByteArray());
-        return s;
+        return Utils10.byteArrayOutputStreamToString(buff, StandardCharsets.UTF_8);
     }
 
     private void shutdownServers() {
@@ -691,7 +718,7 @@ public class TestTools extends TestBase {
         Connection conn = getConnection(url + ";TRACE_LEVEL_FILE=3", "sa", "sa");
         Statement stat = conn.createStatement();
         stat.execute(
-                "create table test(id int primary key, name varchar, amount decimal)");
+                "create table test(id int primary key, name varchar, amount decimal(4, 2))");
         PreparedStatement prep = conn.prepareStatement(
                 "insert into test values(?, ?, ?)");
         prep.setInt(1, 1);
@@ -700,7 +727,7 @@ public class TestTools extends TestBase {
         prep.executeUpdate();
         stat.execute("create table test2(id int primary key,\n" +
                 "a real, b double, c bigint,\n" +
-                "d smallint, e boolean, f binary, g date, h time, i timestamp)",
+                "d smallint, e boolean, f varbinary, g date, h time, i timestamp)",
                 Statement.NO_GENERATED_KEYS);
         prep = conn.prepareStatement(
                 "insert into test2 values(1, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
@@ -743,8 +770,7 @@ public class TestTools extends TestBase {
 
     private void testTraceFile(String url) throws SQLException {
         Connection conn;
-        Recover.main("-removePassword", "-dir", getBaseDir(), "-db",
-                "toolsConvertTraceFile");
+        Recover.main("-dir", getBaseDir(), "-db", "toolsConvertTraceFile");
         conn = getConnection(url, "sa", "");
         Statement stat = conn.createStatement();
         ResultSet rs;
@@ -760,7 +786,7 @@ public class TestTools extends TestBase {
         assertEquals(Double.MIN_VALUE, rs.getDouble("b"));
         assertEquals(Long.MIN_VALUE, rs.getLong("c"));
         assertEquals(Short.MIN_VALUE, rs.getShort("d"));
-        assertTrue(!rs.getBoolean("e"));
+        assertFalse(rs.getBoolean("e"));
         assertEquals(new byte[] { (byte) 10, (byte) 20 }, rs.getBytes("f"));
         assertEquals("2007-12-31", rs.getString("g"));
         assertEquals("23:59:59", rs.getString("h"));
@@ -769,32 +795,6 @@ public class TestTools extends TestBase {
         conn.close();
     }
 
-    private void testRemove() throws SQLException {
-        if (config.mvStore) {
-            return;
-        }
-        deleteDb("toolsRemove");
-        org.h2.Driver.load();
-        String url = "jdbc:h2:" + getBaseDir() + "/toolsRemove";
-        Connection conn = getConnection(url, "sa", "sa");
-        Statement stat = conn.createStatement();
-        stat.execute("create table test(id int primary key, name varchar)");
-        stat.execute("insert into test values(1, 'Hello')");
-        conn.close();
-        Recover.main("-dir", getBaseDir(), "-db", "toolsRemove",
-                "-removePassword");
-        conn = getConnection(url, "sa", "");
-        stat = conn.createStatement();
-        ResultSet rs;
-        rs = stat.executeQuery("select * from test");
-        rs.next();
-        assertEquals(1, rs.getInt(1));
-        assertEquals("Hello", rs.getString(2));
-        conn.close();
-        deleteDb("toolsRemove");
-        FileUtils.delete(getBaseDir() + "/toolsRemove.h2.sql");
-    }
-
     private void testRecover() throws SQLException {
         if (config.memory) {
             return;
@@ -867,7 +867,6 @@ public class TestTools extends TestBase {
     }
 
     private void testScriptRunscriptLob() throws Exception {
-        org.h2.Driver.load();
         String url = getURL("jdbc:h2:" + getBaseDir() +
                 "/testScriptRunscriptLob", true);
         String user = "sa", password = "abc";
@@ -902,8 +901,8 @@ public class TestTools extends TestBase {
                     "SELECT * FROM TEST ORDER BY ID");
             rs.next();
             assertEquals(1, rs.getInt(1));
-            assertTrue(rs.getString(2) == null);
-            assertTrue(rs.getString(3) == null);
+            assertNull(rs.getString(2));
+            assertNull(rs.getString(3));
             rs.next();
             assertEquals(2, rs.getInt(1));
             assertEquals("face", rs.getString(2));
@@ -928,8 +927,7 @@ public class TestTools extends TestBase {
 
     }
 
-    private void testScriptRunscript() throws SQLException {
-        org.h2.Driver.load();
+    private void testScriptRunscript() throws Exception {
         String url = getURL("jdbc:h2:" + getBaseDir() + "/testScriptRunscript",
                 true);
         String user = "sa", password = "abc";
@@ -966,10 +964,10 @@ public class TestTools extends TestBase {
                 "-quiet");
         RunScript tool = new RunScript();
         ByteArrayOutputStream buff = new ByteArrayOutputStream();
-        tool.setOut(new PrintStream(buff));
+        tool.setOut(new PrintStream(buff, false, "UTF-8"));
         tool.runTool("-url", url, "-user", user, "-password", password,
                 "-script", fileName + ".txt", "-showResults");
-        assertContains(buff.toString(), "Hello");
+        assertContains(Utils10.byteArrayOutputStreamToString(buff, StandardCharsets.UTF_8), "Hello");
 
 
         // test parsing of BLOCKSIZE option
@@ -1008,14 +1006,9 @@ public class TestTools extends TestBase {
                 .executeQuery("SELECT * FROM TEST");
         assertTrue(rs.next());
         assertFalse(rs.next());
-        new AssertThrows(ErrorCode.CANNOT_CHANGE_SETTING_WHEN_OPEN_1) {
-            @Override
-            public void test() throws SQLException {
-                // must fail when the database is in use
-                Backup.main("-file", fileName, "-dir", getBaseDir(), "-db",
-                        "testBackupRestore");
-            }
-        };
+        // must fail when the database is in use
+        assertThrows(ErrorCode.CANNOT_CHANGE_SETTING_WHEN_OPEN_1,
+                () -> Backup.main("-file", fileName, "-dir", getBaseDir(), "-db", "testBackupRestore"));
         conn.close();
         DeleteDbFiles.main("-dir", getBaseDir(), "-db", "testBackupRestore",
                 "-quiet");
@@ -1033,21 +1026,16 @@ public class TestTools extends TestBase {
         conn.close();
         String[] args = { "-dir", dir, "-db", "testChangeFileEncryption",
                 "-cipher", "AES", "-decrypt", "abc", "-quiet" };
-        ChangeFileEncryption.main(args);
+        new ChangeFileEncryption().runTool(args);
         args = new String[] { "-dir", dir, "-db", "testChangeFileEncryption",
                 "-cipher", "AES", "-encrypt", "def", "-quiet" };
-        ChangeFileEncryption.main(args);
+        new ChangeFileEncryption().runTool(args);
         conn = getConnection(url, "sa", "def 123");
         stat = conn.createStatement();
         stat.execute("SELECT * FROM TEST");
-        new AssertThrows(ErrorCode.CANNOT_CHANGE_SETTING_WHEN_OPEN_1) {
-            @Override
-            public void test() throws SQLException {
-                ChangeFileEncryption.main(new String[] { "-dir", dir, "-db",
-                        "testChangeFileEncryption", "-cipher", "AES",
-                        "-decrypt", "def", "-quiet" });
-            }
-        };
+        assertThrows(ErrorCode.CANNOT_CHANGE_SETTING_WHEN_OPEN_1,
+                () -> new ChangeFileEncryption().runTool(new String[] { "-dir", dir, "-db", "testChangeFileEncryption",
+                        "-cipher", "AES", "-decrypt", "def", "-quiet" }));
         conn.close();
         args = new String[] { "-dir", dir, "-db", "testChangeFileEncryption",
                 "-quiet" };
@@ -1055,14 +1043,8 @@ public class TestTools extends TestBase {
     }
 
     private void testChangeFileEncryptionWithWrongPassword() throws SQLException {
-        if (config.mvStore) {
-            // the file system encryption abstraction used by the MVStore
-            // doesn't detect wrong passwords
-            return;
-        }
         org.h2.Driver.load();
         final String dir = getBaseDir();
-        // TODO: this doesn't seem to work in MVSTORE mode yet
         String url = "jdbc:h2:" + dir + "/testChangeFileEncryption;CIPHER=AES";
         DeleteDbFiles.execute(dir, "testChangeFileEncryption", true);
         Connection conn = getConnection(url, "sa", "abc 123");
@@ -1072,14 +1054,8 @@ public class TestTools extends TestBase {
         conn.close();
         // try with wrong password, this used to have a bug where it kept the
         // file handle open
-        new AssertThrows(SQLException.class) {
-            @Override
-            public void test() throws SQLException {
-                ChangeFileEncryption.execute(dir, "testChangeFileEncryption",
-                        "AES", "wrong".toCharArray(),
-                        "def".toCharArray(), true);
-            }
-        };
+        assertThrows(SQLException.class, () -> ChangeFileEncryption.execute(dir, "testChangeFileEncryption", "AES",
+                "wrong".toCharArray(), "def".toCharArray(), true));
         ChangeFileEncryption.execute(dir, "testChangeFileEncryption",
                 "AES", "abc".toCharArray(), "def".toCharArray(),
                 true);
@@ -1096,24 +1072,20 @@ public class TestTools extends TestBase {
         Connection conn;
         try {
             deleteDb("test");
-            Server tcpServer = Server.createTcpServer(
+            Server tcpServer = Server.createTcpServer("-ifNotExists",
                             "-baseDir", getBaseDir(),
                             "-tcpAllowOthers").start();
             remainingServers.add(tcpServer);
             final int port = tcpServer.getPort();
-            conn = getConnection("jdbc:h2:tcp://localhost:"+ port +"/test", "sa", "");
+            conn = getConnection("jdbc:h2:tcp://localhost:" + port + "/test", "sa", "");
             conn.close();
             // must not be able to use a different base dir
-            new AssertThrows(ErrorCode.IO_EXCEPTION_1) {
-                @Override
-                public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:"+ port +"/../test", "sa", "");
-            }};
-            new AssertThrows(ErrorCode.IO_EXCEPTION_1) {
-                @Override
-                public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:"+port+"/../test2/test", "sa", "");
-            }};
+            assertThrows(ErrorCode.IO_EXCEPTION_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:" + port + "/../test", "sa", ""));
+            assertThrows(ErrorCode.IO_EXCEPTION_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:" + port + "/../test2/test", "sa", ""));
+            assertThrows(ErrorCode.WRONG_USER_OR_PASSWORD,
+                    () -> Server.shutdownTcpServer("tcp://localhost:" + port, "", true, false));
             tcpServer.stop();
             Server tcpServerWithPassword = Server.createTcpServer(
                             "-ifExists",
@@ -1122,43 +1094,34 @@ public class TestTools extends TestBase {
             final int prt = tcpServerWithPassword.getPort();
             remainingServers.add(tcpServerWithPassword);
             // must not be able to create new db
-            new AssertThrows(ErrorCode.DATABASE_NOT_FOUND_1) {
-                @Override
-                public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:"+prt+"/test2", "sa", "");
-            }};
-            new AssertThrows(ErrorCode.DATABASE_NOT_FOUND_1) {
-                @Override
-                public void test() throws SQLException {
-                    getConnection("jdbc:h2:tcp://localhost:"+prt+"/test2;ifexists=false", "sa", "");
-            }};
-            conn = getConnection("jdbc:h2:tcp://localhost:"+prt+"/test", "sa", "");
+            assertThrows(ErrorCode.REMOTE_DATABASE_NOT_FOUND_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:" + prt + "/test2", "sa", ""));
+            assertThrows(ErrorCode.REMOTE_DATABASE_NOT_FOUND_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:" + prt + "/test2;ifexists=false", "sa", ""));
+            conn = getConnection("jdbc:h2:tcp://localhost:" + prt + "/test", "sa", "");
             conn.close();
-            new AssertThrows(ErrorCode.WRONG_USER_OR_PASSWORD) {
-                @Override
-                public void test() throws SQLException {
-                    Server.shutdownTcpServer("tcp://localhost:"+prt, "", true, false);
-            }};
-            conn = getConnection("jdbc:h2:tcp://localhost:"+prt+"/test", "sa", "");
+            assertThrows(ErrorCode.WRONG_USER_OR_PASSWORD,
+                    () -> Server.shutdownTcpServer("tcp://localhost:" + prt, "", true, false));
+            conn = getConnection("jdbc:h2:tcp://localhost:" + prt + "/test", "sa", "");
             // conn.close();
-            Server.shutdownTcpServer("tcp://localhost:"+prt, "abc", true, false);
+            Server.shutdownTcpServer("tcp://localhost:" + prt, "abc", true, false);
             // check that the database is closed
             deleteDb("test");
             // server must have been closed
-            assertThrows(ErrorCode.CONNECTION_BROKEN_1, this).
-                    getConnection("jdbc:h2:tcp://localhost:"+prt+"/test", "sa", "");
+            assertThrows(ErrorCode.CONNECTION_BROKEN_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:" + prt + "/test", "sa", ""));
             JdbcUtils.closeSilently(conn);
             // Test filesystem prefix and escape from baseDir
             deleteDb("testSplit");
-            server = Server.createTcpServer(
+            server = Server.createTcpServer("-ifNotExists",
                             "-baseDir", getBaseDir(),
                             "-tcpAllowOthers").start();
             final int p = server.getPort();
-            conn = getConnection("jdbc:h2:tcp://localhost:"+p+"/split:testSplit", "sa", "");
+            conn = getConnection("jdbc:h2:tcp://localhost:" + p + "/split:testSplit", "sa", "");
             conn.close();
 
-            assertThrows(ErrorCode.IO_EXCEPTION_1, this).
-                    getConnection("jdbc:h2:tcp://localhost:"+p+"/../test", "sa", "");
+            assertThrows(ErrorCode.IO_EXCEPTION_1,
+                    () -> getConnection("jdbc:h2:tcp://localhost:" + p + "/../test", "sa", ""));
 
             server.stop();
             deleteDb("testSplit");
@@ -1170,7 +1133,7 @@ public class TestTools extends TestBase {
     /**
      * A simple Clob implementation.
      */
-    class SimpleClob implements Clob {
+    static class SimpleClob implements Clob {
 
         private final String data;
 
@@ -1260,7 +1223,7 @@ public class TestTools extends TestBase {
     /**
      * A simple Blob implementation.
      */
-    class SimpleBlob implements Blob {
+    static class SimpleBlob implements Blob {
 
         private final byte[] data;
 
