@@ -1,15 +1,17 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.unit;
 
 import java.io.ByteArrayOutputStream;
 import java.io.PrintStream;
+import java.nio.charset.StandardCharsets;
 import org.h2.message.TraceSystem;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.util.Utils10;
 
 /**
  * Tests the trace system
@@ -22,7 +24,7 @@ public class TestTraceSystem extends TestBase {
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
-        TestBase.createCaller().init().test();
+        TestBase.createCaller().init().testFromMain();
     }
 
     @Override
@@ -48,14 +50,14 @@ public class TestTraceSystem extends TestBase {
         ts.close();
     }
 
-    private void testTraceDebug() {
+    private void testTraceDebug() throws Exception {
         TraceSystem ts = new TraceSystem(null);
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        ts.setSysOut(new PrintStream(out));
+        ts.setSysOut(new PrintStream(out, false, "UTF-8"));
         ts.setLevelSystemOut(TraceSystem.DEBUG);
         ts.getTrace("test").debug(new Exception("error"), "test");
         ts.close();
-        String outString = new String(out.toByteArray());
+        String outString = Utils10.byteArrayOutputStreamToString(out, StandardCharsets.UTF_8);
         assertContains(outString, "error");
         assertContains(outString, "Exception");
         assertContains(outString, "test");
