@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.utils;
@@ -46,7 +46,7 @@ public class OutputCatcher {
         System.setOut(out.print);
         System.err.flush();
         System.setErr(err.print);
-        output = new String(buff.toByteArray());
+        output = buff.toString();
     }
 
     /**
