@@ -1,12 +1,12 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.utils;
 
 import java.sql.Timestamp;
-import java.util.Map;
+import org.h2.util.ThreadDeadlockDetector;
 
 /**
  * This is a self-destructor class to kill a long running process automatically
@@ -55,17 +55,8 @@ public class SelfDestructor {
                     System.out.println(time + " Killing the process after " +
                             minutes + " minute(s)");
                     try {
-                        Map<Thread, StackTraceElement[]> map =
-                                Thread.getAllStackTraces();
-                        for (Map.Entry<Thread, StackTraceElement[]> en :
-                                map.entrySet()) {
-                            System.out.println(en.getKey());
-                            for (StackTraceElement el : en.getValue()) {
-                                System.out.println("  " + el);
-                            }
-                        }
-                        System.out.println();
-                        System.out.flush();
+                        ThreadDeadlockDetector.dumpAllThreadsAndLocks(
+                                "SelfDestructor timed out", System.err);
                         try {
                             Thread.sleep(1000);
                         } catch (Exception e) {
