@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.dev.fs;
@@ -196,7 +196,7 @@ public class ArchiveTool {
         return new InputStream() {
 
             private final String baseDir;
-            private final ArrayList<String> files = new ArrayList<String>();
+            private final ArrayList<String> files = new ArrayList<>();
             private String current;
             private ByteArrayInputStream meta;
             private DataInputStream fileIn;
@@ -255,7 +255,7 @@ public class ArchiveTool {
                     fileIn.close();
                     fileIn = null;
                 }
-                if (files.size() == 0) {
+                if (files.isEmpty()) {
                     // EOF
                     return -1;
                 }
@@ -407,7 +407,7 @@ public class ArchiveTool {
         DataOutputStream tempOut = new DataOutputStream(new BufferedOutputStream(
                 new FileOutputStream(tempFileName), 1024 * 1024));
         byte[] bytes = new byte[bufferSize];
-        List<Long> segmentStart = new ArrayList<Long>();
+        List<Long> segmentStart = new ArrayList<>();
         long outPos = 0;
         long id = 1;
 
@@ -422,18 +422,17 @@ public class ArchiveTool {
                 break;
             }
             log.printProgress(len);
-            TreeMap<Chunk, Chunk> map = new TreeMap<Chunk, Chunk>();
+            TreeMap<Chunk, Chunk> map = new TreeMap<>();
             for (int pos = 0; pos < len;) {
                 int[] key = getKey(bytes, pos, len);
                 int l = key[3];
-                byte[] buff = new byte[l];
-                System.arraycopy(bytes, pos, buff, 0, l);
+                byte[] buff = Arrays.copyOfRange(bytes, pos, pos + l);
                 pos += l;
                 Chunk c = new Chunk(null, key, buff);
                 Chunk old = map.get(c);
                 if (old == null) {
                     // new entry
-                    c.idList = new ArrayList<Long>();
+                    c.idList = new ArrayList<>();
                     c.idList.add(id);
                     map.put(c, c);
                 } else {
@@ -459,7 +458,7 @@ public class ArchiveTool {
             log.setRange(30, 50, tempSize);
             log.println();
             log.println("Merging " + segmentStart.size() + " segments " + blockSize + ":1");
-            ArrayList<Long> segmentStart2 = new ArrayList<Long>();
+            ArrayList<Long> segmentStart2 = new ArrayList<>();
             outPos = 0;
             DataOutputStream tempOut2 = new DataOutputStream(new BufferedOutputStream(
                     new FileOutputStream(tempFileName + ".b"), 1024 * 1024));
@@ -467,7 +466,7 @@ public class ArchiveTool {
                 segmentStart2.add(outPos);
                 int s = Math.min(segmentStart.size(), blockSize);
                 List<Long> start = segmentStart.subList(0, s);
-                TreeSet<ChunkStream> segmentIn = new TreeSet<ChunkStream>();
+                TreeSet<ChunkStream> segmentIn = new TreeSet<>();
                 long read = openSegments(start, segmentIn, tempFileName, true);
                 log.printProgress(read);
                 Chunk last = null;
@@ -477,9 +476,7 @@ public class ArchiveTool {
                     if (last == null) {
                         last = c;
                     } else if (last.compareTo(c) == 0) {
-                        for (long x : c.idList) {
-                            last.idList.add(x);
-                        }
+                        last.idList.addAll(c.idList);
                     } else {
                         outPos += last.write(tempOut2, true);
                         last = c;
@@ -503,7 +500,7 @@ public class ArchiveTool {
             log.println("Combining " + segmentStart.size() + " segments");
         }
 
-        TreeSet<ChunkStream> segmentIn = new TreeSet<ChunkStream>();
+        TreeSet<ChunkStream> segmentIn = new TreeSet<>();
         long read = openSegments(segmentStart, segmentIn, tempFileName, true);
         log.printProgress(read);
 
@@ -521,9 +518,7 @@ public class ArchiveTool {
             if (last == null) {
                 last = c;
             } else if (last.compareTo(c) == 0) {
-                for (long x : c.idList) {
-                    last.idList.add(x);
-                }
+                last.idList.addAll(c.idList);
             } else {
                 last.write(dataOut, false);
                 last = c;
@@ -567,7 +562,7 @@ public class ArchiveTool {
 
             @Override
             public boolean hasNext() {
-                return segmentIn.size() > 0;
+                return !segmentIn.isEmpty();
             }
 
             @Override
@@ -583,11 +578,6 @@ public class ArchiveTool {
                 return c;
             }
 
-            @Override
-            public void remove() {
-                throw new UnsupportedOperationException();
-            }
-
         };
     }
 
@@ -656,7 +646,7 @@ public class ArchiveTool {
             }
         }
         int[] key = new int[4];
-        ; // TODO test if cs makes a difference
+        // TODO test if cs makes a difference
         key[0] = (int) (min >>> 32);
         key[1] = (int) min;
         key[2] = cs;
@@ -790,7 +780,7 @@ public class ArchiveTool {
         }
         long size = readVarLong(dataIn);
         long outPos = 0;
-        List<Long> segmentStart = new ArrayList<Long>();
+        List<Long> segmentStart = new ArrayList<>();
         boolean end = false;
 
         // Temp file: segment* 0
@@ -799,7 +789,7 @@ public class ArchiveTool {
         log.setRange(0, 30, size);
         while (!end) {
             int segmentSize = 0;
-            TreeMap<Long, byte[]> map = new TreeMap<Long, byte[]>();
+            TreeMap<Long, byte[]> map = new TreeMap<>();
             while (segmentSize < bufferSize) {
                 Chunk c = Chunk.read(dataIn, false);
                 if (c == null) {
@@ -839,7 +829,7 @@ public class ArchiveTool {
             log.setRange(30, 50, tempSize);
             log.println();
             log.println("Merging " + segmentStart.size() + " segments " + blockSize + ":1");
-            ArrayList<Long> segmentStart2 = new ArrayList<Long>();
+            ArrayList<Long> segmentStart2 = new ArrayList<>();
             outPos = 0;
             DataOutputStream tempOut2 = new DataOutputStream(new BufferedOutputStream(
                     new FileOutputStream(tempFileName + ".b"), 1024 * 1024));
@@ -847,7 +837,7 @@ public class ArchiveTool {
                 segmentStart2.add(outPos);
                 int s = Math.min(segmentStart.size(), blockSize);
                 List<Long> start = segmentStart.subList(0, s);
-                TreeSet<ChunkStream> segmentIn = new TreeSet<ChunkStream>();
+                TreeSet<ChunkStream> segmentIn = new TreeSet<>();
                 long read = openSegments(start, segmentIn, tempFileName, false);
                 log.printProgress(read);
 
@@ -875,7 +865,7 @@ public class ArchiveTool {
             log.println("Combining " + segmentStart.size() + " segments");
         }
 
-        TreeSet<ChunkStream> segmentIn = new TreeSet<ChunkStream>();
+        TreeSet<ChunkStream> segmentIn = new TreeSet<>();
         DataOutputStream dataOut = new DataOutputStream(out);
         log.setRange(50, 100, size);
 
@@ -933,7 +923,7 @@ public class ArchiveTool {
     static class Chunk implements Comparable<Chunk> {
         ArrayList<Long> idList;
         final byte[] value;
-        private int[] sortKey;
+        private final int[] sortKey;
 
         Chunk(ArrayList<Long> idList, int[] sortKey, byte[] value) {
             this.idList = idList;
@@ -950,7 +940,7 @@ public class ArchiveTool {
          */
         public static Chunk read(DataInputStream in, boolean readKey) {
             try {
-                ArrayList<Long> idList = new ArrayList<Long>();
+                ArrayList<Long> idList = new ArrayList<>();
                 while (true) {
                     long x = readVarLong(in);
                     if (x == 0) {
@@ -958,7 +948,7 @@ public class ArchiveTool {
                     }
                     idList.add(x);
                 }
-                if (idList.size() == 0) {
+                if (idList.isEmpty()) {
                     // eof
                     in.close();
                     return null;
