@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.dev.ftp;
@@ -17,11 +17,10 @@ import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
 import java.net.InetAddress;
 import java.net.Socket;
+import java.nio.charset.StandardCharsets;
 
-import org.h2.engine.Constants;
 import org.h2.util.IOUtils;
 import org.h2.util.NetUtils;
-import org.h2.util.StatementBuilder;
 import org.h2.util.StringUtils;
 
 /**
@@ -57,7 +56,7 @@ public class FtpClient {
         InputStream in = socket.getInputStream();
         OutputStream out = socket.getOutputStream();
         reader = new BufferedReader(new InputStreamReader(in));
-        writer = new PrintWriter(new OutputStreamWriter(out, Constants.UTF8));
+        writer = new PrintWriter(new OutputStreamWriter(out, StandardCharsets.UTF_8));
         readCode(220);
     }
 
@@ -230,12 +229,14 @@ public class FtpClient {
         int last = message.indexOf(')');
         String[] address = StringUtils.arraySplit(
                 message.substring(first, last), ',', true);
-        StatementBuilder buff = new StatementBuilder();
+        StringBuilder builder = new StringBuilder();
         for (int i = 0; i < 4; i++) {
-            buff.appendExceptFirst(".");
-            buff.append(address[i]);
+            if (i > 0) {
+                builder.append('.');
+            }
+            builder.append(address[i]);
         }
-        String ip = buff.toString();
+        String ip = builder.toString();
         InetAddress addr = InetAddress.getByName(ip);
         int port = (Integer.parseInt(address[4]) << 8) | Integer.parseInt(address[5]);
         Socket socketData = NetUtils.createSocket(addr, port, false);
@@ -377,8 +378,7 @@ public class FtpClient {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         IOUtils.copyAndClose(inData, out);
         readCode(226);
-        byte[] data = out.toByteArray();
-        return new String(data);
+        return out.toString();
     }
 
     /**
@@ -394,8 +394,7 @@ public class FtpClient {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         IOUtils.copyAndClose(inData, out);
         readCode(226);
-        byte[] data = out.toByteArray();
-        return new String(data);
+        return out.toString();
     }
 
     /**
