@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.dev.ftp.server;
@@ -28,7 +28,7 @@ import org.h2.util.Tool;
 /**
  * Small FTP Server. Intended for ad-hoc networks in a secure environment.
  * Remote connections are possible.
- * See also http://cr.yp.to/ftp.html http://www.ftpguide.com/
+ * See also https://cr.yp.to/ftp.html http://www.ftpguide.com/
  */
 public class FtpServer extends Tool implements Service {
 
@@ -91,9 +91,10 @@ public class FtpServer extends Tool implements Service {
 
     /**
      * When running without options, -tcp, -web, -browser,
-     * and -pg are started.<br />
-     * Options are case sensitive. Supported options are:
+     * and -pg are started.
+     * Options are case sensitive.
      * <table>
+     * <caption>Supported options</caption>
      * <tr><td>[-help] or [-?]</td>
      * <td>Print the list of options</td></tr>
      * <tr><td>[-web]</td>
@@ -145,7 +146,6 @@ public class FtpServer extends Tool implements Service {
      * <tr><td>[-trace]</td>
      * <td>Print additional trace information; for all servers</td></tr>
      * </table>
-     * @h2.resource
      *
      * @param args the command line arguments
      */
@@ -230,7 +230,7 @@ public class FtpServer extends Tool implements Service {
         buff.append('r');
         buff.append(FileUtils.canWrite(fileName) ? 'w' : '-');
         buff.append("------- 1 owner group ");
-        String size = String.valueOf(FileUtils.size(fileName));
+        String size = Long.toString(FileUtils.size(fileName));
         for (int i = size.length(); i < 15; i++) {
             buff.append(' ');
         }
