@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.dev.hash;
@@ -499,9 +499,7 @@ public class MinimalPerfectHash<K> {
             for (ByteArrayOutputStream temp : outList) {
                 out.write(temp.toByteArray());
             }
-        } catch (InterruptedException e) {
-            throw new RuntimeException(e);
-        } catch (IOException e) {
+        } catch (InterruptedException | IOException e) {
             throw new RuntimeException(e);
         }
     }
@@ -658,7 +656,7 @@ public class MinimalPerfectHash<K> {
             if (index == 0) {
                 return o.hashCode();
             } else if (index < 8) {
-                long x = o.longValue();
+                long x = o;
                 x += index;
                 x = ((x >>> 32) ^ x) * 0x45d9f3b;
                 x = ((x >>> 32) ^ x) * 0x45d9f3b;
@@ -666,7 +664,7 @@ public class MinimalPerfectHash<K> {
             }
             // get the lower or higher 32 bit depending on the index
             int shift = (index & 1) * 32;
-            return (int) (o.longValue() >>> shift);
+            return (int) (o >>> shift);
         }
 
     }
