@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.dev.net;
@@ -30,14 +30,14 @@ public class PgTcpRedirect {
      * @param args the command line parameters
      */
     public static void main(String... args) throws Exception {
-        new PgTcpRedirect().loop(args);
+        loop(args);
     }
 
-    private void loop(String... args) throws Exception {
+    private static void loop(String... args) throws Exception {
         // MySQL protocol:
         // http://www.redferni.uklinux.net/mysql/MySQL-Protocol.html
         // PostgreSQL protocol:
-        // http://developer.postgresql.org/pgdocs/postgres/protocol.html
+        // https://www.postgresql.org/docs/devel/protocol.html
         // int portServer = 9083, portClient = 9084;
         // int portServer = 3306, portClient = 3307;
         // H2 PgServer
@@ -66,7 +66,7 @@ public class PgTcpRedirect {
     /**
      * This is the working thread of the TCP redirector.
      */
-    private class TcpRedirectThread implements Runnable {
+    private static class TcpRedirectThread implements Runnable {
 
         private static final int STATE_INIT_CLIENT = 0, STATE_REGULAR = 1;
         private final Socket read, write;
@@ -92,7 +92,7 @@ public class PgTcpRedirect {
             return buff.toString();
         }
 
-        private void println(String s) {
+        private static void println(String s) {
             if (DEBUG) {
                 System.out.println(s);
             }
@@ -385,7 +385,7 @@ public class PgTcpRedirect {
                         break;
                     }
                     String msg = readStringNull(dataIn);
-                    // http://developer.postgresql.org/pgdocs/postgres/protocol-error-fields.html
+                    // https://www.postgresql.org/docs/devel/protocol-error-fields.html
                     // S Severity
                     // C Code: the SQLSTATE code
                     // M Message
@@ -420,7 +420,7 @@ public class PgTcpRedirect {
                         break;
                     }
                     String msg = readStringNull(dataIn);
-                    // http://developer.postgresql.org/pgdocs/postgres/protocol-error-fields.html
+                    // https://www.postgresql.org/docs/devel/protocol-error-fields.html
                     // S Severity
                     // C Code: the SQLSTATE code
                     // M Message
