@@ -1,15 +1,12 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.dev.util;
 
 import java.util.Iterator;
 
-import org.h2.mvstore.DataUtils;
-
-
 /**
  * A very simple linked list that supports concurrent access.
  * Internally, it uses immutable objects.
@@ -112,11 +109,6 @@ public class ConcurrentLinkedList<K> {
                 return x;
             }
 
-            @Override
-            public void remove() {
-                throw DataUtils.newUnsupportedOperationException("remove");
-            }
-
         };
     }
 
